/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */

/* INCLUDE FILES **************************************************************/

#include "ControlView.h"

#include <QtGui/QLayout>
#include <QtGui/QSplitter>
#include <QtGui/QStringListModel>
#include <QtGui/QPrintDialog>
#include <QtGui/QPrintPreviewDialog>
#include <QtGui/QPrinter>
#include <QtGui/QPainter>
#include <QtGui/QTextDocument>
#include <QtCore/QPointer>
#include <QtCore/QUrl>
#include <QtNetwork/QNetworkProxy>
#include <QtGui/QStandardItemModel>
#include <QtGui/QStandardItem>
#include <QtCore/QFileInfo>

//#include <QTreeView>
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include <MarbleWidget.h>
#include <MarbleModel.h>
#include <MarbleMap.h>
#include <MapThemeManager.h>
#include "PrintOptionsWidget.h"
#include "ViewportParams.h"
#include "ViewParams.h"
#include "PluginManager.h"
#include "MarbleRunnerManager.h"
#include "MarbleWidgetInputHandler.h"
#include "routing/RoutingManager.h"
#include "routing/RoutingModel.h"
#include "routing/RouteRequest.h"
#include "routing/RoutingWidget.h"

#include "CurrentLocationWidget.h"

namespace Marble
{

ControlView::ControlView( QWidget *parent)
: WlanControlSplitter(Qt::Vertical, parent)
,	m_pHSplitter(NULL)
,	m_pLeftVSplitter(NULL)
,	m_pRightVSplitter(NULL)
,	m_pRoutingWidget(NULL)
,   m_pCurrentLocWidget(NULL)
,   m_pRunnerManager(NULL)
,	m_iconView(NULL)
,	m_treeView(NULL)
,	m_marbleWidget(NULL)
{
    setWindowTitle(tr("WlanMap"));
	setObjectName("CentralWidget");

    resize( 680, 640 );

 //   QVBoxLayout *vlayout = new QVBoxLayout( this );
 //   vlayout->setMargin(0);
	/**********************************************************************/
	m_pLeftVSplitter = new WlanControlSplitter(Qt::Vertical, this);
	m_pLeftVSplitter->setObjectName("WlanLeftVSplitter");

	m_pRightVSplitter = new WlanControlSplitter(Qt::Vertical, this);
	m_pRightVSplitter->setObjectName("WlanRightVSplitter");
	/*********************************************************************/

    //m_splitter = new QSplitter (this);
	m_pHSplitter = new WlanControlSplitter(Qt::Horizontal, this);
	m_pHSplitter->setObjectName("WlanControlHSplitter");
//    vlayout->addWidget( m_pHSplitter );

	m_marbleWidget = new MarbleWidget( this );
	m_treeView = new TreeViewWidget(this );
//	m_iconView = new IconViewWidget(m_marbleWidget );
//	m_treeView->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
  //                                              QSizePolicy::MinimumExpanding ) );
	m_pLeftVSplitter->addWidget( m_treeView  );
//	m_pLeftVSplitter->addWidget( m_iconView );

    m_marbleWidget->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
                                                QSizePolicy::MinimumExpanding ) );

	m_pHSplitter->addWidget(m_pLeftVSplitter);
	m_pHSplitter->addWidget( m_marbleWidget );
	m_pHSplitter->addWidget(m_pRightVSplitter);

	m_pHSplitter->setSizes( QList<int>() << 100 << width()-100 );
	m_pLeftVSplitter->setSizes( QList<int>() << height()-height()/4 << height()/4 );
	m_pRightVSplitter->setSizes( QList<int>() << height()-height()/4 << height()/4 );

	m_pHSplitter->setFrameShape(QFrame::StyledPanel);

	m_marbleWidget->model()->setWorkOffline(false);
	m_marbleWidget->setShowAtmosphere(true);
	m_marbleWidget->inputHandler()->setKineticScrollingEnabled(true);


    m_treeView->addMarbleWidget( m_marbleWidget );
	
	m_pCurrentLocWidget = new CurrentLocationWidget(this );
	m_pCurrentLocWidget->setMarbleWidget(m_marbleWidget);
	
	//m_pRightVSplitter->addWidget( m_pCurrentLocWidget );
	m_pLeftVSplitter->addWidget( m_pCurrentLocWidget );
	m_pRightVSplitter->hide();
	m_bRoute=false;

	addWidget(m_pHSplitter);
}

void ControlView::syncSettings(QString proxyUrl,QString proxyUser,QString proxyPass, int iPort)
{
    //d->m_settings->sync();
    
 /*   QNetworkProxy proxy;
    
    // Make sure that no proxy is used for an empty string or the default value: 
    if ( proxyUrl.isEmpty() || proxyUrl == "http://" ) {
        proxy.setType( QNetworkProxy::NoProxy );
    } else 
	{
    /*    if ( proxyType() == Marble::Socks5Proxy ) {
            proxy.setType( QNetworkProxy::Socks5Proxy );
        }
        else if ( proxyType() == Marble::HttpProxy ) {
            proxy.setType( QNetworkProxy::HttpProxy );
        }
        else {
            mDebug() << "Unknown proxy type! Using Http Proxy instead.";
            proxy.setType( QNetworkProxy::HttpProxy );
        }*/
/*		proxy.setType( QNetworkProxy::HttpProxy );
    }
    
    proxy.setHostName( proxyUrl );
    proxy.setPort( iPort );
    
    if ( 1 ) 
	{
        proxy.setUser( proxyUser );
        proxy.setPassword( proxyPass );
    }
    
    QNetworkProxy::setApplicationProxy(proxy);*/
}

ControlView::~ControlView()
{
	if(m_pRoutingWidget!=NULL)
		delete m_pRoutingWidget;

	if(m_pCurrentLocWidget!=NULL)
		delete m_pCurrentLocWidget;

	if(m_pRunnerManager!=NULL)
		delete m_pRunnerManager;
	
	if(m_iconView!=NULL)
		delete m_iconView;

	if(m_treeView!=NULL)
		delete m_treeView;

	if( m_marbleWidget!=NULL)
		delete  m_marbleWidget;

	if(m_pLeftVSplitter!=NULL)
		delete m_pLeftVSplitter;

	if(m_pRightVSplitter!=NULL)
		delete m_pRightVSplitter;

	if(m_pHSplitter!=NULL)
		delete m_pHSplitter;
}

QString ControlView::applicationVersion()
{
    return "1.3.80 (1.4 Beta 1)";
}

void ControlView::slot_resetViewsToFactoryDefaults()
{
//	CfgWrap::restoreSplitterFromSizeString(m_pHSplitter,m_plotSpitterSizesHorz);
//	CfgWrap::restoreSplitterFromSizeString(m_pLeftVSplitter,m_plotSpitterSizesVert);
//	CfgWrap::restoreSplitterFromSizeString(m_pRightVSplitter,m_plotSpitterSizesVert);

	m_pHSplitter->setSizes( QList<int>() << 100 << 560 );
	m_pLeftVSplitter->setSizes( QList<int>() << 480 << 160 );
	m_pRightVSplitter->setSizes( QList<int>() << 480 << 160 );
}

int ControlView::defaultMapThemeId(QStringList& installedThemes) const
{
    QStringList fallBackThemes;
	fallBackThemes << "earth/plain/plain.dgml";
    fallBackThemes << "earth/srtm/srtm.dgml";
    fallBackThemes << "earth/bluemarble/bluemarble.dgml";
    fallBackThemes << "earth/openstreetmap/openstreetmap.dgml";

	QString qsMap;
	QString qsMapName;
	int iRet=0;

    //QList<GeoSceneDocument const*> themes = m_mapThemeManager->mapThemes();
/*	QList<GeoSceneDocument const*> themes = m_marbleWidget->model()->mapThemeManager()->mapThemes();
    foreach(GeoSceneDocument const* theme, themes) 
	{
		qsMap = theme->head()->mapThemeId();
		qsMapName = theme->head()->name();
		if(!fallBackThemes.contains(qsMap))
		{
			installedThemes << qsMap;
			installedThemes << qsMapName;
			iRet++;
		}
    }*/
	
	int i=0;
	QStandardItem* pChild=NULL;
	QStandardItemModel* pItem = m_marbleWidget->model()->mapThemeManager()->mapThemeModel();
	int ic = m_marbleWidget->model()->mapThemeManager()->mapThemeModel()->rowCount();
	const QStringList installedAllThemes = m_marbleWidget->model()->mapThemeManager()->mapThemeIds();
	foreach(const QString &fallback,installedAllThemes) 
	{
        if(!fallBackThemes.contains(fallback))
		{
			qsMap = fallback;
			installedThemes << qsMap;
			if(pItem!=NULL)
			{
				pChild = pItem->item(i,0);
				if(pChild!=NULL)
					qsMapName = pChild->text();
				installedThemes << qsMapName;
				iRet++;
			}
		}
		++i;
    }

 /*   foreach(const QString &fallback, fallBackThemes) {
        if (installedThemes.contains(fallback)) {
            return fallback;
        }
    }

    if (installedThemes.size()) {
        return installedThemes.first();
    }

    return QString();*/

	return iRet;
}
void ControlView::search(const QString &searchTerm, SearchMode searchMode )
{
//    m_control->search( searchTerm, searchMode );

	MarbleModel* pModel = marbleModel();
	if(pModel==NULL)
	{
		emit searchFinished();
		return;
	}
	/*
	QList<RunnerPlugin*> allPlugins = pModel->pluginManager()->runnerPlugins();
    foreach( RunnerPlugin* plugin, allPlugins ) {
        if ( plugin->supports( RunnerPlugin::Search ) ) 
		{
			pModel->pluginManager()->
		}
		*/

	if(m_pRunnerManager==NULL)
	{
		m_pRunnerManager = new MarbleRunnerManager( pModel->pluginManager(), this );
		connect( m_pRunnerManager, SIGNAL( searchResultChanged( QVector<GeoDataPlacemark*> ) ),
		         marbleTree() , SLOT( setLocations( QVector<GeoDataPlacemark*> ) ) );
		connect( m_pRunnerManager, SIGNAL( searchFinished( QString ) ), this, SIGNAL( searchFinished() ) );
		m_pRunnerManager->setModel(pModel);
	}
	else if ( !searchTerm.isEmpty() )
		m_pRunnerManager->findPlacemarks(searchTerm);
	
}

void ControlView::zoomIn()
{
    m_marbleWidget->zoomIn();
}

void ControlView::zoomOut()
{
    m_marbleWidget->zoomOut();
}

void ControlView::moveLeft()
{
    m_marbleWidget->moveLeft();
}

void ControlView::moveRight()
{
    m_marbleWidget->moveRight();
}

void ControlView::moveUp()
{
    m_marbleWidget->moveUp();
}

void ControlView::moveDown()
{
    m_marbleWidget->moveDown();
}

void ControlView::setSideBarShown( bool show )
{
//    m_control->setVisible( show );
}

void ControlView::setNavigationTabShown( bool show )
{
//    m_control->setNavigationTabShown( show );
}

void ControlView::setLegendTabShown( bool show )
{
 //   m_control->setLegendTabShown( show );
}

void ControlView::setMapViewTabShown( bool show )
{
  //  m_control->setMapViewTabShown( show );
}

void ControlView::setCurrentLocationTabShown( bool show )
{
  //  m_control->setCurrentLocationTabShown( show );
}

void ControlView::setFileViewTabShown( bool show )
{
//    m_control->setFileViewTabShown( show );
}

void ControlView::showRoutingWidget(QWidget* pParent)
{
	if(m_pCurrentLocWidget!=NULL)
	{
		if(m_bRoute)
		{
			//m_pRoutingWidget->hide();
			m_pRightVSplitter->hide();
			//delete m_pRoutingWidget;
			//m_pRoutingWidget=NULL;
			m_bRoute=false;
		}
		else
		{
			m_pRightVSplitter->show();// m_pRoutingWidget->show();
			m_bRoute=true;
		}
	}
	/*
	else 
	{
		if(m_pRoutingWidget==NULL)
		{
			m_pRoutingWidget = new RoutingWidget(m_marbleWidget,this );
			m_pCurrentLocWidget = new CurrentLocationWidget(this );
			m_pCurrentLocWidget->setMarbleWidget(m_marbleWidget);
	//	m_pRoutingWidget->setShowDirectionsButtonVisible(true);
		//	QFrame frame(m_pRoutingWidget);
		//	frame.setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);
			m_pRightVSplitter->addWidget( m_pRoutingWidget );
			m_pRightVSplitter->addWidget( m_pCurrentLocWidget );
			
			//m_pHSplitter->addWidget( m_pRoutingWidget );
		}
		else m_pRightVSplitter->show();// m_pRoutingWidget->show();
		m_bRoute=true;
	}*/

//	pRoutingWidget->setShowDirectionsButtonVisible( true );
}

bool ControlView::importRoute(QWidget*)
{
	bool bRet=false;
	if(m_pRoutingWidget==NULL)
	{
		m_pRoutingWidget = new RoutingWidget(m_marbleWidget,this );
		m_pCurrentLocWidget = new CurrentLocationWidget(this );
		m_pCurrentLocWidget->setMarbleWidget(m_marbleWidget);
		//m_pRoutingWidget->setShowDirectionsButtonVisible(true);
		if(m_pRoutingWidget!=NULL)
		{
			//m_pHSplitter->addWidget( m_pRoutingWidget );
			m_pRightVSplitter->addWidget( m_pRoutingWidget );
			m_pRightVSplitter->addWidget( m_pCurrentLocWidget );
			if(m_pRoutingWidget->openRoute())
			{
				bRet=true;
			}
			else m_pRightVSplitter->hide();//m_pRoutingWidget->hide();
			m_bRoute=true;
		}
	}
	else
	{
		if(m_pRoutingWidget->openRoute())
		{
			bRet=true;
			if(!m_bRoute)
			{
				m_bRoute=true;
				//m_pRoutingWidget->show();
				m_pRightVSplitter->show();
			}
		}
	}

	return bRet;
}

void ControlView::setSrtmRunner(const ElevationRunnerPlugin* pRunner)
{
	m_pSrtmRunner = pRunner;
}

void ControlView::printMapScreenShot( QPointer<QPrintDialog> printDialog)
{
#ifndef QT_NO_PRINTER
  /*      PrintOptionsWidget* printOptions = new PrintOptionsWidget( this );
        bool const mapCoversViewport = m_marbleWidget->viewport()->mapCoversViewport();
        printOptions->setBackgroundControlsEnabled( !mapCoversViewport );
        bool hasLegend = m_marbleWidget->model()->legend() != 0;
        printOptions->setLegendControlsEnabled( hasLegend );
        bool hasRoute = marbleWidget()->model()->routingManager()->routingModel()->rowCount() > 0;
        printOptions->setPrintRouteSummary( hasRoute );
        printOptions->setPrintDrivingInstructions( hasRoute );
		printOptions->setPrintDrivingInstructionsAdvice( hasRoute );
        printOptions->setRouteControlsEnabled( hasRoute );
        printDialog->setOptionTabs( QList<QWidget*>() << printOptions );*/

        if ( printDialog->exec() == QDialog::Accepted ) {
            QTextDocument document;
            QString text = "<html><head><title>Marble Printout</title></head><body>";
            QPalette const originalPalette = m_marbleWidget->palette();
            bool const wasBackgroundVisible = m_marbleWidget->showBackground();
     //       bool const hideBackground = !mapCoversViewport && !printOptions->printBackground();
            if (wasBackgroundVisible)// hideBackground ) 
			{
                // Temporarily remove the black background and layers painting on it
                m_marbleWidget->setShowBackground( false );
                m_marbleWidget->setPalette( QPalette ( Qt::white ) );
                m_marbleWidget->update();
            }

            if (1)// printOptions->printMap() ) 
			{
                printMap( document, text, printDialog->printer() );
            }

      /*      if ( printOptions->printLegend() ) {
                printLegend( document, text );
            }

            if ( printOptions->printRouteSummary() ) {
                printRouteSummary( document, text );
            }

            if ( printOptions->printDrivingInstructions() ) {
                printDrivingInstructions( document, text );
            }

			if ( printOptions->printDrivingInstructionsAdvice() ) {
                printDrivingInstructionsAdvice( document, text );
            }*/

            text += "</body></html>";
            document.setHtml( text );
            document.print( printDialog->printer() );

            if (wasBackgroundVisible)// hideBackground ) 
			{
                m_marbleWidget->setShowBackground( wasBackgroundVisible );
                m_marbleWidget->setPalette( originalPalette );
                m_marbleWidget->update();
            }
    }
#endif
}
//print helper
void ControlView::printPixmap( QPrinter * printer, const QPixmap& pixmap  )
{
#ifndef QT_NO_PRINTER
    QSize printSize = pixmap.size();
    QRect mapPageRect = printer->pageRect();
    printSize.scale( printer->pageRect().size(), Qt::KeepAspectRatio );
    QPoint printTopLeft( ( mapPageRect.width() - printSize.width() ) / 2 ,
                         ( mapPageRect.height() - printSize.height() ) / 2 );
    QRect mapPrintRect( printTopLeft, printSize );

    QPainter painter;
    if (!painter.begin(printer))
        return;
    painter.drawPixmap( mapPrintRect, pixmap, pixmap.rect() );
    painter.end();
#endif
}

// QPointer is used because of issues described in http://www.kdedevelopers.org/node/3919
void ControlView::printPreview()
{
#ifndef QT_NO_PRINTER
    QPrinter printer( QPrinter::HighResolution );

	QWidget* pWiget = m_marbleWidget;

    QPointer<QPrintPreviewDialog> preview = new QPrintPreviewDialog( &printer,pWiget );
    preview->setWindowFlags ( Qt::Window );
    connect( preview, SIGNAL( paintRequested( QPrinter * ) ), SLOT( paintPrintPreview( QPrinter * ) ) );
    preview->exec();
    delete preview;
#endif
}

void ControlView::paintPrintPreview( QPrinter * printer )
{
#ifndef QT_NO_PRINTER
    QPixmap mapPixmap = mapScreenShot();
    printPixmap( printer, mapPixmap );
#endif
}

void ControlView::printMap( QTextDocument &document, QString &text, QPrinter *printer )
{
#ifndef QT_NO_PRINTER
    QPixmap image = mapScreenShot();

    if ( m_marbleWidget->viewport()->mapCoversViewport() ) {
        // Paint a black frame. Looks better.
        QPainter painter(&image);
        painter.setPen( Qt::black );
        painter.drawRect( 0, 0, image.width() - 2, image.height() - 2 );
    }

    QString uri = "marble://screenshot.png";
    document.addResource( QTextDocument::ImageResource, QUrl( uri ), QVariant( image) );
    QString img = "<img src=\"%1\" width=\"%2\" align=\"center\">";
    int width = qRound( printer->pageRect( QPrinter::Point ).width() );
    text += img.arg( uri ).arg( width );
#endif
}

void ControlView::printLegend( QTextDocument &document, QString &text )
{
#ifndef QT_NO_PRINTER
    QTextDocument *legend = m_marbleWidget->model()->legend();
    if ( legend ) {
        legend->adjustSize();
        QSize size = legend->size().toSize();
        QSize imageSize = size + QSize( 4, 4 );
        QImage image( imageSize, QImage::Format_ARGB32);
        QPainter painter( &image );
        painter.setRenderHint( QPainter::Antialiasing, true );
        painter.drawRoundedRect( QRect( QPoint( 0, 0 ), size ), 5, 5 );
        legend->drawContents( &painter );
        document.addResource( QTextDocument::ImageResource, QUrl( "marble://legend.png" ), QVariant(image) );
        QString img = "<p><img src=\"%1\" align=\"center\"></p>";
        text += img.arg( "marble://legend.png" );
    }
#endif
}

void ControlView::printRouteSummary( QTextDocument &document, QString &text)
{
#ifndef QT_NO_PRINTER
    RoutingModel* routingModel = m_marbleWidget->model()->routingManager()->routingModel();

    if ( !routingModel ) {
        return;
    }

    RouteRequest* routeRequest = m_marbleWidget->model()->routingManager()->routeRequest();
    if ( routeRequest ) {
        QString summary = "<h3>Route to %1: %2 %3</h3>";
        QString destination;
        if ( routeRequest->size() ) {
            destination = routeRequest->name( routeRequest->size()-1 );
        }

        QString label = "<p>%1 %2</p>";
        qreal distance = routingModel->route().distance();
        QString unit = distance > 1000 ? "km" : "m";
        int precision = distance > 1000 ? 1 : 0;
        if ( distance > 1000 ) {
            distance /= 1000;
        }
        summary = summary.arg(destination).arg( distance, 0, 'f', precision ).arg( unit );
        text += summary;

        text += "<table cellpadding=\"2\">";
        QString pixmapTemplate = "marble://viaPoint-%1.png";
        for ( int i=0; i<routeRequest->size(); ++i ) {
            text += "<tr><td>";
            QPixmap pixmap = routeRequest->pixmap(i);
            QString pixmapResource = pixmapTemplate.arg( i );
            document.addResource(QTextDocument::ImageResource,
                                          QUrl( pixmapResource ), QVariant( pixmap ) );
            QString myimg = "<img src=\"%1\">";
            text += myimg.arg( pixmapResource );
            text += "</td><td>";
            text += routeRequest->name( i );
            text += "</td></tr>";
        }
        text += "</table>";
    }
#endif
}

void ControlView::printDrivingInstructions( QTextDocument &document, QString &text )
{
#ifndef QT_NO_PRINTER
    RoutingModel* routingModel = m_marbleWidget->model()->routingManager()->routingModel();

    if (!routingModel) {
        return;
    }

    GeoDataLineString total = routingModel->route().path();

    text += "<table cellpadding=\"4\">";
    text += "<tr><th>No.</th><th>Distance</th><th>Instruction</th></tr>";
    for ( int i=0; i<routingModel->rowCount(); ++i ) {
        QModelIndex index = routingModel->index(i, 0);
        GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>( index.data( RoutingModel::CoordinateRole ) );
        GeoDataLineString accumulator;
        for (int k=0; k<total.size(); ++k) {
            accumulator << total.at(k);

            if (total.at(k) == coordinates)
                break;
        }

        if ( i%2 == 0 ) {
            text += "<tr bgcolor=\"lightGray\"><td align=\"right\" valign=\"middle\">";
        }
        else {
            text += "<tr><td align=\"right\" valign=\"middle\">";
        }
        text += QString::number( i+1 );
        text += "</td><td align=\"right\" valign=\"middle\">";

        text += QString::number( accumulator.length( EARTH_RADIUS ) * METER2KM, 'f', 1 );
        /** @todo: support localization */
        text += " km</td><td valign=\"middle\">";

        QPixmap instructionIcon = qVariantValue<QPixmap>( index.data( Qt::DecorationRole ) );
        if ( !instructionIcon.isNull() ) {
            QString uri = QString("marble://turnIcon%1.png").arg(i);
            document.addResource( QTextDocument::ImageResource, QUrl( uri ), QVariant( instructionIcon ) );
            text += QString("<img src=\"%1\">").arg(uri);
        }

        text += routingModel->data( index ).toString();
        text += "</td></tr>";
    }
    text += "</table>";
#endif
}

void ControlView::printDrivingInstructionsAdvice( QTextDocument &, QString &text )
{
#ifndef QT_NO_PRINTER
    text += "<p>" + tr( "The Marble development team wishes you a pleasant and safe journey." ) + "</p>";
    text += "<p>" + tr( "Caution: Driving instructions may be incomplete or inaccurate." );
    text += " " + tr( "Road construction, weather and other unforeseen variables can result in this suggested route not to be the most expedient or safest route to your destination." );
    text += " " + tr( "Please use common sense while navigating." ) + "</p>";
#endif
}

void ControlView::addGeoDataFile( QString filename )
{
    QFileInfo const file( filename );
    if ( file.exists() ) {
        m_marbleWidget->model()->addGeoDataFile( file.absoluteFilePath() );
    } else {
        qWarning() << "File" << filename << "does not exist, cannot open it.";
    }
}

/*SunLocator* ControlView::sunLocator()
{
    return m_marbleWidget->model()->sunLocator();
}*/

}

#include "moc_ControlView.cpp"


