/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CONTROLVIEW_H
#define CONTROLVIEW_H


#include <QtGui/QWidget>
#include <QtGui/QPixmap>
#include <QtCore/QPointer>

#include "WlanControlSplitter.h"

#include "MarbleWidget.h"
#include "MarbleControlBox.h"
#include "MarbleNavigator.h"
#include "./treeview/TreeViewWidget.h"
#include "./IconView/IconViewWidget.h"

//class QSplitter;
class QPrintDialog;
class QTextDocument;
//class QTreeView;

namespace Marble
{
class MarbleModel;
class RoutingWidget;
class CurrentLocationWidget;
class ElevationRunnerPlugin;
class MarbleRunnerManager;
//class SunLocator;

class ControlView : public WlanControlSplitter
{
    Q_OBJECT

 public:
    explicit ControlView( QWidget * = 0);
    virtual ~ControlView();

    /**
	* Returns the version of the Marble applications (which differs from
	* the Marble library version).
	*/
    static QString applicationVersion();


    MarbleWidget*			marbleWidget()  const { return m_marbleWidget; }
//  MarbleNavigator*  marbleControl()  const { return m_control;      }
	MarbleModel*			marbleModel()         { return m_marbleWidget->model(); }
	TreeViewWidget*			marbleTree()     const { return m_treeView; }
	IconViewWidget*			marbleIcon()     const { return m_iconView; }
	RoutingWidget*			marbleRouting()  const { return m_pRoutingWidget; }
	const ElevationRunnerPlugin*	srtmRunner()     const { return m_pSrtmRunner;}

	//OpenGl
	WlanControlSplitter*	getHSplitter(void){return m_pHSplitter;}

	int defaultMapThemeId(QStringList&) const;

    void zoomIn();
    void zoomOut();
    void moveLeft();
    void moveRight();
    void moveUp();
    void moveDown();

//    bool sideBarShown() const { return m_control->isVisible(); }

//    void addPlacemarkFile( QString filename ){ m_marbleWidget->addPlacemarkFile( filename ); }

	void addGeoDataFile( QString filename );

    QPixmap mapScreenShot(){ return m_marbleWidget->mapScreenShot(); }
    
   // SunLocator* sunLocator();

	//print
	void printPreview();

	void syncSettings(QString proxyUrl,QString proxyUser,QString proxyPass, int iPort);

	//routing
	void showRoutingWidget(QWidget*);
	bool importRoute(QWidget*);

	//srtm elevation data
	void setSrtmRunner(const ElevationRunnerPlugin* pRunner);
    
 signals:
     void showMapWizard();
     void showUploadDialog();
	 void searchFinished();

 public slots:
    void setSideBarShown( bool );    
    void setNavigationTabShown( bool );
    void setLegendTabShown( bool );
    void setMapViewTabShown( bool );
    void setCurrentLocationTabShown( bool );
    void setFileViewTabShown( bool );

	void printMapScreenShot( QPointer<QPrintDialog> dialog );
	void printPixmap( QPrinter * printer, const QPixmap& pixmap );
    void paintPrintPreview( QPrinter * printer );

	/**
		reset splitters to facotry defaults
	*/
	void slot_resetViewsToFactoryDefaults();

	/**
      * Search for the given term and display the results in the search result lists,
      * which is brought to the front
      */
    void search( const QString &searchTerm, SearchMode searchMode );

private:
	void printMap( QTextDocument &document, QString &text, QPrinter *printer );
    void printLegend( QTextDocument &document, QString &text );
    void printRouteSummary( QTextDocument &document, QString &text );
    void printDrivingInstructions( QTextDocument &document, QString &text );
	void printDrivingInstructionsAdvice( QTextDocument &document, QString &text );

 private:
    //MarbleModel       *m_marbleModel;
    MarbleWidget*					m_marbleWidget;
	const ElevationRunnerPlugin*	m_pSrtmRunner;
	MarbleRunnerManager*			m_pRunnerManager;

//    MarbleNavigator   *m_control;
    //QSplitter           *m_splitter;
	WlanControlSplitter*	m_pHSplitter;
	WlanControlSplitter*	m_pLeftVSplitter;
	WlanControlSplitter*	m_pRightVSplitter;
	TreeViewWidget*			m_treeView;
	IconViewWidget*			m_iconView;
	RoutingWidget*			m_pRoutingWidget;
	CurrentLocationWidget*  m_pCurrentLocWidget;
	bool					m_bRoute;
};

}

#endif // CONTROLVIEW_H
