/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */


/* INCLUDE FILES ******************************************************************/
#include "Splitter.h"
//#include "GuiBase.h"

#include <QPainter>
#include <QPaintEvent>


/* CLASS IMPLEMENTATION ***********************************************************/
Splitter::Splitter(QWidget* parent)
 :	QSplitter(parent)
 ,	m_persistantName("")
{
	init();
}

Splitter::Splitter(Qt::Orientation orientation, QWidget* parent)
 :	QSplitter(orientation, parent)
 ,	m_persistantName("")
{
	init();
}


Splitter::Splitter(Qt::Orientation orientation, QWidget* parent, const QString& name, const QString& persistantName)
 :	QSplitter(orientation, parent)
 ,	m_persistantName(persistantName)
{
	setObjectName(name);
	init();
}

Splitter::~Splitter()
{
}

void Splitter::init()
{
	bool ok = false;
/*	if (!m_persistantName.isEmpty())
	{
		GuiBase::Instance()->ApplySplitterSizes4(m_persistantName+m_persistantVariant, this);
		ok = connect(this, SIGNAL(splitterMoved(int, int)), this, SLOT(slotKeepSettings(int, int)));
	}*/
	setOpaqueResize(false);

	////Beispiel: Zum einfachen ndern des Aussehens des SplitterHandles kann man auch ein StyleSheet setzten
	////Dann msste man nicht den SplitterHandle ableiten 
	//QString splitterStyle = "QSplitter::handle:horizontal {width: 2px;} QSplitter::handle:vertical { height: 2px;} ";
	//splitterStyle += "QSplitter::handle {border-width: 1px; border-style: outset; border-color: #4F4F4F;} ";
	//splitterStyle += "QSplitter::handle:hover {background-color: #FFFFFF; border-color: #7F7F7F;} ";
	//setStyleSheet(splitterStyle);
}
void Splitter::setSizes(const QList<int>& size)
{
	QList<int> s = size;

	if (s.size() < count())
	{
		s.clear();
		int i = 0;
		while (i < count())
		{
			s.push_back(10);
			++i;
		}
	}
	QSplitter::setSizes(s);
	adjustSize();
}

void Splitter::addWidget(QWidget* widget)
{
	QSplitter::addWidget(widget);
//	GuiBase::Instance()->ApplySplitterSizes4(m_persistantName+m_persistantVariant, this);
}

void Splitter::showEvent( QShowEvent* event )
{
	QSplitter::showEvent(event);
//	GuiBase::Instance()->ApplySplitterSizes4(m_persistantName+m_persistantVariant, this);	
}

void Splitter::setVariant(const QString& persistantVariant)
{
	m_persistantVariant = persistantVariant;
	QObjectList childs = children();
	QListIterator<QObject*> it(childs);
	while (it.hasNext())
	{
		Splitter* tst = dynamic_cast<Splitter*>(it.next());
		if (tst)
		{
			tst->setVariant(persistantVariant);
		}
	}
//	GuiBase::Instance()->ApplySplitterSizes4(m_persistantName+m_persistantVariant, this);	
}

void Splitter::slotKeepSettings(int pos, int index)
{
	storeSplitterSizesNow();
}

void Splitter::storeSplitterSizesNow()
{
/*	if (!m_persistantName.isEmpty())
	{
		GuiBase::Instance()->StoreSplitterSizes4(m_persistantName+m_persistantVariant, this);
	}*/
}

QSplitterHandle* Splitter::createHandle()
{
	return new SplitterHandle(orientation(), this);
}



/* CLASS IMPLEMENTATION ***********************************************************/
SplitterHandle::SplitterHandle(Qt::Orientation orientation, QSplitter* parent)
 :	QSplitterHandle(orientation, parent),
 m_bPressed(false)
{
	setAttribute(Qt::WA_Hover);
	// empty
}

void SplitterHandle::paintEvent(QPaintEvent* pEvent)
{
	//Der normale Hintergrund wurde im Style der Application gezeichnet
	//QSplitterHandle::paintEvent(e);

	// Achtung, hier wird nur halbtransparent etwas weiss und Schwarz ber die Fensterfarbe gelegt.
	// Normalerweise sollte man sich Farben raussuchen, alles Opaque zeichnen 
	// und mit Helligkeit und Sttigung arbeiten
	
	QPainter painter(this);
	QLinearGradient gradient;
	
	if(!m_bPressed)
	{
		QPoint cursorPos = mapFromGlobal(QCursor::pos());
		
		if( rect().contains(cursorPos) )
		{	//Hover Event //Hellerer raised look - Cursor ber SplitterHandle aber nicht gepresst/gegrabbed
			gradient.setColorAt(0.0, QColor(255,255,255,255)); //Highlight
			gradient.setColorAt(0.3, QColor(255,255,255, 127)); //Heller
			gradient.setColorAt(0.7, QColor(255,255,255, 127)); 
			gradient.setColorAt(1.0, QColor(64,64,64,127)); //Tieferer Schatten
		}
		else
		{	//Normales Aussehen //normal raised look, Voll Durchsichtig
			gradient.setColorAt(0.0, QColor(255,255,255,255)); //Highlight
			gradient.setColorAt(0.3, QColor(255,255,255, 0)); // Hier sollt die "normale" Fensterfarbe hin
			gradient.setColorAt(0.7, QColor(255,255,255, 0)); // Hier sollt die "normale" Fensterfarbe hin
			gradient.setColorAt(1.0, QColor(127,127,127,127)); //Schatten
		}
	}
	else
	{	//Clicked/Grabbed // sunken look
		gradient.setColorAt(0.0, QColor(0,0,0,127)); //Schatten
		gradient.setColorAt(0.3, QColor(127,127,127,64));
		gradient.setColorAt(0.7, QColor(127,127,127,64));
		gradient.setColorAt(1.0, QColor(255,255,255,255)); //Highlight
	}

	if (orientation() == Qt::Horizontal)
	{
		gradient.setStart(rect().left(), rect().height()/2);
		gradient.setFinalStop(rect().right(), rect().height()/2);
	}
	else
	{
		gradient.setStart(rect().width()/2, rect().top());
		gradient.setFinalStop(rect().width()/2, rect().bottom());
	}
	painter.fillRect(pEvent->rect(), QBrush(gradient));
}

void SplitterHandle::mousePressEvent(QMouseEvent* pEvent)
{
	QSplitterHandle::mousePressEvent(pEvent);
	m_bPressed = true;
	update();
}
void SplitterHandle::mouseReleaseEvent(QMouseEvent* pEvent)
{
	QSplitterHandle::mouseReleaseEvent(pEvent);
	m_bPressed = false;
	update();
}

#include "moc_Splitter.cpp"
/**********************************************************************************/
/* END OF FILE																	  */
/**********************************************************************************/
