/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SPLITTER_H
#define SPLITTER_H


/* INCLUDE FILES ******************************************************************/
//#include "GlobalDefinitions/ARExports.h"
#include <QSplitter>


/* FORWARD DECLARATION ************************************************************/
class SplitterHandle;


/* CLASS DECLARATION **************************************************************/
/** The Splitter is a derivate of QSplitter that is able to show the borders between
  *	its children graphically.
  * Use it like you would use QSplitter.
  * @author Bernhard Mayr
  */
class Splitter : public QSplitter
{
	Q_OBJECT
public:
	Splitter(QWidget* parent = 0);
	Splitter(Qt::Orientation orientation, QWidget* parent);
	Splitter(Qt::Orientation orientation, QWidget* parent, const QString& objectname, const QString& persistantName);

	virtual ~Splitter(); 

	void setSizes(const QList<int>& size);

	void addWidget(QWidget* widget);

	void setVariant(const QString& persistantVariant);
	
	void storeSplitterSizesNow();

public slots:
	void slotKeepSettings(int pos, int index);

protected:
	virtual void showEvent( QShowEvent* event );

	QSplitterHandle* createHandle();
	void init();

protected:
	QString m_persistantName;
	QString m_persistantVariant;
};


/* CLASS DECLARATION **************************************************************/
class SplitterHandle : public QSplitterHandle
{
public:
	SplitterHandle(Qt::Orientation orientation, QSplitter* parent);

protected:
	virtual void paintEvent(QPaintEvent* pEvent);
	virtual void mousePressEvent(QMouseEvent* pEvent);
	virtual void mouseReleaseEvent(QMouseEvent* pEvent);

private: 
	bool m_bPressed;
};


#endif	// SPLITTER_H

/**********************************************************************************/
/* END OF FILE																	  */
/**********************************************************************************/
