/******************************************************************************************************************
 *	@file			EsriRunnerPlugin.h
 *	@copyright		&copy; 2010 Rohde&Schwarz, Munich
 *	@version		$Header: \ISAR_dev\userinterface\MapViewControl\marblewidget\EsriRunnerPlugin.h   \main\3     
 *	5 Oktober 2011  14:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the ESRI runner plugin
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_ESRIRUNNERPLUGIN_H
#define MARBLE_ESRIRUNNERPLUGIN_H

#include <QtCore/QObject>
#include "PluginInterface.h"

namespace Marble
{

class MarbleAbstractRunner;

/**
  * A plugin for Marble to execute ESRI mxd import
  */
class MARBLE_EXPORT EsriRunnerPlugin : public QObject, public PluginInterface
{
    Q_OBJECT

public:
    /** Constructor with optional parent object */
    explicit EsriRunnerPlugin( QObject* parent = 0 );

    /** Destructor */
    virtual ~EsriRunnerPlugin();

    /**
     * @brief Returns the string that should appear in the user interface.
     *
     * For example, "Nominatim" should be returned for the Nominatim search plugin.
     */
    virtual QString guiString() const = 0;

    /** Plugin factory method to create a new runner instance.
      * Method caller gets ownership of the returned object
      */
    virtual MarbleAbstractRunner* newRunner() const = 0;

    /** True if the plugin supports its tasks on the given planet */
    bool supportsCelestialBody( const QString &celestialBodyId ) const;

    /** True if the plugin can execute its tasks without network access */
    bool canWorkOffline() const;

    /**
     * @brief Returns @code true @endcode if the plugin is able to perform its claimed task.
     *
     * The default implementation returns @code true @endcode. This method can be
     * overridden for example to indicate an incomplete installation.
     */
    virtual bool canWork() const;

    // Overridden methods with default implementations

    virtual QIcon icon() const;

protected:
    void setSupportedCelestialBodies( const QStringList &celestialBodies );

    void setCanWorkOffline( bool canWorkOffline );

private:
    class Private;
    Private *const d;
};

}

Q_DECLARE_INTERFACE( Marble::EsriRunnerPlugin, "org.kde.Marble.EsriRunnerPlugin/1.0" )

#endif // MARBLE_SEARCHRUNNERPLUGIN_H
