/******************************************************************************************************************
 *	@file			IconViewWidget.h
 *	@copyright		2010 Rohde&Schwarz, Munich
 *	@version		$Header: IconViewWidget.h    
 *	10 Dezember 2011 16:29:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView dimmer layer.
 *					
 *					
 *****************************************************************************************************************/

#ifndef ICONVIEWWIDGET_H
#define ICONVIEWWIDGET_H

#include <QFrame>
#include "marble_export.h"

#include <QMap>

class QDragEnterEvent;
class QDropEvent;
class QGridLayout;
class QAction;
class QLabel;
class QScrollArea;

namespace Marble
{
	class MarbleWidget;
	class IconViewWidgetPrivate;

	class MARBLE_EXPORT IconViewWidget : public QFrame
	{
	 Q_OBJECT

	 public:
		 explicit IconViewWidget(QWidget *parent );

		/** Destructor */
		~IconViewWidget();

		void addMarbleWidget( MarbleWidget *marbleWidget,QString& ramonData);
		void initializeIconGrid(QString&,bool);

	 protected:
		 void dragEnterEvent(QDragEnterEvent *event);
		 void dragMoveEvent(QDragMoveEvent *event);
		 void dropEvent(QDropEvent *event);
		 void mousePressEvent(QMouseEvent *event);

	private:
		QIcon getIcon(QString&,bool);

	 private Q_SLOTS:
		 void slotSelectActionMenu(QAction*);

	 Q_SIGNALS:
		 void signalIconSelectionChanged(QString&);

	 private:
		IconViewWidgetPrivate *const d;
		QGridLayout				*m_pGrid;
		QStringList				m_qsList;
		QMap<int,QLabel*>		m_iconTab;
		QString					m_qsRamonData;
		int						m_iMvSym;
	 };

 } // namespace Marble

#endif