//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2011	Konrad Enzensberger
//

#ifndef MAPVIEWSETTINGSDIALOG_H
#define MAPVIEWSETTINGSDIALOG_H

#include <QtGui/QDialog>
#include <QtCore/QLocale>

#include "marble_export.h"
#include "MarbleGlobal.h"

//#include "ui_MarbleTimeSettingsWidget.h"


namespace Marble
{

class MarbleWidget;

class MapViewSettingsDialogPrivate;

class MARBLE_EXPORT MapViewSettingsDialog : public QDialog
{
    Q_OBJECT
    
    public:
    explicit MapViewSettingsDialog( MarbleWidget *marbleWidget, QWidget *parent = 0 );
    ~MapViewSettingsDialog();

    // View Settings
    
    QLocale::MeasurementSystem measurementSystem() const;
    Marble::AngleUnit angleUnit() const;
	void setAngleUnit(Marble::AngleUnit val) const;
    Marble::MapQuality stillQuality() const;
    Marble::MapQuality animationQuality() const;
    int labelLocalization() const;
    QFont mapFont() const;

    // View
    Marble::GraphicsSystem graphicsSystem() const;


    // Navigation Settings
    int dragLocation() const;
    int onStartup() const;
    bool animateTargetVoyage() const;
	void setAnimateTargetVoyage(bool) const;
	bool enablePanToObject() const;
	void setEnablePanToObject(bool) const;
	bool enableHeading() const;
	void setEnableHeading(bool) const;
    QString externalMapEditor() const;

    // Cache Settings
    int volatileTileCacheLimit() const;
    int persistentTileCacheLimit() const;

    QString proxyUrl() const;
	void setProxyUrl(QString&) const;

	void setProxyUser(QString&) const;
    QString proxyUser() const;

	QString proxyPass() const;
	void setProxyPassword(QString&) const;

    int proxyPort() const;

    bool proxyType() const;
    bool proxyAuth() const;
	bool proxySavePass() const;

    // Time Settings
    /**
     * Read the value of 'Time/systemTime' key from settings
     */
    bool systemTime() const;

    /**
     * Read the value of 'Time/lastSessionTime' key from settings
     */
    bool lastSessionTime() const;

    /**
     * Read the value of 'Time/systemTimezone' key from settings
     */
    bool systemTimezone() const;

    /**
     * Read the value of 'Time/UTC' key from settings
     */
    bool UTC() const;

    /**
     * Read the value of 'Time/customTimezone' key from settings
     */
    bool customTimezone() const;

    /**
     * Read the value of 'Time/chosenTimezone' key from settings
     */
    int chosenTimezone() const;
    
    void initializeCustomTimezone();

	/**
     * Romes disabled Navigation/Routing
     */
	void disableNavRouting() const;

	private slots:

	void findProxyClicked();

    Q_SIGNALS:
    /**
     * This signal is emitted when when the loaded settings were changed.
     * Either by the user or by loading them initially from disk.
     */
    void settingsChanged();

    /**
     * The user clicked on the button to clear volatile tile cache.
     */
    void clearVolatileCacheClicked();

    /**
     * The user clicked on the button to clear persistent tile cache.
     */
    void clearPersistentCacheClicked();

    public Q_SLOTS:
    /**
     * Read settings and update interface.
     */
    void readSettings();

    /**
     * Write settings to disk.
     */
    void writeSettings();
    
    private Q_SLOTS:
    /**
     * Retrieve the current plugin state for the user interface.
     */
    void retrievePluginState();
    
    /**
     * Apply the plugin state from the user interface.
     */
    void applyPluginState();
    
    /**
     * Synchronize the loaded settings with the file on hard disk.
     */
    void syncSettings();

    /**
     * Shows the about dialog for the plugin with the corresponding @p nameId.
     */
    void showPluginAboutDialog( QString nameId );

    /**
     * Shows the configuration dialog for the plugin with the corresponding @p nameId.
     */
    void showPluginConfigDialog( QString nameId );

    /**
     * Saves the settings of all plugins.
     */
    void writePluginSettings();

    private:
    Q_DISABLE_COPY( MapViewSettingsDialog )

    MapViewSettingsDialogPrivate * const d;
};

} // Marble namespace

#endif
