/******************************************************************************************************************
 *	@file			MapViewLegendLayer.h
 *	@copyright		2013 Rohde&Schwarz, Munich
 *	@version		$Header: MapViewLegendLayer.h    
 *	5 Juli 2011		18:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView legend layer.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_MAPVIEWLEGENDLAYER_H
#define MARBLE_MAPVIEWLEGENDLAYER_H

#include "GeoDataCoordinates.h"
#include "LayerInterface.h"

#include <QtCore/QModelIndex>
#include <QtGui/QItemSelection>

#include "Enum.h"
using namespace GIS;

namespace Marble
{

class MarbleWidget;
class MapViewLegendLayerPrivate;
class TreeViewModel;

/**
  * @brief A shading layer that serves as a view off image data
  */
class MapViewLegendLayer: public QObject, public LayerInterface
{
    Q_OBJECT

public:
    /**
      * @brief Constructor
      * @param widget The marble widget used for geopos <-> screenpos transformations
      *   and repainting of (small) areas. Must not be null
      * @param parent Optional parent widget
      */
    explicit MapViewLegendLayer( MarbleWidget *widget, QWidget *parent = 0, qint64 index=0);

    /** Destructor */
    ~MapViewLegendLayer();

    /** Reimplemented from LayerInterface. We'll hover above the surface */
    QStringList renderPosition() const;

    /** Reimplemented from LayerInterface. Paints route items and placemarks */
    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString &renderPos = "NONE", GeoSceneLayer *layer = 0 );


    /**
      * The route is dirty (needs an update) and should be painted to indicate that
      */
    void setLayerDirty( bool dirty );

	 /**
      * Set the proxy model another QAbstractItemView uses that should share
      * its selection model with us. Needed because this class uses an unfiltered
      * model which has different indices than a filtered one.
      */
    void synchronizeWith( QItemSelectionModel *selection );

	 /**
      * Set the tree model to use. Implicitly removes the placemark model.
      */
    void setModel ( TreeViewModel *model );

	/**
      * Add/Remove new image.
      */
    void updateImageList (long id, QImage *img );

	/**
      * set dimmer value to change text color
      */
	void setDimmerValue(int dValue);

	// Overriding LayerInterface to paint on top of other layers
    virtual qreal zValue() const;


protected:
    /** Overriding QWidget, used to make the layer interactive */
    bool eventFilter( QObject *obj, QEvent *event );

private Q_SLOTS:

	 /**
      * The viewport has changed, recalculate positions accordingly
      */
    void setViewportChanged();

private:
    MapViewLegendLayerPrivate *const d;
    friend class MapViewLegendLayerPrivate;

};

} // namespace Marble

#endif
