/*******************************************************************************
* @file			TreeViewModel.cpp
* @copyright	2011-2013 Rohde&Schwarz, Munich
* @version		$Header: TreeViewModel.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "TreeViewModel.h"

#include "MarbleDebug.h"
#include "MarbleDirs.h"
#include "MarbleMath.h"
#include "GeoDataCoordinates.h"
#include "GeoDataDocument.h"
#include "GeoDataParser.h"
#include "GeoDataPlacemark.h"
//#include "RouteSkeleton.h"

#include <QApplication>
#include <QtCore/QBuffer>
#include <QtCore/QPointer>
#include <QtCore/QRegExp>
#include <QtCore/QVector>
#include <QtGui/QMessageBox>
#include <QtGui/QPixmap>
#include <QVariant>
#include <QObject>
//#include <QStandardItemModel>
//#include <QTimer>

//#include "Enum.h"
//using namespace Model;

namespace Marble
{
	TreeViewModel::TreeViewModel(QObject *parent)
		: QAbstractItemModel(parent)
	{
		m_pRootNode = 0;
	/*	m_pUpdateTimer = NULL;
		m_pUpdateTimer = new QTimer(this);
		m_pUpdateTimer->setSingleShot(true);
		m_pUpdateTimer->setInterval(500);
		connect( m_pUpdateTimer, SIGNAL(timeout()), SLOT(slotUpdateMap()) );*/
		QString qStr = ":/bitmaps/overlay.png";
		QPixmap pPixMap;
		QPixmap pPix;
			//pPixMap = (QPixmap*) new QPixmap();
		if(pPixMap.load(qStr))
		{
			//pChildGroup->m_qsImage = qs;
			//pGroup->m_qsImage = qs;
			//pPix = pPixMap.scaled(16,16);
			m_pixmapTree.insert("Overlay",pPixMap);
		}
		qStr = ":/bitmaps/layer.png"; 
		if(pPixMap.load(qStr))
		{
			//pChildGroup->m_qsImage = qs;
			//pGroup->m_qsImage = qs;
			//pPix = pPixMap.scaled(16,16);
			m_pixmapTree.insert("Layer",pPixMap);
		}
		qStr = ":/bitmaps/plot.png";
		if(pPixMap.load(qStr))
		{
			//pChildGroup->m_qsImage = qs;
			//pGroup->m_qsImage = qs;
			m_pixmapTree.insert("Plot",pPixMap);
		}
		qStr = ":/bitmaps/wlan.png";
		if(pPixMap.load(qStr))
		{
			//pChildGroup->m_qsImage = qs;
			//pGroup->m_qsImage = qs;
			m_pixmapTree.insert("Wlan",pPixMap);
		}

		//((QStandardItemModel*)this)->setSortRole(SortRole);
	}

	TreeViewModel::~TreeViewModel()
	{
	//	if(m_pUpdateTimer!=NULL)
	//		m_pUpdateTimer->stop();

		//m_objects.clear();
		delete m_pRootNode;
	}

	void TreeViewModel::slotUpdateMap()
	{
	//	if(m_pUpdateTimer!=NULL)
	//		m_pUpdateTimer->stop();

		emit layoutChanged();
		emit signalUpdateMap();
	}

	void TreeViewModel::setRootNode(TreeNode *node)
	{
		delete m_pRootNode;
		m_pRootNode = node;
		reset();
	}

	QModelIndex TreeViewModel::index(int row, int column,
                               const QModelIndex &parent) const
	{
		if (!m_pRootNode || row < 0 || column < 0 )//|| column > 0)
			return QModelIndex();
		TreeNode *parentNode = nodeFromIndex(parent);
		TreeNode *childNode = parentNode->children.value(row);
		if (!childNode)
			return QModelIndex();
		return createIndex(row, column, childNode);
	}

	TreeNode *TreeViewModel::nodeFromIndex(const QModelIndex &index) const
	{
		if (index.isValid()) {
			return static_cast<TreeNode *>(index.internalPointer());
		} else {
			return m_pRootNode;
		}
	}

	int TreeViewModel::rowCount(const QModelIndex &parent) const
	{
		 if ( parent.column() > 0 ) {
//			mDebug() << "rowCount bad column";
			return 0;
		}

		TreeNode *parentNode = nodeFromIndex(parent);
		if (!parentNode)
			return 0;
		return parentNode->children.count();
	}

	int TreeViewModel::columnCount(const QModelIndex & /* parent */) const
	{
		return 2;
	}

	void TreeViewModel::sort(int column, Qt::SortOrder order)
	{
		return;
		if (column != 0)
        return;

		emit layoutAboutToBeChanged();

		int iRow = rowCount(QModelIndex());

		QPair<TreeNode*,int> pairNodes;
		QVector < QPair<TreeNode*,int> > sorting(rowCount(QModelIndex()));
		for (int i = 0; i < iRow; ++i) {
			pairNodes = sorting.at(i);
			TreeNode *item = pairNodes.first;
			sorting[i].first = item;
			sorting[i].second = i;
		}
	}

	QModelIndex TreeViewModel::parent(const QModelIndex &child) const
	{
		if(!child.isValid())
			return QModelIndex();
		try{
			TreeNode *node = nodeFromIndex(child);
			if (!node || node->ItemType == Root)
				return QModelIndex();
			TreeNode *parentNode = node->parent;
			if (!parentNode)
				return QModelIndex();
			TreeNode *grandparentNode = parentNode->parent;
			if (!grandparentNode)
				return QModelIndex();
			int row = grandparentNode->children.indexOf(parentNode);
			return createIndex(row, child.column(), parentNode);
		}
		catch(...)
		{
			return QModelIndex();
		}
	}

	QVariant TreeViewModel::data(const QModelIndex &index, int role) const
	{
		if (role > 500)
			return QVariant();

		if ( !index.isValid() ) 
		{
			return QVariant();
		}

		if (role == Qt::DisplayRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
				case Overlay:
					return node->m_qsName;
				case Layer:
					return node->m_qsName;
				case PointLayer:
					return node->m_qsName;
				case Group:
					return node->m_qsName;
				case Object:
					return node->m_qsName;
				case Text:
					if(node->parent->m_qsDesc.isEmpty())
						return node->m_qsName;
					else return QVariant();
				case Desc:
					return node->m_qsDesc;
				case Elipse:
					return node->m_qsName;
				case Plot:
					return node->m_qsName;
				case PlotEx:
					return node->m_qsName;
				case Poly:
					return node->m_qsName;
				case Polygone:
					return node->m_qsName;
				case Shading:
					return node->m_qsName;
				case Arrow:
					return node->m_qsName;
				case Marker:
					return node->m_qsName;
					//return QVariant::fromValue( node->m_dRadius );
				case Icon:
				/*	{
						QString tmp = node->m_qsName.section('/',-1);
						if(tmp.length()==node->m_qsName.length() )
							tmp = node->m_qsName.section('\\',-1);
						return tmp;
					}*/
					return tr("Icon");
				case SvgBts:
					return node->m_qsName;
				case SvgObj:
					return node->m_qsName;
				case HotSpot:
					return node->m_qsName;
				case Rect:
					return node->m_qsName;
				case Azimuth:
					return node->m_qsName;
				case Error:
					return tr("Error");
				default:
					return tr("Unknown");
				}
			}//col=0
			else if (index.column() == 1) 
			{
				switch (node->ItemType) 
				{
					case Layer:
						return node->m_iZVal;
				}
			}
		}
		else if ( role == Qt::EditRole ) 
		{
			TreeNode *pNode = nodeFromIndex(index);
			if (!pNode)
				return QVariant();
			switch (pNode->ItemType) 
			{
				case Text:
					return pNode->m_qsName;
				case Desc:
					return pNode->m_qsDesc;
				case Icon:
					if(!pNode->m_qsDesc.isEmpty())
						return pNode->m_qsDesc;
					else if(!pNode->m_qsName.isEmpty())
						return pNode->m_qsDesc;
					else return "Icon";
				default: return pNode->m_qsName;
			}
		}
		else if (role == Qt::UserRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
				case Overlay:
					return QVariant();
				case Layer:
					//return node->m_qsName;
					return QVariant::fromValue( node->m_lFillCol );
				case PointLayer:
					//return node->m_qsName;
					return QVariant::fromValue( node->m_lFillCol );
				case Group:
					return node->m_qsName;// QVariant::fromValue( node->m_position.at(0) );
				case Object:
					return node->m_qsName;
				case Text:
					return node->m_qsName;
				case Desc:
					return node->m_qsName;
				case Elipse:
					return QVariant::fromValue( node->m_dRadius );
				case Plot:
					return QVariant::fromValue( node->m_dRadius );
				case PlotEx:
					return QVariant::fromValue( node->m_dRadius );
				case Marker:
					//return node->m_qsName;
					return node->parent->parent->MarkerType;
				case Poly:
					return node->m_qsName;
				case Polygone:
					return node->m_qsName;
				case Shading:
					return node->m_qsName;
				case Icon:
					return node->m_qsImage;
				case SvgObj:
					return node->m_qsImage;
				case SvgBts:
					return node->m_qsImage;
				case HotSpot:
					return node->m_qsImage;
				case Rect:
					return node->m_qsName;
				case Azimuth:
					return node->m_qsName;
				case Arrow:
					return node->m_qsName;
				case Error:
					return tr("Error");
				default:
					return QVariant();
				}
			} 
		}
		else if (role == CoordinateRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
				case Overlay:
					return QVariant();
				case Layer:
					return node->m_qsName;
				case PointLayer:
					return node->m_qsName;
				case Group:
					return QVariant::fromValue( node->m_position );
				case Object:
					return node->m_qsName;
				case Text:
					return QVariant::fromValue( node->m_position );
				case Desc:
					return QVariant::fromValue( node->m_position );
				case Elipse:
					return QVariant::fromValue( node->m_position );
				case Plot:
					return QVariant::fromValue( node->m_position );
				case PlotEx:
					return QVariant::fromValue( node->m_position );
				case Poly:
					return QVariant::fromValue( node->m_position );
				case Polygone:
					return QVariant::fromValue( node->m_position );
				case Marker:
					return QVariant::fromValue( node->m_position );
				case Icon:
					return QVariant::fromValue( node->m_position );
				case SvgObj:
					return QVariant::fromValue( node->m_position );
				case SvgBts:
					return QVariant::fromValue( node->m_position );
				case HotSpot:
					return QVariant::fromValue( node->m_position );
				case Rect:
					return QVariant::fromValue( node->m_position );
				case Azimuth:
				//	{
					/*GeoDataLineString* pGdl = & node->m_poly;
					//return QVariant::fromValue( node->m_position.at(0) );
					GeoDataLineString* item = static_cast<GeoDataLineString*>(pGdl);
					QVariant v;
					v.setValue(item);
					return v;
					}*/
			//		GeoDataObject *object = static_cast<GeoDataObject*>( index.internalPointer() );
			//		GeoDataLineString *placemark = dynamic_cast<GeoDataLineString*>( object );
			//		if ( placemark )
			//			return QVariant::fromValue(node->m_poly);
			//	}
					return QVariant::fromValue( node->m_position );
				case Shading:
					return QVariant::fromValue( node->m_position );
				case Arrow:
					return QVariant::fromValue( node->m_position );
				case Error:
					return tr("Error");
				default:
					return QVariant();
				}
			} 
		}
		else if (role == TypeRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				return node->ItemType;
				//return QVariant::fromValue( (unsigned int)node->ItemType );
			}

		}//typeRole
		else if ( role == Qt::CheckStateRole && index.column() == 0 ) 
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
	
			if(0)//node->ItemType == Layer)
			{
				if(node->m_bVisible && node->m_bUpdate)
				{
					return QVariant( Qt::Checked );
				}
				else return QVariant( Qt::Unchecked );

			}
			else if(node->ItemType == Overlay && node->m_pMapDoc!=NULL)
			{
				if(node->m_qsName=="EsriMap")//Esri document is not added as GeoDataDocument
				{
					if ( node->m_bVisible )
						return QVariant( Qt::Checked );
					else
						return QVariant( Qt::Unchecked );
				}
				else 
				{
					if(node->m_pMapDoc->isVisible())
						return QVariant( Qt::Checked );
					else 
						return QVariant( Qt::Unchecked );
				}
			}
			else if ( node->m_bVisible )
				return QVariant( Qt::Checked );
			else
				return QVariant( Qt::Unchecked );
		}
		else if(role==DataRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Marker:
					{
						//return m_iDirection from group
						TreeNode *parentNode = node->parent;
						if (!parentNode)//Group
							return QVariant();
						else if(node->MarkerType == GIS::Direction || parentNode->parent->MarkerType == GIS::Direction) 
						{
							node->m_qsName.clear();
							node->m_qsName = "Dir-%1";
							if(parentNode->m_iDirection<=36000)
								node->m_qsName = node->m_qsName.arg(parentNode->m_iDirection/100);
							else node->m_qsName = node->m_qsName.arg("--");
							return QVariant::fromValue(parentNode->m_iDirection );
						}
						else if(node->MarkerType == GIS::Circle)
						{
							return node->m_qsImage;
						}
						else return QVariant();
					}
					break;
					case Shading:
						{
							TreeNode *parentNode = node->parent;
							if (!parentNode)//Group
								return QVariant();
							return QVariant::fromValue(parentNode->m_iDirection );
						}
					case SvgBts:
						return QVariant::fromValue(node->m_iDirection );
					case Rect:
					case Poly:
					case Polygone:
					case Elipse:
						return QVariant::fromValue(node->m_iDirection );
			/*		case Poly:
						return QVariant::fromValue( node->m_lFillCol );
					case Polygone:
						return QVariant::fromValue( node->m_lFillCol );
					case Elipse:
						return QVariant::fromValue( node->m_lFillCol );
					case Arrow:
						return QVariant::fromValue( node->m_lFillCol );*/
					case Desc:
						return QVariant::fromValue(node->m_iAlignment );
					case Text:
						return QVariant::fromValue(node->m_iAlignment );
					case Azimuth:
						return QVariant::fromValue(node->m_bMap );
					default:
						return QVariant();
				}
			}

		}
		else if ( role == FillColorRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Plot:
					case PlotEx:
					case Rect:
					case Poly:
					case Polygone:
					case Elipse:
					case Arrow:
					case Marker:
					case Icon:
						return QVariant::fromValue( node->m_lFillCol );
					default:
						return QVariant();
				}
			}
		}
		else if ( role == LineColorRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Plot:
					case PlotEx:
					case Rect:
					case Poly:
					case Polygone:
					case Elipse:
					case Arrow:
					case Marker:
						return QVariant::fromValue( node->m_lLineCol );
					default:
						return QVariant();
				}
			}

		}
		else if ( role == TransRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Plot:
					case PlotEx:
					case Rect:
					case Poly:
					case Polygone:
					case Elipse:
					case Arrow:
					case Marker:
						return QVariant::fromValue( node->m_iTrans );
					case Icon:
					//case Marker:		why?
						return QVariant::fromValue( node->m_bMultiSelect );
					case Azimuth:
						return QVariant::fromValue(node->m_iTrans );
					default:
						return QVariant();
				}
			}

		}
		else if(role==InfoRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Marker:
						return node->m_qsName;
					case Shading:
						return node->m_iShadingID;
					case Azimuth:
						return QVariant::fromValue(node->m_iDirection );
				}
			}
		}
		else if ( role == Qt::DecorationRole)
        {
			if (index.column() == 0) 
			{
				TreeNode *node = nodeFromIndex(index);
				switch (node->ItemType) 
				{
					case Overlay:
						if(node->m_bMap)
							return QVariant(m_pixmapTree["Plot"]);
						else if(node->m_qsName=="Adapter")
							return QVariant(m_pixmapTree["Wlan"]);
						else
							return QVariant(m_pixmapTree["Overlay"]);
					case Layer:
						if(node->parent->m_bMap)
							return QVariant(m_pixmapTree["Plot"]);
						else if(node->parent->m_qsName=="Adapter")
							return QVariant(m_pixmapTree["Wlan"]);
						else
							return QVariant(m_pixmapTree["Layer"]);
					case Icon:
						if(!node->m_qsImage.isEmpty())
						{
							QPixmap qPix = pixmap(node->m_qsImage);
							if(!qPix.isNull())
							{
								if(qPix.width()>20)
									return QVariant(qPix.scaled(qPix.width()/2,qPix.height()/2));
								else return QVariant(qPix);
							}
						}
						else return QVariant();
					case Plot:
						return QVariant(m_pixmapTree["Plot"]);
					case PlotEx:
						return QVariant(m_pixmapTree["Plot"]);
					default: return QVariant();
				}
			}
		}
		else if(role==OffXRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Marker:
						return node->m_iXoff;
					case Icon:
						return node->m_iXoff;
					case Desc:
						return node->m_iXoff;
					case Text:
						return node->m_iXoff;
					case Arrow:
						return node->m_iXoff;
					case SvgBts:
						return node->m_iXoff;
					case Azimuth:
						return QVariant::fromValue(node->m_iXoff );
					case Poly:
						return node->m_iXoff;
					default: return QVariant();
				}
			}
		}
		else if(role==OffYRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Marker:
						return node->m_iYoff;
					case Icon:
						return node->m_iYoff;
					case Desc:
						return node->m_iYoff;
					case Text:
						return node->m_iYoff;
					case Arrow:
						return node->m_iYoff;
					case Azimuth:
						return QVariant::fromValue(node->m_iYoff );
					case Poly:
						return node->m_iYoff;
					default: return QVariant();
				}
			}
		}
		else if ( role == ZValueRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Azimuth:
						return QVariant::fromValue(node->m_iZVal );
					default: return QVariant();
				}
			}
		}
		else if ( role == ValueRole)
		{
			TreeNode *node = nodeFromIndex(index);
			if (!node)
				return QVariant();
			if (index.column() == 0) 
			{
				switch (node->ItemType) 
				{
					case Azimuth:
						return QVariant::fromValue(node->m_dRadius );
					default: return QVariant();
				}
			}
		}
	/*	else if (index.column() == 1) 
		{
			return node->;
		}*/
		return QVariant();
	}


	QVariant TreeViewModel::headerData(int section,
									 Qt::Orientation orientation,
									 int role) const
	{
		if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
			if (section == 0) {
				return tr("TreeView");
			} else if (section == 1) {
				return tr("Idx");
			}
		}
		return QVariant();
	}
/*
	bool TreeViewModel::hasChildren( const QModelIndex &parent ) const
	{
		if ( parent.column() > 0 ) 
		{
			return false;
		}

		int iCount=0;
		TreeNode* pNode = static_cast<TreeNode *>(parent.internalPointer());
		if(pNode!=NULL)
			iCount = pNode->children.count();

		return iCount>0;

	}*/

	Qt::ItemFlags TreeViewModel::flags ( const QModelIndex & index ) const
	{
		if ( !index.isValid() )
			return Qt::NoItemFlags;

		if (index.column() == 1) 
			return Qt::NoItemFlags;

		TreeNode* pNode = static_cast<TreeNode *>(index.internalPointer());
		//GeoDataFeature *feature = dynamic_cast<GeoDataFeature*>( object );
		if ( pNode ) {
			return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsUserCheckable;
		}
		return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
	}

	bool TreeViewModel::setData ( const QModelIndex & index, const QVariant & value, int role )
	{
		if ( !index.isValid() )
			return false;
		if (index.column() == 1) 
			return false;
		if(role>500)return false;

		TreeNode* pNode = static_cast<TreeNode *>(index.internalPointer());
		if ( role == Qt::CheckStateRole ) 
		{
			//GeoDataFeature *feature = dynamic_cast<GeoDataFeature*>( object );
			if ( pNode ) 
			{
			//	pNode->m_bVisible = value.toBool();
				if(pNode->ItemType == Overlay && pNode->m_pMapDoc!=NULL)
					pNode->m_pMapDoc->setVisible(!pNode->m_pMapDoc->isVisible());
				else
					pNode->m_bVisible = value.toBool();

				mDebug() << "setData " << pNode->m_qsName << " " << value.toBool();
				emit dataChanged( index, index );
				return true;
			}
		}
		else if ( role == Qt::EditRole ) 
		{
			QString str=value.toString();
			if(str.isEmpty())
				return false;
			if(!pNode->m_qsDesc.isEmpty())
				pNode->m_qsDesc = str;
			else 
				pNode->m_qsName = str;
			return true;
		}

		return false;
	}

	Model::TreeNode* TreeViewModel::AddRow(QModelIndex& parent,GIS::TreeViewItemType type)
	{
		int iRow = rowCount(parent);
		if( parent == QModelIndex() )
			beginInsertRows( QModelIndex(), m_pRootNode->children.count(),m_pRootNode->children.count());
		else beginInsertRows( parent,m_pRootNode->children.count(),m_pRootNode->children.count());

		//createIndex(row, column, parentNode->children[row]);
		TreeNode* pParent = nodeFromIndex(parent);
		int iChildRow = pParent->children.count();
		Model::TreeNode* pNewGisObj = new Model::TreeNode(type);
		pNewGisObj->children.append(pNewGisObj);
		//pNewGisObj->children[pNewGisObj->children.count()]->iRow = pNewGisObj->children.
		//pNewGisObj->iRow = rowCount(parent);//m_gisObjectList.size();	// new element will be inserted at last position
		//m_gisObjectList.push_back(pNewGisObj);

		endInsertRows();

		// check if maximum is exceeded
		//if (rowCount() > m_iMaxResults)
		//	removeRows(0, rowCount()-m_iMaxResults);

		return pNewGisObj;
	}

	bool TreeViewModel::insertRows(int iPosition, int iRows, const QModelIndex& parent)
	{
		beginInsertRows(parent, iPosition, iPosition+iRows-1);
		for (int i=0; i<iRows; ++i)
		{
			if ( parent == QModelIndex() )//parent == root
			{
				Model::TreeNode* pOverlay = new Model::TreeNode(Overlay);
				if(m_pRootNode != NULL)
				{
					// new element will be inserted at last position
					//add overlay to root node
					addChild(m_pRootNode,pOverlay);
					pOverlay->m_iRow = m_pRootNode->children.count();
				}
				
			}
			else
				break;	// do nothing with the internal data structure when a child should be inserted
		/*	{
				GisObject* pNewGisObj = new GisObject();
				pNewGisObj->iRow = m_gisObjectList.size();	// new element will be inserted at last position
				m_gisObjectList.push_back(pNewGisObj);
			}*/
		}
		endInsertRows();

		// check if maximum is exceeded
		//if ( !parent.isValid()  &&  rowCount() > m_iMaxResults)
		//	removeRows(0, rowCount()-m_iMaxResults);

		return true;
	}
/*
	bool TreeViewModel::removeRows(int iPosition, int iRows, const QModelIndex& parent)
	{
		bool ret=false;
	//	if(!parent.isValid())
	//		return false;

		Model::TreeNode* pRoot = nodeFromIndex(parent);
		if(pRoot==NULL)
			return false;
		Model::TreeNode* pNode=NULL;

		//qint64 id1 = parent.internalId();

		if (iRows > 0 && iPosition+iRows <= pRoot->children.count() )	// check for container limits
		{
			beginRemoveRows(parent, iPosition, iPosition+iRows-1);
			for (int i=iPosition; i<iPosition+iRows; ++i)
			{
				pNode = pRoot->children.value(i);
				if(pNode != NULL)
				{
					int iGRow = pNode->m_iRow;
					pNode->children.clear();
				//	pNode->parent=NULL;
					pRoot->children.removeOne(pNode);
					//pNode->children.free();
					delete pNode;
					for(int iG = iGRow;iG < pRoot->children.count();iG++)
					{
						pNode = pRoot->children.value(iG);
						if(pNode!=NULL)
							pNode->m_iRow -= 1;
					}
				}
			}
			endRemoveRows();
			return true; 
		}
		else
			return false;


	}*/

	bool TreeViewModel::removeRows(int iPosition, int iRows, const QModelIndex& parent)
	{
		bool ret=false;
		

		Model::TreeNode* pRoot = nodeFromIndex(parent);
		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;
		Model::TreeNode* pGroup=NULL;
		Model::TreeNode* pChild=NULL;

		qint64 id1 = parent.internalId();

		if (iRows > 0 && iPosition+iRows <= pRoot->children.count() )	// check for container limits
		{
			beginRemoveRows(parent, iPosition, iPosition+iRows-1);
			for (int i=iPosition; i<iPosition+iRows; ++i)
			{
				pOverlay = pRoot->children.value(i);
				if(pOverlay != NULL)
				{
					if(pOverlay->ItemType == Overlay)
					{
						for(int iL =0;iL<pOverlay->children.count();iL++)
						{
							//Layer
							pLayer = pOverlay->children.value(iL);
							if(pLayer != NULL)
							{
								pLayer->children.clear();
								pLayer->parent = NULL;
								
							}

						}
						pOverlay->m_pMapDoc=NULL;
						pOverlay->children.clear();
						pOverlay->parent=NULL;

						int iORow = pOverlay->m_iRow;
						pRoot->children.removeOne(pOverlay);
						for(int iO =iORow;iO<pRoot->children.count();iO++)
						{
							pOverlay = pRoot->children.value(iO);
							pOverlay->m_iRow -= 1;
						}
					}

					else if(pOverlay->ItemType == Layer)
					{
						pLayer = pOverlay;
						pOverlay = pLayer->parent;
						pLayer->children.clear();
						pLayer->parent = NULL;
						int iLRow = pLayer->m_iRow;
						if(pOverlay!=NULL)
						{
							pOverlay->children.removeOne(pLayer);
							for(int iL =iLRow;iL<pOverlay->children.count();iL++)
							{
								pLayer = pOverlay->children.value(iL);
								if(pLayer!=NULL)
									pLayer->m_iRow -= 1;
							}
						}
					}

					else if(pOverlay->ItemType == Group)
					{
						pGroup = pOverlay;
						pLayer = pGroup->parent;
						pGroup->parent=NULL;
						int iGRow = pGroup->m_iRow;
						if(pLayer!=NULL)
						{
							pLayer->children.removeOne(pGroup);
							delete pGroup;
							pGroup=NULL;
							for(int iG = iGRow;iG < pLayer->children.count();iG++)
							{
								pGroup = pLayer->children.value(iG);
								if(pGroup!=NULL)
									pGroup->m_iRow -= 1;
							}
						}
					}
					else //items
					{
						pChild = pOverlay;
						pGroup = pChild->parent;
						pChild->parent=NULL;
						int iGRow = pChild->m_iRow;
						pChild->m_poly.clear();
						pChild->m_position.detach();
						if(pGroup!=NULL)
						{
							pGroup->children.removeOne(pChild);
							pChild->children.clear();
						//crash? render anno?	
							try{
								delete pChild;
								pChild=NULL;
							}
							catch(...)
							{
								pChild=NULL;
							}
							for(int iG = iGRow;iG < pGroup->children.count();iG++)
							{
								pChild = pGroup->children.value(iG);
								if(pChild!=NULL)
									pChild->m_iRow -= 1;
							}
						}
					}
				}//end if(pOverlay)

				
			}//end for
			endRemoveRows();
			return true; 
		}
		else
			return false;
	}

	void TreeViewModel::clear()
	{
	 //   d->m_elTree.clear();
		reset();
	}

	TreeNode* TreeViewModel::getChildNode(TreeNode* pParent)
	{
		if(pParent->children.count()<1)
			return NULL;

		TreeNode* pChild = pParent->children.at(0);

		return pChild;
	}

	TreeNode* TreeViewModel::getNodeFromIndex(const QModelIndex &index)
	{
		TreeNode* pNode = nodeFromIndex(index);
		return pNode;
	}

	qint64 TreeViewModel::AddMapOverlay(QStringList& qsList,GeoDataDocument* pDoc,bool bMap)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		if(pRoot==NULL)
		{
			pRoot = new Model::TreeNode(Root);
			setRootNode(pRoot);
		}

		//overlay

		GeoDataCoordinates position;
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pOverlay = new Model::TreeNode(Overlay);
		//pOverlay->m_qsName = "Overlay";
		pOverlay->m_qsName = qsList.at(2);
		pOverlay->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pOverlay->m_bMap = bMap; //is this the map ?
		pOverlay->m_pMapDoc = pDoc;

		//add overlay to root node
		addChild(pRoot,pOverlay);

		pOverlay->m_iRow = pRoot->children.count()-1;
		QModelIndex idx =  createIndex(pOverlay->m_iRow,0,pOverlay);
		qint64 id = idx.internalId();
		pOverlay->m_lId = id;
		//m_objects.insert(pOverlay->m_lId,pOverlay->ItemType);

		emit layoutChanged();

		return id;

	}

	qint64 TreeViewModel::AddOverlay(QStringList& qsList,bool bMap)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		if(pRoot==NULL)
		{
			pRoot = new Model::TreeNode(Root);
			setRootNode(pRoot);
		}

		//overlay

		GeoDataCoordinates position;
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pOverlay = new Model::TreeNode(Overlay);
		//pOverlay->m_qsName = "Overlay";
		pOverlay->m_qsName = qsList.at(2);
		pOverlay->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pOverlay->m_bMap = bMap; //is this the map ?

		//add overlay to root node
		addChild(pRoot,pOverlay);

		pOverlay->m_iRow = pRoot->children.count()-1;
		QModelIndex idx =  createIndex(pOverlay->m_iRow,0,pOverlay);
		qint64 id = idx.internalId();
		pOverlay->m_lId = id;
		//m_objects.insert(pOverlay->m_lId,pOverlay->ItemType);

		emit layoutChanged();

		return id;

	}

	qint64 TreeViewModel::AddLayer(QStringList& qsList,qint64 id)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;

		pOverlay = getNodeFromIndexId(id);
		if(pOverlay == NULL)return 0; 
		
		if(pOverlay->ItemType != GIS::Overlay)
			return 0;

		if(pOverlay->m_lId != id)
			return 0;

		/*
		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay->m_lId == id)
				i = pRoot->children.count();

		}*/

	//	int row = grandparentNode->children.indexOf(parentNode);
	//	return createIndex(row, child.column(), parentNode);

		GeoDataCoordinates position;
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pLayer = new Model::TreeNode(Layer);
		pLayer->m_qsName = qsList.at(2);
		pLayer->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pLayer->m_iZVal=1;

		long lLine = 0;//(long)qLine.rgba();
		lLine <<= 8;
		lLine+= 0;
		lLine <<= 8;
		lLine+= 0;
		pLayer->m_lLineCol = lLine;

		long lFill = 130;//(long)qFill.rgba();
		lFill <<= 8;
		lFill+= 192;
		lFill <<= 8;
		lFill+= 255;
		pLayer->m_lFillCol = lFill;
		pLayer->m_iTrans=80;

		addChild(pOverlay,pLayer);
		

		pLayer->m_iRow = pOverlay->children.count()-1;
		QModelIndex idx =  createIndex(pLayer->m_iRow,0,pLayer);

		//Qt::CheckState state = Qt::Checked;
		//setData(idx, state, Qt::CheckStateRole);

		qint64 id1 = idx.internalId();
		pLayer->m_lId = id1;
		//m_objects.insert(pLayer->m_lId,pLayer->ItemType);

		emit layoutChanged();

		return id1;

	}

	qint64 TreeViewModel::AddTakticalLayer(QStringList& qsList,qint64 id)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;

		pOverlay = getNodeFromIndexId(id);
		if(pOverlay == NULL)return 0; 
		
		if(pOverlay->ItemType != GIS::Overlay)
			return 0;

		if(pOverlay->m_lId != id)
			return 0;

		/*
		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay->m_lId == id)
				i = pRoot->children.count();

		}*/

	//	int row = grandparentNode->children.indexOf(parentNode);
	//	return createIndex(row, child.column(), parentNode);

		GeoDataCoordinates position;
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pLayer = new Model::TreeNode(Layer);
		pLayer->m_qsName = qsList.at(2);
		pLayer->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pLayer->m_iZVal=1;
		pLayer->m_bTactical=true;

		long lLine = 0;//(long)qLine.rgba();
		lLine <<= 8;
		lLine+= 0;
		lLine <<= 8;
		lLine+= 0;
		pLayer->m_lLineCol = lLine;

		long lFill = 130;//(long)qFill.rgba();
		lFill <<= 8;
		lFill+= 192;
		lFill <<= 8;
		lFill+= 255;
		pLayer->m_lFillCol = lFill;
		pLayer->m_iTrans=80;

		addChild(pOverlay,pLayer);
		

		pLayer->m_iRow = pOverlay->children.count()-1;
		QModelIndex idx =  createIndex(pLayer->m_iRow,0,pLayer);

		//Qt::CheckState state = Qt::Checked;
		//setData(idx, state, Qt::CheckStateRole);

		qint64 id1 = idx.internalId();
		pLayer->m_lId = id1;
		//m_objects.insert(pLayer->m_lId,pLayer->ItemType);

		emit layoutChanged();

		return id1;

	}

	qint64 TreeViewModel::AddMapLayer(QStringList& qsList,qint64 id,bool bVisible)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;

		pOverlay = getNodeFromIndexId(id);
		if(pOverlay == NULL)return 0; 
		
		if(pOverlay->ItemType != GIS::Overlay)
			return 0;

		if(pOverlay->m_lId != id)
			return 0;

		GeoDataCoordinates position;
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pLayer = new Model::TreeNode(Layer);
		pLayer->m_qsName = qsList.at(2);
		pLayer->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pLayer->m_bVisible = bVisible;
		pLayer->m_bMap = pOverlay->m_bMap; //is this the map ?

		addChild(pOverlay,pLayer);
		

		pLayer->m_iRow = pOverlay->children.count()-1;
		QModelIndex idx =  createIndex(pLayer->m_iRow,0,pLayer);

		//Qt::CheckState state = Qt::Checked;
		//setData(idx, state, Qt::CheckStateRole);

		qint64 id1 = idx.internalId();
		pLayer->m_lId = id1;

		emit layoutChanged();

		return id1;

	}

	qint64 TreeViewModel::AddGroup(QStringList& qsList,qint64 id)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;

		pLayer = getNodeFromIndexId(id);
		if(pLayer == NULL)return 0; 
		
		if(pLayer->ItemType != GIS::PointLayer && pLayer->ItemType != GIS::Layer)
			return 0;

		if(pLayer->m_lId != id)
			return 0;

		/*
		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay != NULL)
			{
				for(int iL =0;iL<pOverlay->children.count();iL++)
				{
					//Layer
					pLayer = pOverlay->children.value(iL);
					if(pLayer->m_lId == id)
					{
						iL = pOverlay->children.count();
						i  = pRoot->children.count();
					}

				}
			}

		}*/

	//	int row = grandparentNode->children.indexOf(parentNode);
	//	return createIndex(row, child.column(), parentNode);

		GeoDataCoordinates position; 
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );

		Model::TreeNode* pGroup = new Model::TreeNode(Group);
		
		pGroup->m_qsName = qsList.at(2);
		if(pGroup->m_qsName.isEmpty())
			pGroup->m_qsName = "#0";
		//pGroup->m_position.clear();
		pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		addChild(pLayer,pGroup);
		pLayer->m_bUpdate=true;

		pGroup->m_iRow = pLayer->children.count()-1;
		QModelIndex idx =  createIndex(pGroup->m_iRow,0,pGroup);
		qint64 id1 = idx.internalId();
		pGroup->m_lId = id1;
		//m_objects.insert(pGroup->m_lId,pGroup->ItemType);

		emit layoutChanged();
		
		/*if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/
	
		return id1;

	}

	qint64 TreeViewModel::AddIcon(QString& qs,long id,qreal lon, qreal lat, long xoffs, long yoffs, long align, long idx)
	{
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		Model::TreeNode* pChildGroup = new Model::TreeNode(Icon);
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup==NULL) return 0;

		pGroup->parent->m_bUpdate=true;

		QString qsName;

		if(qs.isEmpty())
		{
			QModelIndex idxGroup = getIndexFromIndexId(qID);
			QModelIndex idxParent = parent(idxGroup);
			Model::TreeNode* pLayer = nodeFromIndex(idxParent);
			if(pLayer != NULL)
			{
				pChildGroup->m_qsImage = pLayer->m_qsImage;
		//		pChildGroup->m_qsName = pLayer->m_qsImage.section('/',0,-1);
				qs = pLayer->m_qsImage;
			}

		}
		else
		{
			qsName = qs.section(';',0,0);
			if(!qsName.isEmpty())
				pChildGroup->m_qsImage = qsName;
			else pChildGroup->m_qsImage = qs;
			qsName = qs.section(';',1,1);
		//	pChildGroup->m_qsName = qs.section('/',0,-1);
		}

		if(!qsName.isEmpty())//pChildGroup->m_qsName.isEmpty())
		{
			Model::TreeNode* pText = new Model::TreeNode(Text);
			pText->m_qsName = qsName;//pGroup->m_qsName;
			pText->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
			addChild(pGroup,pText);
			pText->m_iRow = pGroup->children.count()-1;
			QModelIndex tID =  createIndex(pText->m_iRow,0,pText);
			qint64 id2 = tID.internalId();
			pText->m_lId = id2;
		//	pText->m_bVisible=false;
			//m_objects.insert(pText->m_lId,pText->ItemType);
		}

		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_iXoff = xoffs;
		pChildGroup->m_iYoff = yoffs;

		if (!m_pixmapCache.contains( pChildGroup->m_qsImage ) ) 
		{
			QPixmap pPixMap;
			if(pPixMap.load(pChildGroup->m_qsImage))
			{
				//pChildGroup->m_qsImage = qs;
				//pGroup->m_qsImage = qs;
				m_pixmapCache.insert(pChildGroup->m_qsImage,pPixMap);
			}
			else
			{
				QString qsImg = QApplication::applicationDirPath()+"/"+pChildGroup->m_qsImage;
				if(pPixMap.load(qsImg))
				{
					pChildGroup->m_qsImage = qsImg;
					//pGroup->m_qsImage = qs;
					m_pixmapCache.insert(pChildGroup->m_qsImage,pPixMap);
				}
				else //load default ico
				{
					pChildGroup->m_qsImage.clear();
					pChildGroup->m_qsImage = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
					if(pPixMap.load(pChildGroup->m_qsImage))
					{
						//pChildGroup->m_qsImage = qs;
						//pGroup->m_qsImage = qs;
						m_pixmapCache.insert(pChildGroup->m_qsImage,pPixMap);
					}
				}
			}

		}
	/*	else
		{
			pChildGroup->m_qsImage = qs;
			//pGroup->m_qsImage = qs;
		}*/

		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);
		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;

	}

	qint64 TreeViewModel::AddMarker(QString& qs,long id,qreal lon, qreal lat, long xoffs, long yoffs, long align)
	{
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		Model::TreeNode* pChildGroup = new Model::TreeNode(Marker);
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);

		if(pGroup!=NULL)
		{
			Model::TreeNode* pLayer = pGroup->parent;
			if(pLayer!=NULL)
			{
				pChildGroup->MarkerType = pLayer->MarkerType;
			}
		}

		pGroup->parent->m_bUpdate=true;
		pChildGroup->m_qsName = qs.section(';',0,0);
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

/*		QString qsMarker = qs.section(';',1,1);
		if ( !qsMarker.isEmpty() ) 
		{
			pChildGroup->m_qsImage = qsMarker;
			if(!m_pixmapCache.contains( qsMarker ))
			{
				QPixmap pPixMap;
				//pPixMap = (QPixmap*) new QPixmap();
				if(pPixMap.load(qsMarker))
				{
					//pChildGroup->m_qsImage = qs;
					//pGroup->m_qsImage = qs;
					m_pixmapCache.insert(qsMarker,pPixMap);
				}
			}
		}*/

		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);
		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;
	}

	qint64 TreeViewModel::AddEllipse(QString& qsl,long id,qreal lon, qreal lat,GeoDataLineString& gdl)
	{
		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;

		pChildGroup = new Model::TreeNode(Elipse);
		if(pChildGroup!=NULL)
		{
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			pChildGroup->m_qsName = qsl;
			pChildGroup->m_dRadius = 0;//radius;
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;
			pChildGroup->m_lLineCol=0;//default black

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

			//Qt::CheckState state = Qt::Checked;
			//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

			return id1;
		}

		return 0;
	}

	qint64 TreeViewModel::AddColorEllipse(QString& qs,long id,long lineCol,long fillCol,qreal lon, qreal lat,GeoDataLineString& gdl)
	{
		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;

		pChildGroup = new Model::TreeNode(Elipse);
		if(pChildGroup!=NULL)
		{
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			if(!qs.isEmpty())
				pChildGroup->m_qsName = qs;
			pChildGroup->m_dRadius = 0;//radius;
			pChildGroup->m_lFillCol = fillCol;
			pChildGroup->m_lLineCol = lineCol;
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

			//Qt::CheckState state = Qt::Checked;
			//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

			//add text item
			if(!qs.isEmpty())//!pChildGroup->m_qsName.isEmpty())
			{
				Model::TreeNode* pText = new Model::TreeNode(Text);
				pText->m_qsName = qs;//qsl.at(0);
				pText->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
				addChild(pGroup,pText);
				pText->m_iRow = pGroup->children.count()-1;
				QModelIndex tID =  createIndex(pText->m_iRow,0,pText);
				qint64 id2 = tID.internalId();
				pText->m_lId = id2;
				//m_objects.insert(pText->m_lId,pText->ItemType);
			}

			return id1;
		}

		return 0;
	}

	qint64 TreeViewModel::AddColorPolygone(QString& qs,long id,long lineCol,long fillCol,qreal lon, qreal lat,int width,GeoDataLineString& gdl)
	{
		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;

		pChildGroup = new Model::TreeNode(Polygone);
		if(pChildGroup!=NULL)
		{
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			if(!qs.isEmpty())
				pChildGroup->m_qsName = qs;
			else pChildGroup->m_qsName = tr("Polygon");
			pChildGroup->m_dRadius = 0;//radius;
			pChildGroup->m_lFillCol = fillCol;
			pChildGroup->m_lLineCol = lineCol;
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

			//Qt::CheckState state = Qt::Checked;
			//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

			//add text item
			if(0)//!qs.isEmpty())//!pChildGroup->m_qsName.isEmpty())
			{
				Model::TreeNode* pText = new Model::TreeNode(Text);
				pText->m_qsName = qs;//qsl.at(0);
				pText->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
				addChild(pGroup,pText);
				pText->m_iRow = pGroup->children.count()-1;
				QModelIndex tID =  createIndex(pText->m_iRow,0,pText);
				qint64 id2 = tID.internalId();
				pText->m_lId = id2;
				//m_objects.insert(pText->m_lId,pText->ItemType);
			}

			return id1;
		}

		return 0;
	}

	qint64 TreeViewModel::AddPlot(QStringList& qsl,long id,qreal lon, qreal lat, qreal radius,GeoDataLineString& gdl, long flags)
	{
		/*GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );*/

	/*	#define PV_NAME			0x0001
		#define PV_CLASS		0x0002
		#define PV_COMMENT		0x0004
		#define PV_SYMBOL		0x0008
		#define PV_RADIUS		0x0010
		#define PV_ALL			0xffff*/

		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
		/*	QModelIndex tempID =  createIndex(pGroup->m_iRow,0,pGroup);
			pGroup = parent(tempID);
			if(pGroup->ItemType != Layer)*/
				return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;
		
		int iChildRow=0;
		
		//the plot
		if(flags & 0x0010)
		{
			pChildGroup = new Model::TreeNode(Plot);
			pChildGroup->m_dRadius = radius;
	
			if(qsl.count()>0)
				pChildGroup->m_qsName = qsl.at(0);
			else pChildGroup->m_qsName = "Unknown";
			if(pChildGroup->m_qsName.contains('\n'))
					pChildGroup->m_qsName.remove('\n');
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		}

		//add text item
		QString qsTemp; 
		if(qsl.count()>0)
			qsTemp= qsl.at(0);
		qint64 idText=0;
		if(!qsTemp.isEmpty())//!pChildGroup->m_qsName.isEmpty())
		{
		//	if(qsTemp.contains('\n'))
		//		qsTemp.remove('\n');
			Model::TreeNode* pText = new Model::TreeNode(Text);
			pText->m_qsName = qsTemp;//qsl.at(0);
			pText->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			addChild(pGroup,pText);
			pText->m_iRow = pGroup->children.count()-1;
			QModelIndex tID =  createIndex(pText->m_iRow,0,pText);
			idText = tID.internalId();
			pText->m_lId = idText;
			//m_objects.insert(pText->m_lId,pText->ItemType);
		}

		//QString qsImage;
		qsTemp.clear();
		if(qsl.count()>1)
			qsTemp = qsl.at(1);
		qint64 idIco=0;
		//add icon item
		if( (flags & 0x0008) && !qsTemp.isEmpty())
		{
			Model::TreeNode* pIcon = new Model::TreeNode(Icon);
			pIcon->m_qsImage = qsTemp;
			pIcon->m_qsName  = qsTemp;
			pIcon->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			addChild(pGroup,pIcon);
			pIcon->m_iRow = pGroup->children.count()-1;
			QModelIndex tID =  createIndex(pIcon->m_iRow,0,pIcon);
			idIco = tID.internalId();
			pIcon->m_lId = idIco;
			//m_objects.insert(pIcon->m_lId,pIcon->ItemType);


			if (!m_pixmapCache.contains( qsTemp ) ) 
			{
				QPixmap pPixMap;
				//pPixMap = (QPixmap*) new QPixmap();
				if(pPixMap.load(qsTemp))
				{
				//	pIcon->m_qsName = qsImage;
				//	pGroup->m_qsImage = qsImage;
					m_pixmapCache.insert(qsTemp,pPixMap);
				}
			}
			/*else
			{
				pIcon->m_qsName = qsImage;
			//	pGroup->m_qsImage = qsImage;
			}*/
		}

		if(pChildGroup!=NULL)
		{
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*		if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
			{
				m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}*/

			return id1;
		}
		else 
		{
			if(idIco>0)
				return idIco;//return the icon id if no plot is created
			if(idText>0)
				return idText;//return the text id if no plot is created
			return 0;
		}

	}
	qint64 TreeViewModel::AddPlotEx(QStringList& qsl,long id,qreal lon, qreal lat, GeoDataLineString& gdl, long flags)
	{
	/*	GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );*/

		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid; 
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
		/*	QModelIndex tempID =  createIndex(pGroup->m_iRow,0,pGroup);
			pGroup = parent(tempID);
			if(pGroup->ItemType != Layer)*/
				return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;
		
		int iChildRow=0;

		//the plot
		if(flags & 0x0010)
		{
			pChildGroup = new Model::TreeNode(PlotEx);
			if(qsl.count()>0)
				pChildGroup->m_qsName = qsl.at(0);
			else pChildGroup->m_qsName = "Unknown";
			if(pChildGroup->m_qsName.contains('\n'))
					pChildGroup->m_qsName.remove('\n');
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		}

		//add text item
		QString qsTemp; 
		if(qsl.count()>0)
			qsTemp = qsl.at(0);
		if(!qsTemp.isEmpty())//!pChildGroup->m_qsName.isEmpty())
		{
			//if(qsTemp.contains('\n'))
			//	qsTemp.remove('\n');
			Model::TreeNode* pText = new Model::TreeNode(Text);
			pText->m_qsName = qsTemp;//qsl.at(0);
			//pText->m_qsName = "Radius-%1";
			//pText->m_qsName = pText->m_qsName.arg(radius);
			pText->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			addChild(pGroup,pText);
			pText->m_iRow = pGroup->children.count()-1;
			QModelIndex tID =  createIndex(pText->m_iRow,0,pText);
			qint64 id2 = tID.internalId();
			pText->m_lId = id2;
			//m_objects.insert(pText->m_lId,pText->ItemType);
		}

		//QString qsImage;
		qsTemp.clear();
		if(qsl.count()>1)
			qsTemp = qsl.at(1);

		//add icon item
		if( (flags & 0x0008) && !qsTemp.isEmpty())
		{
			Model::TreeNode* pIcon = new Model::TreeNode(Icon);
			pIcon->m_qsName = qsTemp;
			pIcon->m_qsImage = qsTemp;
			pIcon->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			addChild(pGroup,pIcon);
			pIcon->m_iRow = pGroup->children.count()-1;
			QModelIndex tID =  createIndex(pIcon->m_iRow,0,pIcon);
			qint64 id2 = tID.internalId();
			pIcon->m_lId = id2;
			//m_objects.insert(pIcon->m_lId,pIcon->ItemType);

			if (!m_pixmapCache.contains( qsTemp ) ) 
			{
				QPixmap pPixMap;
				//pPixMap = (QPixmap*) new QPixmap();
				if(pPixMap.load(qsTemp))
				{
				//	pIcon->m_qsName = qsImage;
				//	pGroup->m_qsImage = qsImage;
					m_pixmapCache.insert(qsTemp,pPixMap);
				}
			}
			
		}

		if(pChildGroup!=NULL)
		{
			pChildGroup->m_dRadius = 0;//radius;

			/*int is = pChildGroup->m_poly.size();
			for(int i=is-1;i>=0;i--)
			{
				pChildGroup->m_poly.remove(i);
			}
			pChildGroup->m_poly.clear();
			is = pChildGroup->m_poly.size();

			for(is=0;is<iCount;is++)
				pChildGroup->m_poly.append(gdl.at(is));*/
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

			//Qt::CheckState state = Qt::Checked;
			//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*		if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
			{
				m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}*/

			return id1;
		}
		else return 0;

	}

	qint64 TreeViewModel::AddAzimuthLine(long id,qreal lon, qreal lat,long azi)
	{
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		Model::TreeNode* pChildGroup = new Model::TreeNode(Azimuth);
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}

		pGroup->parent->m_bUpdate=true;
		//int iCount = gdl.size();
		pChildGroup->m_qsName = "Azi-%1";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(azi);
	//	pChildGroup->m_position.clear();
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_iTrans=azi;
	//	pChildGroup->m_poly = gdl;
	//	pChildGroup->m_poly.setTessellate(true);

		//add the plot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;

	}

	qint64 TreeViewModel::AddPolyLine(long id,long color,GeoDataLineString& gdl)
	{
		if(gdl.size()<1)
			return 0;

		GeoDataCoordinates position = gdl.at(0); 
	//	position.setLongitude(lon, GeoDataCoordinates::Degree );
	//	position.setLatitude( lat, GeoDataCoordinates::Degree );
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0; 
		}
		pGroup->parent->m_bUpdate=true;

		if(gdl.size()<2)//color poly from ARGUS
		{
			int iC = pGroup->children.count();
			if(iC>0)
			{
				Model::TreeNode* pChild = pGroup->children.at(--iC);
				if(pChild!=NULL)
				{
					position = pChild->m_poly.at(0);
					gdl << position;
					position = gdl.at(0);
				}
			}
		}


		Model::TreeNode* pChildGroup = new Model::TreeNode(Poly);

		Model::TreeNode* pParentNode = pChildGroup->parent;
		if(pParentNode != NULL)
		{
		//	pParentNode->m_position.clear();
			pParentNode->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}

		int iCount = gdl.size();		
		pChildGroup->m_qsName.clear();
		pChildGroup->m_qsName = "Poly-%1";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(iCount);
	//	pChildGroup->m_position.clear();
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_lLineCol = color;
	//	pChildGroup->m_lFillCol = color;

		int is = pChildGroup->m_poly.size();
		for(int i=is-1;i>=0;i--)
		{
			pChildGroup->m_poly.remove(i);
		}
		pChildGroup->m_poly.clear();
		is = pChildGroup->m_poly.size();

		for(is=0;is<iCount;is++)
			pChildGroup->m_poly.append(gdl.at(is));
//		pChildGroup->m_poly.clear();
//		pChildGroup->m_poly << gdl;
//		pChildGroup->m_poly.setTessellate(true);

		if(pGroup->m_position.longitude(GeoDataCoordinates::Degree)== -399.0)
			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		//add the plot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;


	}

	qint64 TreeViewModel::AddPolyLineEx(long id,long color,int iGridX,int iGridY,int iSID,GeoDataLineString& gdl)
	{
		GeoDataCoordinates position = gdl.at(0); 
	//	position.setLongitude(lon, GeoDataCoordinates::Degree );
	//	position.setLatitude( lat, GeoDataCoordinates::Degree );
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0; 
		}
		pGroup->parent->m_bUpdate=true;
		pGroup->m_iXoff = iGridX;//used as gridx value
		pGroup->m_iYoff = iGridY;//used as gridy value

		Model::TreeNode* pChildGroup = new Model::TreeNode(Shading);

		Model::TreeNode* pParentNode = pChildGroup->parent;
		if(pParentNode != NULL)
		{
		//	pParentNode->m_position.clear();
			pParentNode->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}

		int iCount = gdl.size();
		pChildGroup->m_qsName = "Grid %1x%2";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(iGridX).arg(iGridY);
	//	pChildGroup->m_position.clear();
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_lFillCol = color;
		pChildGroup->m_iXoff = iGridX;//used as GridX value
		pChildGroup->m_iYoff = iGridY;//used as GridY value
		pChildGroup->m_iShadingID = iSID;

		pChildGroup->m_poly.clear();
		pChildGroup->m_poly << gdl;
//		pChildGroup->m_poly.setTessellate(true);

		if(pGroup->m_position.longitude(GeoDataCoordinates::Degree)== -399.0)
			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		//add the plot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;


	}

	qint64 TreeViewModel::AddNamedPolyLine(long id,QString& name,qreal lon,qreal lat,long color,GeoDataLineString& gdl)
	{
		if(gdl.size()<1)
			return 0;

		GeoDataCoordinates position;
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0; 
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;

/*		if(gdl.size()<2)//color poly from ARGUS
		{
			int iC = pGroup->children.count();
			if(iC>0)
			{
				Model::TreeNode* pChild = pGroup->children.at(--iC);
				if(pChild!=NULL)
				{
					position = pChild->m_poly.at(0);
					gdl << position;
					position = gdl.at(0);
				}
			}
		}*/

		Model::TreeNode* pChildGroup = new Model::TreeNode(Poly);

		int iCount = gdl.size();		
		pChildGroup->m_qsName.clear();
		//pChildGroup->m_qsName = "Poly-%1";
		pChildGroup->m_qsName = name;//pChildGroup->m_qsName.arg(iCount);
	//	pChildGroup->m_position.clear();
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_lLineCol = color;
	//	pChildGroup->m_lFillCol = color;

		int is = pChildGroup->m_poly.size();
		for(int i=is-1;i>=0;i--)
		{
			pChildGroup->m_poly.remove(i);
		}
		pChildGroup->m_poly.clear();
		is = pChildGroup->m_poly.size();

		for(is=0;is<iCount;is++)
			pChildGroup->m_poly.append(gdl.at(is));
//		pChildGroup->m_poly.clear();
//		pChildGroup->m_poly << gdl;
//		pChildGroup->m_poly.setTessellate(true);

		//add the plot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;


	}

	qint64 TreeViewModel::UpdatePolyLine(long id,long color,GeoDataLineString& gdl)
	{
		if(gdl.size()<1)
			return 0;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pPoly = getNodeFromIndexId(qID);
		if(pPoly->ItemType != Poly)
		{
			return 0; 
		}
		pPoly->parent->m_bUpdate=true;
		pPoly->m_poly.clear();
		pPoly->m_poly << gdl;
		pPoly->m_lFillCol = color;

		int iCount = gdl.size();		
		pPoly->m_qsName.clear();
		pPoly->m_qsName = "Poly-%1";
		pPoly->m_qsName = pPoly->m_qsName.arg(iCount);

		emit layoutChanged();
		return (qint64)pPoly->m_lId;
	}

	qint64 TreeViewModel::AddPolygon(long id,long color,GeoDataLineString& gdl)
	{
		GeoDataCoordinates position; 
		if(gdl.size()>0)
			position = gdl.at(0); 
		else
		{
			position.setLongitude(-399.0, GeoDataCoordinates::Degree );
			position.setLatitude( -399.0, GeoDataCoordinates::Degree );
		}
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0; 
		}
		pGroup->parent->m_bUpdate=true;

		Model::TreeNode* pChildGroup = new Model::TreeNode(Polygone);

		Model::TreeNode* pParentNode = pChildGroup->parent;
		if(pParentNode != NULL)
		{
		//	pParentNode->m_position.clear();
			pParentNode->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}

		int iCount = gdl.size();		
		pChildGroup->m_qsName.clear();
		pChildGroup->m_qsName = "Polygon-%1";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(iCount);
	//	pChildGroup->m_position.clear();
		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_lFillCol = color;

		int is = pChildGroup->m_poly.size();
		for(int i=is-1;i>=0;i--)
		{
			pChildGroup->m_poly.remove(i);
		}
		pChildGroup->m_poly.clear();
		is = pChildGroup->m_poly.size();

		for(is=0;is<iCount;is++)
			pChildGroup->m_poly.append(gdl.at(is));
//		pChildGroup->m_poly.clear();
//		pChildGroup->m_poly << gdl;
//		pChildGroup->m_poly.setTessellate(true);

		if(pGroup->m_position.longitude(GeoDataCoordinates::Degree)== -399.0)
			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		//add the plot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;


	}

	qint64 TreeViewModel::MoveAzimuthLine(long id,qreal lon, qreal lat,long azi,GeoDataLineString& gdl)
	{
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		qint64 qID = (qint64)id;
		Model::TreeNode* pAzimuth = getNodeFromIndexId(qID);
		if(pAzimuth==NULL)return 0;
		Model::TreeNode* pParentNode = pAzimuth->parent;
		if(pParentNode != NULL)
		{
			//pParentNode->m_position.clear();
			pParentNode->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}
		pParentNode->parent->m_bUpdate=true;

		pAzimuth->m_qsName.clear();
		pAzimuth->m_qsName = "Azi-%1";
		pAzimuth->m_qsName = pAzimuth->m_qsName.arg(azi);

	/*	int is = pAzimuth->m_poly.size();
		for(int i=is-1;i>=0;i--)
		{
			pAzimuth->m_poly.remove(i);
		}*/
		//pAzimuth->m_poly.clear();

		QVector<GeoDataCoordinates>::Iterator itCoords = pAzimuth->m_poly.begin();
		QVector<GeoDataCoordinates>::Iterator itEnd = pAzimuth->m_poly.end();

		if(pAzimuth->m_poly.size()>0)
		{
		/*	iSize-=1;
			for(int i=iSize;i>=0;i--)
				m_lineString.remove(i);*/
			pAzimuth->m_poly.erase(itCoords,itEnd);
			
		}

		pAzimuth->m_poly << gdl;
		pAzimuth->m_iTrans=azi;
	/*	is = pAzimuth->m_poly.size();
		if(is>0)
			is=0;*/

	/*	if(pAzimuth->m_poly.size()>0 && gdl.size()>0)
		{
			for(int is=0;is<gdl.size();is++)
				pAzimuth->m_poly[is] = gdl.at(is);
			//pAzimuth->m_poly << gdl;
		}
		else pAzimuth->m_poly << gdl;*/

//		pAzimuth->m_poly.clear();
//		pAzimuth->m_poly << gdl;
/*		Marble::TessellationFlags f;
		f = Marble::TessellationFlag::Tessellate;
		pAzimuth->m_poly.setTessellationFlags(f);
		pAzimuth->m_poly.setTessellate(true);*/
//		is = pAzimuth->m_poly.size();

	//	emit layoutChanged();

		return pAzimuth->m_lId;
	}

	/*
	qint64 TreeViewModel::MoveAzimuthLine(long id,qreal lon, qreal lat,long azi,TCoord * coords,int iCount)
	{
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

		qint64 qID = (qint64)id;
		Model::TreeNode* pAzimuth = getNodeFromIndexId(qID);
		if(pAzimuth==NULL)return 0;
		Model::TreeNode* pParentNode = pAzimuth->parent;
		if(pParentNode != NULL)
		{
			//pParentNode->m_position.clear();
			pParentNode->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}
		pParentNode->parent->m_bUpdate=true;

		pAzimuth->m_qsName.clear();
		pAzimuth->m_qsName = "Azi-%1";
		pAzimuth->m_qsName = pAzimuth->m_qsName.arg(azi);

		int is = pAzimuth->m_poly.size();

		for(int i=is-1;i>=0;i--)
			pAzimuth->m_poly.remove(i);

	//	pAzimuth->m_poly.clear();
	//	pAzimuth->m_poly.detach();
	//	is = pAzimuth->m_poly.size();
	//	for(is=0;is<gdl.size();is++)
	//		pAzimuth->m_poly.append(gdl.at(is));

		for(int i=0;i<iCount;i++)
		{
			position.set(coords[i].longitude,coords[i].latitude,0,GeoDataCoordinates::Degree);
			
			pAzimuth->m_poly << position;
		}

//		is = pAzimuth->m_poly.size();

		//emit layoutChanged();

		return pAzimuth->m_lId;
	}*/

	qint64 TreeViewModel::AddRect(long id,double left, double top,double rigth,double bottom)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		pGroup->parent->m_bUpdate=true;

		Model::TreeNode* pChildGroup = new Model::TreeNode(Rect);

		const qreal east = rigth;
		const qreal west = left;
		const qreal north = top;
		const qreal south = bottom;

		GeoDataLatLonBox boxR;
		boxR.setEast(east,GeoDataCoordinates::Degree );
		boxR.setWest(west,GeoDataCoordinates::Degree );
		boxR.setNorth(north,GeoDataCoordinates::Degree );
		boxR.setSouth(south,GeoDataCoordinates::Degree );

		GeoDataCoordinates center = boxR.center();

	/*	qreal centerLon = center.longitude(GeoDataCoordinates::Degree );//left+ ((rigth-left)/2);
		qreal centerLat = center.latitude(GeoDataCoordinates::Degree );//top - ((top-bottom)/2);

		GeoDataCoordinates position; 
		position.setLongitude(centerLon, GeoDataCoordinates::Degree );
		position.setLatitude( centerLat, GeoDataCoordinates::Degree );*/

		pGroup->m_position.set(center.longitude(),center.latitude(),0);
		pChildGroup->m_position.set(center.longitude(),center.latitude(),0);

		pChildGroup->m_poly.clear();
		center.setLongitude(left, GeoDataCoordinates::Degree );
		center.setLatitude( top, GeoDataCoordinates::Degree );
		pChildGroup->m_poly.append(center);

		center.setLongitude(rigth, GeoDataCoordinates::Degree );
		center.setLatitude(bottom, GeoDataCoordinates::Degree );
		pChildGroup->m_poly.append(center);

		//pChildGroup->m_poly.setTessellate(true);

		pChildGroup->m_qsName = "Rect-%1";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(pGroup->children.count());

		//add the rect
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		//Qt::CheckState state = Qt::Checked;
		//setData(mID, state, Qt::CheckStateRole);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

	/*	if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;
		
	}

	qint64 TreeViewModel::AddDescription(QString& qs,long id,qreal lon,qreal lat,int xoff,int yoff,int align)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		pGroup->parent->m_bUpdate=true;

		//disable text item to avoid double display same text
		int iGC = pGroup->children.count();
		Model::TreeNode* pText=NULL;
		for(int i=0;i<iGC;i++)
		{
			pText = pGroup->children.at(i);
			if(pText->ItemType == Text)
				pText->m_bVisible = false;
		}

		Model::TreeNode* pChildGroup = new Model::TreeNode(Desc);

		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

	/*	if(pGroup->m_position.latitude(GeoDataCoordinates::Degree)==position.latitude(GeoDataCoordinates::Degree))
		{
			position.setLatitude( lat-0.001, GeoDataCoordinates::Degree );

		}*/

		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pChildGroup->m_qsDesc = qs;
		pChildGroup->m_qsName = pGroup->m_qsName;
		pChildGroup->m_iXoff = xoff;
		pChildGroup->m_iYoff = yoff;
		pChildGroup->m_iAlignment = align;
		pGroup->m_qsDesc = pChildGroup->m_qsDesc;

		//add the description
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;
		//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

/*		if(m_pUpdateTimer!=NULL)// && !m_pUpdateTimer->isActive())
		{
			m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/

		return id1;

	}

	qint64 TreeViewModel::ModifyObjectText(long id,QString& qs)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pNode = getNodeFromIndexId(qID);
		if(pNode==NULL || pNode->parent==NULL)
			return 0;

		pNode->m_qsName.clear();
		pNode->m_qsName = qs;

		emit layoutChanged();

		return (qint64)pNode->m_lId;

	}

	qint64 TreeViewModel::AddArrow(long id,qreal lon, qreal lat,int dir_count1,int dir_count2,long color,GeoDataLineString& gdl)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}

		Model::TreeNode* pChildGroup = new Model::TreeNode(Arrow);
		GeoDataCoordinates position; 
		position.setLongitude(lon, GeoDataCoordinates::Degree );
		position.setLatitude( lat, GeoDataCoordinates::Degree );

	/*	if(pGroup->m_position.latitude(GeoDataCoordinates::Degree)==position.latitude(GeoDataCoordinates::Degree))
		{
			position.setLatitude( lat-0.001, GeoDataCoordinates::Degree );

		}*/

		pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		//pChildGroup->m_qsDesc = qs;
		pChildGroup->m_qsName = "dirC1-%1:dirC2-%2";
		pChildGroup->m_qsName = pChildGroup->m_qsName.arg(dir_count1).arg(dir_count2);
		pChildGroup->m_iXoff = dir_count1;
		pChildGroup->m_iYoff = dir_count2;
		pChildGroup->m_lFillCol = color;
		pChildGroup->m_poly << gdl;

		//add the description
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;

		return id1;
	}

	qint64 TreeViewModel::AddSvgObject(long id, QString& key,double lon, double lat, bool file,bool fixed,bool omni,int bts, QPixmap* pPix)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}

		Model::TreeNode* pChildGroup = NULL;
		if(!file)//Svg created by string
		{
			pChildGroup = new Model::TreeNode(SvgBts);
			GeoDataCoordinates position; 
			position.setLongitude(lon, GeoDataCoordinates::Degree );
			position.setLatitude( lat, GeoDataCoordinates::Degree );

			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
			pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			//use direction to indicate fixed/dynamic size
			if(fixed)
				pChildGroup->m_iDirection=-1;//fixed size
			else pChildGroup->m_iDirection=1;

			//marker type indicates OMNI or Antenna BTS / if omni==false, set MarkerType to Triangle / default marker type==Circle (OMNI)
			if(!omni)
				pChildGroup->MarkerType = GIS::Triangle;

			QString qsKey;
			pChildGroup->m_qsName =key.section(';',0,0);
			qsKey = key.right(key.length()-(pChildGroup->m_qsName.length()+1) );
			if (!m_pixmapCache.contains( qsKey ) ) 
			{
				if(pPix!=NULL && !pPix->isNull())
					m_pixmapCache.insert(qsKey,*pPix);
			}
			pChildGroup->m_qsImage=qsKey;
			pChildGroup->m_qsDesc=key;
			pChildGroup->m_iXoff=bts;
		}
		else //svg file
		{
			pChildGroup = new Model::TreeNode(SvgObj);
			GeoDataCoordinates position; 
			position.setLongitude(lon, GeoDataCoordinates::Degree );
			position.setLatitude( lat, GeoDataCoordinates::Degree );

			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
			pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}

		//add the description
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;

		return id1;
	}

	qint64 TreeViewModel::AddHotSpot(long id, QString& key,double lon, double lat, bool set, QPixmap* pPix)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}

		Model::TreeNode* pChildGroup = new Model::TreeNode(HotSpot);
		GeoDataCoordinates position; 
		position.setLongitude((double)lon, GeoDataCoordinates::Degree );
		position.setLatitude( (double)lat, GeoDataCoordinates::Degree );

		//pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		pGroup->m_position.set(position.longitude(),position.latitude(),0);
		pChildGroup->m_position.set(position.longitude(),position.latitude(),0);

		if (!m_pixmapCache.contains( key ) ) 
		{
			if(pPix!=NULL && !pPix->isNull())
				m_pixmapCache.insert(key,*pPix);
		}

		pChildGroup->m_qsName =key.section('-',0,0);
		pChildGroup->m_qsImage=key;

		//add the HotSpot
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;

		return id1;

	}

	qint64 TreeViewModel::AddPixmapObject(long id, QString& key,double lon, double lat,int xoffs,int yoffs,int alignment, QPixmap* pPix)
	{
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}

		Model::TreeNode* pChildGroup = NULL;
		if(pPix!=NULL)
		{
			pChildGroup = new Model::TreeNode(Icon);
			GeoDataCoordinates position; 
			position.setLongitude(lon, GeoDataCoordinates::Degree );
			position.setLatitude( lat, GeoDataCoordinates::Degree );

			pGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
			pChildGroup->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			pChildGroup->m_iXoff = xoffs;
			pChildGroup->m_iYoff = yoffs;

			//use direction to indicate fixed/dynamic size
			/*if(fixed)
				pChildGroup->m_iDirection=-1;
			else pChildGroup->m_iDirection=1;*/

			QString qsKey=key;
			//pChildGroup->m_qsName =key.section(';',0,0);
			//qsKey = key.right(key.length()-(pChildGroup->m_qsName.length()+1) );
			if (!m_pixmapCache.contains( qsKey ) ) 
			{
				if(pPix!=NULL && !pPix->isNull())
					m_pixmapCache.insert(qsKey,*pPix);
			}
			pChildGroup->m_qsImage=qsKey;
		}

		//add the icon
		addChild(pGroup,pChildGroup);

		pChildGroup->m_iRow = pGroup->children.count()-1;
		QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

		qint64 id1 = mID.internalId();
		pChildGroup->m_lId = id1;

		return id1;

	}

	qint64 TreeViewModel::AddHyperbola(QString& qsl,long id,qreal lon, qreal lat,GeoDataLineString& gdl)
	{
		Model::TreeNode* pChildGroup = NULL;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;
		Model::TreeNode* pGroup = getNodeFromIndexId(qID);
		if(pGroup->ItemType != Group)
		{
			return 0;
		}
		else pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		pGroup->parent->m_bUpdate=true;

		pChildGroup = new Model::TreeNode(Elipse);
		if(pChildGroup!=NULL)
		{
			pChildGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			pChildGroup->m_qsName = qsl;
			pChildGroup->m_dRadius = 0;//radius;
			pChildGroup->m_poly.clear();
			pChildGroup->m_poly << gdl;

			//add the plot
			addChild(pGroup,pChildGroup);

			pChildGroup->m_iRow = pGroup->children.count()-1;
			QModelIndex mID =  createIndex(pChildGroup->m_iRow,0,pChildGroup);

			//Qt::CheckState state = Qt::Checked;
			//setData(mID, state, Qt::CheckStateRole);

			qint64 id1 = mID.internalId();
			pChildGroup->m_lId = id1;
			//m_objects.insert(pChildGroup->m_lId,pChildGroup->ItemType);

			return id1;
		}

		return 0;
	}

	qint64 TreeViewModel::DeleteObject(long id, int type)
	{
		//if(!m_objects.contains(id))
		//	return 0;
		uint uid = (uint)id;
		qint64 qID = (qint64)uid;

		if(uid<=0)
		{
			qID=0;
			return qID;
		}

		Model::TreeNode* pNode = getNodeFromIndexId(qID);
		if(pNode==NULL) 
			return 0;

		qint64 qLID=0;
		if(pNode->ItemType == Layer)
		{
			uid = (uint)pNode->m_lId;
			qLID = uid;
		}

	/*	if(pNode->parent->ItemType == Group)
			pNode->parent->m_bUpdate=true;
		else if(pNode->ItemType == Group)
			pNode->parent->m_bUpdate=true;
		else if(pNode->ItemType == Layer)
			pNode->m_bUpdate=true;*/

	/*	QModelIndex idxNode   = getIndexFromIndexId(qID);
		QModelIndex idxParent = parent(idxNode);*/
		if(pNode!=NULL)
		{
			Model::TreeNode* pParent = pNode->parent;
			if(pParent!=NULL)
			{
				uid = (uint)pParent->m_lId;
				qID = (qint64)uid;
			}
			else 
			{
				uid = (uint)pNode->m_lId;
				qID = uid;
			}
			QModelIndex idxParent = getIndexFromIndexId(qID);
			if(idxParent.isValid())
			{
				bool ret=removeRows(pNode->m_iRow,1,idxParent);
				//m_objects.remove(pNode->m_lId);
			}
			else if(pParent->ItemType == Root)
			{
				bool ret=removeRows(pNode->m_iRow,1,QModelIndex());
			}

		}
		else qID=0;

		if(qLID>0)qID=qLID;//to delete layer from marble
		//else qID = (qint64)pNode->m_lId;

		return qID;

	}

	qint64 TreeViewModel::DeleteByName(QStringList& qsList)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;
		Model::TreeNode* pGroup=NULL;
		Model::TreeNode* pChild=NULL;

		QString qsDel = qsList.at(2);
		qint64 idx=0;
		uint uid=0;

		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay != NULL)
			{
				if(pOverlay->m_qsName.compare(qsDel,Qt::CaseSensitive)==0)
				{
					//idx = (qint64)pRoot->m_lId;
					uid = (uint)pOverlay->m_lId;
					idx = (qint64)uid;
					//QModelIndex parent = getIndexFromIndexId(idx);
					//bool ret=removeRows(pOverlay->m_iRow,1,parent);
					return idx;
				}

				for(int iL =0;iL<pOverlay->children.count();iL++)
				{
					//Layer
					pLayer = pOverlay->children.value(iL);
					if(pLayer&&pLayer->m_qsName.compare(qsDel,Qt::CaseSensitive)==0)
					{
						//qreal lid = pLayer->m_lId;
						uid = (uint)pLayer->m_lId;
						idx = (qint64)uid;
						//QModelIndex parent = getIndexFromIndexId(idx);
						//bool ret=removeRows(pLayer->m_iRow,1,parent);
						return idx;
					}

					for(int iG =0;iG<pLayer->children.count();iG++)
					{
						//Group
						pGroup = pLayer->children.value(iG);
						if(pGroup&&pGroup->m_qsName.compare(qsDel,Qt::CaseSensitive)==0)
						{
							uid = (uint)pGroup->m_lId;
							idx = (qint64)uid;
						//	QModelIndex parent = getIndexFromIndexId(idx);
						//	bool ret=removeRows(pGroup->m_iRow,1,parent);
							return idx;
						}
						
						//child
						for(int iC =0;iC<pGroup->children.count();iC++)
						{
							//Group child
							pChild = pGroup->children.value(iC);
							if(pChild&&pChild->m_qsName.compare(qsDel,Qt::CaseSensitive)==0)
							{
								uid = (uint)pChild->m_lId;
								idx = (qint64)uid;
							//	QModelIndex parent = getIndexFromIndexId(idx);
							//	bool ret=removeRows(pChild->m_iRow,1,parent);
								return idx;
							}
						}
					}

				}
			}
		}

		return 0;

	}

	TreeNode* TreeViewModel::getNodeFromIndexId(qint64 &index)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;
		Model::TreeNode* pGroup=NULL;
		Model::TreeNode* pChild=NULL;

		//TreeNode* pNode = static_cast<TreeNode *>(index.internalPointer());
		//if(!m_objects.contains((long)index))
		//	return NULL;

		TreeNode* pNode = static_cast<TreeNode *>((void*)index);
		return pNode;
	/*	if(!pNode)return NULL;
		QModelIndex idxNode = createIndex(pNode->m_iRow,0,pNode);
		if(idxNode.isValid())
			return pNode;
		else  
			return NULL;*/

		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay != NULL)
			{
				if((uint)pOverlay->m_lId == index)
					return pOverlay;

				for(int iL =0;iL<pOverlay->children.count();iL++)
				{
					//Layer
					pLayer = pOverlay->children.value(iL);
					if((uint)pLayer->m_lId == index)
						return pLayer;

					for(int iG =0;iG<pLayer->children.count();iG++)
					{
						//Group
						pGroup = pLayer->children.value(iG);
						if((uint)pGroup->m_lId == index)
							return pGroup;
						
						//child
						for(int iC =0;iC<pGroup->children.count();iC++)
						{
							//Group child
							pChild = pGroup->children.value(iC);
							if((uint)pChild->m_lId == index)
								return pChild;
						}
					}

				}
			}
		}

		return NULL;
	}

	QModelIndex  TreeViewModel::getIndexFromIndexId(qint64 &index)
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;
		Model::TreeNode* pGroup=NULL;
	
		//if(!m_objects.contains((long)index))
		//	return QModelIndex();

		try{
			TreeNode* pNode = static_cast<TreeNode *>((void*)index);
			if(!pNode)return QModelIndex();
			QModelIndex idx = createIndex(pNode->m_iRow,0,pNode);
			return idx;
		}
		catch(...)
		{
			return QModelIndex();
		}

		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay != NULL)
			{
				if(pOverlay->m_lId == index)
					return createIndex(pOverlay->m_iRow,0,pOverlay);

				for(int iL =0;iL<pOverlay->children.count();iL++)
				{
					//Layer
					pLayer = pOverlay->children.value(iL);
					if(pLayer->m_lId == index)
						return createIndex(pLayer->m_iRow,0,pLayer);

					for(int iG =0;iG<pLayer->children.count();iG++)
					{
						//Group
						pGroup = pLayer->children.value(iG);
						if(pGroup->m_lId == index)
							return createIndex(pGroup->m_iRow,0,pGroup);
					}

				}
			}
		}

		return QModelIndex();
	}

	void TreeViewModel::addChild(TreeNode *parent, TreeNode *child)
	{
		if (child) 
		{
			parent->children += child;
		//	parent->m_qsName += child->m_qsName;
			child->parent = parent;
		}
	}

	QPixmap TreeViewModel::pixmap( QString& key ) const
	{
		if ( m_pixmapCache.contains( key ) ) 
		{
			return m_pixmapCache[key];
		}
		else return QPixmap();
	}

	bool TreeViewModel::checkPixmapKey( QString& key ) const
	{
		if ( m_pixmapCache.contains( key ) ) 
		{
			return true;
		}
		else return false;
	}

	GeoDataLineString* TreeViewModel::getPoly(const QModelIndex &index)
	{
		if(!index.isValid())
			return NULL;

		TreeNode* pNode = nodeFromIndex(index);
		if(pNode==NULL)return NULL;

		GeoDataLineString* pGdl = &(pNode->m_poly);
		return pGdl;

	}

	bool TreeViewModel::RemoveMap()
	{
		Model::TreeNode* pRoot = nodeFromIndex(QModelIndex());
		if(pRoot==NULL)return false;

		Model::TreeNode* pOverlay=NULL;
		Model::TreeNode* pLayer=NULL;

		int iPos=0;
		int iRows=0;

		for(int i=0;i<pRoot->children.count();i++)
		{
			//overlay
			pOverlay = pRoot->children.value(i);
			if(pOverlay != NULL && pOverlay->m_bMap && pOverlay->m_bMultiSelect)
			{
				i = pRoot->children.count();
				iPos = pOverlay->m_iRow;
				iRows = 1;//allways one row for map //pOverlay->children.count();
				removeRows(iPos,iRows,QModelIndex());//parent is root
				clear();
			}

		}
		return true;
	}


} // namespace Marble

#include "moc_TreeViewModel.cpp"
