/*******************************************************************************
* @file			TreeViewWidget.cpp
* @copyright	2011-2012 Rohde&Schwarz, Munich
* @version		$Header: TreeViewWidget.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "TreeViewWidget.h"
#include <MarbleDirs.h>

#include "MarbleMap.h"
#include "MarbleModel.h"
#include "MarblePlacemarkModel.h"
#include "MarbleWidget.h"
#include "MarbleWidgetInputHandler.h"
#include "MarbleRunnerManager.h"
#include "PluginManager.h"
#include "MarbleLocale.h"

#include "GeoDataLineString.h"
#include "GeoDataLinearRing.h"
#include "GeoDataPolygon.h"
#include "GeoDataTrack.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "GeoSceneSettings.h"
#include "GeoSceneProperty.h"

#include "MapViewLayer.h"
#include "MapViewTsLayer.h"
#include "MapViewShadingLayer.h"
#include "MapViewDimLayer.h"
#include "MapViewLegendLayer.h"
#include "MapViewMapLayer.h"

#include "GeoDataDocument.h"
#include "GeoDataContainer.h"
#include "GeoDataPlacemark.h"
#include "GeoDataParser.h"
#include "GeoDataExtendedData.h"
#include "GeoDataStyle.h"
#include "GeoDataTypes.h"
#include "GeoDataGroundOverlay.h"
#include "GeoDataPhotoOverlay.h"
#include "GeoWriter.h"
#include "FileManager.h"
#include "ViewportParams.h"
#include "Planet.h"

#include "GeoDataTreeModel.h"

#include "TreeViewModel.h" 
#include "TreeViewProxyModel.h"
#include "MvComboBox.h"

//#include "modeltest.h"
#include "dtypes.h" //GroupSetData types

#include <QtCore/QTime>
#include <QtCore/QTimer>
#include <QFile>
#include <QFileDialog>
#include <QtGui/QSortFilterProxyModel>
#include <QtGui/QColorDialog>
#include <QDebug>
#include <QtGui/QMessageBox>

#include "plugins/render/MapViewInterface/mapview.h"

#include "uic_TreeViewWidget.h"
#include "uic_LayerProperties.h"
#include "uic_ObjectProperties.h"

namespace Marble
{

class TreeViewWidgetPrivate
{
public:
    Ui::Form m_ui;

    MarbleWidget*						m_marbleWidget;
	MarbleRunnerManager*				m_runnerManager;

    TreeViewModel*						m_treeViewModel;

	QMap<QString, bool>					m_checkBoxMap;
	QMap<qreal,MapViewLayer*>			m_mvLayers;
	QMap<qreal,MapViewMapLayer*>		m_mvMapLayers;
	QMap<qreal,MapViewTSLayer*>			m_mvTsLayers;
	QMap<qreal,MapViewShadingLayer*>	m_mvShadingLayers;
	QMap<int,qint64>					m_mvShadingObjects;

	MapViewLayer*						m_pMvLayer;
//	MapViewShadingLayer*				m_pMvShadingLayer;
	MapViewDimLayer*					m_pMvDimLayer;

    TreeViewProxyModel					*m_treeviewProxyModel;

	// Places on the map
	GeoDataFolder*						m_folder;
	GeoDataDocument*					m_document;
    FileManager*						m_fileManager;

    //RouteSkeleton *m_routeSkeleton;

    //bool m_zoomRouteAfterDownload;

    bool m_workOffline;

	int	 m_iBeamPointCount;

    //QMovie m_progress;

    //QTimer m_progressTimer;

    /** Constructor */
    TreeViewWidgetPrivate();

    /**
      * @brief Toggle between simple search view and route view
      * If only one input field exists, hide all buttons
      */
    //void adjustInputWidgets();

    //void adjustSearchButton();

    /**
      * @brief Change the active input widget
      * The active input widget influences what is shown in the paint layer
      * and in the list view: Either a set of placemarks that correspond to
      * a runner search result or the current route
      */
    //void setActiveInput( RoutingInputWidget* widget );
	//GeoDataLineString				m_poly;
};

TreeViewWidgetPrivate::TreeViewWidgetPrivate() :
        m_marbleWidget( 0 ),
        m_workOffline( false ),
		m_treeViewModel(0),
		// Places on the map
		m_fileManager(0),
		m_pMvLayer(0),m_pMvDimLayer(0),m_treeviewProxyModel(0),
		m_runnerManager(0)
{
    //m_progressTimer.setInterval( 100 );
	m_iBeamPointCount=200;
}
/*
void RoutingWidgetPrivate::adjustInputWidgets()
{
    for ( int i = 0; i < m_inputWidgets.size(); ++i ) {
        m_inputWidgets[i]->setIndex( i );
    }

    //m_ui.optionsLabel->setVisible( !simple );
    adjustSearchButton();
}

void RoutingWidgetPrivate::adjustSearchButton()
{
    QString text = QObject::tr( "Get Directions" );
    QString tooltip = QObject::tr( "Retrieve routing instructions for the selected destinations." );

    bool search = m_inputWidgets.size() < 2;
    if ( m_inputWidgets.size() > 1 ) {
        for ( int i = 0; i < m_inputWidgets.size(); ++i ) {
            if ( !m_inputWidgets[i]->hasInput() ||
                    ( m_inputWidgets[i]->hasInput() && !m_inputWidgets[i]->hasTargetPosition() ) ) {
                search = true;
            }
        }
    }

    if ( search ) {
        text = QObject::tr( "Search" );
        tooltip = QObject::tr( "Find places matching the search term" );
    }

    m_ui.searchButton->setText( text );
    m_ui.searchButton->setToolTip( tooltip );
}

void RoutingWidgetPrivate::setActiveInput( RoutingInputWidget *widget )
{
    Q_ASSERT( widget && "Must not pass null" );
    MarblePlacemarkModel *model = widget->searchResultModel();

    m_activeInput = widget;
    m_ui.directionsListView->setModel( model );
    m_routingLayer->setModel( model );
    m_routingLayer->synchronizeWith( m_routingProxyModel, m_ui.directionsListView->selectionModel() );
}*/

TreeViewWidget::TreeViewWidget( QWidget *parent ) :
        QWidget( parent ), d( new TreeViewWidgetPrivate )
{
	d->m_marbleWidget = 0;

	//layer properties
	m_lpDialog=NULL;
	ui_lpWidget=NULL;
	m_lsComboBox=NULL;
	m_lpDialog = new QDialog();
    ui_lpWidget = new Ui::LayerProperties;
    ui_lpWidget->setupUi( m_lpDialog );
	m_iPropSet=0;

	//object properties
	m_opDialog=NULL;
	ui_opWidget=NULL;
	m_osComboBox=NULL;
	m_opDialog = new QDialog();
    ui_opWidget = new Ui::ObjectProperties;
    ui_opWidget->setupUi( m_opDialog );

	m_lGridMapID=0;
	m_lGrayMapID=0;
	m_lElevationID=0;
	m_selectedID=0;
	m_focusID=0;
	m_sitOverlayID=0;
	m_propOverlayID=0;
	m_vecOverlayID=0;
	m_vecLayerID=0;
	m_tsOverlayID=0;
	m_tsLayerID=0;
	m_tsGroupID=0;
	m_bMOG=false;
	m_bRedraw=true;
	m_bFocusObject=false;
	m_iDimmer=0;
	m_bElevation=false;
	m_bGrid=true;
	m_bGray=false;

	m_esriCacheID=0;
	m_esriOverlayID=0;
	m_minfOverlayID=0;
	m_mapOverlayID=0;

	m_pUpdateTimer = NULL;
	m_pFocusTimer = NULL;

	m_legendRect.setLeft(1000);
	m_legendRect.setTop(1000);
	m_legendRect.setRight(0);
	m_legendRect.setBottom(0);
	m_iARGUS=0;

	m_iRepaintCount=0;

	m_b3D=false;

	d->m_ui.setupUi( this ); 
}

TreeViewWidget::~TreeViewWidget()
{
	if(m_pUpdateTimer!=NULL)
	{
		m_pUpdateTimer->stop();
		delete m_pUpdateTimer;
	}

	if(m_pFocusTimer!=NULL)
	{
		m_pFocusTimer->stop();
		delete m_pFocusTimer;
	}

	if(m_lsComboBox!=NULL)
		delete m_lsComboBox;
	if(ui_lpWidget!=NULL)
		delete ui_lpWidget;
	if(m_lpDialog!=NULL)
		delete m_lpDialog;

	m_lpDialog=NULL;
	ui_lpWidget=NULL;
	m_lsComboBox=NULL;

	if(m_osComboBox!=NULL)
		delete m_osComboBox;
	if(ui_opWidget!=NULL)
		delete ui_opWidget;
	if(m_opDialog!=NULL)
		delete m_opDialog;

	m_opDialog=NULL;
	ui_opWidget=NULL;
	m_osComboBox=NULL;

	// delete all custom layers
	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
        if ( pLayer != NULL ) 
		{
			if(!pLayer->getLegendLayer())
			{
				m_legendRect.setLeft(1000);
				m_legendRect.setTop(1000);
				m_legendRect.setRight(0);
				m_legendRect.setBottom(0);
			}
			d->m_marbleWidget->removeLayer(pLayer); 
			delete pLayer;
			pLayer=NULL;
		}
	}
	d->m_mvLayers.clear();

	// delete all taktical layers
	MapViewTSLayer* pTsLayer=NULL;
    QMap<qreal,MapViewTSLayer*>::const_iterator iterTs = d->m_mvTsLayers.constBegin();
    for ( ; iterTs != d->m_mvTsLayers.constEnd(); ++iterTs ) 
	{
		pTsLayer = iterTs.value();
        if ( pTsLayer != NULL ) 
		{
			d->m_marbleWidget->removeLayer(pTsLayer); 
			delete pTsLayer;
			pTsLayer=NULL;
		}
	}
	d->m_mvTsLayers.clear();

	// delete all shading layers
	MapViewShadingLayer* pShadingLayer=NULL;
    QMap<qreal,MapViewShadingLayer*>::const_iterator iterShade = d->m_mvShadingLayers.constBegin();
    for ( ; iterShade != d->m_mvShadingLayers.constEnd(); ++iterShade ) 
	{
		pShadingLayer = iterShade.value();
        if ( pShadingLayer != NULL ) 
		{
			d->m_marbleWidget->removeLayer(pShadingLayer); 
			delete pShadingLayer;
			pShadingLayer=NULL;
		}
	}
	d->m_mvShadingLayers.clear();

	// delete all custom layers
	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator mapiter = d->m_mvMapLayers.constBegin();
    for ( ; mapiter != d->m_mvMapLayers.constEnd(); ++mapiter ) 
	{
		pMapLayer = mapiter.value();
        if ( pMapLayer != NULL ) 
		{
			d->m_marbleWidget->removeLayer(pMapLayer); 
			delete pMapLayer;
			pMapLayer=NULL;
		}
	}
	d->m_mvMapLayers.clear();

	// delete the shading layer
/*	if(d->m_pMvShadingLayer!=NULL)
	{
		d->m_marbleWidget->removeLayer(d->m_pMvShadingLayer); 
		delete d->m_pMvShadingLayer;
	}
	d->m_pMvShadingLayer=NULL;*/

	// delete the dimmer layer
	if(d->m_pMvDimLayer!=NULL)
	{
		d->m_marbleWidget->removeLayer(d->m_pMvDimLayer); 
		delete d->m_pMvDimLayer;
	}
	d->m_pMvDimLayer=NULL;

	if(d->m_treeViewModel != NULL)
		delete d->m_treeViewModel;
	if(d->m_treeviewProxyModel != NULL)
		delete d->m_treeviewProxyModel;
	
    delete d;
}

void TreeViewWidget::addMarbleWidget(MarbleWidget *marbleWidget)
{
	d->m_marbleWidget = marbleWidget;

	d->m_treeViewModel = new TreeViewModel(this);
	d->m_marbleWidget->model()->setTreeViewModel(d->m_treeViewModel);
	d->m_marbleWidget->model()->setTreeViewWidget(this);

	//new ModelTest(d->m_treeViewModel, this);

	d->m_marbleWidget->setTree(this);
	d->m_ui.treeTreeView->setModel(d->m_treeViewModel);
//org	d->m_ui.treeTreeView->setSortingEnabled( true );


//	d->m_ui.treeTreeView->setSelectionMode( QAbstractItemView::MultiSelection);
//	d->m_ui.treeTreeView->setAlternatingRowColors(true);
//	d->m_ui.treeTreeView->setUniformRowHeights (true);
//	d->m_ui.treeTreeView->setAnimated(false); 

	// disable built-in sorting - new test		
	d->m_ui.treeTreeView->setSortingEnabled(true);
	// use our own sorting method instead
	d->m_ui.treeTreeView->header()->setSortIndicatorShown(true);
	d->m_ui.treeTreeView->header()->setClickable(true);
	connect(d->m_ui.treeTreeView->header(), SIGNAL(sectionClicked(int)), this, SLOT(customSortByColumn(int)));
	customSortByColumn(d->m_ui.treeTreeView->header()->sortIndicatorSection());


	d->m_ui.treeTreeView->setSelectionBehavior(QAbstractItemView::SelectItems); 
	d->m_ui.treeTreeView->setSelectionMode(QAbstractItemView::ExtendedSelection); 
/*	d->m_ui.treeTreeView->header()->setSortIndicatorShown(true);
	d->m_ui.treeTreeView->header()->setClickable(true);*/

//org this is used    d->m_ui.treeTreeView->sortByColumn( 1, Qt::AscendingOrder );
	d->m_ui.treeTreeView->resizeColumnToContents(1);

	d->m_ui.treeTreeView->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(d->m_ui.treeTreeView,SIGNAL(customContextMenuRequested(const QPoint &)),
									this,SLOT(contextTreeMenu(const QPoint &))); 

	//d->m_ui.treeTreeView->setSelectionModel(
	QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
	connect(selectionModel, SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection& )), this, SLOT(slotTreeSelectionChanged( const QItemSelection&, const QItemSelection& )));
	//d->m_ui.treeTreeView->setModel(d->m_marbleWidget->model()->treeModel());
	//d->m_ui.treeTreeView->setModel((QAbstractItemModel*) d->m_marbleWidget->map()->model()->dataFacade()->treeModel());

	d->m_treeviewProxyModel = new TreeViewProxyModel( this );
 //   d->m_treeviewProxyModel->setSourceModel(d->m_treeViewModel);
/*	d->m_treeviewProxyModel->setFilterKeyColumn(0);
	d->m_treeviewProxyModel->setDynamicSortFilter(true);*/
//	d->m_ui.treeTreeView->setModel(d->m_treeviewProxyModel);
/*	d->m_ui.treeTreeView->setSortingEnabled( true );
    d->m_ui.treeTreeView->sortByColumn( 0, Qt::AscendingOrder ); */

	d->m_fileManager = d->m_marbleWidget->model()->fileManager(); 

	d->m_runnerManager = new MarbleRunnerManager( d->m_marbleWidget->model()->pluginManager(), this ) ;
	d->m_runnerManager->setModel( d->m_marbleWidget->model() );
	//elevation results
	connect( d->m_runnerManager, SIGNAL( elevationFinished( GeoDataCoordinates, qreal ) ),
            this, SLOT( addDetectedElevationResult( GeoDataCoordinates, qreal  ) ) );


	/*
	d->m_ui.treeTreeView->setModel(d->m_marbleWidget->map()->model()->treeModel());

	QString qsKml("c:/config/jakobsweg.kml");
	d->m_marbleWidget->addPlacemarkFile(qsKml); 

	QString qsGpx("c:/config/feldbergrunde.gpx");
	d->m_marbleWidget->openGpxFile(qsGpx);
	//d->m_marbleWidget->fileViewModel();

	QString qsPnt("c:/config/river.pnt");
	d->m_marbleWidget->addPlacemarkFile(qsPnt);*/

//	d->m_ui.treeTreeView->setWindowTitle(QObject::tr("Simple Tree Model"));
//	d->m_ui.treeTreeView->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding,
  //                                              QSizePolicy::MinimumExpanding ) );


	d->m_pMvDimLayer = new MapViewDimLayer(d->m_marbleWidget,this,0);
	d->m_pMvDimLayer->setModel(d->m_treeViewModel);
	d->m_marbleWidget->addLayer(d->m_pMvDimLayer); 

	 
	connect (d->m_treeViewModel, SIGNAL( dataChanged(QModelIndex,QModelIndex) ),
                  this, SLOT( showSelectedItem(QModelIndex) ) );	 
	connect (d->m_treeViewModel, SIGNAL( signalUpdateMap() ),
                  this, SLOT( slotUpdateMap() ) );	 
	connect (d->m_ui.treeTreeView, SIGNAL( clicked(QModelIndex) ),
                  this, SLOT( zoomSelectedItem(QModelIndex) ) );
	connect(this , SIGNAL( toggledShowProperty( QString, bool ) ),
             d->m_marbleWidget,SLOT( setPropertyValue( QString, bool ) ) );	 
	connect( d->m_marbleWidget, SIGNAL( themeChanged( QString ) ),
             this,        SLOT( selectTheme( QString ) ) );

	connect( d->m_marbleWidget, SIGNAL( regionSelected( const QList<double>& ) ),
             this,        SLOT( selectRegion( const QList<double>& ) ) );
	

    m_bRedraw=true;
	m_pUpdateTimer = NULL;
	m_pUpdateTimer = new QTimer(this);
	m_pUpdateTimer->setSingleShot(true);
	m_pUpdateTimer->setInterval(50);
	connect( m_pUpdateTimer, SIGNAL(timeout()), SLOT(slotUpdateMap()) );

	m_pFocusTimer = new QTimer(this);
//	m_pUpdateTimer->setSingleShot(true);
	m_pFocusTimer->setInterval(20000);
	connect( m_pFocusTimer, SIGNAL(timeout()), SLOT(slotCenterMap()) );

	initLayerPropertiesWidget(1);
	initObjectPropertiesWidget(1);

}

void TreeViewWidget::customSortByColumn(int column)
{
	// here you can get the order
	Qt::SortOrder order = d->m_ui.treeTreeView->header()->sortIndicatorOrder();
	// and sort the items
	//d->m_ui.treeTreeView->sortItems(column, order);
	d->m_ui.treeTreeView->sortByColumn(column, order);

	// to get more control over actual sorting of items,
	// reimplement QTreeWidgetItem::operator<()

	/*
	bool ascendingRows( const QModelIndex &r1, const QModelIndex &r2 )
	return r1.row() < r2.row();  // or try return r1 < r2; and expand on from there until desired results

	qSort( listeSelections.begin(),listeSelections.end(), ascendingRows );
	*/
}

void TreeViewWidget::slotTreeSelectionChanged( const QItemSelection & selected, const QItemSelection & deselected )
{
	int iSel = selected.size();
}

void TreeViewWidget::addParsingFileResult( GeoDataDocument* document)
{
	if(document==NULL)
		return;

	savePlacemarks(document);

}

void TreeViewWidget::savePlacemarks(const GeoDataContainer *container)
{

    qreal lon;
    qreal lat;
    qreal alt;
	QString qsVal;
	double val=0;
	qint64 val1=0;
	qint16 val2=0;
	qint8  val3=0;

    const QVector<GeoDataPlacemark*> placemarks = container->placemarkList();
    QVector<GeoDataPlacemark*>::const_iterator it = placemarks.constBegin();
    QVector<GeoDataPlacemark*>::const_iterator const end = placemarks.constEnd();
    for (; it != end; ++it ) 
	{
        qsVal = (*it)->name();
        (*it)->coordinate( ).geoCoordinates( lon, lat, alt );

        // Use double to provide a single cache file format across architectures
        
        qsVal = QString( (*it)->role() );
        qsVal = QString( (*it)->description() );
        qsVal = QString( (*it)->countryCode() );
        qsVal = QString( (*it)->state() );
        val = (double) (*it)->area();
        val1 = (qint64) (*it)->population();
        val2 = ( qint16 ) ( (*it)->extendedData().value("gmt").value().toInt() );
        val3 = ( qint8 ) ( (*it)->extendedData().value("dst").value().toInt() );
    }

    const QVector<GeoDataFolder*> folders = container->folderList();
    QVector<GeoDataFolder*>::const_iterator cont = folders.constBegin();
    QVector<GeoDataFolder*>::const_iterator endcont = folders.constEnd();
    for (; cont != endcont; ++cont ) {
            savePlacemarks(*cont);
    }
}

void TreeViewWidget::addDetectedElevationResult( const GeoDataCoordinates pos, const qreal val  )
{
	double dElev = val;
	emit signalElevationValue(dElev);
}

void TreeViewWidget::initLayerPropertiesWidget(int init)
{
	 connect( ui_lpWidget->m_colorLineButton, SIGNAL( clicked() ),
                 SLOT( chooseLayerLineColor() ) );

	 connect( ui_lpWidget->m_colorFillButton, SIGNAL( clicked() ),
                 SLOT( chooseLayerFillColor() ) );

	 connect( ui_lpWidget->m_transSpinBox, SIGNAL( valueChanged( int ) ),
                 SLOT( synchronizeTransbox(int) ) );

	 connect( ui_lpWidget->spinBoxWidth, SIGNAL( valueChanged( int ) ),
                 SLOT( synchronizeWidthbox(int) ) );


	 int iWidth = ui_lpWidget->fontComboBox->width();
	 m_lsComboBox = new MvPenStyleComboBox(ui_lpWidget->labelPS,iWidth);

	 connect( ui_lpWidget->m_colorFontButton, SIGNAL( clicked() ),
                 SLOT( chooseLayerFontColor() ) );

	// ui_lpWidget->comboBoxPenStyle->addItem(pix,QString(),Qt::SolidLine);

}

void TreeViewWidget::initObjectPropertiesWidget(int init)
{
	 connect( ui_opWidget->m_colorLineButton, SIGNAL( clicked() ),
                 SLOT( chooseObjectLineColor() ) );

	 connect( ui_opWidget->m_colorFillButton, SIGNAL( clicked() ),
                 SLOT( chooseObjectFillColor() ) );

	 connect( ui_opWidget->m_transSpinBox, SIGNAL( valueChanged( int ) ),
                 SLOT( synchronizeObjTransbox(int) ) );

	 connect( ui_opWidget->spinBoxWidth, SIGNAL( valueChanged( int ) ),
                 SLOT( synchronizeObjWidthbox(int) ) );


	 int iWidth = ui_opWidget->fontComboBox->width();
	 m_osComboBox = new MvPenStyleComboBox(ui_opWidget->labelPS,iWidth);

	// ui_lpWidget->comboBoxPenStyle->addItem(pix,QString(),Qt::SolidLine);

}

void TreeViewWidget::chooseLayerLineColor()
{
	QColor c = QColorDialog::getColor( m_colLine, 0, 
                                       tr( "Please choose the layer line color" ), 
                                       QColorDialog::ShowAlphaChannel );
    if( c.isValid() )
    {
        m_colLine = c;
    /*    QPalette palette = ui_lpWidget->m_colorLineButton->palette();
        palette.setColor( QPalette::Button, m_colLine );
        ui_lpWidget->m_colorLineButton->setPalette( palette );*/
		QString s = "background-color: ";
		ui_lpWidget->m_colorLineButton->setStyleSheet(s + c.name());
		m_iPropSet|=1;
    }

}

void TreeViewWidget::chooseLayerFillColor()
{
	QColor c = QColorDialog::getColor( m_colFill, 0, 
                                       tr( "Please choose the layer line color" ), 
                                       QColorDialog::ShowAlphaChannel );
    if( c.isValid() )
    {
        m_colFill = c;
    //    QPalette palette = ui_lpWidget->m_colorFillButton->palette();
    //    palette.setColor( QPalette::Button, m_colFill );
    //    ui_lpWidget->m_colorFillButton->setPalette( palette );
		QString s = "background-color: ";
		ui_lpWidget->m_colorFillButton->setStyleSheet(s + c.name());
		m_iPropSet|=2;
    }

}

void TreeViewWidget::chooseLayerFontColor()
{
	QColor c = QColorDialog::getColor( m_colFont, 0, 
                                       tr( "Please choose the layer line color" ), 
                                       QColorDialog::ShowAlphaChannel );
    if( c.isValid() )
    {
        m_colFont = c;
    /*    QPalette palette = ui_lpWidget->m_colorLineButton->palette();
        palette.setColor( QPalette::Button, m_colLine );
        ui_lpWidget->m_colorLineButton->setPalette( palette );*/
		QString s = "background-color: ";
		ui_lpWidget->m_colorFontButton->setStyleSheet(s + c.name());
		m_iPropSet|=16;
    }

}

void TreeViewWidget::synchronizeTransbox(int val)
{
	m_iPropSet|=4;
}

void TreeViewWidget::synchronizeWidthbox(int val)
{
	m_iPropSet|=8;
}

void TreeViewWidget::chooseObjectLineColor()
{
	QColor c = QColorDialog::getColor( m_colLine, 0, 
                                       tr( "Please choose the layer line color" ), 
                                       QColorDialog::ShowAlphaChannel );
    if( c.isValid() )
    {
        m_colLine = c;
    /*    QPalette palette = ui_lpWidget->m_colorLineButton->palette();
        palette.setColor( QPalette::Button, m_colLine );
        ui_lpWidget->m_colorLineButton->setPalette( palette );*/
		QString s = "background-color: ";
		ui_opWidget->m_colorLineButton->setStyleSheet(s + c.name());
		m_iPropSet|=1;
    }

}

void TreeViewWidget::chooseObjectFillColor()
{
	QColor c = QColorDialog::getColor( m_colFill, 0, 
                                       tr( "Please choose the layer line color" ), 
                                       QColorDialog::ShowAlphaChannel );
    if( c.isValid() )
    {
        m_colFill = c;
    //    QPalette palette = ui_lpWidget->m_colorFillButton->palette();
    //    palette.setColor( QPalette::Button, m_colFill );
    //    ui_lpWidget->m_colorFillButton->setPalette( palette );
		QString s = "background-color: ";
		ui_opWidget->m_colorFillButton->setStyleSheet(s + c.name());
		m_iPropSet|=2;
    }

}

void TreeViewWidget::synchronizeObjTransbox(int val)
{
	m_iPropSet|=4;
}

void TreeViewWidget::synchronizeObjWidthbox(int val)
{
	m_iPropSet|=8;
}

void TreeViewWidget::setBeamPointCount(int iBeamCount)
{
	d->m_iBeamPointCount = iBeamCount;
}

void TreeViewWidget::removeLayer(qint64 objId)
{
  	if(objId>0 && d->m_marbleWidget!=NULL)
	{
		if(d->m_mvLayers.contains(objId) )
		{
			MapViewLayer* pLayer = d->m_mvLayers.value(objId);
			if(pLayer!=NULL)
			{
				d->m_marbleWidget->removeLayer(pLayer);
				d->m_mvLayers.remove(objId);
				delete pLayer;
			}
		}
		if(d->m_mvMapLayers.contains(objId) )
		{
			MapViewMapLayer* pMapLayer = d->m_mvMapLayers.value(objId);
			if(pMapLayer!=NULL)
			{
				pMapLayer->slotDeleteTreeItem(MST_DELETE_LAYER,(long)objId);

				QString filename = pMapLayer->getFileName();
				if(!filename.isEmpty())
					d->m_marbleWidget->model()->removeGeoData(filename);

				d->m_marbleWidget->removeLayer(pMapLayer);
				d->m_mvMapLayers.remove(objId);

				delete pMapLayer;
			}
		}
		//taktical layers
		if(d->m_mvTsLayers.contains(objId) )
		{
			MapViewTSLayer* pTsLayer = d->m_mvTsLayers.value(objId);
			if(pTsLayer!=NULL)
			{
				d->m_marbleWidget->removeLayer(pTsLayer);
				d->m_mvTsLayers.remove(objId);
				delete pTsLayer;
			}
		}

	}

}

//esri map slots
void TreeViewWidget::slotAddMapObject(int type,GeoDataDocument* pMap,long& objectID)
{
	if(pMap==NULL&&objectID==0)return;

	QString map;
	QString layer;

	bool bESRI=false;
	QStringList qsl;
	
	switch(type)
	{
		case MST_ADD_OVERLAY:
								if(pMap!=NULL)
								{
									map   = pMap->name();
									if(map=="EsriMap")
										bESRI=true;
									qsl << "-399" << "-399" << map;
									objectID = (long)d->m_treeViewModel->AddMapOverlay(qsl,pMap,true);
									if(bESRI)
										m_esriCacheID=(qint64)objectID;
								}
								break;
		case MST_ADD_LAYER:		
								if(pMap!=NULL)
								{
									map   = pMap->name();
									if(map=="EsriMap")
										bESRI=true;
									layer = pMap->description();
									if(layer.isEmpty())layer=map;
									qsl << "-399" << "-399" << layer;
									if(bESRI && m_esriCacheID>0)
										objectID = (long)d->m_treeViewModel->AddMapLayer(qsl,m_esriCacheID,true);//m_esriOverlayID,true);
								}
								break;
		case MST_DELETE_OVERLAY:
								if(objectID!=0)
								{
									if(objectID==m_esriCacheID)
										m_esriCacheID=0;
									slotDeleteObject(objectID,MST_DELETE_OVERLAY);
									objectID=0;
								}
								break;
		case MST_DELETE_LAYER:
								if(objectID!=0)
								{
									slotDeleteObject(objectID,MST_DELETE_LAYER);
									objectID=0;
								}
								break;

		default: break;
	}

}

//slots
void TreeViewWidget::slotAddObject(QStringList qsl,long id,int type,long& objectID)
{
	switch(type)
	{
		case MST_ADD_OVERLAY:	if(objectID== -1000)//signal sent from GeoObjectsPlugin
								{
									objectID = (long)d->m_treeViewModel->AddOverlay(qsl,false);
									m_sitOverlayID=objectID;
								}
								else objectID = (long)d->m_treeViewModel->AddOverlay(qsl,false);
								/*d->m_document = new GeoDataDocument();
								d->m_document->setName(qsl.at(2));
								d->m_fileManager->addGeoDataDocument(d->m_document);*/
								
								break;
		case MST_ADD_GEOSET_OVERLAY:
								objectID = (long)d->m_treeViewModel->AddOverlay(qsl,true);
								m_minfOverlayID=objectID;
								break;
		case MST_ADD_LEGEND_LAYER:
		case MST_ADD_LAYER:	{
								long lInterfaceID = objectID; 
								objectID=0;
								objectID = (long)d->m_treeViewModel->AddLayer(qsl,id);
								MapViewLayer* pLayer = new MapViewLayer(d->m_marbleWidget,this,(qint64)objectID,lInterfaceID);
								//pLayer->setModel((MarblePlacemarkModel *)d->m_marbleWidget->model()->placemarkModel());
								pLayer->setModel(d->m_treeViewModel);

								QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
								pLayer->synchronizeWith( selectionModel );

								pLayer->setDimmerValue(m_iDimmer);

								d->m_marbleWidget->addLayer(pLayer); 

								if(type==MST_ADD_LAYER)
								{
									connect( pLayer, SIGNAL( placemarkSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activatePlacemark( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( placemarkMultiSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activateMultiPlacemark( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( polygoneSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activatePolygone( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( placemarkDblClick( QModelIndex,long,QPoint&,bool ) ),
											 this, SLOT( dblClickPlacemark( QModelIndex,long,QPoint&,bool ) ) );
									connect( pLayer, SIGNAL( polygoneDblClick( QModelIndex,long,QPoint&,bool ) ),
											 this, SLOT( dblClickPolygone( QModelIndex,long,QPoint&,bool ) ) );
									connect( pLayer, SIGNAL( pointSelected( GeoDataCoordinates ) ),
											 this, SLOT( retrieveSelectedPoint( GeoDataCoordinates ) ) );
									connect( pLayer, SIGNAL( pointSelectionAborted() ),
											 this, SLOT( pointSelectionCanceled() ) );
									connect( pLayer, SIGNAL( placemarkMoved(QModelIndex,long) ),
											 this, SLOT( retrieveMovedPlacemark(QModelIndex,long) ) );
									connect( pLayer, SIGNAL( polygoneMoved(QModelIndex,long) ),
											 this, SLOT( retrieveMovedPolygone(QModelIndex,long) ) );

									connect( pLayer, SIGNAL( contextMenuSelected(QModelIndex,long,int&) ),
											 this, SLOT( retrieveContextMenu(QModelIndex,long,int&) ) );

									connect( pLayer, SIGNAL( deleteItem(QModelIndex,long) ),
											 this, SLOT( itemToDelete(QModelIndex,long) ) );

									connect( pLayer, SIGNAL( mouseHoversObject(QModelIndex,QPoint&,long) ),
											 this, SLOT( hoveredObject(QModelIndex,QPoint&,long) ) );
									

									//multi selection slot
									connect( pLayer, SIGNAL( placemarkMultiSelection( const QMap<int,QModelIndex>,long,bool) ),
											 this, SLOT( activateMultiSelection( const QMap<int,QModelIndex>,long,bool) ) );
									
									//synchronize layer with tree - delete item case
									//connect( this, SIGNAL( signalDeleteObject(int,long) ),
									//		 pLayer, SLOT( slotDeleteTreeItem(int,long) ) );
									
								}
								
								if(type==MST_ADD_LEGEND_LAYER)
								{
									pLayer->setLegendLayer(false);
									//set legend alignment
									if(m_overlayLegend.contains(id))
									{
										//reset legend bounding rect
									/*	m_legendRect.setLeft(1000);
										m_legendRect.setTop(1000);
										m_legendRect.setRight(0);
										m_legendRect.setBottom(0);*/
										pLayer->setLegendLayerAlignment(m_overlayLegend.value(id));
										pLayer->setLegendRect(m_legendRect);
									}
									//pLayer->setTransparency(255);
									QString qLay = qsl.at(2);
									if(qLay == "Background")//dirty - for ARGUS
									{
										m_iARGUS=1;
										pLayer->setLayerZValue(0);
									}
									//else m_iARGUS=0;
								}

								d->m_pMvLayer = pLayer;
								d->m_mvLayers.insert(objectID,pLayer);

								d->m_ui.treeTreeView->resizeColumnToContents(0);

							/*	d->m_folder = new GeoDataFolder();
								d->m_folder->setName(qsl.at(2));
								d->m_document->append(d->m_folder);*/
							}
							break;
		case MST_ADD_TS_OVERLAY:	
								objectID = (long)d->m_treeViewModel->AddOverlay(qsl,false);
								m_tsOverlayID = objectID;
								/*d->m_document = new GeoDataDocument();
								d->m_document->setName(qsl.at(2));
								d->m_fileManager->addGeoDataDocument(d->m_document);*/
								
								break;
		case MST_ADD_TS_LAYER:	{
								long lInterfaceID = objectID; 
								objectID=0;
								objectID = (long)d->m_treeViewModel->AddTakticalLayer(qsl,id);
								MapViewTSLayer* pLayer = new MapViewTSLayer(d->m_marbleWidget,this,(qint64)objectID,lInterfaceID);
								//pLayer->setModel((MarblePlacemarkModel *)d->m_marbleWidget->model()->placemarkModel());
								pLayer->setModel(d->m_treeViewModel);

								QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
								pLayer->synchronizeWith( selectionModel );

								//pLayer->setDimmerValue(m_iDimmer);

								d->m_marbleWidget->addLayer(pLayer); 

								if(type==MST_ADD_LAYER)
								{
									connect( pLayer, SIGNAL( placemarkSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activatePlacemark( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( placemarkMultiSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activateMultiPlacemark( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( polygoneSelected( QModelIndex,long,bool ) ),
											 this, SLOT( activatePolygone( QModelIndex,long,bool ) ) );
									connect( pLayer, SIGNAL( placemarkDblClick( QModelIndex,long,QPoint&,bool ) ),
											 this, SLOT( dblClickPlacemark( QModelIndex,long,QPoint&,bool ) ) );
									connect( pLayer, SIGNAL( polygoneDblClick( QModelIndex,long,QPoint&,bool ) ),
											 this, SLOT( dblClickPolygone( QModelIndex,long,QPoint&,bool ) ) );
									connect( pLayer, SIGNAL( pointSelected( GeoDataCoordinates ) ),
											 this, SLOT( retrieveSelectedPoint( GeoDataCoordinates ) ) );
									connect( pLayer, SIGNAL( pointSelectionAborted() ),
											 this, SLOT( pointSelectionCanceled() ) );
									connect( pLayer, SIGNAL( placemarkMoved(QModelIndex,long) ),
											 this, SLOT( retrieveMovedPlacemark(QModelIndex,long) ) );
									connect( pLayer, SIGNAL( polygoneMoved(QModelIndex,long) ),
											 this, SLOT( retrieveMovedPolygone(QModelIndex,long) ) );

									connect( pLayer, SIGNAL( contextMenuSelected(QModelIndex,long,int&) ),
											 this, SLOT( retrieveContextMenu(QModelIndex,long,int&) ) );

									connect( pLayer, SIGNAL( deleteItem(QModelIndex,long) ),
											 this, SLOT( itemToDelete(QModelIndex,long) ) );

									connect( pLayer, SIGNAL( mouseHoversObject(QModelIndex,QPoint&,long) ),
											 this, SLOT( hoveredObject(QModelIndex,QPoint&,long) ) );
									

									//synchronize layer with tree - delete item case
									//connect( this, SIGNAL( signalDeleteObject(int,long) ),
									//		 pLayer, SLOT( slotDeleteTreeItem(int,long) ) );
									
								}

								m_tsLayerID = objectID;

								//??d->m_pMvLayer = pLayer;
								d->m_mvTsLayers.insert(objectID,pLayer);

								d->m_ui.treeTreeView->resizeColumnToContents(0);

							/*	d->m_folder = new GeoDataFolder();
								d->m_folder->setName(qsl.at(2));
								d->m_document->append(d->m_folder);*/
							}
							break;
		case MST_ADD_TS_GROUP:		
								objectID = (long)d->m_treeViewModel->AddGroup(qsl,id);
								m_bRedraw=true;
								m_tsGroupID=objectID;
								break;

		case MST_ADD_SHADING_LAYER:	
							{
								long lInterfaceID = objectID;
								objectID = (long)d->m_treeViewModel->AddLayer(qsl,id);
								//id=objectID;

								MapViewShadingLayer* pMvShadingLayer = new MapViewShadingLayer(d->m_marbleWidget,this,(qint64)objectID,lInterfaceID);
								if(pMvShadingLayer!=NULL)
								{
									pMvShadingLayer->setModel(d->m_treeViewModel);

									QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
									pMvShadingLayer->synchronizeWith( selectionModel );

									d->m_marbleWidget->addLayer(pMvShadingLayer);

									d->m_mvShadingLayers.insert(objectID,pMvShadingLayer);

									connect( pMvShadingLayer, SIGNAL( contextMenuSelected(QModelIndex,long,int&) ),
											 this, SLOT( retrieveContextMenu(QModelIndex,long,int&) ) );

									connect( pMvShadingLayer, SIGNAL( signalNotifyMouseAction(QPoint&,int,long) ),
											 this, SLOT( notifyMouseAction(QPoint&,int,long) ) );
									
								}

							}
							break;
		case MST_ADD_GROUP:			objectID = (long)d->m_treeViewModel->AddGroup(qsl,id);
								/*	{
									GeoDataFolder* pFolder = new GeoDataFolder();
									pFolder->setName(qsl.at(2));
									d->m_folder->append(pFolder);
									}*/
									/*if(m_pUpdateTimer!=NULL)
									{
										m_pUpdateTimer->stop();
										m_pUpdateTimer->start();
									}*/
									m_bRedraw=true;
									break;


		case MST_DELETE_BY_NAME:	deleteObjects(qsl,0,type);
									//objectID = (long)d->m_treeViewModel->DeleteByName(qsl);
									//removeLayer(objectID);
									break;
		
		default: break;
	}
	
}
void TreeViewWidget::slotAddImageObject(QString& qs,long id,int type,long xoffs, long yoffs, long align, long idx, long& objectID,bool bMulti)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode==NULL)return;

	//enable region calculation for objects
	MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
	if(pLayer!=NULL)
		pLayer->setLayerUpdate(true);

	switch(type) 
	{
		case MST_ADD_ICON:		
			{
				if(pNode!=NULL)
				{
					//GeoDataCoordinates position;// = pNode->m_position;//.at(0);
					//position.set(pNode->m_position.longitude(),pNode->m_position.latitude,0,GeoDataCoordinates::Degree);
					//QStringList qsl;
					//qsl << position.lonToString() << position.latToString();

					qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
					qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

					objectID = (long)d->m_treeViewModel->AddIcon(qs,id,lon,lat,xoffs,yoffs,align,idx);

				/*	GeoDataCoordinates* cor = new GeoDataCoordinates();
					cor->set(lon,lat,0,GeoDataCoordinates::Degree);
					m_objectCache.insert((qint64)objectID,cor);*/

					//3D
					if(m_b3D)
						emit signal3dAddImageObject(objectID,qs,lon,lat);
				
					if(lon!=-399.0 && lat!=-399.0)
					{
						if(objectID>0 && d->m_marbleWidget!=NULL)
						{
							QRegion rg;
							QRect rc;
							qreal lon1=lon;
							qreal lat1=lat;
							qreal lx=0;
							qreal ly=0;
							d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
							rc.setLeft(lx-20);
							rc.setRight(lx+20);
							rc.setTop(ly-20);
							rc.setBottom(ly+20);
							rg.setRects(&rc,0);
							m_dirtyRG |= rg;
						//	d->m_marbleWidget->update(rc);
							//d->m_marbleWidget->repaint(rc);
						}

						if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
						{
							//m_pUpdateTimer->stop();
							if(!bMulti)
								m_pUpdateTimer->start();
						}
						else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
						{
							if(m_iRepaintCount<10)
							{
								m_pUpdateTimer->stop();
								m_iRepaintCount++;
								if(!bMulti)
									m_pUpdateTimer->start();
							}
						}
					}//valid coords ?
			
				}
			}
			break;	
		case MST_ADD_MARKER:		
			{
				if(pNode!=NULL)
				{
					//draw location plot always on top
					if(pNode->ItemType == Group)
					{
						if(pNode->parent->MarkerType == Circle)
						{
						/*	qint64 layerID = pNode->parent->m_lId;
							if(d->m_mvLayers.contains(layerID) )
							{
								MapViewLayer* pLayer = d->m_mvLayers.value(layerID);
								if(pLayer!=NULL)
									pLayer->setLayerZValue(3.0);
							}*/
							m_bRedraw=true;
						}
					}
					//GeoDataCoordinates position;// = pNode->m_position;//.at(0);
					//position.set(pNode->m_position.longitude(),pNode->m_position.latitude,0,GeoDataCoordinates::Degree);
					//QStringList qsl;
					//qsl << position.lonToString() << position.latToString();
					qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
					qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

					objectID = (long)d->m_treeViewModel->AddMarker(qs,id,lon,lat,xoffs,yoffs,align);

					//3D view
					if(m_b3D && objectID>0)
						emit signal3dAddMarkerObject(objectID,qs,pNode->parent->MarkerType,lon,lat);

					if(lon!=-399.0 && lat!=-399.0)
					{
						if(objectID>0 && d->m_marbleWidget!=NULL)
						{
							QRegion rg;
							QRect rc;
							qreal lon1=lon;
							qreal lat1=lat;
							qreal lx=0;
							qreal ly=0;
							d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
							rc.setLeft(lx-20);
							rc.setRight(lx+20);
							rc.setTop(ly-20);
							rc.setBottom(ly+20);
							rg.setRects(&rc,0);
							m_dirtyRG |= rg;
						//	d->m_marbleWidget->update(rc);
						//	d->m_marbleWidget->repaint(rc);
						}
						if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
						{
							//m_pUpdateTimer->stop();
							if(!bMulti)
								m_pUpdateTimer->start();
						}
						else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
						{
							if(m_iRepaintCount<10)
							{
								m_pUpdateTimer->stop();
								m_iRepaintCount++;
								if(!bMulti)
									m_pUpdateTimer->start();
							}
						}
					}//valid coords?
				}
			}
			break;	
		
		default: break;
	}

}

void TreeViewWidget::slotAddEllipse(QString& qsl,long id,int type,double radius,double longitude,double latitude,double minor,double major, long& objectID)
{
//	QRegion rg = d->m_marbleWidget->activeRegion();
	switch(type)
	{
		
		case 1:		
			{
				qint64 qID = (qint64)id;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
				MapViewLayer* pLayer = NULL;
				if(pNode!=NULL)
				{
					//enable region calculation for objects
					pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
					if(pLayer!=NULL)
						pLayer->setLayerUpdate(true);
				}

				if(qID>0)//pNode!=NULL)
				{
					// Initialize variables
					qreal centerLon = longitude;
					qreal centerLat = latitude;
					qreal altitude = 0.0;//centerPosition.altitude();
					//centerPosition.geoCoordinates( centerLon, centerLat,
					//							   GeoDataCoordinates::Degree );

					qreal width = (qreal)minor; 
					qreal height= (qreal)major;
					qreal DEG2RAD = PI / 180.0;
					qreal RAD2DEG = 180.0 / PI;

					// Ensure a valid latitude range: 
					if ( centerLat + 0.5 * height > 90.0 || centerLat - 0.5 * height < -90.0 ) {
						return;
					}

					// Don't show the ellipse if it's too small:
					GeoDataLatLonBox ellipseBox( centerLat + 0.5 * height, centerLat - 0.5 * height,
												 centerLon + 0.5 * width,  centerLon - 0.5 * width, 
												 GeoDataCoordinates::Degree );
					
					if ( !d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( ellipseBox ) ||
						 !d->m_marbleWidget->viewport()->resolves( ellipseBox ) ) return;

					GeoDataLineString ellipse;
					qreal lon = 0.0;
					qreal lat = 0.0;

					// Optimizing the precision by determining the size which the 
					// ellipse covers on the screen:
					qreal degreeResolution = d->m_marbleWidget->viewport()->angularResolution() * RAD2DEG;
					// To create a circle shape even for very small precision we require uneven numbers:
					int precision = width / degreeResolution / 8 + 1; 
					if ( precision > 81 ) precision = 81;
					precision = 180;

					// Calculate the shape of the upper half of the ellipse:
					for ( int i = 0; i < precision; ++i ) {
						qreal t = 1.0 - 2.0 * (qreal)(i) / (qreal)(precision);
						lat = centerLat + 0.5 * height * sqrt( 1.0 - t * t );
						lon = centerLon + 0.5 * width * t;
						ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
					}
					// Calculate the shape of the lower half of the ellipse:
					for ( int i = 0; i < precision; ++i ) {
						qreal t = 2.0 * (qreal)(i) / (qreal)(precision) -  1.0;
						lat = centerLat - 0.5 * height * sqrt( 1.0 - t * t );
						lon = centerLon + 0.5 * width * t;
						ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
					}

					objectID = (long)d->m_treeViewModel->AddEllipse(qsl,id,centerLon,centerLat,ellipse);
					if(objectID>0 && d->m_marbleWidget!=NULL)
					{
						QRegion rg;
						QRect rc;
						qreal lon1=lon;
						qreal lat1=lat;
						qreal lx=0;
						qreal ly=0;
						d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
						rc.setLeft(lx-20);
						rc.setRight(lx+20);
						rc.setTop(ly-20);
						rc.setBottom(ly+20);
						rg.setRects(&rc,0);
						m_dirtyRG |= rg;
					//	d->m_marbleWidget->update(rc);
					//	d->m_marbleWidget->repaint(rc);
					}

					if(m_b3D)
					{
						QColor col;
						long fillcol = 0;
						long linecol = 0;
						if(pLayer!=NULL)
						{
							col = pLayer->getFillColor();
							fillcol = col.blue();
							fillcol <<=8;
							fillcol += col.green();
							fillcol <<=8;
							fillcol += col.red();
	
							col = pLayer->getLineColor();
							linecol = col.blue();
							linecol <<=8;
							linecol += col.green();
							linecol <<=8;
							linecol += col.red();
						}

						emit signal3dAddGeoObject(objectID,2,lon,lat,fillcol,linecol,ellipse);
					}

					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}
				//	if(objectID>0 && d->m_marbleWidget!=NULL)
				//		d->m_marbleWidget->update(rg);
				//		d->m_marbleWidget->repaint(); 
				}
			}
			break;	
		
		default: break;
	}

}

void TreeViewWidget::slotAddPlotObject(QStringList& qsl,long id,int type,double radius,double longitude,double latitude,long flags,GeoDataLineString& gdl,long& objectID)
{
//	QRegion rg = d->m_marbleWidget->activeRegion();

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	//enable region calculation for objects
	if(pNode!=NULL)
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);
	}

	switch(type)
	{
		case MST_ADD_PLOT:		
			{
				
				//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
				if(qID>0)//pNode!=NULL)
				{
//					
					objectID = (long)d->m_treeViewModel->AddPlot(qsl,id,(qreal)longitude,(qreal)latitude,radius,gdl,flags);
					if(objectID>0 && d->m_marbleWidget!=NULL)
					{
						//d->m_marbleWidget->update(rg);
						//GeoDataLineString gdl; 
						//gdl.append(GeoDataCoordinates(lon,lat,0));
						if(1)//d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( gdl.latLonAltBox()) )
						{
							QRegion rg;
							QRect rc;
							qreal lon1=(qreal)longitude;
							qreal lat1=(qreal)latitude;
							qreal lx=0;
							qreal ly=0;
							d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
							rc.setLeft(lx-20);
							rc.setRight(lx+20);
							rc.setTop(ly-20);
							rc.setBottom(ly+20);
							rg.setRects(&rc,0);
							m_dirtyRG |= rg;
						//	d->m_marbleWidget->update(rc);
						//	d->m_marbleWidget->repaint(rc);
						}

						//3D
						if(m_b3D)
						{
							//add plot ?
							if(flags & 0x0010)
								emit signal3dAddPlot(objectID,qsl,longitude,latitude,radius,gdl);
							else
								emit signal3dAddPlot(objectID,qsl,longitude,latitude,0,gdl);
						}

						if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
						{
							//m_pUpdateTimer->stop();
							m_pUpdateTimer->start();
						}
						else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
						{
							if(m_iRepaintCount<10)
							{
								m_pUpdateTimer->stop();
								m_iRepaintCount++;
								m_pUpdateTimer->start();
							}
						}
					}
				//	if(objectID>0 && d->m_marbleWidget!=NULL)
				//		d->m_marbleWidget->repaint();
				}
			}
			break;	
		case MST_ADD_PLOT_EX:		
			{
				//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
				if(qID>0)//pNode!=NULL)
				{
//					GeoDataCoordinates position;// = pNode->m_position;//.at(0); //.set(lon,lat,0,GeoDataCoordinates::Degree);
					//position.set(pNode->m_position.longitude(),pNode->m_position.latitude,0,GeoDataCoordinates::Degree);
				//	pNode->m_position.setLongitude(longitude,GeoDataCoordinates::Degree);
				//	pNode->m_position.setLatitude(latitude,GeoDataCoordinates::Degree);
//					pNode->m_position.clear();
//					pNode->m_position.set(position.longitude(),position.latitude(),0,GeoDataCoordinates::Degree);

					objectID = (long)d->m_treeViewModel->AddPlotEx(qsl,id,(qreal)longitude,(qreal)latitude,gdl,flags);
					if(objectID>0 && d->m_marbleWidget!=NULL)
					{
						QRegion rg;
						QRect rc;
						qreal lon1=(qreal)longitude;
						qreal lat1=(qreal)latitude;
						qreal lx=0;
						qreal ly=0;
						d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
						rc.setLeft(lx-20);
						rc.setRight(lx+20);
						rc.setTop(ly-20);
						rc.setBottom(ly+20);
						rg.setRects(&rc,0);
						m_dirtyRG |= rg;
					//	d->m_marbleWidget->update(rc);
					//	d->m_marbleWidget->repaint(rc);
					}

					//3D
					if(m_b3D)
					{
						//add plot ?
						if(flags & 0x0010)
							emit signal3dAddPlotEx(objectID,qsl,longitude,latitude,gdl);
						else
							emit signal3dAddPlotEx(objectID,qsl,longitude,latitude,gdl);
					}

					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}
				//	if(objectID>0 && d->m_marbleWidget!=NULL)
				//		d->m_marbleWidget->update(rg);
				//		d->m_marbleWidget->repaint(); 
				}
			}
			break;	

		case MST_ADD_COLOR_PLOT:
			{
				if(qID>0)//pNode!=NULL)
				{
					// flags = lineCol
					// objectID = fillCol
					QString qsTemp; 
					if(qsl.count()>0)
						qsTemp= qsl.at(0);
					objectID = (long)d->m_treeViewModel->AddColorEllipse(qsTemp,id,flags,objectID,(qreal)longitude,(qreal)latitude,gdl);
					if(objectID>0 && d->m_marbleWidget!=NULL)
					{
						QRegion rg;
						QRect rc;
						qreal lon1=(qreal)longitude;
						qreal lat1=(qreal)latitude;
						qreal lx=0;
						qreal ly=0;
						d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
						rc.setLeft(lx-20);
						rc.setRight(lx+20);
						rc.setTop(ly-20);
						rc.setBottom(ly+20);
						rg.setRects(&rc,0);
						m_dirtyRG |= rg;
					//	d->m_marbleWidget->update(rc);
					//	d->m_marbleWidget->repaint(rc);
					}
					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}
				}


			}
			break;
		
		default: break;
	}

}

#define CALCLEN(alpha)\
			double cosa=cos((alpha)*PI/180);\
			if (fabs(cosa)>0.0005) length=length/cosa;

void TreeViewWidget::slotAddAzimuthLine(long id,int type,long azimuth,double longitude,double latitude,GeoDataLineString& gdl,long& objectID,bool bMulti)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && (pNode->ItemType == Group || pNode->ItemType == Azimuth) )
	{

	//enable region calculation for objects
		if(pNode->ItemType==Group)
		{
			MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
			if(pLayer!=NULL)
				pLayer->setLayerUpdate(true);
		}
		else
		{
			MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->parent->m_lId);
			if(pLayer!=NULL)
				pLayer->setLayerUpdate(true);
		}

		GeoDataCoordinates position;// = pNode->m_position.at(0); //.set(lon,lat,0,GeoDataCoordinates::Degree);
//		position.setLongitude(longitude,GeoDataCoordinates::Degree);
//		position.setLatitude(latitude,GeoDataCoordinates::Degree);
//		pNode->m_position.clear();
//		pNode->m_position.append(position);	

		//draw azimuth always on top
	/*	if(pNode->ItemType == Group)
		{
			qint64 layerID = pNode->parent->m_lId;
			if(d->m_mvLayers.contains(layerID) )
			{
				MapViewLayer* pLayer = d->m_mvLayers.value(layerID);
				if(pLayer!=NULL)
					pLayer->setLayerZValue(2.0);
			}
		}*/


		QRegion rg = d->m_marbleWidget->mapRegion();
		QRect qr = rg.boundingRect();

		qreal lon = (qreal)longitude;//position.longitude(GeoDataCoordinates::Degree);
		qreal lat = (qreal)latitude;//position.latitude(GeoDataCoordinates::Degree);

		//Marble::TessellationFlags f;
		//f = Marble::TessellationFlag::NoTessellation;
		//GeoDataLineString gdl(f) ;
		//position.detach();
		//pNode->m_poly.clear();


		pNode->m_position.setLongitude(lon,GeoDataCoordinates::Degree);
		pNode->m_position.setLatitude(lat,GeoDataCoordinates::Degree);	
		//also set the group coord
		if(pNode->ItemType == Azimuth)
		{
			TreeNode* pParent = pNode->parent;
			if(pParent!=NULL)
			{
				pParent->m_position.setLongitude(lon,GeoDataCoordinates::Degree);
				pParent->m_position.setLatitude(lat,GeoDataCoordinates::Degree);	
			}

		}

			switch(type)
			{
				case MST_ADD_AZIMUTH:		
				{
				//	position.setLongitude(lon,GeoDataCoordinates::Degree);
				//	position.setLatitude(lat,GeoDataCoordinates::Degree);
				//	gdl.append(position);
					objectID = (long)d->m_treeViewModel->AddAzimuthLine(id,lon,lat,azimuth);

					if(m_b3D)
						emit signal3dAddAzimuth(objectID,lon,lat,azimuth,gdl);

				//	if(objectID>0 && d->m_marbleWidget!=NULL)
				//			d->m_marbleWidget->repaint();
				//	m_bRedraw=false;
					if(0)//objectID>0)
					{
						QRect rc;
						qreal lon1=lon;
						qreal lat1=lat;
						qreal lx=0;
						qreal ly=0;
						d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
						rc.setLeft(lx-20);
						rc.setRight(lx+20);
						rc.setTop(ly-20);
						rc.setBottom(ly+20);
						//d->m_marbleWidget->update(rc);
					}
				}
				break;
				case MST_MOVE_AZIMUTH:		
				{
						//make history beams longer
					/*	if(pNode->parent->parent->parent->m_qsName == "LHistory")
						{
							if(totalDistance<2000000)
								totalDistance=2000000;
						}*/
					
						objectID = (long)d->m_treeViewModel->MoveAzimuthLine(id,lon,lat,azimuth,gdl);

						if(m_b3D)
							emit signal3dAddAzimuth(objectID,lon,lat,azimuth,gdl);
						
						if(0)//!m_bRedraw )//&&!m_pUpdateTimer->isActive() )
						{
							m_pUpdateTimer->stop();
							d->m_marbleWidget->repaint(rg);
						}
						else
						{
							m_dirtyRG = rg;
							if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
							{
								//m_pUpdateTimer->stop();
								m_pUpdateTimer->start();
							}
						} 

					//	if(objectID>0 && d->m_marbleWidget!=NULL)
					//		d->m_marbleWidget->repaint(rg);
						if(0)//objectID>0 && d->m_marbleWidget!=NULL)
						{
						//	QRegion rg;
							rg = QRegion();
							QRect rc;
							qreal lx1=0;
							qreal ly1=0;
							qreal lx2=0;
							qreal ly2=0;

							position = gdl.at(0);
							qreal lon1= position.longitude();
							qreal lat1= position.latitude();
							d->m_marbleWidget->screenCoordinates(lon1,lat1,lx1,ly1);
							
							position = gdl.at(gdl.size()-1);
							qreal lon2= position.longitude();
							qreal lat2= position.latitude();
							d->m_marbleWidget->screenCoordinates(lon2,lat2,lx2,ly2);

							if(lx1 < lx2)//right
							{
								if(ly1>ly2)//top
								{
									rc.setLeft(lx1);
									rc.setRight(lx2);
									rc.setTop(ly2);
									rc.setBottom(ly1);
									rg.setRects(&rc,0);
								}
								else
								{
									rc.setLeft(lx1);
									rc.setRight(lx2);
									rc.setTop(ly1);
									rc.setBottom(ly2);
									rg.setRects(&rc,0);
								}
							}
							else
							{
								if(ly1>ly2)//top
								{
									rc.setLeft(lx2);
									rc.setRight(lx1);
									rc.setTop(ly2);
									rc.setBottom(ly1);
									rg.setRects(&rc,0);
								}
								else
								{
									rc.setLeft(lx2);
									rc.setRight(lx1);
									rc.setTop(ly1);
									rc.setBottom(ly2);
									rg.setRects(&rc,0);
								}
							}

							m_dirtyRG = rg;
							if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
							{
								//m_pUpdateTimer->stop();
								if(!bMulti)
									m_pUpdateTimer->start();
							}
						}
					
				}
				break;

				default: break;
			}
		
	}

}

void TreeViewWidget::slotAddPoly(long id,int type,long lCol,GeoDataLineString& gdl, long& objectID)
{

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && pNode->parent!=NULL && pNode->ItemType == Group )
	{	
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		bool bProfile=false;

		if(lCol==-1 || lCol==77777)
		{
			if(lCol==77777)
				bProfile=true;

			TreeNode* pParent = pNode->parent;
			if(pParent!=NULL)
				lCol = pParent->m_lLineCol;
		}

		switch(type)
		{
			case MST_ADD_POLY:
			case MST_ADD_COLOR_POLY_LIST:
				//qDebug() << "PolyLine: " << objectID;

				if(type==MST_ADD_POLY)
					objectID = (long)d->m_treeViewModel->AddPolyLine(id,lCol,gdl);
				else
				{
					if(pNode->children.count()>0)
					{
						int gi = 0;
						int is = 0;
						long actCol=0;
						TreeNode* pChild=NULL;
						for(int i=0;i<pNode->children.count();i++)
						{
							pChild = pNode->children.at(i);
							if(pChild!=NULL && pChild->ItemType==Poly)
							{
								actCol = pChild->m_lLineCol;
								if(actCol==lCol)
								{
									gi = gdl.size();
									is = pChild->m_poly.size();
									GeoDataCoordinates pos1 = pChild->m_poly.at(pChild->m_poly.size()-1);
									GeoDataCoordinates pos2 = gdl.at(gi-1);
									if( (pos1.latitude()==pos2.latitude() && pos1.longitude()==pos2.longitude())||gi==1)
									{
										for(int x=gi-1;x<gdl.size();x++)
											pChild->m_poly.append(gdl.at(x));
										i=pNode->children.count();
										objectID = pChild->m_lId;
										is = pChild->m_poly.size();
										pChild->m_qsName.clear();
										pChild->m_qsName = "ColorPoly-%1";
										pChild->m_qsName = pChild->m_qsName.arg(is);
									}
									else actCol=0;
								}
								else actCol=0;
							}
						}
						if(actCol==0)
							objectID = (long)d->m_treeViewModel->AddPolyLine(id,lCol,gdl);
					}
					else
						objectID = (long)d->m_treeViewModel->AddPolyLine(id,lCol,gdl);
				}
				//qDebug() << "PolyLine: " << objectID;

				if(pLayer!=NULL && !pLayer->getLegendLayer()&&m_iARGUS<2)
				{
					m_legendRect.setLeft(1000);
					m_legendRect.setTop(1000);
					m_legendRect.setRight(0);
					m_legendRect.setBottom(0);
					GeoDataCoordinates qPos;
					for(int i=0;i<gdl.size();i++)
					{
						qPos = gdl.at(i);
						if(qPos.longitude(GeoDataCoordinates::Degree)<m_legendRect.left())
							m_legendRect.setLeft(qPos.longitude(GeoDataCoordinates::Degree));
						if(qPos.longitude(GeoDataCoordinates::Degree)>m_legendRect.right())
							m_legendRect.setRight(qPos.longitude(GeoDataCoordinates::Degree));
						if(qPos.latitude(GeoDataCoordinates::Degree)<m_legendRect.top())
							m_legendRect.setTop(qPos.latitude(GeoDataCoordinates::Degree));
						if(qPos.latitude(GeoDataCoordinates::Degree)>m_legendRect.bottom())
							m_legendRect.setBottom(qPos.latitude(GeoDataCoordinates::Degree));
					}
					pLayer->setLegendRect(m_legendRect);
					if(m_iARGUS==1)
						m_iARGUS=2;
				}
							
				if(objectID>0 && d->m_marbleWidget!=NULL)
				{
					QRegion rg = d->m_marbleWidget->mapRegion();
			//		d->m_marbleWidget->update(rg);
					if(0)//d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( gdl.latLonAltBox()) )
					{
						/*GeoPainter pt;
						QRegion rg = pt.regionFromPolyline(gdl);
						if(!rg.isEmpty())
							d->m_marbleWidget->update(rg);*/
					//	d->m_marbleWidget->update();
					}
					m_dirtyRG |= rg;
					
					if(m_b3D)
					{
						QColor col;
						long fillcol = 0;
						long linecol = 0;
						if(pLayer!=NULL)
						{
							col = pLayer->getLineColor();
							linecol = col.blue();
							linecol <<=8;
							linecol += col.green();
							linecol <<=8;
							linecol += col.red();

							if(lCol>0)
								linecol=lCol;
						}
						else if(lCol>0)
							linecol=lCol;
						emit signal3dAddGeoObject(objectID,3,0,0,fillcol,linecol,gdl);
					}

					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}

					//from GeoObjects plugin - measure tool
					if(bProfile)
					{
						qreal qRad = d->m_marbleWidget->model()->planet()->radius();
						qreal qDist = gdl.length(qRad);
						QString qsGroup;

						QLocale::MeasurementSystem measurementSystem;
						measurementSystem = MarbleGlobal::getInstance()->locale()->measurementSystem();

						if ( measurementSystem == QLocale::MetricSystem ) {
							if ( qDist >= 1000.0 ) {
								qsGroup = tr("%1 km").arg( qDist/1000.0 );
							}
							else {
								qsGroup = tr("%1 m").arg( qDist );
							}
						}
						else {
							qsGroup = QString("%1 mi").arg( qDist/1000.0 * KM2MI );
						}

						if(!qsGroup.isEmpty())
							d->m_treeViewModel->ModifyObjectText(objectID,qsGroup);
						emit signalUpdateElevationProfile(gdl);
					}
				}
				break;
			case MST_CREATE_POLY:
				{
					TreeNode* pLayer   = pNode->parent;
					TreeNode* pOverlay = pLayer->parent;
					//objectID = (long)d->m_treeViewModel->AddPolyLine(id,lCol,gdl);
					objectID = (long)d->m_treeViewModel->AddPolygon(id,lCol,gdl);
					if(objectID>0)
					{
						emit signalAddItem(MST_CREATE_POLY,pOverlay->m_lId,pLayer->m_lId,pNode->m_lId,objectID);
					}
				}
				break;
			default: break;
		}//switch
	//	gdl.clear();

	}//if marble

}

void TreeViewWidget::slotUpdatePoly(long id,int type,long lCol,GeoDataLineString& gdl, long& objectID)
{

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL )//&& pNode->ItemType == Group )
	{	
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		if(lCol==-1)
		{
			TreeNode* pParent = pNode->parent->parent;
			if(pParent!=NULL)
				lCol = pParent->m_lLineCol;
		}

		objectID = (long)d->m_treeViewModel->UpdatePolyLine(id,lCol,gdl);
	}
}
void TreeViewWidget::slotAddPolygon(long id,int type,long lCol,GeoDataLineString& gdl, long& objectID)
{

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && pNode->ItemType == Group )
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		if(lCol==-1)
		{
			TreeNode* pParent = pNode->parent;
			if(pParent!=NULL)
				lCol = pParent->m_lFillCol;
		}

		if(objectID==0)//add new object
		{
			objectID = (long)d->m_treeViewModel->AddPolygon(id,lCol,gdl);
							
			if(objectID>0 && d->m_marbleWidget!=NULL)
			{
				QRegion rg = d->m_marbleWidget->mapRegion();
		//		d->m_marbleWidget->update(rg);
				if(0)//d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( gdl.latLonAltBox()) )
				{
					/*GeoPainter pt;
					QRegion rg = pt.regionFromPolyline(gdl);
					if(!rg.isEmpty())
						d->m_marbleWidget->update(rg);*/
				//	d->m_marbleWidget->update();
				}
				m_dirtyRG |= rg;

				if(m_b3D)
				{
					QColor col;
					long fillcol = 0;
					long linecol = 0;
					if(pLayer!=NULL)
					{
						col = pLayer->getFillColor();
						fillcol = col.blue();
						fillcol <<=8;
						fillcol += col.green();
						fillcol <<=8;
						fillcol += col.red();

						col = pLayer->getLineColor();
						linecol = col.blue();
						linecol <<=8;
						linecol += col.green();
						linecol <<=8;
						linecol += col.red();

						if(lCol>0)
							linecol=lCol;
					}
					else if(lCol>0)
						linecol=lCol;

					emit signal3dAddGeoObject(objectID,1,0,0,fillcol,linecol,gdl);
				}
				if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
				{
					//m_pUpdateTimer->stop();
					m_pUpdateTimer->start();
				}
				else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
				{
					if(m_iRepaintCount<10)
					{
						m_pUpdateTimer->stop();
						m_iRepaintCount++;
						m_pUpdateTimer->start();
					}
				}
			}//if objectID>0
		}//if objectID
		else //modify existing
		{
			qID = (qint64)objectID;
			TreeNode* pItem = d->m_treeViewModel->getNodeFromIndexId(qID);
			if(pItem!=NULL)
			{
				pItem->m_poly.clear();
				pItem->m_poly << gdl;

				GeoDataCoordinates pos = gdl.at(0);
				pItem->parent->m_position.set(pos.longitude(),pos.latitude(),0);
				d->m_marbleWidget->update();

				QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
				if(idx.isValid())
				{
					if(d->m_mvLayers.contains(pItem->parent->parent->m_lId) )
					{
						MapViewLayer* pLayer = d->m_mvLayers.value(pItem->parent->parent->m_lId);
						if(pLayer!=NULL)
							retrieveMovedPolygone(idx,pLayer->getInterfaceID());
					}
				}
			}//POLY
	/*		else if(pItem->ItemType==Rect)
			{
				QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
				if(idx.isValid())
				{
					if(d->m_mvLayers.contains(pItem->parent->parent->m_lId) )
					{
						MapViewLayer* pLayer = d->m_mvLayers.value(pItem->parent->parent->m_lId);
						if(pLayer!=NULL)
							retrieveMovedPolygone(idx,pLayer->getInterfaceID());
					}
				}

			}//RECT*/
		}//else

	//	gdl.clear();

	}

}

void TreeViewWidget::slotAddColorPolygon(long id,QString& qs,long fCol,long lCol,int width,GeoDataLineString& gdl, long& objectID)
{

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && pNode->ItemType == Group )
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		if(lCol==-1)
		{
			TreeNode* pParent = pNode->parent;
			if(pParent!=NULL)
				lCol = pParent->m_lFillCol;
		}

		if(objectID==0)//add new object
		{
			GeoDataCoordinates pos = gdl.at(0);
			objectID = (long)d->m_treeViewModel->AddColorPolygone(qs,id,lCol,fCol,pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),width,gdl);
							
			if(objectID>0 && d->m_marbleWidget!=NULL)
			{
				QRegion rg = d->m_marbleWidget->mapRegion();
		//		d->m_marbleWidget->update(rg);
				if(0)//d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( gdl.latLonAltBox()) )
				{
					/*GeoPainter pt;
					QRegion rg = pt.regionFromPolyline(gdl);
					if(!rg.isEmpty())
						d->m_marbleWidget->update(rg);*/
				//	d->m_marbleWidget->update();
				}
				m_dirtyRG |= rg;

				if(m_b3D)
				{
					QColor col;
					long fillcol = 0;
					long linecol = 0;
					if(pLayer!=NULL)
					{
						col = pLayer->getFillColor();
						fillcol = col.blue();
						fillcol <<=8;
						fillcol += col.green();
						fillcol <<=8;
						fillcol += col.red();

						col = pLayer->getLineColor();
						linecol = col.blue();
						linecol <<=8;
						linecol += col.green();
						linecol <<=8;
						linecol += col.red();

						if(lCol>0)
							linecol=lCol;
					}
					else if(lCol>0)
						linecol=lCol;

					emit signal3dAddGeoObject(objectID,1,0,0,fillcol,linecol,gdl);
				}
				if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
				{
					//m_pUpdateTimer->stop();
					m_pUpdateTimer->start();
				}
				else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
				{
					if(m_iRepaintCount<10)
					{
						m_pUpdateTimer->stop();
						m_iRepaintCount++;
						m_pUpdateTimer->start();
					}
				}
			}//if objectID>0
		}//if objectID
		else //modify existing
		{
			qID = (qint64)objectID;
			TreeNode* pItem = d->m_treeViewModel->getNodeFromIndexId(qID);
			if(pItem!=NULL)
			{
				pItem->m_poly.clear();
				pItem->m_poly << gdl;

				GeoDataCoordinates pos = gdl.at(0);
				pItem->parent->m_position.set(pos.longitude(),pos.latitude(),0);
				d->m_marbleWidget->update();

				QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
				if(idx.isValid())
				{
					if(d->m_mvLayers.contains(pItem->parent->parent->m_lId) )
					{
						MapViewLayer* pLayer = d->m_mvLayers.value(pItem->parent->parent->m_lId);
						if(pLayer!=NULL)
							retrieveMovedPolygone(idx,pLayer->getInterfaceID());
					}
				}
			}//POLY
	/*		else if(pItem->ItemType==Rect)
			{
				QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
				if(idx.isValid())
				{
					if(d->m_mvLayers.contains(pItem->parent->parent->m_lId) )
					{
						MapViewLayer* pLayer = d->m_mvLayers.value(pItem->parent->parent->m_lId);
						if(pLayer!=NULL)
							retrieveMovedPolygone(idx,pLayer->getInterfaceID());
					}
				}

			}//RECT*/
		}//else

	//	gdl.clear();

	}

}

void TreeViewWidget::slotAddRect(long id,int type,double left, double top,double right,double bottom,long& objectID)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode != NULL && pNode->ItemType == Group)
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
		{
			pLayer->setLayerUpdate(true);

			if(!pLayer->getLegendLayer())
			{
		//		if(left < m_legendRect.left())
					m_legendRect.setLeft((int)left);
		//		if(right > m_legendRect.right())
					m_legendRect.setRight((int)right);
		//		if(top < m_legendRect.top())
					m_legendRect.setTop((int)top);
		//		if(bottom > m_legendRect.bottom())
					m_legendRect.setBottom((int)bottom);
				pLayer->setLegendRect(m_legendRect);
			}
		}
				
//		if(left<-175.0)left=-175.0;
//		if(right>175.0)right=175.0;
//		if(top>89.0)top=80.05;
//		if(bottom<-89.0)bottom=-80.05;
					
		objectID = (long)d->m_treeViewModel->AddRect(id,left,top,right,bottom);
	//	if(objectID>0 && d->m_marbleWidget!=NULL)
	//		d->m_marbleWidget->update();
		if(objectID>0)
		{
			if(m_b3D)
			{
				QColor col;
				long fillcol = 0;
				long linecol = 0;
				if(pLayer!=NULL)
				{
					col = pLayer->getFillColor();
					fillcol = col.blue();
					fillcol <<=8;
					fillcol += col.green();
					fillcol <<=8;
					fillcol += col.red();

					col = pLayer->getLineColor();
					linecol = col.blue();
					linecol <<=8;
					linecol += col.green();
					linecol <<=8;
					linecol += col.red();
				}

				qID = (qint64)objectID;
				pNode = d->m_treeViewModel->getNodeFromIndexId(qID);

				if(pNode!=NULL)
					emit signal3dAddGeoObject(objectID,0,0,0,fillcol,linecol,pNode->m_poly);
			}

			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}
		}
	}
}

void TreeViewWidget::slotAddHyperbola(long id,qint64 time,double lon1, double lat1,double lon2,double lat2,long& objectID)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	MapViewLayer* pLayer = NULL;
	if(pNode!=NULL)
	{
		//enable region calculation for objects
		pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);
	}

	if(qID>0)//pNode!=NULL)
	{
		QString hyp("Hyperbola");
		// Initialize variables
		qreal centerLon = lon1+( (lon2-lon1)/2);
		qreal centerLat = lat1-( (lat1-lat2)/2);
		qreal altitude = 0.0;//centerPosition.altitude();
		//centerPosition.geoCoordinates( centerLon, centerLat,
		//							   GeoDataCoordinates::Degree );

		qreal width = (qreal)lon2-lon1; 
		qreal height= (qreal)lon2-lon1;
		qreal DEG2RAD = PI / 180.0;
		qreal RAD2DEG = 180.0 / PI;

		// Ensure a valid latitude range: 
		if ( centerLat + 0.5 * height > 90.0 || centerLat - 0.5 * height < -90.0 ) {
			return;
		}

		// Don't show the ellipse if it's too small:
		GeoDataLatLonBox ellipseBox( centerLat + 0.5 * height, centerLat - 0.5 * height,
										centerLon + 0.5 * width,  centerLon - 0.5 * width, 
										GeoDataCoordinates::Degree );
					
		if ( !d->m_marbleWidget->viewport()->viewLatLonAltBox().intersects( ellipseBox ) ||
				!d->m_marbleWidget->viewport()->resolves( ellipseBox ) ) return;

		GeoDataLineString ellipse;
		qreal lon = 0.0;
		qreal lat = 0.0;

		// Optimizing the precision by determining the size which the 
		// ellipse covers on the screen:
		qreal degreeResolution = d->m_marbleWidget->viewport()->angularResolution() * RAD2DEG;
		// To create a circle shape even for very small precision we require uneven numbers:
		int precision = width / degreeResolution / 8 + 1; 
		if ( precision > 81 ) precision = 81;
		precision = 180;

		// Calculate the shape of the upper half of the ellipse:
		for ( int i = 0; i < precision; ++i ) {
			qreal t = 1.0 - 2.0 * (qreal)(i) / (qreal)(precision);
			lat = centerLat + 0.5 * height * sqrt( 1.0 - t * t );
			lon = centerLon + 0.5 * width * t;
			ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
		}
		// Calculate the shape of the lower half of the ellipse:
	/*	for ( int i = 0; i < precision; ++i ) {
			qreal t = 2.0 * (qreal)(i) / (qreal)(precision) -  1.0;
			lat = centerLat - 0.5 * height * sqrt( 1.0 - t * t );
			lon = centerLon + 0.5 * width * t;
			ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
		}*/

		objectID = (long)d->m_treeViewModel->AddHyperbola(hyp,id,centerLon,centerLat,ellipse);
		if(objectID>0 && d->m_marbleWidget!=NULL)
		{
			QRegion rg;
			QRect rc;
			qreal lon1=lon;
			qreal lat1=lat;
			qreal lx=0;
			qreal ly=0;
			d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
			rc.setLeft(lx-20);
			rc.setRight(lx+20);
			rc.setTop(ly-20);
			rc.setBottom(ly+20);
			rg.setRects(&rc,0);
			m_dirtyRG |= rg;
		//	d->m_marbleWidget->update(rc);
		//	d->m_marbleWidget->repaint(rc);
		}
		if(objectID>0)
		{

			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}

		}
	}
}

void TreeViewWidget::slotAddArrow(long id,double longitude,double latitude,long lCol,int dirc1,int dirc2,GeoDataLineString& gdl, long& objectID)
{
	if(dirc1<=0 && dirc2<=0) return;
	if(gdl.size()<2)return;

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode != NULL && pNode->ItemType == Group)
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		objectID = (long)d->m_treeViewModel->AddArrow(id,longitude,latitude,dirc1,dirc2,lCol,gdl);
	//	if(objectID>0 && d->m_marbleWidget!=NULL)
	//		d->m_marbleWidget->update();
		if(objectID>0)
		{
			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}
		}
	}

}

void TreeViewWidget::slotAddSvgObject(long id, QString& key,double lon, double lat, bool file,bool fixed,bool omni,int bts, QPixmap* pPix,long& objectID)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode != NULL && pNode->ItemType == Group)
	{
		objectID = (long)d->m_treeViewModel->AddSvgObject(id,key,lon,lat,file,fixed,omni,bts,pPix);
	//	if(objectID>0 && d->m_marbleWidget!=NULL)
	//		d->m_marbleWidget->update();
		if(objectID>0)
		{
			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}
		}

	}

}

void TreeViewWidget::slotAddHotSpot(long id, QString& key,double lon, double lat, bool set, QPixmap* pPix,long& objectID)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode != NULL && pNode->ItemType == Group)
	{
		objectID = (long)d->m_treeViewModel->AddHotSpot(id,key,lon,lat,set,pPix);
	//	if(objectID>0 && d->m_marbleWidget!=NULL)
	//		d->m_marbleWidget->update();
		if(objectID>0)
		{
			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}
		}

	}

}

void TreeViewWidget::slotAddPixmapObject(long id, QString& key,double lon, double lat,int xoffs,int yoffs,int alignment, QPixmap* pPix,long& objectID)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode != NULL && pNode->ItemType == Group)
	{
		objectID = (long)d->m_treeViewModel->AddPixmapObject(id,key,lon,lat,xoffs,yoffs,alignment,pPix);
	//	if(objectID>0 && d->m_marbleWidget!=NULL)
	//		d->m_marbleWidget->update();
		if(objectID>0)
		{
			if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
			{
				//m_pUpdateTimer->stop();
				m_pUpdateTimer->start();
			}
			else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
			{
				if(m_iRepaintCount<10)
				{
					m_pUpdateTimer->stop();
					m_iRepaintCount++;
					m_pUpdateTimer->start();
				}
			}
		}

	}

}

bool TreeViewWidget::checkSvgObjectKey(QString& key)
{
	bool ret = d->m_treeViewModel->checkPixmapKey(key);
	return ret;
}

void TreeViewWidget::PixelToCoord(int xpos,int ypos, double& lon,double& lat)
{
	qreal longitude=0;
	qreal latitude =0;

	bool ret=false;

	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();
    for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
	{
		pMapLayer = iter.value();
        if ( pMapLayer != NULL ) 
		{
			ret = pMapLayer->pixelToCoord(xpos,ypos,longitude,latitude);
			if(ret)
			{
				lon=longitude;
				lat=latitude;
				return;
			}
		}
	}

	//normal conversion
	d->m_marbleWidget->geoCoordinates(xpos,ypos,longitude,latitude);
	lon=longitude;
	lat=latitude;
}
	
void TreeViewWidget::CoordToPixel(int& xpos,int& ypos, double lon,double lat)
{
	int posx =0;
	int posy =0;

	qreal qposx =0;
	qreal qposy =0;

	bool ret=false;

	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();
    for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
	{
		pMapLayer = iter.value();
        if ( pMapLayer != NULL ) 
		{
			ret = pMapLayer->coordToPixel(posx,posy,lon,lat);
			if(ret)
			{
				xpos=posx;
				ypos=posy;
				return;
			}
		}
	}

	d->m_marbleWidget->screenCoordinates(lon,lat,qposx,qposy);

	xpos=(int)qposx;
	ypos=(int)qposy;
}

void TreeViewWidget::geosetFilename(QString& gst)
{
	gst = m_qsGeoset;
}

bool TreeViewWidget::mapLayerList(QStringList& list)
{
	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();
    for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
	{
		pMapLayer = iter.value();
        if ( pMapLayer != NULL ) 
		{
			qint64 qID = (qint64)iter.key();
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
			
			if(pNode!=NULL && pNode->m_bGeoSet)//get all geoset layers
			{
				TreeNode* pOverlay = pNode->parent;
				if(pOverlay!=NULL)
				{
					TreeNode* pChild=NULL;
					int count = pOverlay->children.count();
					for(int i=0;i<count;i++)
					{
						pChild = pOverlay->children.at(i);
						if(pChild!=NULL && pChild->m_bGeoSet)//do not add the geoset name itself
						{
							if( pChild->m_lId!=qID)
								list << pChild->m_qsName;
							//else list << m_qsGeoset;
						}
					}
				}
	//			m_qsGeoset.clear();
			}
			else list << pMapLayer->getFileName();//pMapLayer->runtimeTrace();
		}//if MapLayer
	}//for iter

	return (list.count()>0);
}

void TreeViewWidget::disableSelection(bool flag,bool temp)
{
	// iterate all layers
	MapViewLayer* pMapLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pMapLayer = iter.value();
        if ( pMapLayer != NULL )
		{
			if(!temp)
				pMapLayer->setInputEnabled(flag);
			else pMapLayer->setInputTempEnabled(flag);
		}
	}
}

bool TreeViewWidget::setLayerZorder(QString& name,int flag,qint64 idx)
{
	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
	MapViewMapLayer* pMyMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();

	qint64 key=0;
	qreal zval=0;
	qreal mzval=0;
	switch(flag)
	{
		case 0://to front
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{
					zval = pMapLayer->zValue();
					if(zval>mzval)mzval=zval;
					QString layer = pMapLayer->runtimeTrace();//LayerName
					QString path  = pMapLayer->getFileName();//Layer file path
					if(name==layer || name==path)
					{
						key = (qint64)iter.key();
						pMyMapLayer=pMapLayer;
					}
					else//check geoset layers
					{
						qint64 qid = (qint64)iter.key();
						if(verifyGeosetLayer(flag,qid,name,idx))
						{
							return true;
						}
					}
				}
			}
			if(pMyMapLayer!=NULL)
			{
				mzval+=1.0;
				pMyMapLayer->setLayerZValue(mzval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(key);
				if(pNode!=NULL && pNode->ItemType==Layer)
				{
					pNode->m_iZVal = mzval;
					updateDocumentListing((long)key,2);
				}
				return true;
			}
			break;
		case 1://to back
			mzval=1000.0;
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{
					zval = pMapLayer->zValue();
					if(zval<mzval)mzval=zval;
					QString layer = pMapLayer->runtimeTrace();//LayerName
					QString path  = pMapLayer->getFileName();//Layer file path
					if(name==layer || name==path)
					{
						key = (qint64)iter.key();
						pMyMapLayer=pMapLayer;
					}
					else//check geoset layers
					{
						qint64 qid = (qint64)iter.key();
						if(verifyGeosetLayer(flag,qid,name,idx))
						{
							return true;
						}
					}
				}
			}
			if(pMyMapLayer!=NULL)
			{
				mzval-=1.0;
				if(mzval<1)mzval=1.0;
				pMyMapLayer->setLayerZValue(mzval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(key);
				if(pNode!=NULL && pNode->ItemType==Layer)
				{
					pNode->m_iZVal = mzval;
					updateDocumentListing((long)key,2);
				}
				return true;
			}
			break;
		case 2://up
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{
					QString layer = pMapLayer->runtimeTrace();//LayerName
					QString path  = pMapLayer->getFileName();//Layer file path
					if(name==layer || name==path)
					{
						key = (qint64)iter.key();
						zval = pMapLayer->zValue()+1.0;
						pMapLayer->setLayerZValue(zval);
						TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(key);
						if(pNode!=NULL && pNode->ItemType==Layer)
						{
							pNode->m_iZVal = zval;
							updateDocumentListing((long)key,2);
						}
						return true;
					}
					else//check geoset layers
					{
						qint64 qid = (qint64)iter.key();
						if(verifyGeosetLayer(flag,qid,name,idx))
						{
							return true;
						}
					}
				}
			}
		case 3://down
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{
					QString layer = pMapLayer->runtimeTrace();//LayerName
					QString path  = pMapLayer->getFileName();//Layer file path
					if(name==layer || name==path)
					{
						key = (qint64)iter.key();
						zval = pMapLayer->zValue()-1.0;
						if(zval<1)zval=1.0;
						pMapLayer->setLayerZValue(zval);
						TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(key);
						if(pNode!=NULL && pNode->ItemType==Layer)
						{
							pNode->m_iZVal = zval;
							updateDocumentListing((long)key,2);
						}
						return true;
					}
					else//check geoset layers
					{
						qint64 qid = (qint64)iter.key();
						if(verifyGeosetLayer(flag,qid,name,idx))
						{
							return true;
						}
					}
				}
			}
			break;
		default:break;
	}

/*	QString qsLayer;
	QStringList layerList;
	bool bMap = mapLayerList(layerList);
	if(layerList.count()<1)
		return false;

	int iLC = layerList.count();
	for(int i=0;i<iLC;i++)
	{
		qsLayer=layerList.at(i);
	}
*/
	return false;
}
bool TreeViewWidget::verifyGeosetLayer(int flag,qint64 qID,QString& name,qint64 idx)
{

	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	TreeNode* pTreeNode=NULL;
	if(idx>0)
		pTreeNode = d->m_treeViewModel->getNodeFromIndexId(idx);
	
	if(pNode!=NULL && pNode->m_bGeoSet)//get all geoset layers
	{
		TreeNode* pOverlay = pNode->parent;
		if(pOverlay!=NULL)
		{
			TreeNode* pChild=NULL;
			int count = pOverlay->children.count();
			for(int i=0;i<count;i++)
			{
				pChild = pOverlay->children.at(i);
				if(pTreeNode==NULL)
				{
					if(pChild!=NULL && pChild->m_bGeoSet && pChild->m_lId!=qID)//do not verify the geoset name itself
					{
						if(pChild->m_qsName==name)
						{
							emit signalMapLayerItem(flag,pChild->parent->m_lId,pChild->m_lId,QString());//send to MapXRender
							return true;
						}
					}
				}
				else if(pTreeNode == pChild)
				{
					if(pChild->m_qsName==name)
					{
						emit signalMapLayerItem(flag,pChild->parent->m_lId,pChild->m_lId,QString());//send to MapXRender
						return true;
					}
				}
			}
		}
	}

	return false;
}

void TreeViewWidget::updateDocumentListing(long id,int flag)
{
	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
	MapViewMapLayer* pMyMapLayer=NULL;
   // QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode==NULL)return;

	//Overlay
	qID = pNode->parent->m_lId;
	pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode==NULL)return;

	//sort layer by zVal
	int sort=10000;
	TreeNode* pLayerNode = NULL;
	QMap<int,MapViewMapLayer*> qLayerMap;
	int layerCount = pNode->children.count();

	for(int i=0;i<layerCount;i++)
	{
		pLayerNode = pNode->children.at(i);
		if(pLayerNode!=NULL && d->m_mvMapLayers.contains((qreal)(pLayerNode->m_lId)) )
		{
			pMapLayer = d->m_mvMapLayers.value((qreal)pLayerNode->m_lId);
			if(pMapLayer!=NULL)
			{
				qLayerMap.insert(sort-pLayerNode->m_iZVal,pMapLayer);
				//remove it first
				pMapLayer->updateDocumentListing(0,pLayerNode->m_iZVal);
			}
		}

	}

//	qreal zval=0;
//	qreal mzval=0;
	int zVal=0;

	QMap<int,MapViewMapLayer*>::const_iterator iter = qLayerMap.constBegin();
	for ( ; iter != qLayerMap.constEnd(); ++iter )
	{
		pMapLayer = iter.value();
		zVal = sort-iter.key();
		if(pMapLayer!=NULL)
		{
			//add document
			pMapLayer->updateDocumentListing(1,zVal);
		}

	}
/*	switch(flag)
	{
		case 0://top
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{

				}
			}
			break;
		case 1://bottom
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{

				}
			}
			break;
		case 2://up
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{

				}
			}
			break;
		case 3://down
			for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
			{
				pMapLayer = iter.value();
				if ( pMapLayer != NULL ) 
				{

				}
			}
			break;
		default:break;
	}*/
}

void TreeViewWidget::setViewContext( ViewContext viewContext )
{
	// iterate all map layers
	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
        if ( pLayer != NULL ) 
		{
			pLayer->setViewContext(viewContext);
		}
	}

}

void TreeViewWidget::slotMoveAllObjects(long id,int x,int y,int& count)
{
	uint uid = (uint)id;
	qint64 qID = (qint64)uid;

	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode == NULL)return;

	//move Icon and Poly
	if(pNode->ItemType == Group)
	{
		int iCC = pNode->children.count();
		for(int ic=0;ic<iCC;ic++)
		{
			Model::TreeNode* pChild = pNode->children.at(ic);
			if(pChild!=NULL)
			{
				pChild->m_iXoff = x;
				pChild->m_iYoff = y;
				count++;
			}//pChild
		}//for CC

	}
	else if(pNode->ItemType == Layer)
	{
		int iGC = pNode->children.count();
		
		for(int ig=0;ig<iGC;ig++)
		{
			Model::TreeNode* pGroup = pNode->children.at(ig);
			if(pGroup!=NULL)
			{
				int iCC = pGroup->children.count();
				for(int ic=0;ic<iCC;ic++)
				{
					Model::TreeNode* pChild = pGroup->children.at(ic);
					if(pChild!=NULL)
					{
						pChild->m_iXoff = x;
						pChild->m_iYoff = y;
						count++;
					}
				}//for CC
			}//if pGroup
		}//for GC

	}//Layer
	else if(pNode->ItemType == Poly || pNode->ItemType == Icon)
	{
		pNode->m_iXoff = x;
		pNode->m_iYoff = y;
		count=1;

	}//Poly or Icon

	if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
	{
		m_pUpdateTimer->start();
	}

}

void TreeViewWidget::slotMoveObject(long id,double lon, double lat)
{
	/*
	if(lon < -360.0 || lon > 360.0)
		return;

	if(lat < -90.0 || lat > 90.0)
		return; */
	if(lon == -399.0 || lat == -399.0)
		return;

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);

	if(pNode != NULL && pNode->ItemType != Group && pNode->ItemType != Layer && pNode->ItemType != Overlay)
	{
		TreeNode* pGroup = pNode->parent;
		if(pGroup==NULL)return;
		TreeNode* pLayer = pGroup->parent;
		if(pLayer==NULL)return;

		//enable region calculation for layer objects
		MapViewLayer* pMVLayer = d->m_mvLayers.value(pLayer->m_lId);
		if(pMVLayer!=NULL)
			pMVLayer->setLayerUpdate(true);

		pGroup->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
		for(int i=0;i<pGroup->children.count();i++)
		{
			pNode = pGroup->children.value(i);
			if(pNode!=NULL)
			{			
				pNode->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);
			}

			//3D
			if(m_b3D)
				emit signal3dMoveObject(pNode->m_lId,lon,lat);
		}

		if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
		{
			//m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}
		else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
		{
			if(m_iRepaintCount<10)
			{
				m_pUpdateTimer->stop();
				m_iRepaintCount++;
				m_pUpdateTimer->start();
			}
		}

	}
	else if(pNode != NULL && pNode->ItemType == Group)
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

	/*	GeoDataCoordinates position; 
		position.setLongitude( qsList.at( 0 ).toDouble(), GeoDataCoordinates::Degree );
		position.setLatitude( qsList.at( 1 ).toDouble(), GeoDataCoordinates::Degree );*/
		//qreal lon1=lon;
		//qreal lat1=lat;
		//GeoDataCoordinates position = pNode->m_position;//l(lon,lat,0,GeoDataCoordinates::Degree)
		
		//position.setLongitude(lon,GeoDataCoordinates::Degree);
		//position.setLatitude(lat,GeoDataCoordinates::Degree);
//		pNode->m_position.clear();
		pNode->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);

		Model::TreeNode* pChild=NULL;
		for(int i=0;i<pNode->children.count();i++)
		{
			pChild = pNode->children.value(i);
			if(pChild!=NULL)
			{
//				pChild->m_position.clear();
			/*	if(pChild->ItemType == Desc)
				{
					qreal xpos=0;
					qreal ypos=0;
					qreal lon1=lon;
					qreal lat1=lat;
					d->m_marbleWidget->screenCoordinates(lon1,lat1,xpos,ypos);
					if(ypos<0)
						ypos+=100;
					else ypos-=100;
					//d->m_marbleWidget->geoCoordinates(
					d->m_marbleWidget->geoCoordinates((int)xpos,(int)ypos,lon1,lat1);
					position.setLongitude(lon1,GeoDataCoordinates::Degree);
					position.setLatitude(lat1,GeoDataCoordinates::Degree);
				}*/
			
				//pChild->m_position.set(pNode->m_position.longitude(GeoDataCoordinates::Degree),pNode->m_position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
				if(pChild->ItemType==GIS::Plot||pChild->ItemType==GIS::PlotEx )
				{
					if(pChild->m_position!=pNode->m_position)
						calcPlotCoords((qint64)pChild->m_lId,pChild->m_position,pNode->m_position);
				}
				else
					pChild->m_position.set(lon,lat,0,GeoDataCoordinates::Degree);

				//3D
				if(m_b3D)
					emit signal3dMoveObject(pChild->m_lId,lon,lat);
			}


		}

		/*if(d->m_marbleWidget!=NULL)
		{
			QRegion rg = d->m_marbleWidget->mapRegion();
			d->m_marbleWidget->repaint(rg);
		}*/

/*		Model::TreeNode* pParent=NULL;
		pParent = pNode->parent;
		if(pParent!=NULL)
		{
			pParent->m_bUpdate=true;
//			pParent->m_position.clear();
			pParent->m_position.set(position.longitude(GeoDataCoordinates::Degree),position.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);
		}*/

/*		QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
		QModelIndex parent = d->m_treeViewModel->parent(idx);
		pNode = d->m_treeViewModel->getNodeFromIndex(parent);
		pNode->m_position.clear();
		pNode->m_position.append(position);*/
		m_bRedraw=true;
		if(d->m_marbleWidget!=NULL)
		{
			QRegion rg;
			QRect rc;
			qreal lon1=pNode->m_position.longitude(GeoDataCoordinates::Degree);
			qreal lat1=pNode->m_position.latitude(GeoDataCoordinates::Degree);
			qreal lx=0;
			qreal ly=0;
			d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
			rc.setLeft(lx-100);
			rc.setRight(lx+100);
			rc.setTop(ly-100);
			rc.setBottom(ly+100);
			rg.setRects(&rc,0);
		//	m_dirtyRG |= rg;
		//	d->m_marbleWidget->update(rc);//d->m_marbleWidget->repaint();
		//	d->m_marbleWidget->repaint(rc);
			lon1=lon;
			lat1=lat;
			lx=0;
			ly=0;
			d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
			rc.setLeft(lx-100);
			rc.setRight(lx+100);
			rc.setTop(ly-100);
			rc.setBottom(ly+100);
			rg.setRects(&rc,1); 

			//center this object
			if(m_bFocusObject&&(long)m_focusID==id)
			{
				if(!centerFocusObject((qreal)lon,(qreal)lat))
				{
					m_dirtyRG |= rg;
					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}
				}
			}
			else
			{

				if(0)//!m_bRedraw)//!m_pUpdateTimer->isActive() )
					d->m_marbleWidget->repaint(rg);//d->m_marbleWidget->repaint();
				else 
				{
					m_dirtyRG |= rg;
					if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
					{
						//m_pUpdateTimer->stop();
						m_pUpdateTimer->start();
					}
					else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
					{
						if(m_iRepaintCount<10)
						{
							m_pUpdateTimer->stop();
							m_iRepaintCount++;
							m_pUpdateTimer->start();
						}
					}
				}
			}
		}
	}

}

void TreeViewWidget::slotSetObjectProperties(QString& qs,long id,int type)
{
//	qDebug() << "setGroupData " << qs; 

	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode == NULL) return;

	QString qsProp;
	QString qsSec;
	int i=0;
	int iPos=0;

	MapViewLayer* pLayer=NULL;
	MapViewShadingLayer* pShadingLayer=NULL;

	if(pNode->ItemType == Layer)
	{
		if(d->m_mvLayers.contains(pNode->m_lId))
			pLayer = d->m_mvLayers.value(pNode->m_lId);
		else if(d->m_mvShadingLayers.contains(pNode->m_lId) )
		{
			pShadingLayer = d->m_mvShadingLayers.value(pNode->m_lId);
		}
	}

	switch(type)
	{
		case MST_SET_OVERLAY_PROPERTIES:
		{
			iPos=qs.count(";");
			if(pNode->ItemType == Overlay)
			do{
				qsProp = qs.section(';',i,i);
				qsProp = qsProp.simplified();//remove whitespace
				if(!qsProp.isEmpty())
				{
					qsSec = qsProp.section('=',0,0);
					if(qsSec == "LEGEND")
					{
						qsSec = qsProp.section('=',1,1);
						//pNode->m_qsImage = qsSec.section(',',0,0);
						bool bEdit = false;
						if(qsSec=="TRUE")
							bEdit=true;
						if(bEdit)
							emit signalSendEvent(MST_EVENT_LEGEND_OVERLAY,0,NULL,pNode->m_lId);
						i++;
					}
					else if(qsSec == "LEGEND_ALIGN")
					{
						int iAlign=3;//lower-right default
						qsSec = qsProp.section('=',1,1);
						QString qsAlign = qsSec.section(',',0,0);
						if(qsAlign=="UPPERLEFT")
							iAlign=0;
						else if(qsAlign=="UPPERRIGHT")
							iAlign=1;
						else if(qsAlign=="LOWERLEFT")
							iAlign=2;
						else if(qsAlign=="LOWERRIGHT")
							iAlign=3;

						m_overlayLegend.insert(pNode->m_lId,iAlign);
						i++;
					}
					else 
					{
						qsSec.clear();
						++i;
					}
				}
				else i=-1;
				if(i>iPos)i=-1;
			}while(i>=0);
		}
		break;
		case MST_SET_LAYER_PROPERTIES:
		{
			iPos=qs.count(";");
			if(pNode->ItemType == Layer)
			do{
				qsProp = qs.section(';',i,i);
				qsProp = qsProp.simplified();//remove whitespace
				if(!qsProp.isEmpty())
				{
					qsSec = qsProp.section('=',0,0);
					if(qsSec == "DEFAULTICON")
					{
						qsSec = qsProp.section('=',1,1);
						pNode->m_qsImage = qsSec.section(',',0,0);
						i++;
					}
					else if(qsSec == "LINE_COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						pNode->m_lLineCol = qsCol.toLong();
						if(pLayer!=NULL)
							pLayer->setLineColor(pNode->m_lLineCol);
						i++;
					}
					else if(qsSec == "LINE_WIDTH")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						//pNode->m_lCol = qsCol.toLong();
						int iWidth = qsCol.toInt();
						if(pLayer!=NULL)
							pLayer->setLineWidth(iWidth);
						i++;
					}
					else if(qsSec == "FILL_COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						long col = qsCol.toLong();
						pNode->m_lFillCol = qsCol.toLong();
						if(0)//pNode->m_lFillCol<1)
						{
							pNode->m_lFillCol=255;
							col=pNode->m_lFillCol;
						}
						if(pLayer!=NULL)
							pLayer->setFillColor(col);
						i++;
					}
					else if(qsSec == "EDIT")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						bool bEdit = false;
						if(qsCol=="TRUE")
							bEdit=true;
						if(pLayer!=NULL)
							pLayer->setEdit(bEdit);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setEdit(bEdit);
						i++;
					}
					else if(qsSec == "O_EDIT")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						bool bEdit = false;
						if(qsCol=="TRUE")
							bEdit=true;
						if(pLayer!=NULL)
							pLayer->setObjectEdit(bEdit);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setObjectEdit(bEdit);
						i++;
					}
					else if(qsSec == "O_MOVE")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						bool bEdit = false;
						if(qsCol=="TRUE")
							bEdit=true;
						if(pLayer!=NULL)
							pLayer->setObjectMove(bEdit);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setObjectMove(bEdit);
						i++;
					}
					else if(qsSec == "O_DELETE")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						bool bEdit = false;
						if(qsCol=="TRUE")
							bEdit=true;
						if(pLayer!=NULL)
							pLayer->setObjectDelete(bEdit);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setObjectDelete(bEdit);
						i++;
					}
					else if(qsSec == "DELETE")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						bool bDel = false;
						if(qsCol=="TRUE")
							bDel=true;
						if(pLayer!=NULL)
							pLayer->setDelete(bDel);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setDelete(bDel);
						i++;
					}
					else if(qsSec == "MARK_TYPE")
					{
						qsSec = qsProp.section('=',1,1);
						if(qsSec == "DIRECTION")
							pNode->MarkerType = GIS::Direction;
						else if(qsSec == "CIRCLE")
							pNode->MarkerType = GIS::Circle;
						else if(qsSec == "CROSS")
							pNode->MarkerType = GIS::Cross;
						else if(qsSec == "XCROSS")
							pNode->MarkerType = GIS::XCross;
						else if(qsSec == "LEVEL")
							pNode->MarkerType = GIS::Level;
						else if(qsSec == "ICON")
							pNode->MarkerType = GIS::MIcon;
						else if(qsSec == "RECTANGLE")
							pNode->MarkerType = GIS::Rectangle;
						i++;
					}
					else if(qsSec == "MARK_POLY")
					{
						int ff=0;
						qsSec = qsProp.section('=',1,1);
						if(qsSec == "START")
							ff=1;
						else if(qsSec == "END")
							ff=1;
						else if(qsSec == "POINT")
							ff=1;
						else if(qsSec == "OFF")
							ff=1;
						i++;
					}
					else if(qsSec == "MARK_SIZE")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int iWidth = qsCol.toInt();
						if(pLayer!=NULL)
							pLayer->setMarkerSize(iWidth);
						i++;
					}
					else if(qsSec == "MARK_LINE_WIDTH")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int iWidth = qsCol.toInt();
						i++;
					}
					else if(qsSec == "MARK_COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						long col = qsCol.toLong();
						i++;
					}
					else if(qsSec == "LCM")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						if(pLayer!=NULL)
							pLayer->setContextMenu(qsSec);
						else if(pShadingLayer!=NULL)
							pShadingLayer->setContextMenu(qsSec);
						i++;
					}
					else if(qsSec == "TRANS")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int trans = qsCol.toInt();
						if(pLayer!=NULL)
							pLayer->setTransparency(trans);
						i++;
					}
					else if(qsSec == "ZORDER")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsVal = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int trans = qsVal.toInt();
						if(pLayer!=NULL)
						{
							pLayer->setLayerZValue((qreal)trans);
							pNode->m_iZVal=trans;
						//	d->m_treeViewModel->sort(0);
						}
						i++;
					}
					else if(qsSec == "VISIBLE")
					{
						bool vis=true;
						qsSec = qsProp.section('=',1,1);
						if(qsSec == "TRUE")
							vis=true;
						if(qsSec == "FALSE")
							vis=false;
						if(pLayer!=NULL)
						{
							pNode->m_bVisible=vis;
						//	d->m_treeViewModel->sort(0);
						}
						i++;
					}
					else if(qsSec == "FONT_COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						long col = qsCol.toLong();
						if(pLayer!=NULL)
						{
							pLayer->setFontColor(col);
						//	d->m_treeViewModel->sort(0);
						}
						i++;
					}
					else if(qsSec == "FONT_BACK_COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						long col = qsCol.toLong();
						if(pLayer!=NULL)
						{
							pLayer->setFontBackColor(col);
						//	d->m_treeViewModel->sort(0);
						}
						i++;
					}
					else if(qsSec == "FILL")
					{
						qsSec = qsProp.section('=',1,1);
					//	QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int trans = pLayer->getTransparency();
						if(qsSec == "TRUE" && trans==0)
							trans=80;
						else if(qsSec == "FALSE")
							trans=0;
						if(pLayer!=NULL)
							pLayer->setTransparency(trans);
						i++;
					}
					else if(qsSec == "FILL_STYLE")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
					//	pNode->m_lCol = qsCol.toLong();
						int iStyle = qsCol.toInt();
						if(pLayer!=NULL)
						{
						//	pLayer->setFontColor(col);
						//	d->m_treeViewModel->sort(0);
						}
						i++;
					}
					else if(qsSec == "IGNORE_CACHE")
					{
						qsSec = qsProp.section('=',1,1);
					/*	if(qsSec == "TRUE")
							trans=80;
						else if(qsSec == "FALSE")
							trans=0;
						if(pLayer!=NULL)
							pLayer->setTransparency(trans);*/
						i++;
					}
					else 
					{
						qsSec.clear();
						++i;
					}
				}
				else i=-1;
				if(i>iPos)
					i=-1;
			}while(i>=0);
		}
		break;

		case MST_SET_ICON_PROPERTIES:
		{
			iPos=qs.count(";");
			if(pNode->ItemType == Icon)
			do{
				qsProp = qs.section(';',i,i);
				qsProp = qsProp.simplified();//remove whitespace
				if(!qsProp.isEmpty())
				{
					qsSec = qsProp.section('=',0,0);
					if(qsSec == "COLOR")
					{
						qsSec = qsProp.section('=',1,1);
						QString qsCol = qsSec.section(',',0,0);
						pNode->m_lFillCol = qsCol.toLong();
					//	if(pLayer!=NULL)
						//	pLayer->setLineColor(pNode->m_lLineCol);
						i++;
					}
					else 
					{
						qsSec.clear();
						++i;
					}
				}
				else i=-1;
				if(i>iPos)
					i=-1;
			}while(i>=0);
		}
		break;
		case MST_GROUP_SET_DATA:
		{
			iPos=qs.count(";");
			if(pNode->ItemType == Group)
			{
				TreeNode* pAzimuth=NULL;
				bool bAzi=false;
				int iC = pNode->children.count();
				for(int iA=0;iA<iC;iA++)//search for azimuth
				{
					pAzimuth = pNode->children.at(iA);
					if(pAzimuth!=NULL && pAzimuth->ItemType==Azimuth)
					{
						bAzi=true;
						iA=iC;
					}
				}
				do{
					qsProp = qs.section(';',i,i);
					qsProp = qsProp.simplified();//remove whitespace
					if(!qsProp.isEmpty())
					{
						qsSec = qsProp.section('=',0,0);
						if(qsSec == DTYPE_DIRECTION)//"d")
						{
							qsSec = qsProp.section('=',1,1);
							QString qsDir = qsSec.section(',',0,0);
							pNode->m_iDirection = qsDir.toInt();
						//	pNode->m_iDirection = (pNode->m_iDirection/100)*-1; //??
							pNode->parent->m_iDirection = pNode->m_iDirection;
							i++; 
						}
						else if(qsSec == DTYPE_COMPASS)//"c")
						{
							qsSec = qsProp.section('=',1,1);
							QString qsDir = qsSec.section(',',0,0);
							pNode->m_iDirection = qsDir.toInt();
						//	pNode->m_iDirection = (pNode->m_iDirection/100)*-1;
						//	pNode->parent->m_iDirection = pNode->m_iDirection;
							slotCenterMap();
							i++;
						}
						else if(qsSec == "COMPASS")
						{
							qsSec = qsProp.section('=',1,1);
							QString qsDir = qsSec.section(',',0,0);
							pNode->m_iDirection = qsDir.toInt();
						//	pNode->m_iDirection = (pNode->m_iDirection/100)*-1;
							pNode->parent->m_iDirection = pNode->m_iDirection;
							slotCenterMap();
							i++;
						}
						else if(qsSec == DTYPE_FOCUS || qsSec == "FOCUS")
						{
							qsSec = qsProp.section('=',1,1);
							QString qsVal = qsSec.section(',',0,0);
							int foc = qsVal.toInt();
						//	pNode->m_lCol = qsCol.toLong();
							if(foc)
							{
								if(m_pFocusTimer!=NULL && !m_pFocusTimer->isActive())
									m_pFocusTimer->start();
								m_focusID = (qint64)pNode->m_lId;
							}
							else
							{
								if(m_pFocusTimer!=NULL && m_pFocusTimer->isActive())
									m_pFocusTimer->stop();
								m_focusID = 0;
							}
							
							i++;
						}
						else if(qsSec == DTYPE_ON)//"o")
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								int on = qsSec.toInt();
								if(on)
									pAzimuth->m_bMap=true;
								else pAzimuth->m_bMap=false;
							}
						
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_AZIMUTH)
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								pAzimuth->m_iDirection = qsSec.toInt();
							}
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_FREQUENCY)
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								pAzimuth->m_dRadius = qsSec.toDouble();
							}
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_LEVEL)
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								pAzimuth->m_iXoff = qsSec.toInt();
							}
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_QUALITY)
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								pAzimuth->m_iYoff = qsSec.toInt();
							}
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_ELEVATION)
						{
							if(bAzi && pAzimuth!=NULL)
							{
								qsSec = qsProp.section('=',1,1);
								//QString qsVal = qsSec.section(',',0,0);
								pAzimuth->m_iZVal = qsSec.toInt();
							}
							qsSec.clear();
							++i;
						}
						else if(qsSec == DTYPE_NAME || qsSec == "NAME")
						{
							qsSec = qsProp.section('=',1,1);
							QString qsVal1 = qsSec.section(',',0,0);
							int val1 = qsVal1.toInt();
							QString qsVal2 = qsSec.section(',',1,1);
							int val2 = qsVal2.toInt();
							//val1=set , val2=alignment
							if(val1>0)
							{
								TreeNode* pChildNode=NULL;
								long objectID=0;

								qsSec.clear();
								qsSec = pNode->m_qsName;
								int iC = pNode->children.count();
								for(int i=0;i<iC;i++)
								{
									pChildNode = pNode->children.at(i);
									if(pChildNode!=NULL && (pChildNode->ItemType != Marker && pChildNode->ItemType != Desc && pChildNode->ItemType != Shading && pChildNode->ItemType != Arrow) )
									{
										if(pChildNode->ItemType == Text)
											pChildNode->m_qsDesc = qsSec;
										else
										{
											objectID=0;
											slotAddDescription(qsSec,pNode->m_lId,(long)val2,0,0,objectID);
										}

									}//child node
								}//for childs

							}//if set
							++i;
						}
						else 
						{
							qsSec.clear();
							++i;
						}
					}
					else i=-1;
					if(i>iPos)i=-1;
				}while(i>=0);
			}
		}

		break;
	}

}

void TreeViewWidget::slotDeleteObject(long id,int type)
{
	deleteObjects(QStringList(),id,type);

	if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
	{
		//m_pUpdateTimer->stop();
		m_pUpdateTimer->start();
	}
	else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
	{
		if(m_iRepaintCount<10)
		{
			m_pUpdateTimer->stop();
			m_iRepaintCount++;
			m_pUpdateTimer->start();
		}
	}
}

void TreeViewWidget::slotDeleteObjectId(long id,int type,int& count, long& ret)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && pNode->ItemType == Group)
	{
		TreeNode* pLayer = pNode->parent;
		if(pLayer!=NULL)
		{
			count = pLayer->children.count()-1;
			ret = pLayer->m_lId;
		}
	}
	
	deleteObjects(QStringList(),id,type);
	m_dirtyRG = QRegion();

	if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
	{
		//m_pUpdateTimer->stop();
		m_pUpdateTimer->start();
	}
	else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
	{
		if(m_iRepaintCount<10)
		{
			m_pUpdateTimer->stop();
			m_iRepaintCount++;
			m_pUpdateTimer->start();
		}
	}
}

void TreeViewWidget::slotZoomToObject(long id ,int flag)
{
	qint64 qID = (qint64)id;
	QModelIndex objIdx = d->m_treeViewModel->getIndexFromIndexId(qID);
	if(objIdx.isValid())
	{
		const GeoDataCoordinates targetPosition =
			objIdx.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();

		if(targetPosition.latitude(GeoDataCoordinates::Degree) != -399.0 &&
			targetPosition.longitude(GeoDataCoordinates::Degree) != -399.0 )
		{

			TreeNode* pItem = d->m_treeViewModel->getNodeFromIndexId(qID);
			if(pItem == NULL) return;

			if(pItem->ItemType == Group)
			{
				d->m_ui.treeTreeView->scrollTo(objIdx);
				if(d->m_mvLayers.contains(pItem->parent->m_lId) )
				{
					MapViewLayer* pLayer = d->m_mvLayers.value(pItem->parent->m_lId);
					if(pLayer!=NULL)
					{
						TreeNode* pNode=NULL;
						int iC = pItem->children.count();
						for(int i=0;i<iC;i++)
						{
							pNode = pItem->children.at(i);
							if(pNode!=NULL && pNode->ItemType == Icon )//first try to find a icon for selection
							{
								qID = (qint64)pNode->m_lId;
								objIdx = d->m_treeViewModel->getIndexFromIndexId(qID);
								d->m_ui.treeTreeView->setCurrentIndex( objIdx );
								pLayer->synchronizeWithExtern(objIdx,true);
								d->m_marbleWidget->centerOn(targetPosition);
								return;
							}
						}//for
						if(iC>0)
						{
							pNode = pItem->children.at(0);
							if(pNode!=NULL)//select first elemetn, if no icon fond
							{
								qID = (qint64)pNode->m_lId;
								objIdx = d->m_treeViewModel->getIndexFromIndexId(qID);
								d->m_ui.treeTreeView->setCurrentIndex( objIdx );
								pLayer->synchronizeWithExtern(objIdx,true);
								d->m_marbleWidget->centerOn(targetPosition);
								return;
							}
						}
					}//pLayer
				}//if
			}//== Group
			else d->m_ui.treeTreeView->setCurrentIndex( objIdx );//select this item in tree
			d->m_marbleWidget->centerOn(targetPosition);
		}//target pos
	}//valid index
}

void TreeViewWidget::slotAddDescription(QString& qs,long id,long align,long xoff,long yoff,long& objectID)
{
	if(id==0 && align==222)//create map screenshot
	{
		// Take the case into account where no file format is indicated
        const char * format = 0;
        if ( !qs.endsWith("png", Qt::CaseInsensitive)
           | !qs.endsWith("jpg", Qt::CaseInsensitive) )
        {
            format = "JPG";
        }

		QPixmap mapPixmap = d->m_marbleWidget->mapScreenShot();
        bool success = mapPixmap.save( qs, format,100 );

		if(success)objectID=1;

		return;
	}
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode == NULL) return;

	if(pNode->ItemType == Group && align!=111)
	{
		qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
		qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

	/*	if(xoff!=0 || yoff!=0)
		{
			qreal lx=0;
			qreal ly=0;
			d->m_marbleWidget->screenCoordinates(lon,lat,lx,ly);

			lx+=xoff;
			ly+=yoff;
			d->m_marbleWidget->geoCoordinates(lx,ly,lon,lat);
		}*/

	/*	QModelIndex objIdx = d->m_treeViewModel->getIndexFromIndexId(qID);
		int iChildCount = d->m_treeViewModel->rowCount(objIdx);
		int iWidth=0;
		int iHeigth=0;

		if(iChildCount>0)
		{
			QPixmap qPix;
			QModelIndex idxChild;
			QVariant qVar;
			QString qsImage;
			
			for(int i=0;i<iChildCount;i++)
			{
				idxChild = d->m_treeViewModel->index( i, 0,objIdx);
				if(idxChild.isValid())
				{
					GIS::TreeViewItemType cType = (GIS::TreeViewItemType) qVariantValue<unsigned int>( idxChild.data( TreeViewModel::TypeRole ) );
					if ( cType == GIS::Icon )
					{
						qVar = d->m_treeViewModel->data(idxChild,Qt::UserRole);
						qsImage = qVar.toString();
						if(!qsImage.isEmpty())
						{
							qPix = d->m_treeViewModel->pixmap(qsImage);
							iWidth=qPix.width();
							iHeigth=qPix.height();
						}
					}
				}
			}
		}*/

		objectID = (long)d->m_treeViewModel->AddDescription(qs,id,lon,lat,xoff,yoff,align);
		
		if(objectID>0 && d->m_marbleWidget!=NULL && lon<-390 && lat<-390)//legend lon/lat = -399
		{
		/*	int iLen = qs.length()*5;
			QRect rc;
			qreal lon1=lon;
			qreal lat1=lat;
			qreal lx=0;
			qreal ly=0;
			d->m_marbleWidget->screenCoordinates(lon1,lat1,lx,ly);
			rc.setLeft(lx-iLen);
			rc.setRight(lx+iLen);
			rc.setTop(ly-iLen);
			rc.setBottom(ly+iLen);
			m_dirtyRG.setRects(&rc,0);*/
		//	d->m_marbleWidget->update(rc);
		//	d->m_marbleWidget->repaint(rc);
			m_dirtyRG = d->m_marbleWidget->mapRegion();

			if(m_b3D)
				emit signal3dAddText(objectID,qs,lon,lat,align,xoff,yoff);
		}
		//m_dirtyRG = QRegion();
		if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
		{
			//m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}
		else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
		{
			if(m_iRepaintCount<10)
			{
				m_pUpdateTimer->stop();
				m_iRepaintCount++;
				m_pUpdateTimer->start();
			}
		}
	}
	else if(pNode->ItemType == Group && align==111)
	{
	//	qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
	//	qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

	//	QModelIndex objIdx = d->m_treeViewModel->getIndexFromIndexId(qID);
	//	int iChildCount = d->m_treeViewModel->rowCount(objIdx);
		if(xoff>0)
		{
			pNode->m_qsToolTip=qs;
			objectID = 1;
		}
		else pNode->m_qsToolTip.clear();
	}
	


}

void TreeViewWidget::slotAddShadingParameter(long& id,int iGridX,int iGridY,qreal left,qreal top,qreal right,qreal bottom, long& objectID)
{
	if(d->m_marbleWidget==NULL)
		return;

	int iSID = d->m_mvShadingObjects.size();
	
//	if(left<-175.0)left=-175.0;
//	if(right>175.0)right=175.0;
//	if(top>89.0)top=80.05;
//	if(bottom<-89.0)bottom=-80.05;

	if(iGridX<0)//delete a shading object
	{
		if(d->m_mvShadingObjects.contains((int)id) )
		{

			//this is a group id
			qint64 objID = d->m_mvShadingObjects.value(id);
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objID);
			if(pNode->ItemType == Group)
			{
				if(d->m_mvShadingLayers.contains(pNode->parent->m_lId) )
				{
					MapViewShadingLayer* pLayer = d->m_mvShadingLayers.value(pNode->parent->m_lId);
					if(pLayer!=NULL)
					{
						long ovlID=pNode->parent->parent->m_lId;
						d->m_treeViewModel->DeleteObject((long)objID,0);
						d->m_mvShadingObjects.remove(id);
						pLayer->updateImageList(id,NULL);
						pLayer->slotDeleteTreeItem(MST_DELETE_SHAPE,(long)objID);

						if(d->m_mvShadingObjects.count()==0)
						{
							if(m_propOverlayID==ovlID)
							{
								objectID=0;//set the MvInt plugin m_idSL to zero
								m_propOverlayID=0;
								deleteOverlay(ovlID);
							}
						}
						
						d->m_marbleWidget->update();
					}//pLayer
				}//contains layer
			}//Group
		}//contains

		return;
	}

	if(objectID==0)//add shading tree entry
	{
		QStringList qsl;
		qsl << "-399" << "-399" << "Propagation";
		objectID = (long)d->m_treeViewModel->AddOverlay(qsl,false);
		m_propOverlayID=objectID;

		qsl.clear();
		qsl << "-399" << "-399" << "Data";
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,objectID);
		//id=objectID;

		MapViewShadingLayer* pMvShadingLayer = new MapViewShadingLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMvShadingLayer==NULL)return;
		d->m_mvShadingLayers.insert(objectID,pMvShadingLayer);

		pMvShadingLayer->setModel(d->m_treeViewModel);

		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMvShadingLayer->synchronizeWith( selectionModel );

		d->m_marbleWidget->addLayer(pMvShadingLayer); 
	
	}
/*	else if(id<0)//delete shading object from shading tree
	{
		if(d->m_mvLayers.contains(objId) )

	}*/
	
	if(objectID > 0)//add new shading object to shading tree
	{
		int iPos=iSID+1;
		for(int i=0;i<iSID;i++)
		{
			if(!d->m_mvShadingObjects.contains((int)i+1) )
			{
				iPos=i+1;
				i=iSID;
			}
		}

		//if(iSID<1)iPos=1;

		QString qs("Grid-ID %1"); 
		qs = qs.arg(iPos);

		qreal centerLong = left + (right-left)/2;
		qreal centerLat  = top - (top-bottom)/2;

		QString qsPosLo("%1"); 
		qsPosLo = qsPosLo.arg(centerLong);
		QString qsPosLa("%1"); 
		qsPosLa = qsPosLa.arg(centerLat);

		QStringList qsl;
		qsl << qsPosLo << qsPosLa << qs;
							
		qint64 groupID = (long)d->m_treeViewModel->AddGroup(qsl,(qint64)objectID);
		d->m_mvShadingObjects.insert(iPos,groupID);

		GeoDataLineString gdl;
		GeoDataCoordinates pos;

		pos.set(left,top,0,GeoDataCoordinates::Degree);
		gdl << pos;
		pos.set(right,top,0,GeoDataCoordinates::Degree);
		gdl << pos;
		pos.set(right,bottom,0,GeoDataCoordinates::Degree);
		gdl << pos;
		pos.set(left,bottom,0,GeoDataCoordinates::Degree);
		gdl << pos;
		pos.set(left,top,0,GeoDataCoordinates::Degree);
		gdl << pos;

		qint64 polyID = (long)d->m_treeViewModel->AddPolyLineEx(groupID,0,iGridX,iGridY,iPos,gdl);

		//GroundOverlay
/*
		GeoDataGroundOverlay* pGroundOverlay = new GeoDataGroundOverlay();
		GeoDataLatLonBox bbox(top,bottom,right,left,GeoDataCoordinates::Degree);
		pGroundOverlay->setLatLonBox(bbox);

		qint64 qID = (qint64)groupID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode == NULL) return;

		pNode->m_pMapDoc = new GeoDataDocument();
		DocumentRole role = MapDocument;
		pNode->m_pMapDoc->setDocumentRole( role );
		pNode->m_pMapDoc->append(pGroundOverlay);*/

		id = iPos;
	}


}

void TreeViewWidget::slotAddShadingData(long idObj,long lID,long lCount,void* pData, long& objectID)
{
	if(d->m_marbleWidget==NULL)
		return;
	if(d->m_mvShadingLayers.size()<1)
		return;

	int iSID = d->m_mvShadingObjects.size();
	int iGridX=0;
	int iGridY=0;

	if(lID<1)//the grid ID
		return;

	if(!d->m_mvShadingObjects.contains((int)lID) )
		return;

	qint64 qID = d->m_mvShadingObjects.value((int)lID);
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode == NULL) return;

	MapViewShadingLayer* pShadingLayer=NULL;
	if(pNode->ItemType == Group)
	{
		if(d->m_mvShadingLayers.contains(pNode->parent->m_lId) )
		{
			pShadingLayer = d->m_mvShadingLayers.value(pNode->parent->m_lId);
			if(pShadingLayer!=NULL)
			{
				iGridX = pNode->m_iXoff;
				iGridY = pNode->m_iYoff;
			}
		}
	}

	if(iGridX==0)return;
	if(iGridY==0)return;
	QImage qImg(iGridX,iGridY,QImage::Format_ARGB32);
	qImg.fill(255);


#pragma pack(push,1)
		struct t_MPTdata{
			int iCol;
			int iRow;
			long lData;
		};
#pragma pack(pop)

		t_MPTdata* pMptData = (t_MPTdata*)pData;
		if(lCount>0)
		{
			int	iCol=0;
			int iRow=0;
			long lId=0;
			long lData=0;

			short	alpha=0;
			short	red=0;
			short	green=0;
			short	blue=0;

			double	longitude = 0;
			double	latitude = 0;
		/*	double  dMinX=m_dLeft;
			double  dMinY=m_dBottom;
			double  dMaxX=m_dRight;
			double  dMaxY=m_dTop;*/

		//	double dXstep  = (m_dRight-m_dLeft)/m_iGrid;
		//	double dYstep  = (m_dTop-m_dBottom)/m_iGrid;
			long lSize=iGridX*iGridY;
			if(lSize!=lCount)
				lSize=lCount;

			int iX=0;
			int iY=0;
			int iPosX=iGridX-1; //??
			int iPosY=iGridY-1; //??

			for(int i=0;i<lSize;i++)
			{
				iCol  = pMptData[i].iCol;
				iRow  = pMptData[i].iRow;
				lData = pMptData[i].lData;

				//longitude = dMinX + (iCol*dXstep); 
				//latitude  = dMaxY - (iRow*dYstep); 

				if(lData != 0)
				{
					blue = (short)(lData & 0x000000FF);
					lData >>= 8;
					green = (short)(lData & 0x000000FF);
					lData >>= 8;
					red = (short)(lData & 0x000000FF);
					lData >>= 8;
					alpha = (short)(lData & 0x000000FF);

					if(alpha>255)alpha=255;

			//		s << red << green << blue << alpha;
			/*		double dAlpha = (double)alpha/255.0;
					double val=0;
					val = (double)red*dAlpha;
					red = (int)val;
					val = (double)green*dAlpha;
					green = (int)val;
					val = (double)blue*dAlpha;
					blue = (int)val;*/
				
					
					QRgb rgb = qRgba(red,green,blue,alpha);
					qImg.setPixel(iCol,iRow,rgb);
					iX++;
					if(iX==iGridX)//??
					{
						iX=0;
						iY++;
					}
					
				}//lData
			}//for

			pShadingLayer->updateImageList(lID,&qImg);
			objectID=lCount;

			//GrounOverlay
			//GeoDataObject *parent = static_cast< GeoDataObject* >( feature->parent() );
	/*		GeoDataDocument* pDoc = pNode->m_pMapDoc;
			GeoDataContainer *container = static_cast<GeoDataContainer*>( pDoc );
			QVector<GeoDataFeature*> vFeatureList = container->featureList();
			QVector<GeoDataFeature*>::const_iterator cont = vFeatureList.constBegin();
			QVector<GeoDataFeature*>::const_iterator endcont = vFeatureList.constEnd();
						
			for (; cont != endcont; ++cont ) 
			{
					GeoDataGroundOverlay const * groundOverlay = static_cast<GeoDataGroundOverlay const *>( *cont );
					if(groundOverlay!=NULL)
					{
						GeoDataOverlay* pOvl = (GeoDataOverlay*) groundOverlay;
						pOvl->setIcon(qImg);
						d->m_marbleWidget->model()->treeModel()->addDocument( pNode->m_pMapDoc);
					}
			}*/

			if(objectID>0 && d->m_marbleWidget!=NULL)
				d->m_marbleWidget->update();
		}//lCount

}

void TreeViewWidget::slotAddColoredAreaData(long idObj,long lID,long lCount,void* pData, long& objectID)
{
	if(d->m_marbleWidget==NULL)
		return;
	if(d->m_mvShadingLayers.size()<1)
		return;

	int iSID = d->m_mvShadingObjects.size();
	int iGridX=0;
	int iGridY=0;

	if(lID<1)//the grid ID
		return;

	if(!d->m_mvShadingObjects.contains((int)lID) )
		return;

	qint64 qID = d->m_mvShadingObjects.value((int)lID);//Group ID
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode == NULL) return;

	MapViewShadingLayer* pShadingLayer=NULL;
	if(pNode->ItemType == Group)
	{
		if(pNode->parent->m_lId != idObj)return;//not correct layer id

		//if(d->m_mvShadingLayers.contains(pNode->parent->m_lId) )
		if(d->m_mvShadingLayers.contains(idObj) )
		{
			//pShadingLayer = d->m_mvShadingLayers.value(pNode->parent->m_lId);
			pShadingLayer = d->m_mvShadingLayers.value(idObj);
			if(pShadingLayer!=NULL)
			{
				iGridX = pNode->m_iXoff;
				iGridY = pNode->m_iYoff;
			}
		}
	}

	if(iGridX==0)return;
	if(iGridY==0)return;
	QImage qImg(iGridX,iGridY,QImage::Format_ARGB32);
	QRgb rgb = qRgba(255,255,255,0);
	qImg.fill(rgb);

	long lData=0;
	long* pLong  = (long*)pData;
	long lId=0;

/*	QByteArray ba;
	ba.setRawData((char*)pData, lCount );
	QDataStream s(&ba,QIODevice::ReadOnly);
	s.setByteOrder(QDataStream::LittleEndian);  //for MapView
*/
	short	alpha=0;
	short	red=0;
	short	green=0;
	short	blue=0;

	double	longitude = 0;
	double	latitude = 0;

	long lSize=iGridX*iGridY;
	if(lSize!=lCount)
		lSize=lCount;

	int iX=0;
	int iY=0;
	int iPosX=iGridX-1; //??
	int iPosY=iGridY-1; //??
		
	if(lCount>0)
	{
		for(int iRow=0;iRow<iGridY;iRow++)
		{
		
			for(int iCol=0;iCol<iGridX;iCol++)
			{
				lData = *pLong;
				pLong++;
			//	s >> lData;

				//longitude = dMinX + (iCol*dXstep); 
				//latitude  = dMaxY - (iRow*dYstep); 

				if(1)//lData != 0)
				{
					//argb <-> gbar
					blue = (short)(lData & 0x000000FF);
					lData >>= 8;
					green = (short)(lData & 0x000000FF);
					lData >>= 8;
					red = (short)(lData & 0x000000FF);
					lData >>= 8;
					alpha = (short)(lData & 0x000000FF);

					if(alpha>255)alpha=255;
					
					rgb = qRgba(red,green,blue,alpha);
					//rgb = qRgba(255,0,0,128);
					qImg.setPixel(iCol,iRow,rgb);

					iX++;

					if(iX>=lSize)
					{
						iCol=iGridX;
						iRow=iGridY;
					}
					
				}//lData
			}//for iCol
		}//for iRow

/*		QImageWriter qImgw;
		qImgw.setFileName("D:/klaus/test.jpg");
		qImgw.write(qImg);*/

		pShadingLayer->updateImageList(lID,&qImg);
		objectID=lCount;

		if(objectID>0 && d->m_marbleWidget!=NULL)
			d->m_marbleWidget->update();
	}//lCount

}

void TreeViewWidget::slotLayerObjects(int iType,long id,long lCount,QString& qs,long& ret)
{
	qint64 qID = (qint64)id;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL && pNode->ItemType == Layer )
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);

		switch(iType)
		{
			case MST_LAYER_OBJECTS_COUNT:
				{
					m_layerObj.clear();
					int iGroupCount = pNode->children.count();
					if(iGroupCount<1)return;

					qreal left=0;
					qreal right=0;
					qreal top=0;
					qreal bott=0;

					GeoDataLatLonBox llBox = d->m_marbleWidget->viewport()->viewLatLonAltBox();
					llBox.boundaries(top,bott,right,left,GeoDataCoordinates::Degree);

				//	qreal xpos=0;
				//	qreal ypos=0;
					int iTotal=0;

					GeoDataCoordinates pos;
					TreeNode* pGroup=NULL;
					TreeNode* pChild=NULL;

					int iChildCount=0;
					qreal lon=0;
					qreal lat=0;
					for(int i=0;i<iGroupCount;i++)
					{
						pGroup = pNode->children.value(i);
						if(pGroup!=NULL)
						{
							lon = pGroup->m_position.longitude(GeoDataCoordinates::Degree);
							lat = pGroup->m_position.latitude(GeoDataCoordinates::Degree);

							if(lon >=left && lat<=right && lat<=top && lat>=bott)
							{
								iChildCount = pGroup->children.count();
								//d->m_marbleWidget->screenCoordinates(lon,lat,xpos,ypos);
								if(iChildCount>0)
								{
									if(iTotal==0)//also save the layer id
									{
										m_layerObj.insert(iTotal,pNode->m_lId);
										iTotal++;
									}
									for(int x=0;x<iChildCount;x++)
									{
										pChild = pGroup->children.value(x);
										if(pChild!=NULL)
										{
											m_layerObj.insert(iTotal,pChild->m_lId);
											iTotal++;
										}

									}//for

								}//iChildCount

							}//lon/lat
						}//pGroup

					}//for
					ret=iTotal-1;//do not count the layer id
				}
				break;

			case MST_LAYER_OBJECTS_DATA:
				{
					long idLayer = m_layerObj.value(0);
					if(idLayer!=id)return;

					if(lCount > m_layerObj.size()-1)return;

					long* pSendData=NULL;
					int iSize = lCount*3+2;

					pSendData = (long*) new long[iSize];
					if(pSendData==NULL)return;

					int iPos=0;
					pSendData[iPos++]=idLayer;
					pSendData[iPos++]=lCount;

					TreeNode* pNode=NULL;
					qint64 qID = 0;

					MapViewLayer* pLayer=NULL;
					long intID=0;
					if(d->m_mvLayers.contains(idLayer))
					{
						pLayer = d->m_mvLayers.value(idLayer);
						if(pLayer!=NULL)
							intID = pLayer->getInterfaceID();
					}

					qreal lon=0;
					qreal lat=0;
					qreal xpos=0;
					qreal ypos=0;

					int i=1;
					for(i=1;i<=lCount;i++)
					{
						qID = (qint64)m_layerObj.value(i);
						pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
						if(pNode!=NULL)
						{
							lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
							lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

							d->m_marbleWidget->screenCoordinates(lon,lat,xpos,ypos);

							pSendData[iPos++]=pNode->m_lId;
							pSendData[iPos++]=(long)xpos;
							pSendData[iPos++]=(long)ypos;

						}//pNode

					}//for lCount
					--i;

					// Take the case into account where no file format is indicated
					const char * format = 0;
					if ( !qs.endsWith("png", Qt::CaseInsensitive)
					   | !qs.endsWith("jpg", Qt::CaseInsensitive) )
					{
						format = "JPG";
					}

					QPixmap mapPixmap = d->m_marbleWidget->mapScreenShot();
					bool success = mapPixmap.save( qs, format ,100);

					if(success && pSendData!=NULL)
					{
						ret = i;
						iSize = (i*3*sizeof(long))+2*(sizeof(long));
						emit signalSendEvent(MST_EVENT_LAYER_OBJECTS,iSize,pSendData,intID);

						delete []pSendData;
					}


				}//case

				break;
		}

	}//if pNode

}

void TreeViewWidget::slotUpdateMap()
{
//	qDebug() << "slotUpdateMapIn " ; 

	if(d->m_marbleWidget!=NULL)
	{
		m_bRedraw=true;
		if(m_pUpdateTimer!=NULL)
			m_pUpdateTimer->stop();

		m_iRepaintCount=0;
	
		if(m_dirtyRG.isEmpty())
		{
			QRegion rg = d->m_marbleWidget->mapRegion();
		//	qDebug() << "slotUpdateMap-1 " << rg.rectCount(); 
			d->m_marbleWidget->update(rg);
			m_dirtyRG = QRegion();
		}
		else 
		{
		//	QRegion rg;
		//	qDebug() << "slotUpdateMap-2 " << m_dirtyRG.rectCount(); 
			d->m_marbleWidget->update(m_dirtyRG);
		//	qDebug() << "slotUpdateMap-3 " << m_dirtyRG.rectCount(); 
			m_dirtyRG = QRegion();
		}
	}
	//qDebug() << "slotUpdateMapOut " ; 
}

void TreeViewWidget::slotCenterMap()
{
//	qDebug() << "slotUpdateMapIn " ; 
	if(m_bFocusObject && d->m_treeViewModel!=NULL && m_focusID>0)
	{
		QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_focusID);
		if(idx.isValid())
		{
			zoomSelectedItem (idx);
			TreeNode* pChild = d->m_treeViewModel->getNodeFromIndexId(m_focusID);
			if(pChild!=NULL && pChild->m_iDirection!=0)
			{
				qreal heading = ((qreal)pChild->m_iDirection/100)* -1;
				d->m_marbleWidget->setHeading(heading);//-90
			}
		}
	}
	else 
	{
		//d->m_marbleWidget->update();
		if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
		{
			//m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}
	}
}

void TreeViewWidget::slotObjectVisible(long id,bool state)
{
	qint64 qID = (qint64)id;
	QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
	//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexID(qID);
	//if(pNode!=NULL)
	if(idx.isValid())
	{
		//pNode->m_bVisible=state;
		QVariant qVar(state);
		//QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
		d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
	}
}

void TreeViewWidget::slotLayerVisible(long id,QString& name,bool state)
{
	uint uid = (uint)id;
	qint64 qID = (qint64)uid;
	if(qID>0)
	{
		QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexID(qID);
		//if(pNode!=NULL)
		if(idx.isValid())
		{
			//pNode->m_bVisible=state;
			QVariant qVar(state);
			//QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
			d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
		}
	}
	else
	{
		// iterate all map layers
		QString layer;
		QString path;

		MapViewMapLayer* pMapLayer=NULL;
		QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();
		for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
		{
			pMapLayer = iter.value();
			if ( pMapLayer != NULL ) 
			{
				qint64 qID = (qint64)iter.key();
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
				if(pNode!=NULL)
				{
					layer = pMapLayer->runtimeTrace();//LayerName
					path  = pMapLayer->getFileName();//Layer file path
					if(name==layer || name==path)
					{
						QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
						if(idx.isValid())
						{
							QVariant qAktVar = d->m_treeViewModel->data(idx,Qt::CheckStateRole);
							if(state!=qAktVar.toBool())
							{
								QVariant qVar(state);
								d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
							}
							return;
						}
					}
					else if(pNode->m_bGeoSet)//get all geoset layers
					{
						TreeNode* pOverlay = pNode->parent;
						if(pOverlay!=NULL)
						{
							TreeNode* pChild=NULL;
							int count = pOverlay->children.count();
							for(int i=0;i<count;i++)
							{
								pChild = pOverlay->children.at(i);
								if(pChild!=NULL && pChild->m_bGeoSet)//do not add the geoset name itself
								{
									if( pChild->m_qsName == name)
									{
										qID = (qint64)pChild->m_lId;
										QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
										if(idx.isValid())
										{
											QVariant qAktVar = d->m_treeViewModel->data(idx,Qt::CheckStateRole);
											if(state!=qAktVar.toBool())
											{
												QVariant qVar(state);
												d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
											}
											return;
										}

									}//child name
								}//child geoset layer
							}//for
						}//pOverlay
					}
				}//if pNode
				
			}//if MapLayer
		}//for iter

	}//else
}

void TreeViewWidget::slotObjectIntersection(long id,double longitude,double latitude,long& visible)
{
	qint64 qID = (qint64)id;
	//QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(qID);
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode==NULL)return;

	if(pNode->ItemType != GIS::Plot && pNode->ItemType != GIS::PlotEx && pNode->ItemType != GIS::Rect && pNode->ItemType != GIS::Poly && pNode->ItemType != GIS::Polygone && pNode->ItemType != GIS::Elipse)
		return;

	GeoDataLineString gdl = pNode->m_poly;

	if( pNode->ItemType == GIS::Rect)
	{
		gdl.clear();
		GeoDataCoordinates lt;
		GeoDataCoordinates rb;
		lt = pNode->m_poly.at(0);
		gdl << lt;//lt
		rb = pNode->m_poly.at(1);
		lt.setLongitude(rb.longitude());
		gdl << lt;//rt
		gdl << rb;//rb
		lt = pNode->m_poly.at(0);
		rb.setLongitude(lt.longitude());
		gdl << rb;//lb
	}

	
//	GeoDataLinearRing glr(gdl);
	GeoDataPolygon poly( Tessellate );
	poly.setOuterBoundary( GeoDataLinearRing(gdl) );

	GeoDataCoordinates pos;
	pos.set(longitude,latitude,0,GeoDataCoordinates::Degree);

	if(poly.contains(pos))
		visible=1;
}

void TreeViewWidget::showSelectedItem ( const QModelIndex & index)
{
	QString qsLayer;
	if(index.isValid())
	{
		//QVariant qVar = d->m_marbleWidget->map()->model()->treeModel()->data(index,Qt::DisplayRole);//d->m_treeViewModel->data(index,Qt::DisplayRole);
		QVariant qVar = d->m_treeViewModel->data(index,Qt::DisplayRole);
		qsLayer = qVar.toString();
		if(!qsLayer.isEmpty())
		{
			if ( d->m_checkBoxMap.contains( qsLayer ) ) 
			{
				d->m_checkBoxMap[ qsLayer ] = !d->m_checkBoxMap.value( qsLayer );
				emit toggledShowProperty( qsLayer, d->m_checkBoxMap.value( qsLayer ) );
				if(qsLayer=="coordinate-grid")
				{
					m_bGrid=d->m_checkBoxMap.value( qsLayer );
					emit signalSetGrid(m_bGrid);
				}
				if(qsLayer=="grayscaled")
				{
					m_bGray=d->m_checkBoxMap.value( qsLayer );
					emit signalSetGray(m_bGray);
				}
				if(qsLayer=="elevation-shading")
				{
					m_bElevation=d->m_checkBoxMap.value( qsLayer );
					emit signalSetElevation(m_bElevation);
				}
			}
			else
			{
				//m_esriCacheID
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
				if(pNode == NULL)return;
				qVar = d->m_treeViewModel->data(index,Qt::CheckStateRole);
				if( (pNode->ItemType == Overlay || pNode->ItemType == Layer) && pNode->m_bMap )//Esri cache map?
				{
					if(pNode->ItemType == Overlay && pNode->m_pMapDoc!=NULL)//map overlay
					{
						if(pNode->m_pMapDoc->isVisible())
						{
							pNode->m_pMapDoc->setVisible(false);
							d->m_marbleWidget->model()->treeModel()->updateFeature(pNode->m_pMapDoc);
						}
						else 
						{
							pNode->m_pMapDoc->setVisible(true);
							d->m_marbleWidget->model()->treeModel()->updateFeature(pNode->m_pMapDoc);
						}
					/*	GeoDataDocument* pDoc = pNode->m_pMapDoc;
						GeoDataContainer *container = static_cast<GeoDataContainer*>( pDoc );
						QVector<GeoDataFeature*> vFeatureList = container->featureList();
						QVector<GeoDataFeature*>::const_iterator cont = vFeatureList.constBegin();
						QVector<GeoDataFeature*>::const_iterator endcont = vFeatureList.constEnd();
						if(qVar.toBool())
						{
						/*	for (; cont != endcont; ++cont ) {
									(*cont)->setVisible(true);
							/*		GeoDataGroundOverlay const * groundOverlay = static_cast<GeoDataGroundOverlay const *>( *cont );
									if(groundOverlay!=NULL)
										groundOverlay->setVisible(true);*/
						/*	}
							pNode->m_pMapDoc->setVisible(true);*/
					/*		d->m_marbleWidget->model()->treeModel()->addDocument(pDoc);
						}
						else
						{
						/*	for (; cont != endcont; ++cont ) {
									(*cont)->setVisible(false);
							}
							pNode->m_pMapDoc->setVisible(false);*/
					/*		d->m_marbleWidget->model()->treeModel()->removeDocument(pDoc);
						}*/
					}
					else if(pNode->ItemType == Layer && pNode->parent->m_pMapDoc!=NULL)//map overlay
					{
						if(pNode->parent->m_pMapDoc->isVisible())
						{
							pNode->parent->m_pMapDoc->setVisible(false);
							d->m_marbleWidget->model()->treeModel()->updateFeature(pNode->parent->m_pMapDoc);
						}
						else 
						{
							pNode->parent->m_pMapDoc->setVisible(true);
							d->m_marbleWidget->model()->treeModel()->updateFeature(pNode->parent->m_pMapDoc);
						}
			/*			GeoDataDocument* pDoc = pNode->parent->m_pMapDoc;
						GeoDataContainer *container = static_cast<GeoDataContainer*>( pDoc );
						QVector<GeoDataFeature*> vFeatureList = container->featureList();
						QVector<GeoDataFeature*>::const_iterator cont = vFeatureList.constBegin();
						QVector<GeoDataFeature*>::const_iterator endcont = vFeatureList.constEnd();
						if(qVar.toBool())
						{
						/*	for (; cont != endcont; ++cont ) {
									(*cont)->setVisible(true);
							/*		GeoDataGroundOverlay const * groundOverlay = static_cast<GeoDataGroundOverlay const *>( *cont );
									if(groundOverlay!=NULL)
										groundOverlay->setVisible(true);*/
						/*	}
							pNode->m_pMapDoc->setVisible(true);*/
			/*				d->m_marbleWidget->model()->treeModel()->addDocument(pDoc);
						}
						else
						{
						/*	for (; cont != endcont; ++cont ) {
									(*cont)->setVisible(false);
							}
							pNode->m_pMapDoc->setVisible(false);*/
			/*				d->m_marbleWidget->model()->treeModel()->removeDocument(pDoc);
						}*/
					}
					else if(pNode->ItemType == Overlay && pNode->m_bMap)//map overlay
					{
						int iC = pNode->children.count();
						for(int i=0;i<iC;i++)
						{
							TreeNode* pLayerNode = pNode->children.at(i);
							if(pLayerNode!=NULL && d->m_mvMapLayers.contains((qreal)(pLayerNode->m_lId)) )
							{
								MapViewMapLayer* pMapLayer = d->m_mvMapLayers.value((qreal)pLayerNode->m_lId);
								if(pMapLayer!=NULL)
								{
									//QVariant qVar(m_bGrid);
									qint64 id=(qint64)pLayerNode->m_lId;
									QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(id);
									if(idx.isValid())
										d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
								}
							}
						}

					}
				}
				else if(pNode->ItemType == Layer && pNode->m_bGeoSet && pNode->parent->m_bMap )//map layer?
				{
					int value=0;
					if(qVar.toBool())
						value=4;//visible
					else value=5;//invisible

					emit signalMapLayerItem(value,pNode->parent->m_lId,pNode->m_lId,qsLayer);

				}
				d->m_marbleWidget->update();
			}
		}
	}

}

void TreeViewWidget::zoomSelectedItem ( const QModelIndex & index)
{
	 if ( index.isValid() ) 
	 {
		 struct {
					long objectID;
					long visible;
					long size;
					void* pData;
			} data;			

		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
		if(pNode != NULL && (pNode->ItemType != Layer && pNode->ItemType != Overlay)  )
		{
			m_selectedID=0;

			//synchronize with layer selection model
			//QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
			MapViewLayer* pLayer=NULL;
			MapViewTSLayer* pTsLayer=NULL;
			TreeNode* pParent = NULL;

			if(pNode->ItemType == Group)
				pParent = pNode->parent;
			else pParent = pNode->parent->parent;

			if(pParent->ItemType == Layer)
			{
				if(!pParent->m_bTactical)
				{
					if(d->m_mvLayers.contains(pParent->m_lId))
						pLayer = d->m_mvLayers.value(pParent->m_lId);

					if(pLayer!=NULL)
					{
						//return if this layer is a legend layer
						if(!pLayer->getLegendLayer())
							return;
						pLayer->synchronizeWithTree(index);
					}
				}
				else if(d->m_mvTsLayers.contains(pParent->m_lId) )
				{
					pTsLayer = d->m_mvTsLayers.value(pParent->m_lId);

					if(pTsLayer!=NULL)
					{
						//return if this layer is a legend layer
						//if(!pLayer->getLegendLayer())
						//	return;
						pTsLayer->synchronizeWithTree(index);
					}

				}
			}

			const GeoDataCoordinates targetPosition =
				index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();

			if(targetPosition.latitude(GeoDataCoordinates::Degree) != -399.0 &&
				targetPosition.longitude(GeoDataCoordinates::Degree) != -399.0 )
			{
			
				d->m_marbleWidget->centerOn(targetPosition);
			}

			//send to GeoObjects and MapViewControl (to set Export Overlay or layer menu)
			if(pNode->ItemType == Group)
				emit signalSelectedItem(pNode->m_lId,0,0);
			else emit signalSelectedItem(pNode->parent->m_lId,pNode->m_lId,0);

			//recalc profile
			if(pNode->ItemType == Poly && pParent->m_qsName=="Distance")
				emit signalUpdateElevationProfile(pNode->m_poly);
		}
		else if (pNode->ItemType == Layer) 
		{
			m_selectedID = pNode->m_lId;
			//send to GeoObjects plugin
			if(m_sitOverlayID == pNode->parent->m_lId)//use situation overlay if existing
				emit signalSelectedItem(pNode->parent->m_lId,pNode->m_lId,3);
			else emit signalSelectedItem(pNode->parent->m_lId,pNode->m_lId,1);

			if(pNode->parent->m_bMap && !pNode->m_bMap)//map layer
			{
				MapViewMapLayer* pLayer = NULL;
				if(d->m_mvMapLayers.contains((qint64)pNode->m_lId) )
					pLayer = d->m_mvMapLayers.value((qint64)pNode->m_lId);
				else
				{
					QMap<qreal,MapViewMapLayer*>::const_iterator iterMap = d->m_mvMapLayers.constBegin();
					for ( ; iterMap != d->m_mvMapLayers.constEnd(); ++iterMap ) 
					{
						MapViewMapLayer* pMapLayer = iterMap.value();
						qint64 qID = (qint64)iterMap.key();
						TreeNode* pLayerNode = d->m_treeViewModel->getNodeFromIndexId(qID);
						if ( pMapLayer != NULL && pLayerNode->m_bGeoSet )
							pLayer=pMapLayer;
					}

				}
			
				if(pLayer!=NULL)
				{
					GeoDataLatLonBox box;
					qreal lon=0;
					qreal lat=0;
					if(pLayer->getLatLonBox(box,lon,lat))
					{
						d->m_marbleWidget->centerOn(box);
					}
					else if(lon!=lat)
					{
						GeoDataCoordinates targetPosition(lon,lat);
						d->m_marbleWidget->centerOn(targetPosition);
					}

					//send event if visibility changed
					data.objectID = pNode->m_lId;
					data.visible  = (long)pNode->m_bVisible;
					QString qsLayer=pNode->m_qsName;
					qsLayer+='\0';
					int len = qsLayer.length();
					QByteArray pByte = qsLayer.toUtf8();
					const char* pName = (char*)pByte.constData();
					data.pData = new char[len];
					memcpy(data.pData,pName,len);
					data.size = len;
					long size = 3 * sizeof(long) + len*sizeof(char);
					emit signalSendEvent( MST_EVENT_OBJECT_VISIBLE,size,&data,pLayer->getInterfaceID());

					delete[] data.pData;
				}

			}//map layer
			else//other layers
			{
				if(d->m_mvLayers.contains((qint64)pNode->m_lId) )
				{
					MapViewLayer* pLayer = d->m_mvLayers.value((qint64)pNode->m_lId);
					if(pLayer!=NULL)
					{
						//send event if visibility changed
						data.objectID = pNode->m_lId;
						data.visible  = (long)pNode->m_bVisible;
						QString qsLayer=pNode->m_qsName;
						qsLayer+='\0';
						int len = qsLayer.length();
						QByteArray pByte = qsLayer.toUtf8();
						const char* pName = (char*)pByte.constData();
						data.pData = new char[len];
						memcpy(data.pData,pName,len);
						data.size = len;
						long size = 3 * sizeof(long) + len*sizeof(char);
						emit signalSendEvent( MST_EVENT_OBJECT_VISIBLE,size,&data,pLayer->getInterfaceID());

						delete[] data.pData;
					}
				}

			}
		}//Layer
		else if (pNode->ItemType == Overlay)
		{
			m_selectedID=pNode->m_lId;
			if(pNode->children.count()>0)////send to GeoObjects plugin
			{
				if(m_sitOverlayID == m_selectedID)//use situation overlay if existing
					emit signalSelectedItem(pNode->m_lId,pNode->children.at(0)->m_lId,4);
				else emit signalSelectedItem(pNode->m_lId,pNode->children.at(0)->m_lId,2);
			}
			else 
			{
				if(m_sitOverlayID == m_selectedID)//use situation overlay if existing
					emit signalSelectedItem(pNode->m_lId,0,4);
				else emit signalSelectedItem(pNode->m_lId,0,2);
			}
		}


        //GeoDataLookAt target = d->m_map->lookAt();
        //target.setLongitude( targetPosition.longitude() );
        //target.setLatitude( targetPosition.latitude() );
	 }

	
/*	if(index.isValid())
	{
	//	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
		if(1)//pNode != NULL && pNode->ItemType == Group)
			d->m_marbleWidget->centerOn(index);
	}*/

}

void TreeViewWidget::contextTreeMenu(const QPoint &pt)
{
	Q_UNUSED(pt);      

	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;

	long id=0;
	bool bEdit=false;
	bool bObj=false;
	bool bDel=true;
	bool bExt=true;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
	if(pNode!=NULL)
	{
		if(pNode->ItemType == Overlay)
		{
			//the map
			if(pNode->m_bMap && pNode->m_lId != m_esriCacheID)
			{
				if(pNode->children.count()>0)
				{
					if(pNode->children.at(0)->m_bMap)
						return;
				}
				else
				{
					bEdit=false;
					bExt=false;
				}
			}
			else if(pNode->m_lId == m_esriCacheID)
				bExt=false;

			if(pNode->children.count()>0)
				id = pNode->children.at(0)->m_lId;
		}
		else if(pNode->ItemType == Layer)
		{
			if(pNode->parent->m_bMap&&pNode->m_bMap)return;
			if(pNode->m_bGeoSet || pNode->parent->m_bMap)
			{
				bEdit=false;
				bExt=false;
			}
			id = pNode->m_lId;
		}
		else if(pNode->ItemType == Group)
		{
			id = pNode->parent->m_lId;
		}
		else
		{
			id = pNode->parent->parent->m_lId;
			bObj=true;
		
		}

		if(d->m_mvLayers.contains((qint64)id) )
		{
			MapViewLayer* pLayer = d->m_mvLayers.value((qint64)id);
			if(pLayer!=NULL)
			{
				if(!pLayer->getDelete())
					bDel=false;
				if(bObj && !pLayer->getObjectDelete())
					bDel=false;
				if(pLayer->getEdit())
					bEdit=true;
			}
		}
		if(d->m_mvMapLayers.contains((qint64)id) )
		{
			MapViewMapLayer* pLayer = d->m_mvMapLayers.value((qint64)id);
			if(pLayer!=NULL)
			{
				if(pNode->m_bGeoSet)
					bEdit=false;
				if(pNode->m_bMap)
					bEdit=false;
			}
		}
				
		if(d->m_mvShadingLayers.contains((qint64)id) )
		{
			MapViewShadingLayer* pLayer = d->m_mvShadingLayers.value((qint64)id);
			if(pLayer!=NULL)
			{
				if(!pLayer->getDelete())
					bDel=false;
				if(bObj && !pLayer->getObjectDelete())
					bDel=false;
				if(pLayer->getEdit())
					bEdit=true;
			}
		}
	}


	QMenu menu(this); 

	QAction* deleteObjectAction = new QAction( QObject::tr( "&Delete" ), this);
	connect( deleteObjectAction, SIGNAL( triggered() ), this, SLOT( deleteTreeObject() ) );

	QAction* exportAction = new QAction( QObject::tr( "&Export" ), this);
	connect( exportAction, SIGNAL( triggered() ), this, SLOT( exportObjects() ) );

	deleteObjectAction->setEnabled(bDel);
	exportAction->setEnabled(bExt);
   
	menu.addAction(deleteObjectAction );
	menu.addAction(exportAction );

	if(pNode->ItemType == Layer)
	{
		QAction* moveUpAction = new QAction( QObject::tr( "Move&Up" ), this);
		connect( moveUpAction, SIGNAL( triggered() ), this, SLOT( moveTreeItemUp() ) );
		QAction* moveDownAction = new QAction( QObject::tr( "Move&Down" ), this);
		connect( moveDownAction, SIGNAL( triggered() ), this, SLOT( moveTreeItemDown() ) );
		QAction* moveToTopAction = new QAction( QObject::tr( "Move&ToTop" ), this);
		connect( moveToTopAction, SIGNAL( triggered() ), this, SLOT( moveTreeItemToTop() ) );
		QAction* moveToBottAction = new QAction( QObject::tr( "MoveTo&Bottom" ), this);
		connect( moveToBottAction, SIGNAL( triggered() ), this, SLOT( moveTreeItemToBott() ) );
		QAction* propAction = new QAction( QObject::tr( "&Properties" ), this);
		connect( propAction, SIGNAL( triggered() ), this, SLOT( setTreeItemProperties() ) );
		menu.addSeparator();
		menu.addAction(moveUpAction );
		menu.addAction(moveDownAction );
		menu.addAction(moveToTopAction );
		menu.addAction(moveToBottAction );
		menu.addSeparator();
		propAction->setEnabled(bEdit);
		menu.addAction(propAction );
	}
	else if(pNode->ItemType != Overlay && pNode->ItemType != Layer && pNode->ItemType != Group  && pNode->ItemType != SvgBts  && pNode->ItemType != SvgObj)
	{
		menu.addSeparator();
		QAction* propAction = new QAction( QObject::tr( "&Properties" ), this);
		connect( propAction, SIGNAL( triggered() ), this, SLOT( setTreeItemObjectProperties() ) );
		menu.addAction(propAction );

	}


	menu.exec(QCursor::pos()); 

}

void TreeViewWidget::moveTreeItemUp()
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;

	int flag=2;
	qreal zval=0;
	if(d->m_mvLayers.contains(idx.internalId()) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			zval = pLayer->zValue();
			zval++;
			pLayer->setLayerZValue(zval);
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
			if(pNode!=NULL && pNode->ItemType==Layer)
				pNode->m_iZVal = zval;
		}
	}
	else if(d->m_mvMapLayers.contains(idx.internalId()) )
	{
		MapViewMapLayer* pLayer = d->m_mvMapLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			zval = pLayer->zValue();
			zval++;
			pLayer->setLayerZValue(zval);
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
			if(pNode!=NULL && pNode->ItemType==Layer)
			{
				pNode->m_iZVal = zval;
				updateDocumentListing(idx.internalId(),2);
			}
		}
	}
	else 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
		if(pNode && pNode->m_bGeoSet)
		{
			emit signalMapLayerItem(flag,pNode->parent->m_lId,pNode->m_lId,QString());
		}
	}

	d->m_marbleWidget->update();

}
	
void TreeViewWidget::moveTreeItemDown()
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;
	int flag=3;
	qreal zval=0;
	if(d->m_mvLayers.contains(idx.internalId()) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			zval = pLayer->zValue();
			if(zval>0)
			{
				zval--;
				if(zval<0)
					zval=0;
				pLayer->setLayerZValue(zval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
				if(pNode!=NULL && pNode->ItemType==Layer)
					pNode->m_iZVal = zval;
			}
		}
	}
	else if(d->m_mvMapLayers.contains(idx.internalId()) )
	{
		MapViewMapLayer* pLayer = d->m_mvMapLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			zval = pLayer->zValue();
			if(zval>0)
			{
				zval--;
				if(zval<0)
					zval=0;
				pLayer->setLayerZValue(zval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
				if(pNode!=NULL && pNode->ItemType==Layer)
				{
					pNode->m_iZVal = zval;
					updateDocumentListing(idx.internalId(),2);
				}
			}
		}
	}
	else 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
		if(pNode && pNode->m_bGeoSet)
		{
			emit signalMapLayerItem(flag,pNode->parent->m_lId,pNode->m_lId,QString());
		}
	}

	d->m_marbleWidget->update();

}

void TreeViewWidget::moveTreeItemToTop()
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;
	int flag=0;
	qreal zval=0;
	if(d->m_mvLayers.contains(idx.internalId()) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			TreeNode* pNodeLayer = d->m_treeViewModel->getNodeFromIndex(idx);
			TreeNode* pNodeOverlay = pNodeLayer->parent;
			MapViewLayer* pChild=NULL;
			if(pNodeOverlay!=NULL)
			{
				int iChildCount = pNodeOverlay->children.count();
				qint64 qID=0;
				for(int i=0;i<iChildCount;i++)
				{
					pNodeLayer = pNodeOverlay->children.at(i);
					if(pNodeLayer!=NULL)
					{
						qID = (qint64)pNodeLayer->m_lId;
						pChild = d->m_mvLayers.value(qID);

						if(zval < pChild->zValue())
							zval = pChild->zValue();
					}
				}//for
					
				pLayer->setLayerZValue(++zval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
				if(pNode!=NULL && pNode->ItemType==Layer)
					pNode->m_iZVal = zval;
			}//if(pNodeOverlay!=NULL)
		}//if pLayer
	}
	else if(d->m_mvMapLayers.contains(idx.internalId()) )
	{
		MapViewMapLayer* pLayer = d->m_mvMapLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			QString name = pLayer->runtimeTrace();
			setLayerZorder(name,flag,idx.internalId());//move to top
		}
	}
	else 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
		if(pNode && pNode->m_bGeoSet)
		{
			emit signalMapLayerItem(flag,pNode->parent->m_lId,pNode->m_lId,QString());
		}
	}

	d->m_marbleWidget->update();
}

void TreeViewWidget::moveTreeItemToBott()
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;
	int flag=1;
	qreal zval=100;
	if(d->m_mvLayers.contains(idx.internalId()) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			TreeNode* pNodeLayer = d->m_treeViewModel->getNodeFromIndex(idx);
			TreeNode* pNodeOverlay = pNodeLayer->parent;
			MapViewLayer* pChild=NULL;
			if(pNodeOverlay!=NULL)
			{
				int iChildCount = pNodeOverlay->children.count();
				qint64 qID=0;
				for(int i=0;i<iChildCount;i++)
				{
					pNodeLayer = pNodeOverlay->children.at(i);
					if(pNodeLayer!=NULL)
					{
						qID = (qint64)pNodeLayer->m_lId;
						pChild = d->m_mvLayers.value(qID);

						if(zval > pChild->zValue())
							zval = pChild->zValue();
					}
				}//for
					
				if(zval<1)
					zval=1;
					
				pLayer->setLayerZValue(--zval);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
				if(pNode!=NULL && pNode->ItemType==Layer)
					pNode->m_iZVal = zval;
			}//if(pNodeOverlay!=NULL)
		}//if pLayer
	}
	else if(d->m_mvMapLayers.contains(idx.internalId()) )
	{
		MapViewMapLayer* pLayer = d->m_mvMapLayers.value(idx.internalId());
		if(pLayer!=NULL)
		{
			QString name = pLayer->runtimeTrace();
			setLayerZorder(name,flag,idx.internalId());//move to bottom
		}
	}
	else 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
		if(pNode && pNode->m_bGeoSet)
		{
			emit signalMapLayerItem(flag,pNode->parent->m_lId,pNode->m_lId,QString());
		}
	}

	d->m_marbleWidget->update();
}

void TreeViewWidget::setTreeItemProperties()
{
	QColor lineColor;
	QColor fillColor;
	QColor fontColor;
	int    iTrans=0;
	int	   iWidth=0;
	Qt::PenStyle pStyle;
	int ret = 0;
	m_iPropSet=0;

	if(m_lpDialog!=NULL)
	{
		QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
		if(!idx.isValid())
			return;

		if(d->m_mvLayers.contains(idx.internalId()) )
		{
			MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
			if(pLayer!=NULL)
			{
				lineColor = pLayer->getLineColor();
				fillColor = pLayer->getFillColor();
				fontColor = pLayer->getFontColor();
				iTrans    = pLayer->getTransparency();
				iWidth    = pLayer->getLineWidth();
				pStyle = pLayer->getPenStyleValue();

				QString s = "background-color: ";
				ui_lpWidget->m_colorLineButton->setStyleSheet(s + lineColor.name());
				s.clear();
				s = "background-color: ";
				ui_lpWidget->m_colorFillButton->setStyleSheet(s + fillColor.name());
				s.clear();
				s = "background-color: ";
				ui_lpWidget->m_colorFontButton->setStyleSheet(s + fontColor.name());

				ui_lpWidget->m_transSpinBox->setValue(iTrans);
				ui_lpWidget->spinBoxWidth->setValue(iWidth);
				m_lsComboBox->setPenStyle(pStyle);				

				ret = m_lpDialog->exec();
				if(ret==1)
				{
					TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
					int red     = 0;
					int green   = 0;
					int blue    = 0;
					long val=0;

					if(m_iPropSet&1)
					{
						pLayer->setLineColor(m_colLine);
						red = m_colLine.red();
						green = m_colLine.green();
						blue = m_colLine.blue();
						val = blue;
						val <<= 8;
						val+=green;
						val<<=8;
						val+=red;
						if(pNode!=NULL)
							pNode->m_lLineCol=val;
					}
					if(m_iPropSet&2)
					{
						pLayer->setFillColor(m_colFill);
						red = m_colFill.red();
						green = m_colFill.green();
						blue = m_colFill.blue();
						val = blue;
						val <<= 8;
						val+=green;
						val<<=8;
						val+=red;
						if(pNode!=NULL)
							pNode->m_lFillCol=val;
					}
					if(m_iPropSet&16)
					{
						//pLayer->setFontColor(m_colFont);
						red = m_colFont.red();
						green = m_colFont.green();
						blue = m_colFont.blue();
						val = blue;
						val <<= 8;
						val+=green;
						val<<=8;
						val+=red;
						//if(pNode!=NULL)
						//	pNode->m_lFillCol=val;
						pLayer->setFontColor(val);
					}
					if(m_iPropSet&4)
					{
						iTrans = ui_lpWidget->m_transSpinBox->value();
						pLayer->setTransparency(iTrans);
						if(pNode!=NULL)pNode->m_iTrans=iTrans;
					}
					if(m_iPropSet&8)
					{
						iWidth = ui_lpWidget->spinBoxWidth->value();
						pLayer->setLineWidth(iWidth);
					}
					if(m_lsComboBox!=NULL)
					{
						pStyle = m_lsComboBox->penStyle();
						pLayer->setPenStyleValue(pStyle);
					}

					QFont font = ui_lpWidget->fontComboBox->currentFont();
					pLayer->setDefaultFont(font);
				}
			}//pLayer
		}//contains
	}//lpDialog

}

void TreeViewWidget::setTreeItemObjectProperties()
{
	QColor lineColor;
	QColor fillColor;
	int    iTrans=0;
	int	   iWidth=0;
	Qt::PenStyle pStyle;
	int ret = 0;
	m_iPropSet=0;

	if(m_opDialog!=NULL)
	{
		QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
		if(!idx.isValid())
			return;

		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
		if(pNode==NULL)return;

		long val	= 0;
		int red     = 0;
		int green   = 0;
		int blue    = 0;

		val	= pNode->m_lFillCol;
		if(val<0)//get from layer
		{
			val = pNode->parent->parent->m_lFillCol;
		}
		red     = val & 255;
		green   = ( val>>8 ) & 255;
		blue    = ( val>>16 ) & 255;

		if(pNode->m_iTrans<0)
			iTrans = pNode->parent->parent->m_iTrans;
		else iTrans = pNode->m_iTrans;
		if(iTrans!=-1)
			fillColor = QColor( red, green, blue,iTrans );
		else 
		{	
			fillColor = QColor( red, green, blue);
			iTrans=0;
		}

		iWidth = pNode->m_iDirection;

		val	   = pNode->m_lLineCol;
		if(val<0)//get from layer
		{
			val = pNode->parent->parent->m_lLineCol;
		}
		red    = val & 255;
		green  = ( val>>8 ) & 255;
		blue   = ( val>>16 ) & 255;
		lineColor = QColor( red, green, blue);

		QString s = "background-color: ";
		ui_opWidget->m_colorLineButton->setStyleSheet(s + lineColor.name());
		s.clear();
		s = "background-color: ";
		ui_opWidget->m_colorFillButton->setStyleSheet(s + fillColor.name());

		ui_opWidget->m_transSpinBox->setValue(iTrans);
		ui_opWidget->spinBoxWidth->setValue(iWidth);

		int ret = m_opDialog->exec();
		if(ret==1)
		{
			red = m_colFill.red();
			green = m_colFill.green();
			blue = m_colFill.blue();
			//iTrans = m_colFill.alpha();
			val = blue;
			val <<= 8;
			val+=green;
			val<<=8;
			val+=red;

			iTrans = ui_opWidget->m_transSpinBox->value();

			if(m_iPropSet&2)
				pNode->m_lFillCol=val;
			if(m_iPropSet&4)
				pNode->m_iTrans=iTrans;

			red = m_colLine.red();
			green = m_colLine.green();
			blue = m_colLine.blue();
			//iTrans = m_colFill.alpha();
			val = blue;
			val <<= 8;
			val+=green;
			val<<=8;
			val+=red;
			if(m_iPropSet&1)
				pNode->m_lLineCol=val;

			if(m_iPropSet&8)
			{
				iWidth = ui_opWidget->spinBoxWidth->value();
				pNode->m_iDirection=iWidth;
			}

		}

	}
}

void TreeViewWidget::exportObjects()
{
	QString filename;
	QString qsDir = MarbleDirs::marbleDataPath();
	qsDir += "/LocationMap";

	QDir dir(qsDir);
	if(!dir.exists())
	{
		dir.mkdir(qsDir);
	}

    filename = QFileDialog::getSaveFileName( 0, tr("Export file name"),
                            qsDir,
                            tr("All Supported Files (*.kml);;KML file (*.kml)"));

	if(!filename.isEmpty())
		exportLayer(filename);

}

void TreeViewWidget::deleteTreeObject()
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
	if(!idx.isValid())
		return;

	long intID=0;
	if(d->m_mvLayers.contains(idx.internalId()) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value(idx.internalId());
		if(pLayer!=NULL)
			intID = pLayer->getInterfaceID();
	}
	//taktical layers
/*	if(d->m_mvTsLayers.contains(idx.internalId()) )
	{
		MapViewTsLayer* pTsLayer = d->m_mvTsLayers.value(idx.internalId());
		if(pTsLayer!=NULL)
			intID = pTsLayer->getInterfaceID();
	}*/

	else if(d->m_mvMapLayers.contains(idx.internalId()) )
	{
		MapViewMapLayer* pMapLayer = d->m_mvMapLayers.value(idx.internalId());
		if(pMapLayer!=NULL)
		{
			intID = pMapLayer->getInterfaceID();
			//Romes
			//if(pLayer->parent->m_bMap)
			//long id=(long)idx.internalId();
			//emit signalSendEvent(MST_EVENT_MAP_RELEASE,sizeof(long),&id,0);
		}
	}

	itemToDelete (idx,intID);
	d->m_marbleWidget->update();
}

//signal sent by TcpIp plugin
void TreeViewWidget::deleteObjects(QStringList& qsl,long id, int type)
{
	 switch(type)
	 {
		case MST_DELETE_OVERLAY:
						deleteOverlay((long)id);
						 break;
		case MST_DELETE_LAYER:
						deleteLayer((long)id);
						 break;
		case MST_DELETE_GROUP:
						deleteGroup((long)id);
						 break;
		case MST_DELETE_SHAPE:
						 deleteShape((long)id);
						 break;
		case MST_DELETE_BY_NAME: 
			{
				qint64 ret =  d->m_treeViewModel->DeleteByName(qsl);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(ret);
				if(pNode!=NULL)
				{
					if(pNode->ItemType==Overlay)
						deleteOverlay((long)ret);
					else if(pNode->ItemType==Layer)
						deleteLayer((long)ret);
					else if(pNode->ItemType==Group)
						deleteGroup((long)ret);
					else deleteShape((long)ret);
				}
			}
			break;
	 }
		

	if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
	{
		//m_pUpdateTimer->stop();
		m_pUpdateTimer->start();
	}
	else if(m_pUpdateTimer!=NULL && m_pUpdateTimer->isActive())
	{
		if(m_iRepaintCount<10)
		{
			m_pUpdateTimer->stop();
			m_iRepaintCount++;
			m_pUpdateTimer->start();
		}
	}

}

void TreeViewWidget::deleteOverlay(long id)
{
	uint uid = (uint)id;
	qint64 idx = (qint64)uid;
	TreeNode* pOverlay = d->m_treeViewModel->getNodeFromIndexId(idx);
	if(pOverlay==NULL)return;

	if(pOverlay->ItemType != Overlay)
		return;

	if(pOverlay->m_lId == m_esriCacheID)
	{
		emit signalSelectedItem(-100,-100,-100);
		return;
	}

	//first delete child
	TreeNode* pLayer=NULL;
	int iL = 0;
	do{
		iL = pOverlay->children.count();
		if(iL>0)
		{
			//Layer
			pLayer = pOverlay->children.value(0);
			if(pLayer!=NULL)
			{
				deleteLayer(pLayer->m_lId);
			}
		}
		if(iL == pOverlay->children.count())//should not be equal
			iL=0;

	}while(iL>0);

	/*for(int iL =0;iL<pOverlay->children.count();iL++)
	{
		//Layer
		pLayer = pOverlay->children.value(iL);
		if(pLayer!=NULL)
		{
			deleteLayer(pLayer->m_lId);
		}
	}*/

	d->m_treeViewModel->DeleteObject(pOverlay->m_lId,0);
	
	if(m_sitOverlayID==idx)m_sitOverlayID=0;//set situation overlay to 0

	//remove overlay from legend overlay list
	if(m_overlayLegend.contains(id))
		m_overlayLegend.remove(id);

	if(m_tsOverlayID==id)m_tsOverlayID=0;//set taktical overlay to 0

	if(m_minfOverlayID==idx)m_minfOverlayID=0;//set MapInfo overlay to 0
	if(m_esriOverlayID==idx)m_esriOverlayID=0;//set ESRI overlay to 0
	if(m_mapOverlayID==idx)m_mapOverlayID=0;//set MapLayer overlay to 0
		
	//remove overlay from legend overlay list
	if(d->m_mvTsLayers.contains((qreal)idx))
		d->m_mvTsLayers.remove((qreal)idx);

	//remove overlay from legend overlay list
	if(d->m_mvMapLayers.contains((qreal)idx))
		d->m_mvMapLayers.remove((qreal)idx);

	//delete file from file list
	if(!pOverlay->m_qsDesc.isEmpty())
	{
		QString key = pOverlay->m_qsDesc;
		d->m_marbleWidget->model()->removeGeoData(key);
		//Romes
		//emit signalSendEvent(MST_EVENT_MAP_RELEASE,sizeof(long),&id,0);
	}


	//synchronize MvInt plugin QHash
	emit signalDeleteObject(MST_DELETE_OVERLAY,id);
}

void TreeViewWidget::deleteLayer(long id)
{
	uint uid = (uint)id;
	qint64 idx = (qint64)uid;
	TreeNode* pLayer = d->m_treeViewModel->getNodeFromIndexId(idx);
	if(pLayer==NULL)return;

	if(pLayer->ItemType != Layer)
	{
		if(pLayer->ItemType == Overlay)
		{
			deleteOverlay(pLayer->m_lId);
			return;
		}
		else if(pLayer->ItemType == Group)
		{
			deleteGroup(pLayer->m_lId);
			return;
		}
		else return;
	}
	//first delete child
	TreeNode* pGroup=NULL;
	int iG = 0;
	do{
		iG=pLayer->children.count();
		if(iG>0)
		{
			//Group
			pGroup = pLayer->children.value(0);
			if(pGroup!=NULL)
			{
				deleteGroup(pGroup->m_lId);
			}
		}
		if(iG==pLayer->children.count())//should not be equal
			iG=0;

	}while(iG>0);

	//Romes
	if(pLayer->parent!=NULL&&pLayer->parent->m_bMap)
	{
		if(d->m_mvMapLayers.contains((qreal)idx) )
			emit signalSendEvent(MST_EVENT_MAP_RELEASE,sizeof(long),&id,0);
		else
			emit signalSendEvent(MST_EVENT_LAYER_REMOVE,sizeof(long),&id,0);
	}

	/*for(int iG =0;iG<pLayer->children.count();iG++)
	{
		//Group
		pGroup = pLayer->children.value(iG);
		if(pGroup!=NULL)
		{
			deleteGroup(pGroup->m_lId);
		}
	}*/

	if(m_tsLayerID==idx)m_tsLayerID=0;//set taktical layer to 0

	if(pLayer->m_bGeoSet && d->m_mvMapLayers.contains((qreal)idx) )//delete all geoset layers
	{
		TreeNode* pOverlay = pLayer->parent;
		if(pOverlay!=NULL)
		{
			TreeNode* pChild=NULL;
			int count = pOverlay->children.count();
			for(int i=0;i<count;i++)
			{
				pChild = pOverlay->children.at(0);
				if(pChild!=NULL && pChild->m_bGeoSet)// && pChild->m_lId!=id)
				{
					 d->m_treeViewModel->DeleteObject(pChild->m_lId,0);
					 emit signalDeleteObject(MST_DELETE_LAYER,pChild->m_lId);
				}
			}
		}
		m_qsGeoset.clear();
	}
	else
		qint64 ret = d->m_treeViewModel->DeleteObject(pLayer->m_lId,0);

	//synchronize MvInt plugin QHash
	emit signalDeleteObject(MST_DELETE_LAYER,id);
	removeLayer(idx);
}

void TreeViewWidget::deleteGroup(long id)
{
	uint uid = (uint)id;
	qint64 idx = (qint64)uid;
	TreeNode* pGroup = d->m_treeViewModel->getNodeFromIndexId(idx);
	if(pGroup==NULL)return;

	if(pGroup->ItemType != Group)
	{
		if(pGroup->ItemType == Overlay)
		{
			deleteOverlay(pGroup->m_lId);
			return;
		}
		else if(pGroup->ItemType == Layer)
		{
			deleteLayer(pGroup->m_lId);
			return;
		}
		else return;
	}

	//reset focus object id
	if(m_focusID==idx)
	{
		if(m_pFocusTimer!=NULL && m_pFocusTimer->isActive())
			m_pFocusTimer->stop();
		m_focusID=0;
	}
	//first delete child
	TreeNode* pChild=NULL;
	int iC = 0;
	do{
		iC = pGroup->children.count();
		if(iC>0)
		{
			//Group child
			pChild = pGroup->children.value(0);//delete first element
			if(pChild!=NULL)
			{
				deleteShape(pChild->m_lId);
			}
		}
		if(iC == pGroup->children.count())//should not be equal
			iC=0;

	}while(iC>0);
	/*for(int iC =0;iC<pGroup->children.count();iC++)
	{
		//Group child
		pChild = pGroup->children.value(iC);
		if(pChild!=NULL)
		{
			deleteShape(pChild->m_lId);
		}
	}*/

	if(m_tsGroupID==idx)m_tsGroupID=0;//set taktical group to 0

	d->m_treeViewModel->DeleteObject(pGroup->m_lId,0);
	//synchronize MvInt plugin QHash
	emit signalDeleteObject(MST_DELETE_GROUP,id);
}

void TreeViewWidget::deleteShape(long id)
{
	uint uid = (uint)id;
	//reset focus object id
	if(m_focusID==uid)
	{
		if(m_pFocusTimer!=NULL && m_pFocusTimer->isActive())
			m_pFocusTimer->stop();
		m_focusID=0;
	}

	//only emit the signal to the layer containing this object
	qint64 qID=(qint64)uid;
	Model::TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode==NULL || pNode->parent==NULL || pNode->parent->parent==NULL)
	{
		emit signalDeleteObject(MST_DELETE_SHAPE,id);
		return;
	}

	qint64 layerID=0;
	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
		layerID = (qint64)iter.key();
        if ( pLayer != NULL && layerID) 
		{
			if(pNode->parent->parent->m_lId==layerID)
			{
				//synchronize MvInt plugin QHash
				emit signalDeleteObject(MST_DELETE_SHAPE,id);

				//delete from layer
				pLayer->slotDeleteTreeItem(MST_DELETE_SHAPE,id);

				d->m_treeViewModel->DeleteObject(id,0);

				return;
			}

		}
	}

	//shading layer grid
	MapViewShadingLayer* pShadingLayer=NULL;
    QMap<qreal,MapViewShadingLayer*>::const_iterator itersh = d->m_mvShadingLayers.constBegin();
    for ( ; itersh != d->m_mvShadingLayers.constEnd(); ++itersh ) 
	{
		pShadingLayer = itersh.value();
		layerID = (qint64)itersh.key();
        if ( pShadingLayer != NULL && layerID) 
		{
			if(pNode->parent->parent->m_lId==layerID)
			{
				//synchronize MvInt plugin QHash
				emit signalDeleteObject(MST_DELETE_SHAPE,id);

				//delete from layer
				pShadingLayer->slotDeleteTreeItem(MST_DELETE_SHAPE,id);

				d->m_treeViewModel->DeleteObject(id,0);

				return;
			}

		}
	}

	//ts symbole
	MapViewTSLayer* pTsLayer=NULL;			
	QMap<qreal,MapViewTSLayer*>::const_iterator tsiter = d->m_mvTsLayers.constBegin();
	for ( ; tsiter != d->m_mvTsLayers.constEnd(); ++tsiter ) 
	{
		pTsLayer = tsiter.value();
		layerID = (qint64)tsiter.key();
        if ( pTsLayer != NULL && layerID) 
		{
			if(pNode->parent->parent->m_lId==layerID)
			{
				//synchronize MvInt plugin QHash
				emit signalDeleteObject(MST_DELETE_SHAPE,id);

				//delete from layer
				pTsLayer->slotDeleteTreeItem(MST_DELETE_SHAPE,id);

				d->m_treeViewModel->DeleteObject(id,0);

				return;
			}

		}
	}
}

bool TreeViewWidget::centerFocusObject(qreal lon, qreal lat)
{
	if(!m_bFocusObject)return false;
	if(d->m_marbleWidget==NULL)return false;
	if(m_focusID==0)return false;

	bool ret=false;

	qreal left=0;
	qreal right=0;
	qreal top=0;
	qreal bott=0;

	GeoDataLatLonBox llBox = d->m_marbleWidget->viewport()->viewLatLonAltBox();
	llBox.boundaries(top,bott,right,left,GeoDataCoordinates::Degree);

//	qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
//	qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

	qreal xDist = ((right-left)/100)*5;//5%
	qreal yDist = ((top-bott)/100)*5;//5%

	if(lon-xDist < left || lon+xDist>right || lat+yDist>top || lat-yDist<bott)
	{
		if(m_pFocusTimer!=NULL && m_pFocusTimer->isActive())
				m_pFocusTimer->stop();

		if(d->m_treeViewModel!=NULL)
		{
			QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_focusID);
			if(idx.isValid())
			{
				zoomSelectedItem (idx);
				//turn the map
				//TreeNode* pChild = d->m_treeViewModel->getNodeFromIndexId(m_focusID);
				//if(pChild!=NULL && pChild->m_iDirection!=0)
				//	d->m_marbleWidget->setHeading(pChild->m_iDirection);

				ret = true;
			}
		}

		if(m_pFocusTimer!=NULL)
			m_pFocusTimer->start();
	}

	return ret;

}

void TreeViewWidget::selectTheme( const QString& themeId )
{
	if(d->m_treeViewModel!=NULL)
	{
		removeMapFromTree();
		addMapToTree();
		d->m_ui.treeTreeView->resizeColumnToContents(0);

/*		struct {
					double left;
					double top;
					double right;
					double bottom;
				} data; 

				pos = pNode->m_poly.at(0);;
				data.left = pos.longitude(GeoDataCoordinates::Degree);
				data.top = pos.latitude(GeoDataCoordinates::Degree);
				pos = pNode->m_poly.at(1);;
				data.right = pos.longitude(GeoDataCoordinates::Degree);
				data.bottom = pos.latitude(GeoDataCoordinates::Degree);
				long size = sizeof(long) + 4*sizeof(double);

				emit signalSendEvent(MST_EVENT_GROUP_MOVE_EX,size,&data,intID);*/
	}
}

void TreeViewWidget::selectRegion(const QList<double>& list)
{
	if( d->m_marbleWidget==NULL)return;
	if(list.count()<4)return;

	double west = list.at(0);
	double north = list.at(1);
	double east = list.at(2);
	double south = list.at(3);

	qreal x1=0;
	qreal y1=0;
	d->m_marbleWidget->screenCoordinates(west,north,x1,y1);

	qreal x2=0;
	qreal y2=0;
	d->m_marbleWidget->screenCoordinates(east,south,x2,y2);

	QRect rect;//
	rect.setTopLeft(QPoint((int)x1,(int)y1));
	rect.setBottomRight(QPoint((int)x2,(int)y2));

	MapViewLayer* pLayer = NULL;
	QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
		if(pLayer!=NULL)
		{
			QModelIndex idx;
			//detect all object from rect
			int count = pLayer->intersectionRegionObjects(idx,rect,true,0);
		}
	}

}

void TreeViewWidget::removeMapFromTree()
{
	//d->m_treeViewModel->setRootNode(NULL);
	if(d->m_treeViewModel!=NULL)
		d->m_treeViewModel->RemoveMap();
}

void TreeViewWidget::addMapToTree()
{
	// Check for a theme specific legend.html first
    if ( d->m_marbleWidget != 0
     && d->m_marbleWidget->model() != 0
     && d->m_marbleWidget->model()->mapTheme() != 0 )
    {
        GeoSceneDocument *currentMapTheme = d->m_marbleWidget->model()->mapTheme();		

		QString qsmap = currentMapTheme->head()->name();
		QStringList item;
		item << "180.0" << "90.0" << qsmap ;
		qint64 id = d->m_treeViewModel->AddOverlay(item,true);
		item.clear();

		if(id==0)return;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(id);
		if(pNode!=NULL)
			pNode->m_bMultiSelect=true;//this makes the difference from e.g. TAB overlay

		//GeoDataTreeModel* pTree = (GeoDataTreeModel*) d->m_marbleWidget->model()->treeModel();
	/*	GeoDataFolder* pFolder;
		d->m_document = new GeoDataDocument();
		d->m_document->setName(qsmap);*/
	//	d->m_fileManager->addGeoDataDocument(d->m_document);

        QVector<GeoSceneProperty*> allProperties = currentMapTheme->settings()->allProperties();

        d->m_checkBoxMap.clear();

        QVector<GeoSceneProperty*>::const_iterator it = allProperties.constBegin();
        QVector<GeoSceneProperty*>::const_iterator const end = allProperties.constEnd();
        for (; it != end; ++it ) 
		{
            if ( (*it)->available() ) 
			{
                d->m_checkBoxMap[ (*it)->name() ] = (*it)->value();
				qsmap = (*it)->name();
				if(qsmap.compare("compass")!=0 && qsmap.compare("overviewmap")!=0 && qsmap.compare("scalebar")!=0)
				{
					item << "180.0" << "90.0" << qsmap ;
					qint64 lID = d->m_treeViewModel->AddMapLayer(item,id,(bool)(*it)->value());
					item.clear();

					if(qsmap=="elevation-shading")
					{
						m_lElevationID=lID;//save the grid layer id
						if(m_bElevation)
						{
							QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(lID);
							if(idx.isValid())
							{
		
								QVariant qVar(m_bElevation);
								d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
							}
							d->m_checkBoxMap[qsmap] = m_bElevation;
							emit toggledShowProperty(qsmap, m_bElevation );
						}
					}
					if(qsmap=="coordinate-grid")
					{
						m_lGridMapID=lID;//save the grid layer id
						if(!m_bGrid)
						{
							QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(lID);
							if(idx.isValid())
							{
		
								QVariant qVar(m_bGrid);
								d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
							}
							d->m_checkBoxMap[qsmap] = m_bGrid;
							emit toggledShowProperty(qsmap, m_bGrid );
						}
					}
					if(qsmap=="grayscaled")
					{
						m_lGrayMapID=lID;//save the grayscaled layer id
						if(m_bGray)
						{
							QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(lID);
							if(idx.isValid())
							{
		
								QVariant qVar(m_bGray);
								d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
							}
							d->m_checkBoxMap[qsmap] = m_bGray;
							emit toggledShowProperty(qsmap, m_bGray );
						}
					}

				/*	pFolder= new GeoDataFolder();
					pFolder->setName(qsmap);
					d->m_document->append(pFolder); */
				}
            }
        }

        disconnect ( currentMapTheme, SIGNAL( valueChanged( QString, bool ) ), 0, 0 );
        connect ( currentMapTheme, SIGNAL( valueChanged( QString, bool ) ),
                  this, SLOT( setCheckedProperty( QString, bool ) ) );

/*		if(0)//bGrid)
		{
			QString qsmap="coordinate-grid";
			emit toggledShowProperty(qsmap, m_bGrid );
		}*/
    }
}

void TreeViewWidget::setCheckedProperty( const QString& name, bool checked )
{
	if(name == "elevation-shading")
	{
	//	m_bElevation=checked;
		if ( d->m_checkBoxMap[ name ] != checked )
		{
		//	d->m_checkBoxMap[ name ] = checked;
			emit signalSetElevation(checked);
			m_bElevation=checked;
			QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_lElevationID);
			if(idx.isValid()&&m_lElevationID)
			{
		
				QVariant qVar(m_bElevation);
				d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
			}
			emit toggledShowProperty(name, m_bElevation );
		}
	}

	if(name == "coordinate-grid")
	{
		if ( d->m_checkBoxMap[ name ] != checked )
		{
		//	d->m_checkBoxMap[ name ] = checked;
			emit signalSetGrid(checked);
			m_bGrid=checked;
			QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_lGridMapID);
			if(idx.isValid()&&m_lGridMapID)
			{
		
				QVariant qVar(m_bGrid);
				d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
			}
			emit toggledShowProperty(name, m_bGrid );
		}
	}
	if(name == "grayscaled")
	{
		if ( d->m_checkBoxMap[ name ] != checked )
		{
		//	d->m_checkBoxMap[ name ] = checked;
			emit signalSetGray(checked);
			m_bGray=checked;
			QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_lGrayMapID);
			if(idx.isValid()&&m_lGrayMapID)
			{
		
				QVariant qVar(m_bGray);
				d->m_treeViewModel->setData(idx,qVar,Qt::CheckStateRole);
			}
			emit toggledShowProperty(name, m_bGray );
		}
	}

    // If there is no change then leave immediately
    if ( d->m_checkBoxMap[ name ] == checked )
        return;
    
    d->m_checkBoxMap[ name ] = checked;

/*    setUpdatesEnabled( false );
    {
        int scrollPosition = verticalScrollBar()->sliderPosition();
        loadLegend();

        verticalScrollBar()->setSliderPosition( scrollPosition );
    }
    setUpdatesEnabled( true );*/

    update();
}
void TreeViewWidget::slotMouseGeoPos(const qreal lon, const qreal lat)
{
	if(!m_bElevation)return;

	const GeoDataCoordinates pos(lon,lat);

	if(d->m_runnerManager!=NULL)
		d->m_runnerManager->detectElevation(pos);

}
/*
void TreeViewWidget::getElevation(const QString& pos)
{
	if(!m_bElevation)return;

	GeoDataCoordinates geo;
	bool success=false;
	geo = geo.fromString(pos,success);

	if(d->m_runnerManager!=NULL)
		d->m_runnerManager->detectElevation(geo);

}*/

/*
void RoutingWidget::retrieveRoute()
{
    if ( d->m_inputWidgets.size() == 1 ) {
        // Search mode
        d->m_inputWidgets.first()->findPlacemarks();
        return;
    }

    int index = d->m_ui.routePreferenceComboBox->currentIndex();
    RouteSkeleton::RoutePreference pref = RouteSkeleton::CarFastest;
    if ( index == 1 ) {
        pref = RouteSkeleton::CarShortest;
    }
    if ( index == 2 ) {
        pref = RouteSkeleton::Bicycle;
    }
    if ( index == 3 ) {
        pref = RouteSkeleton::Pedestrian;
    }
    RouteSkeleton::AvoidFeatures avoid = RouteSkeleton::AvoidNone;
    if ( d->m_ui.highwaysCheckBox->isChecked() ) {
        avoid |= RouteSkeleton::AvoidHighway;
    }
    if ( d->m_ui.tollWaysCheckBox->isChecked() ) {
        avoid |= RouteSkeleton::AvoidTollWay;
    }

    d->m_routeSkeleton->setRoutePreference( pref );
    d->m_routeSkeleton->setAvoidFeatures( avoid );

    Q_ASSERT( d->m_routeSkeleton->size() == d->m_inputWidgets.size() );
    for ( int i = 0; i < d->m_inputWidgets.size(); ++i ) {
        RoutingInputWidget *widget = d->m_inputWidgets.at( i );
        if ( !widget->hasTargetPosition() && widget->hasInput() ) {
            widget->findPlacemarks();
            return;
        }
    }

    d->m_activeInput = 0;
    if ( d->m_routeSkeleton->size() > 1 ) {
        d->m_zoomRouteAfterDownload = true;
        d->m_routingLayer->setModel( d->m_routingManager->routingModel() );
        d->m_routingManager->retrieveRoute( d->m_routeSkeleton );
        d->m_ui.directionsListView->setModel( d->m_routingProxyModel );
        d->m_routingLayer->synchronizeWith( d->m_routingProxyModel,
                                            d->m_ui.directionsListView->selectionModel() );
    }
}*/

void TreeViewWidget::activateItem ( const QModelIndex &index )
{
    // Underlying model can be both a placemark model and a routing model
    // We rely on the same role index for coordinates
    Q_ASSERT( int( TreeViewModel::CoordinateRole ) == int( MarblePlacemarkModel::CoordinateRole ) );

	if ( !index.isValid() ) 
		return;

    QVariant data = index.data( TreeViewModel::CoordinateRole );

    if ( !data.isNull() ) {
        GeoDataCoordinates position = qvariant_cast<GeoDataCoordinates>( data );
        d->m_marbleWidget->centerOn( position, true );

		d->m_ui.treeTreeView->scrollTo(index);
    }

/*	MapViewLayer* pLayer=NULL;
	TreeNode* pParent = pNode->parent->parent;
	if(pParent->ItemType == Layer)
	{
		if(d->m_mvLayers.contains(pParent->m_lId))
			pLayer = d->m_mvLayers.value(pParent->m_lId);

	}*/
}

void TreeViewWidget::itemToDelete ( const QModelIndex &index, long intID )
{
	if ( !index.isValid() ) 
		return;

	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
    if ( pNode!=NULL)
	{
		long id = pNode->m_lId;
		long iType = 0;
		if(pNode->ItemType == Overlay)
		{
			deleteObjects(QStringList(),pNode->m_lId,MST_DELETE_OVERLAY);
			iType=MST_EVENT_OVERLAY_DELETE;
		}
		else if(pNode->ItemType == Layer)
		{
			deleteObjects(QStringList(),pNode->m_lId,MST_DELETE_LAYER);
			iType=MST_EVENT_LAYER_DELETE;
		}
		else if(pNode->ItemType == Group)
		{
			deleteObjects(QStringList(),pNode->m_lId,MST_DELETE_GROUP);
			iType=MST_EVENT_GROUP_DELETE;
		}
		else 
		{
			TreeNode* pGroup = pNode->parent;
			if(pGroup!=NULL)
			{
				id = pGroup->m_lId;
				deleteObjects(QStringList(),pGroup->m_lId,MST_DELETE_GROUP);
				iType=MST_EVENT_GROUP_DELETE;
			}
			else
			{
				deleteObjects(QStringList(),pNode->m_lId,MST_DELETE_SHAPE);
				iType=MST_EVENT_SHAPE_DELETE;
			}
		}

		if(intID)
		{
			emit signalSendEvent(iType,sizeof(long),&id,intID);

		}
	}

}
/*
void RoutingWidget::handleSearchResult( RoutingInputWidget *widget )
{
    d->setActiveInput( widget );
    MarblePlacemarkModel *model = widget->searchResultModel();

    if ( model->rowCount() ) {
        // Make sure we have a selection
        activatePlacemark( model->index( 0, 0 ) );
    }

    GeoDataLineString placemarks;
    for ( int i = 0; i < model->rowCount(); ++i ) {
        QVariant data = model->index( i, 0 ).data( MarblePlacemarkModel::CoordinateRole );
        if ( !data.isNull() ) {
            placemarks << qVariantValue<GeoDataCoordinates>( data );
        }
    }

    if ( placemarks.size() > 1 ) {
        d->m_widget->centerOn( GeoDataLatLonBox::fromLineString( placemarks ) );
        d->m_ui.descriptionLabel->setVisible( false );
    }
}

void RoutingWidget::centerOnInputWidget( RoutingInputWidget *widget )
{
    if ( widget->hasTargetPosition() ) {
        d->m_widget->centerOn( widget->targetPosition() );
    }
}*/

void TreeViewWidget::activatePlacemark( const QModelIndex &index,long intID,bool sel )
{
    if ( index.isValid() ) 
	{
        //QVariant data = index.data( TreeViewModel::CoordinateRole );
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL && pNode->parent!=NULL)//!data.isNull() ) 
		{
//            d->m_activeInput->setTargetPosition( qVariantValue<GeoDataCoordinates>( data) );
			long id = pNode->m_lId;
			if(id==0)return;
			if(pNode->parent->ItemType == Group)
				 id = pNode->parent->m_lId;

			if(sel)
			{
				d->m_ui.treeTreeView->setCurrentIndex( index );
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_SELECT,sizeof(long),&id,intID);
			}
			else 
			{
				d->m_ui.treeTreeView->setCurrentIndex( QModelIndex() );
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_DESELECT,sizeof(long),&id,intID);
			}
        }
    }
}

void TreeViewWidget::activateMultiPlacemark( const QModelIndex &index,long intID,bool sel )
{
    if ( index.isValid() ) 
	{
        //QVariant data = index.data( TreeViewModel::CoordinateRole );
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL && pNode->parent!=NULL)//!data.isNull() ) 
		{
//            d->m_activeInput->setTargetPosition( qVariantValue<GeoDataCoordinates>( data) );
			long id = pNode->m_lId;
			if(id==0)return;
			if(pNode->parent->ItemType == Group)
				 id = pNode->parent->m_lId;

			if(sel)
			{
		//		d->m_ui.treeTreeView->setCurrentIndex( index,QItemSelectionModel::NoUpdate );
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_SELECT,sizeof(long),&id,intID);
			}
			else 
			{
		//		d->m_ui.treeTreeView->setCurrentIndex( QModelIndex(),QItemSelectionModel::NoUpdate );
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_DESELECT,sizeof(long),&id,intID);
			}
        }
    }
}

void TreeViewWidget::activateMultiSelection( const QMap<int,QModelIndex> &idxMap,long intID,bool sel )
{
	int iCount = idxMap.size();
	if(iCount<1)return;

	struct {
			long count;
			void* pData;
	} data;
			
	long size = sizeof(long)*(iCount+1);
				 
	data.count = iCount;
	long* pValue = (long*) new long [iCount];
	if(pValue==NULL)return;
	data.pData = pValue;
					
	QModelIndex idx;
	QMap<int,QModelIndex>::const_iterator iter = idxMap.constBegin();
	for ( ; iter != idxMap.constEnd(); ++iter )
	{
		idx = iter.value();
		if ( idx.isValid() ) 
		{
			//QVariant data = index.data( TreeViewModel::CoordinateRole );
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
			if ( pNode!=NULL && pNode->parent!=NULL)//!data.isNull() ) 
			{
				*pValue++ = pNode->m_lId;
			}
		}

	}

	emit signalSendEvent(MST_EVENT_SELECTION,size,&data,intID);
	delete data.pData;
}

void TreeViewWidget::dblClickPlacemark( const QModelIndex &index,long intID,QPoint& pt,bool sel )
{
	// "Normal" DblClk opens properties, or OLE server...
	// MST_EVENT_GROUP_OPEN
    if ( index.isValid() && intID ) 
	{
        //QVariant data = index.data( TreeViewModel::CoordinateRole );
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
		if(pNode==NULL)return;

		if(pNode->parent->ItemType == Group)
		{
			struct {
							long groupptr;
							long screenx;
							long screeny;
						} data;
			data.groupptr = pNode->parent->m_lId;
			data.screenx  = pt.x();
			data.screeny  = pt.y();

			long size = 3 * sizeof(long);

			emit signalSendEvent( MST_EVENT_GROUP_OPEN,size,&data,intID);
		}
	}
}

void TreeViewWidget::activatePolygone( const QModelIndex &index,long intID,bool sel )
{
    if ( index.isValid() ) 
	{
     /*   QVariant data = index.data( TreeViewModel::TypeRole );
        if ( !data.isNull() ) 
		{
			GIS::TreeViewItemType type = (GIS::TreeViewItemType) qVariantValue<unsigned int>( index.data( TreeViewModel::TypeRole ) );
			if ( type == GIS::Poly ) 
			{
			}
        }*/
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL)//!data.isNull() ) 
		{
//            d->m_activeInput->setTargetPosition( qVariantValue<GeoDataCoordinates>( data) );
			long id = pNode->m_lId;
			if(pNode->parent->ItemType == Group)
				 id = pNode->parent->m_lId;

			if(sel)
			{
				d->m_ui.treeTreeView->setCurrentIndex( index );
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_SELECT,sizeof(long),&id,intID);
			}
			else 
			{
				d->m_ui.treeTreeView->setCurrentIndex( QModelIndex());
				if(intID)
					emit signalSendEvent(MST_EVENT_GROUP_DESELECT,sizeof(long),&id,intID);
			}
        }
    }
}

void TreeViewWidget::dblClickPolygone( const QModelIndex &index,long intID,QPoint& pt,bool sel )
{
    if ( index.isValid() && intID ) 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
		if(pNode==NULL)return;

		if(pNode->parent->ItemType == Group)
		{
			struct {
							long groupptr;
							long screenx;
							long screeny;
						} data;
			data.groupptr = pNode->parent->m_lId;
			data.screenx  = pt.x();
			data.screeny  = pt.y();

			long size = 3 * sizeof(long);

			emit signalSendEvent( MST_EVENT_GROUP_OPEN,size,&data,intID);
		}

	}
}

void  TreeViewWidget::retrieveMovedPlacemark( const QModelIndex &index,long intID )
{
	if ( index.isValid() && intID) 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL)
		{
			long id = pNode->m_lId;
			GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
			if(pNode->parent->ItemType == Group)
			{
				id = pNode->parent->m_lId;
				pos = qVariantValue<GeoDataCoordinates>( index.parent().data( TreeViewModel::CoordinateRole ) );
			}
			struct {
					long groupptr;
					double longitude;
					double latitude;
			} data; 

			data.groupptr = id;//pNode->m_lId;
			data.longitude = pos.longitude(GeoDataCoordinates::Degree);
			data.latitude = pos.latitude(GeoDataCoordinates::Degree);
			long size = sizeof(long) + 2*sizeof(double);

			emit signalSendEvent(MST_EVENT_GROUP_MOVE,size,&data,intID);

		}
	}


}

void  TreeViewWidget::retrieveMovedPolygone( const QModelIndex &index,long intID )
{
	if ( index.isValid() && intID) 
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL)
		{
			long id = pNode->m_lId;
			GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
			if(pNode->parent->ItemType == Group)
			{
				id = pNode->parent->m_lId;
				pos = qVariantValue<GeoDataCoordinates>( index.parent().data( TreeViewModel::CoordinateRole ) );
			}

			if(pNode->ItemType == Rect)
			{

				struct {
						long groupptr;
						double left;
						double top;
						double right;
						double bottom;
				} data; 

				data.groupptr = id;//pNode->m_lId;
				pos = pNode->m_poly.at(0);;
				data.left = pos.longitude(GeoDataCoordinates::Degree);
				data.top = pos.latitude(GeoDataCoordinates::Degree);
				pos = pNode->m_poly.at(1);;
				data.right = pos.longitude(GeoDataCoordinates::Degree);
				data.bottom = pos.latitude(GeoDataCoordinates::Degree);
				long size = sizeof(long) + 4*sizeof(double);

				emit signalSendEvent(MST_EVENT_GROUP_MOVE_EX,size,&data,intID);
			}//if rect
			else if(pNode->ItemType == Poly || pNode->ItemType == Polygone)
			{
				GeoDataLineString gdl = pNode->m_poly;
				int iCount = gdl.size();
				struct {
						long shapeptr;
						long count;
						void* pData;
				} data;
				// id = pNode->m_lId;
				 data.shapeptr = pNode->m_lId;
				 long size = sizeof(long)*2;
				 if(iCount < 2)
				 {
					data.count = 0;
					data.pData = NULL;

					emit signalSendEvent(MST_EVENT_POLY_CREATE,size,&data,intID);
					return;
				 }//if count < 2
				 else
				 {
					data.count = iCount;
					size = (sizeof(long)*2)+iCount*2*sizeof(double);
					double* pValue = (double*) new double [data.count*2];
					if(pValue==NULL)return;
					data.pData = pValue;
					for(int i=0;i<iCount;i++)
					{
						pos = gdl.at(i);
						*pValue++ = pos.longitude(GeoDataCoordinates::Degree);
						*pValue++ = pos.latitude(GeoDataCoordinates::Degree);

					}

					 emit signalSendEvent(MST_EVENT_POLY_CREATE,size,&data,intID);
					 delete data.pData;

				 }


			}//if poly

		}//if pNode
	}//if index


}

void TreeViewWidget::hoveredObject( const QModelIndex &index,QPoint& pos,long intID )
{
	if ( index.isValid() && intID==1) //this event is only used by MapView embedded
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL)
		{
			//GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
			//int xpos = pos.x();
			//int ypos = pos.y();
			/*
			qreal lon=0;
			qreal lat=0;
			d->m_marbleWidget->geoCoordinates(xpos,ypos,lon,lat);
			*/

			int iSection=0;					//-1 = omni bts flag	-2 = Icon
			if(pNode->ItemType == Icon)
				iSection=-2;
			else if(pNode->ItemType == SvgBts)
			{
				int iFixed = pNode->m_iDirection;
				if(pNode->MarkerType == GIS::Triangle)//BTS with antennas
				{
					//first send event for all other intersected objects
					if(pNode->parent!=NULL && pNode->parent->parent!=NULL)
						intersectionObjects(pNode->parent->parent->m_lId,pNode->m_lId,pos,intID);

					iSection=-5;
					bool ret = false;
					if(iFixed<0)//fixed size
						ret = detectBTSflag(index,pos,false,true,iSection);
					else
						ret = detectBTSflag(index,pos,false,false,iSection);

					if(!ret)return;
					
				}
				else if(pNode->MarkerType == GIS::Circle)//Omni BTS
				{
					//first send event for all other intersected objects
					intersectionObjects(pNode->parent->parent->m_lId,pNode->m_lId,pos,intID);

					iSection=-1;
					bool ret = false;
					if(iFixed<0)//fixed size
						ret = detectBTSflag(index,pos,true,true,iSection);
					else ret = detectBTSflag(index,pos,true,false,iSection);

					if(!ret)return;
				}
			}

	#pragma pack(push,1)
			struct data{
				long	shapeptr;
				double	section;
				double  xpos;
				double  ypos;
			};

			data mousHovered;
	#pragma pack(pop)

			mousHovered.shapeptr = pNode->m_lId;
			mousHovered.section = iSection;
			mousHovered.xpos = (double)pos.x();
			mousHovered.ypos = (double)pos.y();
			long size = sizeof(long) + 3*sizeof(double);
			emit signalSendEvent(MST_EVENT_MOUSE_OVER,size,&mousHovered,intID);

		}//pNode
	}//index valid
	else if (0)// index.isValid() && intID==0) //this event is only used by MapView embedded
	{
		QRect rect;
		rect.setRect(pos.x()-10,pos.y()-10,20,20);
		QPoint pt;
		int iCount=0;
		qreal xpos=0;
		qreal ypos=0;
		GeoDataCoordinates* cor=NULL;
		qint64 objID=0;
		QHash<qint64,GeoDataCoordinates*>::const_iterator objiter = m_objectCache.constBegin();
		for ( ; objiter != m_objectCache.constEnd(); ++objiter ) 
		{
			cor = objiter.value();
			objID = objiter.key();
			if(cor!=NULL)
			{
				qreal lon = cor->longitude(GeoDataCoordinates::Degree);
				qreal lat = cor->latitude(GeoDataCoordinates::Degree);
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objID);
				d->m_marbleWidget->screenCoordinates(lon,lat,xpos,ypos);
				pt.setX((int)xpos);
				pt.setY((int)ypos);
				if(rect.contains(pt))
				{
					iCount++;
				}
			}
		}
	}
}

void TreeViewWidget::intersectionObjects(long layerID, long objectID,QPoint& pt,long intID )
{
	if(d->m_mvLayers.contains((qint64)layerID) )
	{
		MapViewLayer* pLayer = d->m_mvLayers.value((qint64)layerID);
		if(pLayer!=NULL)
		{
			QModelIndex idx;
			//detect all object at position pt
			int count = pLayer->intersectionObjects(idx,pt,true,0);
			if(count<2)return;

			int iFixed   = 0;
			int iSection = 0;
			bool ret = false;

			for(int i=0;i<count;i++)
			{
				iFixed   = 0;
				iSection = 0;
				ret = false;

				pLayer->intersectionObjects(idx,pt,false,i);
				if(idx.isValid())
				{
					TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(idx);
					if( pNode!=NULL && pNode->ItemType==SvgBts && pNode->m_lId!=objectID)
					{
						if(pNode->MarkerType == GIS::Triangle)
						{
							iFixed = pNode->m_iDirection;
							iSection=-5;
							ret = false;
							if(iFixed<0)//fixed size
								ret = detectBTSflag(idx,pt,false,true,iSection);
							else
								ret = detectBTSflag(idx,pt,false,false,iSection);

						}//antenna BTS
						else if(pNode->MarkerType == GIS::Circle)
						{
							iFixed = pNode->m_iDirection;
							iSection=-1;
							ret = false;
							if(iFixed<0)//fixed size
								ret = detectBTSflag(idx,pt,true,true,iSection);
							else ret = detectBTSflag(idx,pt,true,false,iSection);

						}//OMNI BTS

						if(ret)
						{
					#pragma pack(push,1)
							struct data{
								long	shapeptr;
								double	section;
								double  xpos;
								double  ypos;
							};

							data mouseHovered;
					#pragma pack(pop)

							mouseHovered.shapeptr = pNode->m_lId;
							mouseHovered.section = iSection;
							mouseHovered.xpos = (double)pt.x();
							mouseHovered.ypos = (double)pt.y();
							long size = sizeof(long) + 3*sizeof(double);
							emit signalSendEvent(MST_EVENT_MOUSE_OVER,size,&mouseHovered,intID);
						}

					}//pNode

				}//index valid

			}//for

		}//pLayer
	}//contains
}

bool TreeViewWidget::detectBTSflag( const QModelIndex &index,QPoint& mousePT,bool omni,bool fixed,int& iVal)
{
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
    if ( pNode==NULL)return false;

	int iBH = pNode->m_iXoff; //BTS high
	if(omni)
	{
		QPixmap qPix = d->m_treeViewModel->pixmap(pNode->m_qsImage);
		if(!qPix.isNull() )
		{
			QSize imgSize = qPix.size();
			iBH = iBH - ( ((double)iBH/100.0)*10);
		}
	}
	else
	{
		if(fixed)
		{
			iBH = iBH - ( ((double)iBH/100.0)*10); //iBH=pixel 
		}
		else
		{
			qreal distance = d->m_marbleWidget->distance()*1000;//meter
			int height = d->m_marbleWidget->viewport()->height();
			int width = d->m_marbleWidget->viewport()->width(); 
			distance = distance/(qreal)width;					//meter per pixel
			distance = (qreal)iBH/distance;						//pixel for BTS high
			iBH = (int)distance;
		}

		GeoDataCoordinates cor = pNode->m_position;
		qreal xPos=0;
		qreal yPos=0;
		d->m_marbleWidget->screenCoordinates(cor.longitude(GeoDataCoordinates::Degree),cor.latitude(GeoDataCoordinates::Degree),xPos,yPos);

		QRect rect;
		rect.setLeft((int)xPos - iBH);
		rect.setRight((int)xPos + iBH);
		rect.setTop((int)yPos - iBH);
		rect.setBottom((int)yPos + iBH);
		if(!rect.contains(mousePT))
			return false;

		double a = (double)abs((mousePT.x()-(int)xPos));
		double b = (double)abs(((int)yPos - mousePT.y()));
		double h = a*a + b*b;
		double c = sqrt(h);
		double alpha = b/c;//sin
		double sin2=0;
		//alpha = (alpha * 180)/3.1415926535;
		sin2 = asin(alpha);//Rad
		sin2 = (sin2 * 180)/3.1415926535;//Grad
		if(mousePT.x() >= (int)xPos && mousePT.y() < (int)yPos)
			alpha = 90 - sin2;
		if(mousePT.x() >= (int)xPos && mousePT.y() > (int)yPos)
			alpha = 90 + sin2;
		if(mousePT.x() < (int)xPos && mousePT.y() >= (int)yPos)
			alpha = 270 - sin2;
		if(mousePT.x() < (int)xPos && mousePT.y() <= yPos)
			alpha = 270 + sin2;

		QString filename = pNode->m_qsDesc;
		QString tmp;
		QString tmp2;
		int iID=0;
		//BTS name
		tmp = filename.section(';',iID,iID);
		iID++;
		//BTS size
		tmp = filename.section(';',iID,iID);
		iID++;
		int iSize = tmp.toInt();
		//Antenna count
		tmp = filename.section(';',iID,iID);
		iID++;
		int iCount = tmp.toInt();

		int* iAangle = (int*) calloc(iCount,sizeof(int));

		//get the size string
		tmp = filename.section(';',iID,iID);
		iID++;

		//get the angle string
		tmp = filename.section(';',iID,iID);
		iID++;

		iBH = -1;
		int ix=0;
		int iNeg=0;
		int iPos=0;
		for(ix=0;ix<iCount;ix++)
		{
			//get the size string
			tmp2 = tmp.section(',',ix,ix);
			iAangle[ix] = tmp2.toInt();
			iNeg = iAangle[ix]-20;
			iPos = iAangle[ix]+20;
			if(iNeg<=0)
			{
				iNeg=20-iAangle[ix];
				iNeg = 360 - iNeg;
				iPos = iAangle[ix]+20;
				if(alpha<20)
				{
					if( ((360-alpha) >= iNeg) && (alpha <= iPos) )
					{
						iVal = ix;//Bts antenna flag
						ix=iCount;
					}
				}
				else
				{
					if( (alpha >= iNeg) && (360-alpha <= iPos) )
					{
						iVal = ix;//Bts antenna flag
						ix=iCount;
					}
				}

			}
			else if(iPos>=360)
			{
				iNeg = iAangle[ix]-20;
				iPos = (20+iAangle[ix])-360;
				if( (alpha >= iNeg) && ( (360-alpha) <= iPos) )
				{
					iVal = ix;//Bts antenna flag
					ix=iCount;
				}

			}
			else if(alpha > iAangle[ix]-20 && alpha < iAangle[ix]+20)
			{
				iVal = ix;//Bts antenna flag
				ix=iCount;
			}
		}//for

		if(iAangle!=NULL)
			free(iAangle);

		if(iVal>=0)return true;
	}

	return true;
}

void TreeViewWidget::notifyMouseAction(QPoint& pos,int flag,long intID)
{
	//left button - flag=1 //left button dbl - flag=2 //right button - flag=3 //mid button - flag=4
	qreal lon=0;
	qreal lat=0;

    bool isOnGlobe = d->m_marbleWidget->geoCoordinates( pos.x(),pos.y(),lon, lat, GeoDataCoordinates::Degree);
	if(isOnGlobe)
	{
//		#pragma pack(push,1)
			struct {
				long	shapeptr;
				int		type;
				double	lon;
				double	lat;
				int		xpos;
				int		ypos;
			}data;

	//		data mouseAction;
//	#pragma pack(pop)

			data.shapeptr = (long)0;
			data.type = flag;
			data.lon  = (double)lon;
			data.lat  = (double)lat;
			data.xpos = pos.x();
			data.ypos = pos.y();
			long size = (sizeof(long) + 2*sizeof(double)+ 3*sizeof(int));

			emit signalSendEvent(MST_EVENT_MOUSE_ACT,size,&data,intID);
	}

}

void TreeViewWidget::notifyImageMouseAction(QPoint& pos,QPoint& posImg,int flag,long intID)
{
	//left button - flag=1 //left button dbl - flag=2 //right button - flag=3 //mid button - flag=4
	qreal lon=0;
	qreal lat=0;

    bool isOnGlobe = d->m_marbleWidget->geoCoordinates( pos.x(),pos.y(),lon, lat, GeoDataCoordinates::Degree);
	if(isOnGlobe)
	{
//		#pragma pack(push,1)
			struct {
				long	shapeptr;
				int		type;
				double	lon;
				double	lat;
				int		xpos;
				int		ypos;
			}data;

	//		data mouseAction;
//	#pragma pack(pop)

			data.shapeptr = (long)0;
			data.type = flag;
			data.lon  = (double)lon;
			data.lat  = (double)lat;
			data.xpos = posImg.x();
			data.ypos = posImg.y();
			long size = (sizeof(long) + 2*sizeof(double)+ 3*sizeof(int));

			emit signalSendEvent(MST_EVENT_MOUSE_ACT,size,&data,intID);
	}

}

void TreeViewWidget::setLocations( QVector<GeoDataPlacemark*> locations )
{
	if( locations.isEmpty() ) {
        return;
    }

	GeoDataPlacemark *placemark = locations.at(0);
	if(placemark==NULL)
		return;
	qreal lon=0;
	qreal lat=0;
	qreal alt=0;
	placemark->coordinate(lon,lat,alt);
	GeoDataCoordinates cor;
	cor.setLongitude(lon);
	cor.setLatitude(lat);

	//d->m_marbleWidget->centerOn(lon,lat,alt);
	d->m_marbleWidget->centerOn(cor);

}

void TreeViewWidget::retrieveContextMenu( const QModelIndex &index,long intID, int& iVal )
{
    if ( index.isValid() ) 
	{
		d->m_ui.treeTreeView->setCurrentIndex( index );
     
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndex(index);
        if ( pNode!=NULL && pNode->parent!=NULL)//!data.isNull() ) 
		{
			GeoDataCoordinates pos;
			long idGroup = pNode->m_lId;
			long idLayer = 0;
			if(pNode->parent->ItemType == Group)
			{
				 idGroup = pNode->parent->m_lId;
				 idLayer = pNode->parent->parent->m_lId;
				 pos = pNode->parent->m_position;
			}
			else 
			{
				idLayer = pNode->parent->m_lId;
				pos = pNode->m_position;
			}
#pragma pack(push,1)
			typedef struct tag_LCData{
				long groupptr;
				long layerptr;
				int  screen_x;
				int  screen_y;
				int  menu_id;
			} t_LCData;

			typedef struct tag_LCDataHeader {
				short nLCCount; 		// no of df stations (1..10)
				t_LCData sLCData[1]; 	// placeholder for 1..10 DFData structures 
			} t_LCDataHeader;
#pragma pack(pop) 
			int nLCcount=1;
			short size=(nLCcount)*sizeof(t_LCData);
			size+= sizeof(t_LCDataHeader);
			t_LCDataHeader* pLCH=(t_LCDataHeader*)new char[size];
			pLCH->nLCCount=nLCcount;

			for (int i=0;i<pLCH->nLCCount;i++) 
			{
				t_LCData* pD=&pLCH->sLCData[i];

				pD->groupptr = idGroup;
				pD->layerptr = idLayer;
				pD->menu_id  = iVal;

				qreal xpos=0;
				qreal ypos=0;
				d->m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),xpos,ypos);

				pD->screen_x=(int)xpos;
				pD->screen_y=(int)ypos;
			}

			emit signalSendEvent(MST_EVENT_MENU_ID,size,pLCH,intID);

			//free memory
			delete [] pLCH;
        }
    }
}

void TreeViewWidget::addDropItem(QPixmap& qPix, QPoint& qPoint, QString& name)
{
	TreeNode* pOverlayNode=NULL;
	long objectID=0;
	long itemID=0;
	QStringList qsl;
	qsl  << "399.0" << "399.0" << "Situation";
	int iType = MST_ADD_OVERLAY;

	if(m_sitOverlayID==0)
	{
		slotAddObject(qsl,0,iType,objectID);
		if(objectID==0)return;

		m_sitOverlayID=(qint64)objectID;
		itemID = objectID;
		objectID=0;
		qsl.clear();

		qsl  << "399.0" << "399.0" << "Layer_0";
		iType = MST_ADD_LAYER;
		slotAddObject(qsl,itemID,iType,objectID);
		if(objectID==0)return;
		qsl.clear();

		pOverlayNode = d->m_treeViewModel->getNodeFromIndexId(m_sitOverlayID);
	}
	else
	{
		pOverlayNode = d->m_treeViewModel->getNodeFromIndexId(m_sitOverlayID);
		if(pOverlayNode==NULL)
		{
			m_sitOverlayID=0;
			return;
		}
		if(pOverlayNode->children.count()>0)
		{
			TreeNode* pLayer=NULL;
			objectID = 0;
			for(int i=0;i<pOverlayNode->children.count();i++)
			{
				pLayer=pOverlayNode->children.at(i);
				if(pLayer!=NULL && pLayer->m_lId==(long)m_selectedID)
				{
					objectID=(long)m_selectedID;
					i=pOverlayNode->children.count();
				}
			}
			if(objectID==0)
				objectID = pOverlayNode->children.at(0)->m_lId;
			
		}
		else
		{
			qsl  << "399.0" << "399.0" << "Layer_0";
			iType = MST_ADD_LAYER;
			slotAddObject(qsl,m_sitOverlayID,iType,objectID);
			if(objectID==0)return;
		}
		qsl.clear();
	}

	qreal lon=0, lat=0;
	bool isOnGlobe = d->m_marbleWidget->geoCoordinates(qPoint.x(),qPoint.y(),
		lon, lat,GeoDataCoordinates::Degree );

	if(!isOnGlobe)return;

	QString qsLon("%1");
	qsLon = QString("%1").arg(lon);
	QString qsLat("%1");
	qsLat = QString("%1").arg(lat);
	QString qsGroup("Group_%1");
	qsGroup = QString("Group_%1").arg(pOverlayNode->children.at(0)->children.count());

	qsl << qsLon << qsLat << qsGroup;
	iType = MST_ADD_GROUP;
	slotAddObject(qsl,objectID,iType,objectID);

	qint64 qID = (qint64)objectID;
	TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
	if(pNode!=NULL)
	{
		//enable region calculation for layer objects
		MapViewLayer* pLayer = d->m_mvLayers.value(pNode->parent->m_lId);
		if(pLayer!=NULL)
			pLayer->setLayerUpdate(true);
		//GeoDataCoordinates position;// = pNode->m_position;//.at(0);
		//position.set(pNode->m_position.longitude(),pNode->m_position.latitude,0,GeoDataCoordinates::Degree);
		//QStringList qsl;
		//qsl << position.lonToString() << position.latToString();
		qreal lon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
		qreal lat = pNode->m_position.latitude(GeoDataCoordinates::Degree);

		objectID = (long)d->m_treeViewModel->AddIcon(name,objectID,lon,lat,0,0,0,0);

/*		GeoDataCoordinates* cor = new GeoDataCoordinates();
		cor->set(lon,lat,0,GeoDataCoordinates::Degree);
		m_objectCache.insert((qint64)objectID,cor);*/

		d->m_marbleWidget->update();
	/*	if(m_pUpdateTimer!=NULL && !m_pUpdateTimer->isActive())
		{
			//m_pUpdateTimer->stop();
			m_pUpdateTimer->start();
		}*/
	}

}

void TreeViewWidget::slotSituationOverlay(long ovlID)
{
	m_sitOverlayID=ovlID;
}

void TreeViewWidget::slotAddTsSymbol(long id,long symID,long linecol,long fillcol,long width,long& objectID)
{
	long ret = objectID;

	//need new group object ?
	if(id!=m_tsGroupID)m_tsGroupID=0;

	QString qsDir = MarbleDirs::path("Symbols");
	qsDir += "/MOG/";
	QString qsSymbol = "%1.kml";
	qsSymbol = qsSymbol.arg(symID);
	qsDir+=qsSymbol;
	
	d->m_marbleWidget->model()->addGeoDataFileFromSymbol(qsDir);
}

/*
void RoutingWidget::addInputWidget()
{
    int index = d->m_ui.routingLayout->count() - 4;
    d->m_routeSkeleton->append( GeoDataCoordinates() );
    insertInputWidget( index );
}

void RoutingWidget::insertInputWidget( int index )
{
    if ( index >= 0 && index <= d->m_inputWidgets.size() ) {
        RoutingInputWidget *input = new RoutingInputWidget( d->m_routeSkeleton, index, this );
        input->setWorkOffline( d->m_workOffline );
        d->m_inputWidgets.insert( index, input );
        connect( input, SIGNAL( searchFinished( RoutingInputWidget* ) ),
                 this, SLOT( handleSearchResult( RoutingInputWidget* ) ) );
        connect( input, SIGNAL( removalRequest( RoutingInputWidget* ) ),
                 this, SLOT( removeInputWidget( RoutingInputWidget* ) ) );
        connect( input, SIGNAL( activityRequest( RoutingInputWidget* ) ),
                 this, SLOT( centerOnInputWidget( RoutingInputWidget* ) ) );
        connect( input, SIGNAL( mapInputModeEnabled( RoutingInputWidget*, bool ) ),
                 this, SLOT( requestMapPosition( RoutingInputWidget*, bool ) ) );
        connect( input, SIGNAL( targetValidityChanged( bool ) ),
                 this, SLOT( adjustSearchButton() ) );

        d->m_ui.routingLayout->insertWidget( index, input );
        d->adjustInputWidgets();
    }
}

void RoutingWidget::removeInputWidget( RoutingInputWidget *widget )
{
    int index = d->m_inputWidgets.indexOf( widget );
    if ( index >= 0 ) {
        if ( d->m_inputWidgets.size() < 3 ) {
            widget->clear();
        } else {
            d->m_routeSkeleton->remove( index );
        }
        d->m_routingManager->updateRoute();
    }
}

void RoutingWidget::removeInputWidget( int index )
{
    if ( index >= 0 && index < d->m_inputWidgets.size() ) {
        RoutingInputWidget *widget = d->m_inputWidgets.at( index );
        d->m_inputWidgets.remove( index );
        d->m_ui.routingLayout->removeWidget( widget );
        widget->deleteLater();
        if ( widget == d->m_activeInput ) {
            d->m_activeInput = 0;
            d->m_routingLayer->setModel( d->m_routingManager->routingModel() );
        }
        d->adjustInputWidgets();
    }

    if ( d->m_inputWidgets.size() < 2 ) {
        addInputWidget();
    }
}*/
/*
void RoutingWidget::updateRouteState( RoutingManager::State state, RouteSkeleton *route )
{
    Q_UNUSED( route );

    if ( state == RoutingManager::Retrieved ) {
        // Parts of the route may lie outside the route trip points
        GeoDataLineString bbox;
        for ( int i = 0; i < d->m_routingManager->routingModel()->rowCount(); ++i ) {
            QModelIndex index = d->m_routingManager->routingModel()->index( i, 0 );
            QVariant pos = index.data( RoutingModel::CoordinateRole );
            if ( !pos.isNull() ) {
                bbox << qVariantValue<GeoDataCoordinates>( pos );
            }
        }

        if ( bbox.size() > 1 ) {
            qreal distance = d->m_routingManager->routingModel()->totalDistance();
            unsigned int days = d->m_routingManager->routingModel()->duration().days;
            QTime time = d->m_routingManager->routingModel()->duration().time;
            QString timeString = time.toString( Qt::DefaultLocaleShortDate );
            if ( days ) {
                QString label = tr( "Estimated travel time: %1 days, %2 (%3 km)", 0, days );
                d->m_ui.descriptionLabel->setText( label.arg( days ).arg( timeString ).arg( distance, 0, 'f', 1 ) );
            } else {
                QString label = tr( "Estimated travel time: %1 (%2 km)" );
                d->m_ui.descriptionLabel->setText( label.arg( timeString ).arg( distance, 0, 'f', 1 ) );
            }
            d->m_ui.descriptionLabel->setVisible( true );

            if ( d->m_zoomRouteAfterDownload ) {
                d->m_zoomRouteAfterDownload = false;
                d->m_widget->centerOn( GeoDataLatLonBox::fromLineString( bbox ) );
            }
        }
    }

    d->m_routingLayer->setRouteDirty( state == RoutingManager::Downloading );

    if ( state == RoutingManager::Downloading ) {
        d->m_progressTimer.start();
    } else {
        d->m_progressTimer.stop();
        d->m_ui.searchButton->setIcon( QIcon() );
    }
}*/
/*
void RoutingWidget::requestMapPosition( RoutingInputWidget *widget, bool enabled )
{
    pointSelectionCanceled();

    if ( enabled ) {
        d->m_inputRequest = widget;
        d->m_routingLayer->setPointSelectionEnabled( true );
        d->m_widget->setFocus( Qt::OtherFocusReason );
    } else {
        d->m_routingLayer->setPointSelectionEnabled( false );
    }
}*/

void TreeViewWidget::retrieveSelectedPoint( const GeoDataCoordinates &coordinates )
{
/*    if ( d->m_inputRequest && d->m_inputWidgets.contains( d->m_inputRequest ) ) {
        d->m_inputRequest->setTargetPosition( coordinates );
        d->m_inputRequest = 0;
        d->m_widget->update();
    }*/

    d->m_pMvLayer->setPointSelectionEnabled( false );
}
/*
void RoutingWidget::adjustSearchButton()
{
    d->adjustSearchButton();
}*/

void TreeViewWidget::pointSelectionCanceled()
{
  /*  if ( d->m_inputRequest ) {
        d->m_inputRequest->abortMapInputRequest();
    }*/
}
/*
void RoutingWidget::toggleOptionsVisibility()
{
    bool visible = !d->m_ui.routePreferenceComboBox->isVisible();
    d->m_ui.routePreferenceComboBox->setVisible( visible );
    d->m_ui.highwaysCheckBox->setVisible( visible );
    d->m_ui.tollWaysCheckBox->setVisible( visible );
    d->m_ui.preferenceLabel->setVisible( visible );
    d->m_ui.avoidLabel->setVisible( visible );
}*/
/*
void RoutingWidget::exportRoute()
{
    QString fileName = QFileDialog::getSaveFileName( this, tr( "Export Route" ), // krazy:exclude=qclasses
                       QDir::homePath(),
                       tr( "GPX files (*.gpx)" ) );

    if ( !fileName.isEmpty() ) {
        QFile gpx( fileName );
        if ( gpx.open( QFile::WriteOnly) ) {
            d->m_routingManager->routingModel()->exportGpx( &gpx );
            gpx.close();
        }
    }
}*/

void TreeViewWidget::setWorkOffline( bool offline )
{
  /*  foreach ( RoutingInputWidget *widget, d->m_inputWidgets ) {
        widget->setWorkOffline( offline );
    }

    d->m_workOffline = offline;
    d->m_routingManager->setWorkOffline( offline );*/

}
/*
void RoutingWidget::updateProgress()
{
    d->m_progress.jumpToNextFrame();
    QPixmap frame = d->m_progress.currentPixmap();
    d->m_ui.searchButton->setIcon( frame );
}*/

void TreeViewWidget::slotUpdateFeature(long id,qreal left,qreal top,qreal right,qreal bottom)
{
	MapViewMapLayer* pLayer = d->m_mvMapLayers.value(id);
	if(pLayer!=NULL)
		pLayer->updateFeature(left,top,right,bottom);
}

void TreeViewWidget::setDimmerShading(int value)
{
	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
        if ( pLayer != NULL ) 
		{
			pLayer->setDimmerValue(value);
		}
	}
	if(d->m_pMvDimLayer!=NULL)
		d->m_pMvDimLayer->setDimmerValue(value);

	m_iDimmer=value;
}

void TreeViewWidget::coordinateFormat(qreal lon,qreal lat)
{
	QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 

	long id=0;
	bool bObj=false;

	TreeNode* pNode = NULL;
	if(idx.isValid())
		pNode = d->m_treeViewModel->getNodeFromIndex(idx);
	if(pNode!=NULL)
	{
		if(pNode->ItemType != Overlay && pNode->ItemType != Layer && !pNode->m_bMap)
		{
			qreal posLon = pNode->m_position.longitude(GeoDataCoordinates::Degree);
			qreal posLat = pNode->m_position.latitude(GeoDataCoordinates::Degree);
			int iPLon = (int)(posLon*10000);
			int iPLat = (int)(posLat*10000);
			int iLon  = (int)(lon*10000);
			int iLat  = (int)(lat*10000);

		//	QRect rect(iPLon-4,iPLat-4,8,8);
		//	QPoint(iLon,iLat);

			ViewportParams* vp = d->m_marbleWidget->viewport();
			qreal dist = d->m_marbleWidget->distance();

			int iH = vp->height();
			int iW = vp->width();
			qreal posX=0;
			qreal posY=0;
			d->m_marbleWidget->screenCoordinates(posLon,posLat,posX,posY);
			qreal rX=0;
			qreal rY=0;
			d->m_marbleWidget->screenCoordinates(lon,lat,rX,rY);
			QRectF rect(posX-5,posY-5,10,10);
			
			if(rect.left()<rX && rect.right()>rX && rect.top()<rY && rect.bottom()>rY)
			{
				id = pNode->m_lId;
				lon = posLon;
				lat = posLat;
			}
		}
	}
	
	emit signalCoordinateFormat(lon,lat,id);

}

void TreeViewWidget::gpxPlacemark(long id,qreal lon,qreal lat,qreal azimuth, QString& name)
{
	GeoDataCoordinates coord(lon,lat,0,GeoDataCoordinates::Degree);
	qreal longitude = coord.longitude(GeoDataCoordinates::Degree);
	qreal latitude  = coord.latitude(GeoDataCoordinates::Degree);

	emit signalGpxPlacemark(id,m_vecOverlayID,m_vecLayerID,longitude,latitude,azimuth,name);

}

void TreeViewWidget::addGeoDataDocument(GeoDataDocument* doc)
{
	//d->m_marbleWidget->model()->treeModel()->addDocument(doc );
	//return;
	GeoDataContainer *container =  doc;
	if(container==NULL)return;
	
	QString ad;
	m_vecOverlayID=0;
	m_vecLayerID=0;
	m_bMOG=false;

	long ret=0;
	int iExt=0;
	if ( !doc->fileName().isEmpty() )
    {
        QFileInfo file( doc->fileName() );
		QString ext = file.suffix();
		if(ext!="kml" && ext!="kmz")
		{
			ext = ext.toLower();
			if(ext=="tab" )
				iExt=3;
			else if(ext=="shp")
				iExt=4;
			else if(ext=="gpx")
				iExt=1;
			else iExt=8;
		}
		else if(ext=="kml")//not possible for kmz
		{
			//is this a MOG kml ?
			const char* chType=NULL;
			const char* chFolder = GeoDataTypes::GeoDataFolderType;
			QString qsFolder(chFolder);
			QVector<GeoDataFeature*>::Iterator i = container->begin();
			if(i != container->end())
			{
				chType = (*i)->nodeType();
				ad = (*i)->address();
				QString desc = (*i)->description();
				QString name = (*i)->name();
	
				QString qsType(chType);

				if ( qsType == qsFolder)
				{
					if(desc=="RSMOG")
						iExt=2;
			
					if(desc=="IndoorMap")
						iExt=5;

					if(desc=="GeoRefMap")
						iExt=5;
			
					if(desc=="MapInfoGeoSet")
						iExt=6;
				}
			}
		}
    }
	
	switch(iExt)
	{
		case 0:
				ret = createKmlPlacemarks(container);
				break;
		case 1:
				ret = createGpxPlacemarks(container);
				break;
		case 2:
				m_bMOG=true;
				m_tsOverlayID=0;
				m_tsLayerID=0;
				m_tsGroupID=0;
				ret = createKmlTsPlacemarks(container);
				break;
		case 3:
				ret = addMapLayerFromDocument(doc,iExt);
				break;
		case 4:
				ret = addMapLayerFromDocument(doc,iExt);
				break;
		case 5:
				ret = addMapLayerFromDocument(doc,iExt);
				break;
		case 6:
				ret = addMapLayerFromDocument(doc,iExt,ad);
				break;
		case 8:
				ret = addMapLayerFromDocument(doc,iExt,ad);
				break;
		default: 
				ret = createKmlPlacemarks(container);
				break;
	}

	/*if(iExt==100)//GrondOverlay
	{
		d->m_marbleWidget->model()->treeModel()->addDocument( doc );
	}*/
	if(ret>0)
		emit signalSendEvent(MST_EVENT_LAYER_ADD,sizeof(long),&ret,0);
}

long TreeViewWidget::addMapLayerFromDocument(GeoDataDocument* doc,int type, QString& add)
{
	GeoDataContainer *container =  doc;
	if(container==NULL)return 0;

	GeoDataDocument* pDoc=NULL;
	QStringList qsl;
	QString qsMap = doc->name();
	long objectID=0;

/*	if(m_minfOverlayID==0)
	{
		pDoc = new GeoDataDocument();
		pDoc->setName("MapInfoMap");
		DocumentRole role = MapDocument;
		pDoc->setDocumentRole( role );
		slotAddMapObject(MST_ADD_OVERLAY,pDoc,objectID);
		d->m_marbleWidget->model()->treeModel()->addDocument(pDoc);
	}
	if(m_minfOverlayID!=0)
	{
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_minfOverlayID);
		if(pNode!=NULL && pNode->m_pMapDoc!=NULL)
		{
			pDoc = pNode->m_pMapDoc;
			objectID=0;
			slotAddMapObject(MST_ADD_LAYER,doc,objectID);
			//d->m_marbleWidget->model()->treeModel()->addDocument((GeoDataDocument*)container);
			//pDoc->append(doc);
			d->m_marbleWidget->model()->treeModel()->addFeature(pDoc,container);
		}
	}*/

	//create the overlay
	if(type==3 && m_minfOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "MapInfoMap";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_minfOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_minfOverlayID);
		if(pNode!=NULL)
		{
			//pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}
	if(type==4 && m_esriOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "EsriMap";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_esriOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_esriOverlayID);
		{
		//	pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}
	if(type==5 && m_mapOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "MapOverlay";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_mapOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_mapOverlayID);
		{
	//		pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}
	if(type==8 && m_mapOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "MapOverlay";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_mapOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_mapOverlayID);
		{
	//		pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}
	if(type==7 && m_vecOverlayID==0)//m_mapOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "MapOverlay";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_vecOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
		{
	//		pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}

	if(type==6 && m_minfOverlayID==0)
	{
		qsl << "399.0" << "399.0" << "MapOverlay";
		slotAddObject(qsl,0,MST_ADD_OVERLAY,objectID);
		m_mapOverlayID=objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_mapOverlayID);
		{
	//		pNode->m_qsDesc=doc->fileName();
			pNode->m_bMap=true;
		}
		qsl.clear();
	}


	//create the layers
	if(type==3 && m_minfOverlayID!=0)
	{
		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_minfOverlayID);
		qint64 qID=(qint64)objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode!=NULL)
		{
			pNode->m_qsDesc=doc->fileName();
			pNode->parent->m_qsDesc=doc->fileName();
			pNode->m_bMap=false;
		}
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;
		connect( pMapLayer, SIGNAL( signalNotifyImageMouseAction(QPoint&,QPoint&,int,long) ),this, SLOT( notifyImageMouseAction(QPoint&,QPoint&,int,long) ) );

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		QString filename = doc->fileName();
		pMapLayer->setFileName(filename);

		pMapLayer->addDocument(doc);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);
		GeoDataLatLonBox box;
		qreal lon=0;
		qreal lat=0;
		if(pMapLayer->getLatLonBox(box,lon,lat))
		{
			d->m_marbleWidget->centerOn(box);
		}
		else if(lon!=lat)
		{
			GeoDataCoordinates targetPosition(lon,lat);
			d->m_marbleWidget->centerOn(targetPosition);
		}
	}
	if(type==6 && m_minfOverlayID!=0)
	{
		qint64 qID=0;
		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_minfOverlayID);
		TreeNode* pNode = NULL;
		if(objectID>0)
		{
			qID=(qint64)objectID;
			pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
			pNode->m_bGeoSet=true;	
			pNode->m_bMap=false;
			pNode->m_qsDesc=doc->fileName();
			pNode->parent->m_qsDesc=doc->fileName();
		}
		emit signalMapLayerItem(MST_ADD_LAYER,m_minfOverlayID,objectID,qsMap);
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;
		connect( pMapLayer, SIGNAL( signalNotifyImageMouseAction(QPoint&,QPoint&,int,long) ),this, SLOT( notifyImageMouseAction(QPoint&,QPoint&,int,long) ) );

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		QString filename = doc->fileName();
		pMapLayer->setFileName(filename);
		//save the geoset filename - ROMES
		m_qsGeoset = filename.left(filename.length()-3);
		m_qsGeoset += "gst";

		pMapLayer->addDocument(doc);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);

		//the geoset layer names
		//emit signalAddItem(MST_CREATE_POLY,pOverlay->m_lId,pLayer->m_lId,pNode->m_lId,objectID);
		if(!add.isEmpty())
		{
			QString qsGroup;
			GeoDataLatLonBox box;
			qreal lon=0;
			qreal lat=0;
			pMapLayer->getLatLonBox(box,lon,lat);
			GeoDataCoordinates pos = box.center();
			QString qsLon("%1");
			qsLon = QString("%1").arg(pos.longitude(GeoDataCoordinates::Degree));
			QString qsLat("%1");
			qsLat = QString("%1").arg(pos.latitude(GeoDataCoordinates::Degree));

			int count = add.count(';');
			for(int i=0;i<count;i++)
			{
				qsGroup = add.section(';',i,i);
				qsl.clear();
				qsl << qsLon << qsLat << qsGroup;

				long oID = (long)d->m_treeViewModel->AddLayer(qsl,m_minfOverlayID);
				//long groupID = (long)d->m_treeViewModel->AddLayer(qsl,objectID);
				pNode = NULL;
				if(oID>0)
				{
					qID=(qint64)oID;
					pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
					pNode->m_bGeoSet=true;	
					pNode->m_bMap=false;
					emit signalMapLayerItem(MST_ADD_LAYER,m_minfOverlayID,oID,qsGroup);
				}

			}

		}
	}
	if(type==4 && m_esriOverlayID!=0)
	{
		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_esriOverlayID);
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);
		qint64 qID=(qint64)objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode!=NULL)
		{
			pNode->m_qsDesc=doc->fileName();
			pNode->parent->m_qsDesc=doc->fileName();
			pNode->m_bMap=false;
		}

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		QString filename = doc->fileName();
		pMapLayer->setFileName(filename);

		pMapLayer->addDocument(doc);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);

		GeoDataLatLonBox box;
		qreal lon=0;
		qreal lat=0;
		if(pMapLayer->getLatLonBox(box,lon,lat))
		{
			d->m_marbleWidget->centerOn(box);
		}
		else if(lon!=lat)
		{
			GeoDataCoordinates targetPosition(lon,lat);
			d->m_marbleWidget->centerOn(targetPosition);
		}
	}
	if( (type==5||type==6) && m_mapOverlayID!=0)
	{
		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_mapOverlayID);
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);
		qint64 qID=(qint64)objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode!=NULL)
		{
			pNode->m_qsDesc=doc->fileName();
			pNode->parent->m_qsDesc=doc->fileName();
			pNode->m_bMap=false;
		}

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;

		connect( pMapLayer, SIGNAL( signalNotifyImageMouseAction(QPoint&,QPoint&,int,long) ),this, SLOT( notifyImageMouseAction(QPoint&,QPoint&,int,long) ) );

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		QString filename = doc->fileName();
		pMapLayer->setFileName(filename);

		pMapLayer->addDocument(doc);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);

		GeoDataLatLonBox box;
		qreal lon=0;
		qreal lat=0;
		if(pMapLayer->getLatLonBox(box,lon,lat))
		{
			d->m_marbleWidget->centerOn(box);
		}
		else if(lon!=lat)
		{
			GeoDataCoordinates targetPosition(lon,lat);
			d->m_marbleWidget->centerOn(targetPosition);
		}
	}

	if(type==7 && m_vecOverlayID!=0)//m_mapOverlayID!=0)
	{
		GeoDataFeature* pFeature = static_cast<GeoDataFeature*>(doc);
		GeoDataGroundOverlay* pOvl = static_cast<GeoDataGroundOverlay*>(pFeature);
		if(pOvl==NULL)return 0;

		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_vecOverlayID);

		QString filename = pOvl->absoluteIconFile();
		QFile imgFile( filename );
		if ( !imgFile.exists() ) 
		{
			QString name = add;//pDoc->fileName();
			QFileInfo filePath(name);
			QString path = filePath.absolutePath();
			QFileInfo fileInf(filename);
			QString iconname = fileInf.fileName();
			path = path + '/' + iconname;
			filename=path;
			pOvl->setIconFile(filename);
		}
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);
		qint64 qID=(qint64)objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode!=NULL)
		{
			pNode->m_qsDesc=add;//pOvl->absoluteIconFile();
			pNode->parent->m_qsDesc=add;//pOvl->absoluteIconFile();
			pNode->m_bMap=false;
		}

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;

		connect( pMapLayer, SIGNAL( signalNotifyImageMouseAction(QPoint&,QPoint&,int,long) ),this, SLOT( notifyImageMouseAction(QPoint&,QPoint&,int,long) ) );

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		pMapLayer->setFileName(add);

		pMapLayer->addDocument(doc,true);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);

		GeoDataLatLonBox box;
		qreal lon=0;
		qreal lat=0;
		if(pMapLayer->getLatLonBox(box,lon,lat))
		{
			d->m_marbleWidget->centerOn(box);
		}
		else if(lon!=lat)
		{
			GeoDataCoordinates targetPosition(lon,lat);
			d->m_marbleWidget->centerOn(targetPosition);
		}
	}

	if( type==8 && m_mapOverlayID!=0)
	{
		qsl << "399.0" << "399.0" << qsMap;
		objectID = (long)d->m_treeViewModel->AddLayer(qsl,m_mapOverlayID);
		//slotAddObject(qsl,m_minfOverlayID,MST_ADD_LAYER,objectID);
		qint64 qID=(qint64)objectID;
		TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
		if(pNode!=NULL)
		{
			pNode->m_qsDesc=doc->fileName();
			pNode->parent->m_qsDesc=doc->fileName();
			pNode->m_bMap=false;
		}

		//TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(objectID);
		//if(pNode!=NULL)pNode->m_qsDesc=doc->fileName();//needed to delete file from file list

		MapViewMapLayer* pMapLayer = new MapViewMapLayer(d->m_marbleWidget,this,(qint64)objectID);
		if(pMapLayer==NULL)return objectID;

		connect( pMapLayer, SIGNAL( signalNotifyImageMouseAction(QPoint&,QPoint&,int,long) ),this, SLOT( notifyImageMouseAction(QPoint&,QPoint&,int,long) ) );

		pMapLayer->setModel(d->m_treeViewModel);
		QItemSelectionModel *selectionModel = d->m_ui.treeTreeView->selectionModel();
		pMapLayer->synchronizeWith( selectionModel );

		//needed for deleting file from list
		QString filename = doc->fileName();
		pMapLayer->setFileName(filename);

		pMapLayer->addDocument(doc);
		d->m_marbleWidget->addLayer(pMapLayer); 
		d->m_mvMapLayers.insert(objectID,pMapLayer);

		GeoDataLatLonBox box;
		qreal lon=0;
		qreal lat=0;
		if(pMapLayer->getLatLonBox(box,lon,lat))
		{
			d->m_marbleWidget->centerOn(box);
		}
		else if(lon!=lat)
		{
			GeoDataCoordinates targetPosition(lon,lat);
			d->m_marbleWidget->centerOn(targetPosition);
		}
	}

	return objectID;
}

void TreeViewWidget::addGeoDataDocumentSymbole(GeoDataDocument* doc)
{
	//d->m_marbleWidget->model()->treeModel()->addDocument(doc );
	//return;
	GeoDataContainer *container =  doc;
	if(container==NULL)return;
	//const QVector<GeoDataPlacemark*> placemarks = container->placemarkList();
	//doc->name();

//	m_tsOverlayID=0;
//	m_tsLayerID=0;
	m_bMOG=false;

	int iExt=0;
	if ( !doc->fileName().isEmpty() )
    {
        QFileInfo file( doc->fileName() );
		QString ext = file.suffix();
		if(ext!="kml")
			iExt=1;
        //document->setName( file.baseName() );
    }
	
	switch(iExt)
	{
		case 0:
				iExt = createKmlTsPlacemarks(container);
				break;
		case 1:
				iExt = createGpxPlacemarks(container);
				break;
		default: 
				iExt = createKmlPlacemarks(container);
				break;
	}

	/*if(iExt==100)//GrondOverlay
	{
		d->m_marbleWidget->model()->treeModel()->addDocument( doc );
	}*/
}

long TreeViewWidget::createKmlPlacemarks( GeoDataContainer *container,long id )
{
	QStringList qsl;
	QString ad;
	QString desc;
	QString qsOvl;

	long objectID=0;
	long layerID=0;
	long groupID=0;
	qreal lon=0;
    qreal lat=0;
    qreal alt=0;

	GeoDataCoordinates		pos;
	const GeoDataStyle*		gdStyle;
	GeoDataStyleSelector	gdGeoDataStyleSelector;
	GeoDataLabelStyle		gdLabelStyle;
	GeoDataPolyStyle		gdPolyStyle;
	GeoDataIconStyle		gdIconStyle;
	GeoDataLineStyle		gdLineStyle;

	const char* chType=NULL;
	const char* chFolder = GeoDataTypes::GeoDataFolderType;
	const char* chDoc = GeoDataTypes::GeoDataDocumentType;
	const char* chGroudOverlay = GeoDataTypes::GeoDataGroundOverlayType;
	const char* chPhotoOverlay = GeoDataTypes::GeoDataPhotoOverlayType;

	QString qsFolder(chFolder);
	QString qsDoc(chDoc);

    QVector<GeoDataFeature*>::Iterator i = container->begin();
    QVector<GeoDataFeature*>::Iterator const end = container->end();
    for (; i != end; ++i ) 
	{
		chType = (*i)->nodeType();
		ad = (*i)->address();
		desc = (*i)->description();
		//Marble::GeoDataExtendedData exd = (*i)->extendedData();
		//const char* chT = exd.nodeType();
		ad = (*i)->name();
		//ad = (*i)->styleUrl();
		QString qsType(chType);

        if ( qsType == qsFolder || qsType == qsDoc ) 
		{
            GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
			/*if(id==0&&qsType == qsFolder)
			{
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				qsl.clear();
				ad="GeoData";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}
			else if(objectID==0)
			{
				if(ad.isEmpty())
					ad="GeoData";
				qsl << "-399" << "-399" << ad;
				if(id>0)
					slotAddObject(qsl,id,MST_ADD_LAYER,objectID);
				else slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}*/
			if(qsType == qsFolder)
			{
				//m_vecOverlayID=0;
				m_vecLayerID=0;
			}
			if(m_vecOverlayID==0)
			{
				objectID=0;
				qsOvl = container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				m_vecOverlayID=objectID;

				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				//	emit signalObjectImport(m_vecOverlayID,0,pNode->m_qsDesc);
				}
				qsl.clear();
			}
			if(m_vecLayerID==0)
			{
				objectID=0;
				if(ad.isEmpty())
					ad = container->name()+"_Layer";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,m_vecOverlayID,MST_ADD_LAYER,objectID);
				m_vecLayerID=objectID;

				GeoDataDocument* doc = (GeoDataDocument*)container;
				if(doc!=NULL&&id==0)//sent to plugins who created the import
				{
					try{
							QString fN = doc->fileName();
							emit signalObjectImport(m_vecOverlayID,m_vecLayerID,fN);
					}
					catch(...)
					{
						qsl.clear();
					}
				}
				qsl.clear();
			}
            createKmlPlacemarks( child,objectID);
        }
		else if( qsType == chGroudOverlay)
		{
			QString qsFilePath;

			if(m_vecOverlayID!=0)//&&objectID==0)//actually no objects are added
			{
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(pNode!=NULL)
				{
					qsFilePath=pNode->m_qsDesc;
			//		pNode->m_qsDesc.clear();//without we have a crash becaus DELETE_OVERLAY will also remove this file
				}
			//	slotDeleteObject(m_vecOverlayID,MST_DELETE_OVERLAY);
			}
			
/*			GeoDataGroundOverlay* pOvl = static_cast<GeoDataGroundOverlay*>( *i );
			QString qs = pOvl->description();
			qs = pOvl->absoluteIconFile();
			QFile imgFile( qs );
			if ( !imgFile.exists()&&m_vecOverlayID!=0 ) 
			{
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(!pNode->m_qsDesc.isEmpty())
					qsFilePath=pNode->m_qsDesc;
				QFileInfo filePath(qsFilePath);
				QString path = filePath.absolutePath();
				QFileInfo fileInf(qs);
				QString iconname = fileInf.fileName();
				path = path + '/' + iconname;
				pOvl->setIconFile(path);
			}
			else if(m_vecOverlayID==0)
			{
				GeoDataDocument* doc = (GeoDataDocument*)container;
				objectID=addMapLayerFromDocument(doc,5, QString());
				GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
				createKmlPlacemarks( child,objectID);
			}
			else
				pOvl->setIconFile(qs);
			d->m_marbleWidget->model()->treeModel()->addDocument((GeoDataDocument*)container );
			return m_vecLayerID;*/

			if(qsFilePath.isEmpty())
			{
				GeoDataDocument* doc = (GeoDataDocument*)container;
				qsFilePath = doc->fileName();
			}

			GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
			GeoDataDocument* pDoc = (GeoDataDocument*)child;
			if(pDoc==NULL)return m_vecLayerID;
			objectID=addMapLayerFromDocument(pDoc,7,qsFilePath);
			//m_vecLayerID=objectID;

		}
		else if( qsType == chPhotoOverlay )
		{
			/*
			GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );
			GeoDataGeometry* pG=placemark->geometry();
			if( dynamic_cast<GeoDataLatLonBox*>( placemark->geometry() ) ) 
			{
				pos = placemark->coordinate();
				QString qs = placemark->name();
				qs = placemark->styleUrl();
			
			}*/
			slotDeleteObject(m_vecOverlayID,MST_DELETE_OVERLAY);
			GeoDataPhotoOverlay* pOvl = static_cast<GeoDataPhotoOverlay*>( *i );
			QString qs = pOvl->description();
			//qs = pOvl->iconPath();
			return m_vecLayerID;

		}
		else {
			if(m_vecOverlayID==0)
			{
				objectID=0;
				qsOvl = container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				m_vecOverlayID=objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				}
				qsl.clear();
			}
			if(m_vecLayerID==0)
			{
				objectID=0;
				if(ad.isEmpty())
					ad = container->name()+"_Layer";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,m_vecOverlayID,MST_ADD_LAYER,objectID);
				m_vecLayerID=objectID;
				qsl.clear();
			}

            GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );
			ad=placemark->name();
			placemark->coordinate(lon,lat,alt);
			pos.setAltitude(alt);
			pos.setLongitude(lon);
			pos.setLatitude(lat);
			qsl.clear();
			qsl << QString::number(pos.longitude(GeoDataCoordinates::Degree),'f',8) << QString::number(pos.latitude(GeoDataCoordinates::Degree),'f',8) << ad;
			slotAddObject(qsl,m_vecLayerID,MST_ADD_GROUP,groupID);
			lon=pos.longitude(GeoDataCoordinates::Degree);
			lat=pos.latitude(GeoDataCoordinates::Degree);

			if(lon==-399.0 || lat==-399.0)
			{
				QString qsFilePath;

				if(m_vecOverlayID!=0)//&&objectID==0)//actually no objects are added
				{
					TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
					if(pNode!=NULL)
					{
						qsFilePath=pNode->m_qsDesc;
				//		pNode->m_qsDesc.clear();//without we have a crash becaus DELETE_OVERLAY will also remove this file
					}
				}
				if(qsFilePath.isEmpty())
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					qsFilePath = doc->fileName();
				}

				GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
				GeoDataDocument* pDoc = (GeoDataDocument*)child;
			//	GeoDataDocument* pDoc = (GeoDataDocument*)container;
				if(pDoc==NULL)return m_vecLayerID;
				d->m_marbleWidget->model()->treeModel()->addDocument(pDoc);

			}
			else if( dynamic_cast<GeoDataLineString*>( placemark->geometry() ) )
			{
				GeoDataLineString* gdl = dynamic_cast<GeoDataLineString*>( placemark->geometry() );
				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long col = color.blue();
				col <<= 8;
				col += color.green();
				col <<= 8;
				col += color.red();

				int iWidht = (int)gdLineStyle.width();
				slotAddPoly(groupID,MST_ADD_POLY,col,*gdl,objectID);
				qint64 oID = (qint64)objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(oID);
				if(pNode!=NULL)
				{
					pNode->m_iDirection=iWidht;
				}

/*
				gdStyle = placemark->style();
				QString qs = gdStyle->iconStyle().iconPath();
				if(!qs.isEmpty())
				{
					//qs = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
				
					qint64 idx=(qint64)objectID;
					idx = d->m_treeViewModel->AddIcon(qs,groupID,lon,lat,0,0,0,objectID);
				}
				*/
			}
			else if( dynamic_cast<GeoDataPolygon*>( placemark->geometry() ) ) 
			{
				GeoDataPolygon* gdl = dynamic_cast<GeoDataPolygon*>( placemark->geometry() );
				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				QString qs = gdStyle->styleId();
				int ic = qs.count('-');
				if(ic>0)
				{
					--ic;
					qs=qs.section('-',0,ic);
				}

				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				GeoDataLinearRing gdLinearRing = gdl->outerBoundary();
				GeoDataLineString gdLineString;
				for(int i=0; i<gdLinearRing.size();i++)
					gdLineString.append(gdLinearRing.at(i));
				
				//slotAddPoly(groupID,MST_ADD_POLY,col,gdLineString,objectID);
				qint64 idx=(qint64)objectID;
				//TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;

				TreeNode* pNode=NULL;
				int iWidht = (int)gdLineStyle.width();

				if(gdLineString.size()==4)//maybe a rect
				{
					if(gdLineString.at(0).longitude()==gdLineString.at(3).longitude() && gdLineString.at(1).longitude()==gdLineString.at(2).longitude())
					{
						if(gdLineString.at(0).latitude()==gdLineString.at(1).latitude() && gdLineString.at(2).latitude()==gdLineString.at(3).latitude())
						{
							idx = d->m_treeViewModel->AddRect(groupID,gdLineString.at(0).longitude(GeoDataCoordinates::Degree),gdLineString.at(0).latitude(GeoDataCoordinates::Degree),gdLineString.at(2).longitude(GeoDataCoordinates::Degree),gdLineString.at(2).latitude(GeoDataCoordinates::Degree));
							pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
							if(pNode!=NULL)
							{
								pNode->m_lFillCol = fillcol;
								pNode->m_lLineCol = linecol;
								pNode->m_qsName = qs;
							}
						}
						else
							idx = d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,gdLineString);
					}
					else
						idx = d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,gdLineString);

				}
				else
					idx = d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,gdLineString);

				//set the width
				if(pNode==NULL)
					pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
				if(pNode!=NULL)
					pNode->m_iDirection=iWidht;

			}
			else if( dynamic_cast<GeoDataIconStyle*>( placemark->geometry() ) ) 
			{
				pos = placemark->coordinate();
				QString qs = placemark->name();
				qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdIconStyle = gdStyle->iconStyle();
				ad=gdIconStyle.iconPath();
				QImage pix = gdIconStyle.icon();
			
			}
			else if( dynamic_cast<GeoDataPoint*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				QString qs = gdStyle->iconStyle().iconPath();
				if(!qs.isEmpty())
				{
					//qs = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
				
					qint64 idx=(qint64)objectID;
					idx = d->m_treeViewModel->AddIcon(qs,groupID,lon,lat,0,0,0,objectID);
				}
				else if(!ad.isEmpty())//only text label
					slotAddDescription(ad,groupID,0,0,0,objectID);

				//set the ToolTip
				if(!desc.isEmpty())
					slotAddDescription(desc,groupID,111,1,0,objectID);
				desc.clear();
			}
			else if( dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;

				bool bExtendedData=false;


				GeoDataMultiGeometry* pMulti = dynamic_cast<GeoDataMultiGeometry*>(placemark->geometry());
				if(pMulti!=NULL)
				{
					QVector<GeoDataGeometry*>::ConstIterator it = pMulti->constBegin();
					QVector<GeoDataGeometry*>::ConstIterator end = pMulti->constEnd();
					for ( ; it != end; ++it ) 
					{
						if ( (*it)->nodeType() == GeoDataTypes::GeoDataPointType ) {
							
							GeoDataCoordinates coord = GeoDataCoordinates( *static_cast<GeoDataPoint *>(*it) );
							qreal lon1=coord.longitude(GeoDataCoordinates::Degree);
							qreal lat1=coord.latitude(GeoDataCoordinates::Degree);
							slotMoveObject(groupID,lon1,lat1);
							gdStyle = placemark->style();
							QString qs = gdStyle->iconStyle().iconPath();
							if(!qs.isEmpty())
							{
								//qs = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
							
								qint64 idx=(qint64)objectID;
								idx = d->m_treeViewModel->AddIcon(qs,groupID,lon1,lat1,0,0,0,objectID);
							}
							qs.clear(); 
							qs = placemark->name();
							GeoDataExtendedData extData = placemark->extendedData();
							if(!extData.isEmpty())
							{
								if(extData.contains(qs)&&! bExtendedData)
								{
									GeoDataData data = extData.value(qs);

									QString qslabel = data.name();
									QVariant var = data.value();
									QString qsValue = var.toString();
									slotAddDescription(qsValue,groupID,0,0,0,objectID);
									bExtendedData=true;
								}
								else
								{
									GeoDataData gDat;
									QString exD;
									QHash<QString,GeoDataData>::const_iterator iter = extData.constBegin();
									int align=0;
									for ( ; iter != extData.constEnd(); ++iter )
									{
										gDat = iter.value();
										qs = gDat.name();
										exD = gDat.value().toString();
										slotAddDescription(exD,groupID,0,0,align,objectID);
										align+=10;
									}

								}
							}
							//set the ToolTip
							if(!desc.isEmpty())
								slotAddDescription(desc,groupID,111,1,0,objectID);
							desc.clear();
							break;
						}
					}
					GeoDataLineString gdLineString;
					//GeoDataMultiGeometry* geometry = dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() );
					for ( int i = 0; pMulti && i < pMulti->size(); ++i ) 
					{
						GeoDataLinearRing* poly = dynamic_cast<GeoDataLinearRing*>( pMulti->child( i ) );
						GeoDataLineString* line = dynamic_cast<GeoDataLineString*>( pMulti->child( i ) );
						GeoDataPolygon* polygon = dynamic_cast<GeoDataPolygon*>( pMulti->child( i ) );
						GeoDataTrack* track = dynamic_cast<GeoDataTrack*>( pMulti->child( i ) );
						
						if ( poly ) 
						{
							for ( int j = 0; j < poly->size(); ++j ) {
								gdLineString << poly->at( j );
							}

							gdStyle = placemark->style();
							//gdPolyStyle = gdStyle->polyStyle();
							QString qs = gdStyle->styleId();
							int ic = qs.count('-');
							if(ic>0)
							{
								--ic;
								qs=qs.section('-',0,ic);
							}
						
							d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,gdLineString);
 
							qs = placemark->name();
							GeoDataExtendedData extData = placemark->extendedData();
							if(extData.contains(qs)&& !bExtendedData)
							{
								GeoDataData data = extData.value(qs);

								QString qslabel = data.name();
								QVariant var = data.value();
								QString qsValue = var.toString();
								slotAddDescription(qsValue,groupID,0,0,0,objectID);
								bExtendedData=true;
							}
						}
						else if ( polygon ) 
						{
							GeoDataLinearRing gdlr=polygon->outerBoundary();
							for ( int j = 0; j <gdlr.size(); ++j ) {
								gdLineString << gdlr.at( j );
							}
						
							gdStyle = placemark->style();
							//gdPolyStyle = gdStyle->polyStyle();
							QString qs = gdStyle->styleId();
							int ic = qs.count('-');
							if(ic>0)
							{
								--ic;
								qs=qs.section('-',0,ic);
							}
							d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,gdLineString);

							qs = placemark->name();
							GeoDataExtendedData extData = placemark->extendedData();
							if(extData.contains(qs)&& !bExtendedData)
							{
								GeoDataData data = extData.value(qs);

								QString qslabel = data.name();
								QVariant var = data.value();
								QString qsValue = var.toString();
								slotAddDescription(qsValue,groupID,0,0,0,objectID);
								bExtendedData=true;
							}
						}
						else if ( track ) 
						{
							line=(GeoDataLineString*)track->lineString();
							if(line)
							{
								for ( int j = 0; j <line->size(); ++j )
									gdLineString << line->at( j );
							
								//d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
								slotAddPoly(groupID,MST_ADD_POLY,linecol,gdLineString,objectID);
							}
						}//track
						else if(line)
						{
							gdStyle = placemark->style();
							//gdPolyStyle = gdStyle->polyStyle();
							QString qs = gdStyle->styleId();
							int ic = qs.count('-');
							if(ic>0)
							{
								--ic;
								qs=qs.section('-',0,ic);
							}
							//d->m_treeViewModel->AddColorPolygone(qs,groupID,linecol,fillcol,lon,lat,2,*line);
							qs = placemark->name();
							d->m_treeViewModel->AddNamedPolyLine(groupID,qs,lon,lat,linecol,*line);
							GeoDataExtendedData extData = placemark->extendedData();
							if(extData.contains(qs)&& !bExtendedData)
							{
								GeoDataData data = extData.value(qs);

								QString qslabel = data.name();
								QVariant var = data.value();
								QString qsValue = var.toString();
								slotAddDescription(qsValue,groupID,0,0,0,objectID);
								bExtendedData=true;
							}
						}//line
					}
				}
			}

		}
	}

	return m_vecLayerID;
}

long TreeViewWidget::createGpxPlacemarks( GeoDataContainer *container,long id )
{
	QStringList qsl;
	QString ad;
	QString qsProp;
	QString qsOvl;

	int iCount=0;
	long objectID=0;
	long layerID=0;
	long groupID=0;
	qreal lon=0;
    qreal lat=0;
    qreal alt=0;

	GeoDataCoordinates		pos;
	const GeoDataStyle*		gdStyle;
	GeoDataStyleSelector	gdGeoDataStyleSelector;
	GeoDataLabelStyle		gdLabelStyle;
	GeoDataPolyStyle		gdPolyStyle;
	GeoDataIconStyle		gdIconStyle;
	GeoDataLineStyle		gdLineStyle;

	const char* chType=NULL;
	const char* chFolder = GeoDataTypes::GeoDataFolderType;
	const char* chDoc = GeoDataTypes::GeoDataDocumentType;
	const char* chGroudOverlay = GeoDataTypes::GeoDataGroundOverlayType;
	const char* chPhotoOverlay = GeoDataTypes::GeoDataPhotoOverlayType;

	QString qsFolder(chFolder);
	QString qsDoc(chDoc);

    QVector<GeoDataFeature*>::Iterator i = container->begin();
    QVector<GeoDataFeature*>::Iterator const end = container->end();
    for (; i != end; ++i ) 
	{
		chType = (*i)->nodeType();
		ad = (*i)->address();
		ad = (*i)->description();
		Marble::GeoDataExtendedData exd = (*i)->extendedData();
		const char* chT = exd.nodeType();
		ad = (*i)->name();
		//ad = (*i)->styleUrl();
		QString qsType(chType);

        if ( qsType == qsFolder || qsType == qsDoc ) 
		{
            GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
			/*if(id==0&&qsType == qsFolder)
			{
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				qsl.clear();
				ad="GeoData";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}
			else
			{
				if(ad.isEmpty())
					ad="GeoData";
				qsl << "-399" << "-399" << ad;
				if(id>0)
					slotAddObject(qsl,id,MST_ADD_LAYER,objectID);
				else slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}*/
			if(qsType == qsFolder)
			{
				//m_vecOverlayID=0;
				m_vecLayerID=0;
			}
			if(m_vecOverlayID==0)
			{
				objectID=0;
				//if(ad.isEmpty())
				qsOvl = container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				m_vecOverlayID=objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				}
				qsl.clear();
			}
			if(m_vecLayerID==0)
			{
				objectID=0;
				if(ad.isEmpty())
					ad = container->name()+"_Layer";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,m_vecOverlayID,MST_ADD_LAYER,objectID);
				m_vecLayerID=objectID;
				qsl.clear();
			}
            createGpxPlacemarks( child,objectID);
        }
		else {
			if(m_vecOverlayID==0)
			{
				objectID=0;
				qsOvl = container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				m_vecOverlayID=objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_vecOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				}
				qsl.clear();
			}
			if(m_vecLayerID==0)
			{
				objectID=0;
				if(ad.isEmpty())
					ad = container->name()+"_Layer";
				qsl << "-399" << "-399" << "GeoData";
				slotAddObject(qsl,m_vecOverlayID,MST_ADD_LAYER,objectID);
				m_vecLayerID=objectID;
				qsl.clear();
			}
            GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );
			ad=placemark->name();
			placemark->coordinate(lon,lat,alt);
			pos.setAltitude(alt);
			pos.setLongitude(lon);
			pos.setLatitude(lat);
			qsl.clear();
			if(ad.isEmpty())
				ad=QString::number(iCount++);

			qsl << QString::number(pos.longitude(GeoDataCoordinates::Degree)) << QString::number(pos.latitude(GeoDataCoordinates::Degree)) << ad;
			slotAddObject(qsl,m_vecLayerID,MST_ADD_GROUP,groupID);
			lon=pos.longitude(GeoDataCoordinates::Degree);
			lat=pos.latitude(GeoDataCoordinates::Degree);

			if( dynamic_cast<GeoDataLineString*>( placemark->geometry() ) )
			{
				GeoDataLineString* gdl = dynamic_cast<GeoDataLineString*>( placemark->geometry() );
				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long col = color.blue();
				col <<= 8;
				col += color.green();
				col <<= 8;
				col += color.red();
				slotAddPoly(groupID,MST_ADD_POLY,col,*gdl,objectID);
			}
			else if( dynamic_cast<GeoDataPolygon*>( placemark->geometry() ) ) 
			{
				GeoDataPolygon* gdl = dynamic_cast<GeoDataPolygon*>( placemark->geometry() );
				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				GeoDataLinearRing gdLinearRing = gdl->outerBoundary();
				GeoDataLineString gdLineString;
				for(int i=0; i<gdLinearRing.size();i++)
					gdLineString.append(gdLinearRing.at(i));
				
				//slotAddPoly(groupID,MST_ADD_POLY,col,gdLineString,objectID);
				qint64 idx=(qint64)objectID;
				//TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;

				idx = d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);

			}
			else if( dynamic_cast<GeoDataIconStyle*>( placemark->geometry() ) ) 
			{
				pos = placemark->coordinate();
				QString qs = placemark->name();
				qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdIconStyle = gdStyle->iconStyle();
				ad=gdIconStyle.iconPath();
				QImage pix = gdIconStyle.icon();
			
			}
			else if( dynamic_cast<GeoDataPoint*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				QString qs = gdStyle->iconStyle().iconPath();
				if(qs.isEmpty())
				{
					qs = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
				}
				objectID=0;
				qint64 idx=0;
				qreal azimuth=399;
				idx = d->m_treeViewModel->AddIcon(qs,groupID,lon,lat,0,0,0,objectID);

				Marble::GeoDataExtendedData exd = placemark->extendedData();
				if(exd.size()>0)
				{
					qs.clear();
					ad.clear();
					GeoDataData gDat;
					QHash<QString,GeoDataData>::const_iterator iter = exd.constBegin();
					for ( ; iter != exd.constEnd(); ++iter )
					{
						gDat = iter.value();
						qs = gDat.name();
						
						if(qs=="Azimuth")
						{
							azimuth = gDat.value().toDouble();
						}
						if(qs=="desc")
						{
							qsOvl = gDat.value().toString();
						}

						qs +=" : ";
						qs+=gDat.value().toString();
						qs +=";";
						ad+=qs;
						qs.clear();
					}
					//qint64 idx=(qint64)objectID;
					TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
					if(pNode&&pNode->parent)pNode->parent->m_qsToolTip=ad;

					if(azimuth!=399)
					{
						pNode->parent->m_qsName=qsOvl; 
						qs = placemark->name();
						gpxPlacemark(groupID,lon,lat,azimuth,qs);
					}
				}
			}
			else if( dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;


				GeoDataMultiGeometry* pMulti = dynamic_cast<GeoDataMultiGeometry*>(placemark->geometry());
				if(pMulti!=NULL)
				{
					QVector<GeoDataGeometry*>::ConstIterator it = pMulti->constBegin();
					QVector<GeoDataGeometry*>::ConstIterator end = pMulti->constEnd();
					for ( ; it != end; ++it ) 
					{
						if ( (*it)->nodeType() == GeoDataTypes::GeoDataPointType ) {
							
							GeoDataCoordinates coord = GeoDataCoordinates( *static_cast<GeoDataPoint *>(*it) );
							break;
						}
					}
					GeoDataLineString gdLineString;
					//GeoDataMultiGeometry* geometry = dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() );
					for ( int i = 0; pMulti && i < pMulti->size(); ++i ) 
					{
						GeoDataLinearRing* poly = dynamic_cast<GeoDataLinearRing*>( pMulti->child( i ) );
						GeoDataLineString* line = dynamic_cast<GeoDataLineString*>( pMulti->child( i ) );
						GeoDataPolygon* polygon = dynamic_cast<GeoDataPolygon*>( pMulti->child( i ) );
						GeoDataTrack* track = dynamic_cast<GeoDataTrack*>( pMulti->child( i ) );
						
						if ( poly ) 
						{
							for ( int j = 0; j < poly->size(); ++j ) {
								gdLineString << poly->at( j );
							}
						
							d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
						}
						else if ( polygon ) 
						{
							GeoDataLinearRing gdlr=polygon->outerBoundary();
							for ( int j = 0; j <gdlr.size(); ++j ) {
								gdLineString << gdlr.at( j );
							}
						
							d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
						}
						else if ( track ) 
						{
							if(0)//qsProp.isEmpty())
							{
								qsProp="EDIT=FALSE;DELETE=FALSE;O_MOVE=FALSE;O_EDIT=FALSE";
								slotSetObjectProperties(qsProp,m_vecLayerID,MST_SET_LAYER_PROPERTIES);
							}

							line=(GeoDataLineString*)track->lineString();
							if(line)
							{
								for ( int j = 0; j <line->size(); ++j )
									gdLineString << line->at( j );
							
								//d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
								slotAddPoly(groupID,MST_ADD_POLY,linecol,gdLineString,objectID);
							}
							/****** extended data **********************/
							QString qs;
							qint64 idx=(qint64)objectID;
							Marble::GeoDataExtendedData exd = placemark->extendedData();
							if(exd.size()>0)
							{
								qs.clear();
								ad.clear();
								GeoDataData gDat;
								QHash<QString,GeoDataData>::const_iterator iter = exd.constBegin();
								for ( ; iter != exd.constEnd(); ++iter )
								{
									gDat = iter.value();
									qs = gDat.name();
									qs +=" : ";
									qs+=gDat.value().toString();
									qs +=";";
									ad+=qs;
									qs.clear();
								}
								//qint64 idx=(qint64)objectID;
								TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
								if(pNode&&pNode->parent)pNode->parent->m_qsToolTip=ad;
							}
							else
							{
								placemark->coordinate( lon, lat, alt );
								qs.clear();
								ad.clear();
								
								//QList<QDateTime> time = track->whenList();
								QDateTime dt = track->firstWhen();//time.at(iCount);
								qs +=tr("Timestamp start: ");
								qs += dt.toString();
								qs +=";";
								dt = track->lastWhen();//time.at(iCount);
								qs +=tr("Timestamp stop: ");
								qs += dt.toString();
								qs +=";";
								qs += tr("Elevation : ");
								qs += QString::number(alt);
								ad+=qs;
								qs.clear();

								TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
								if(pNode&&pNode->parent)pNode->parent->m_qsToolTip=ad;
							}
							/******************************************/
						}
						else if(line)
						{
							//d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,*line);
							//slotAddPoly(groupID,MST_ADD_POLY,linecol,*line,objectID);
							d->m_treeViewModel->AddNamedPolyLine(groupID,ad,lon,lat,linecol,*line);
						}
					}
				}
			}

		}
	}

	return m_vecLayerID;
}

long TreeViewWidget::createKmlTsPlacemarks( GeoDataContainer *container,long id )
{
	QStringList qsl;
	QString ad;
	QString desc;
	QString qsOvl;

	long objectID=0;
	long layerID=0;
	long groupID=0;
	qreal lon=0;
    qreal lat=0;
    qreal alt=0;

	qreal vblon=0;
    qreal vblat=0;
	GeoDataLatLonAltBox viewBox = d->m_marbleWidget->viewport()->viewLatLonAltBox();
	GeoDataCoordinates center = viewBox.center();
	vblon = center.longitude();
	vblat = center.latitude();

	GeoDataCoordinates		pos;
	const GeoDataStyle*		gdStyle;
	GeoDataStyleSelector	gdGeoDataStyleSelector;
	GeoDataLabelStyle		gdLabelStyle;
	GeoDataPolyStyle		gdPolyStyle;
	GeoDataIconStyle		gdIconStyle;
	GeoDataLineStyle		gdLineStyle;

	const char* chType=NULL;
	const char* chFolder = GeoDataTypes::GeoDataFolderType;
	const char* chDoc = GeoDataTypes::GeoDataDocumentType;
	const char* chGroudOverlay = GeoDataTypes::GeoDataGroundOverlayType;
	const char* chPhotoOverlay = GeoDataTypes::GeoDataPhotoOverlayType;

	QString qsFolder(chFolder);
	QString qsDoc(chDoc);

    QVector<GeoDataFeature*>::Iterator i = container->begin();
    QVector<GeoDataFeature*>::Iterator const end = container->end();
    for (; i != end; ++i ) 
	{
		chType = (*i)->nodeType();
		ad = (*i)->address();
		desc = (*i)->description();
		//Marble::GeoDataExtendedData exd = (*i)->extendedData();
		//const char* chT = exd.nodeType();
		ad = (*i)->name();
		//ad = (*i)->styleUrl();
		QString qsType(chType);

        if ( qsType == qsFolder || qsType == qsDoc ) 
		{
            GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
			/*if(id==0&&qsType == qsFolder)
			{
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,id,MST_ADD_OVERLAY,objectID);
				qsl.clear();
				ad="GeoData";
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}
			else if(objectID==0)
			{
				if(ad.isEmpty())
					ad="GeoData";
				qsl << "-399" << "-399" << ad;
				if(id>0)
					slotAddObject(qsl,id,MST_ADD_LAYER,objectID);
				else slotAddObject(qsl,objectID,MST_ADD_LAYER,objectID);
			}*/
			if(qsType == qsFolder)
			{
				//m_vecOverlayID=0;
				//m_tsLayerID=0;
			}
			if(m_tsOverlayID==0)
			{
				objectID=0;
				qsOvl = tr("TakticalSituation");//container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_TS_OVERLAY,objectID);
				m_tsOverlayID=objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_tsOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				}
				qsl.clear();
			}
			if(m_tsLayerID==0)
			{
				objectID=0;
				if(1)//ad.isEmpty())
					//ad = container->name()+"TakticalLayer";
					ad = tr("TakticalLayer");
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,m_tsOverlayID,MST_ADD_TS_LAYER,objectID);
				m_tsLayerID=objectID;
				qsl.clear();
			}
            createKmlTsPlacemarks( child,objectID);
        }
		else {	//the taktical object
			if(m_tsOverlayID==0)
			{
				objectID=0;
				qsOvl = tr("TakticalSituation");//container->name();
				qsl << "-399" << "-399" << qsOvl;
				slotAddObject(qsl,id,MST_ADD_TS_OVERLAY,objectID);
				m_tsOverlayID=objectID;
				TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(m_tsOverlayID);
				if(pNode!=NULL)
				{
					GeoDataDocument* doc = (GeoDataDocument*)container;
					pNode->m_qsDesc = doc->fileName();
				}
				qsl.clear();
			}
			if(m_tsLayerID==0)
			{
				objectID=0;
				if(1)//ad.isEmpty())
					//ad = container->name()+"_Layer";
					ad = tr("TakticalLayer");
				qsl << "-399" << "-399" << ad;
				slotAddObject(qsl,m_vecOverlayID,MST_ADD_TS_LAYER,objectID);
				m_tsLayerID=objectID;
				qsl.clear();
			}

            GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );
			ad=placemark->name();
			placemark->coordinate(lon,lat,alt);
			pos.setAltitude(alt);
			pos.setLongitude(lon);
			pos.setLatitude(lat);
			qsl.clear();
			qsl << QString::number(pos.longitude(GeoDataCoordinates::Degree),'f',8) << QString::number(pos.latitude(GeoDataCoordinates::Degree),'f',8) << ad;
			if(m_tsGroupID==0)
				slotAddObject(qsl,m_tsLayerID,MST_ADD_TS_GROUP,groupID);
			else groupID=m_tsGroupID;
			//lon=pos.longitude(GeoDataCoordinates::Degree);
			//lat=pos.latitude(GeoDataCoordinates::Degree);
			lon = center.longitude(GeoDataCoordinates::Degree);
			lat = center.latitude(GeoDataCoordinates::Degree);

			if( dynamic_cast<GeoDataLineString*>( placemark->geometry() ) )
			{
				GeoDataLineString* gdl = dynamic_cast<GeoDataLineString*>( placemark->geometry() );
				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long col = color.blue();
				col <<= 8;
				col += color.green();
				col <<= 8;
				col += color.red();
				slotAddPoly(groupID,MST_ADD_POLY,col,*gdl,objectID);
			}
			else if( dynamic_cast<GeoDataPolygon*>( placemark->geometry() ) ) 
			{
				GeoDataPolygon* gdl = dynamic_cast<GeoDataPolygon*>( placemark->geometry() );
				//move to viewbox  center
				GeoDataLinearRing gdLinearRing;
				GeoDataLinearRing gdRing = gdl->outerBoundary();
				GeoDataLatLonAltBox box = gdRing.latLonAltBox();
				GeoDataCoordinates cor = box.center();
				GeoDataCoordinates via;

				if(!m_bMOG)
				{
					for(int i=0;i<gdRing.size();i++)
					{
						via = gdRing.at(i);
						vblon = cor.longitude()-via.longitude();//GeoDataCoordinates::Degree);
						vblat = cor.latitude()-via.latitude();//GeoDataCoordinates::Degree);
						
						vblon = center.longitude()-vblon;
						vblat = center.latitude()-vblat;

						via.set(vblon,vblat,0);//,GeoDataCoordinates::Degree);
						gdLinearRing << via;
					}
				}
				else gdLinearRing = gdl->outerBoundary();

				//QString qs = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				//GeoDataLinearRing gdLinearRing = gdl->outerBoundary();
				GeoDataLineString gdLineString;
				for(int i=0; i<gdLinearRing.size();i++)
					gdLineString.append(gdLinearRing.at(i));
				
				//slotAddPoly(groupID,MST_ADD_POLY,col,gdLineString,objectID);
				qint64 idx=(qint64)objectID;
				//TreeNode*pNode = d->m_treeViewModel->getNodeFromIndexId(idx);
				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;

				QString mog = ad.section('-',1,1);
				idx = d->m_treeViewModel->AddColorPolygone(mog,groupID,linecol,fillcol,lon,lat,2,gdLineString);

				if(!m_bMOG)
					placemark->setCoordinate(center);

			}
			else if( dynamic_cast<GeoDataIconStyle*>( placemark->geometry() ) ) 
			{
				pos = placemark->coordinate();
				QString qs = placemark->name();
				qs = placemark->styleUrl();
				gdStyle = placemark->style();
				gdIconStyle = gdStyle->iconStyle();
				ad=gdIconStyle.iconPath();
				QImage pix = gdIconStyle.icon();
			
			}
			else if( dynamic_cast<GeoDataPoint*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				//qs = placemark->styleUrl();
				gdStyle = placemark->style();
				QString qs = gdStyle->iconStyle().iconPath();
				if(qs.isEmpty())
				{
					qs = MarbleDirs::marbleDataPath()+"/bitmaps/city_2_yellow.png";
				}
				qint64 idx=(qint64)objectID;
				idx = d->m_treeViewModel->AddIcon(qs,groupID,lon,lat,0,0,0,objectID);

				//set the ToolTip
				if(!desc.isEmpty())
					slotAddDescription(desc,groupID,111,1,0,objectID);
				desc.clear();
			}
			else if( dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() ) ) 
			{
				//placemark->coordinate( lon, lat, alt );
				ad = placemark->name();
				gdStyle = placemark->style();
				gdPolyStyle = gdStyle->polyStyle();
				gdLineStyle = gdStyle->lineStyle();
				QColor color = gdLineStyle.color();
				long linecol = color.blue();
				linecol <<= 8;
				linecol += color.green();
				linecol <<= 8;
				linecol += color.red();

				color = gdPolyStyle.color();
				long fillcol = color.alpha();
				//pNode->m_iTrans=col;
				fillcol <<= 8;
				fillcol += color.blue();
				fillcol <<= 8;
				fillcol += color.green();
				fillcol <<= 8;
				fillcol += color.red();
				//pNode->m_lFillCol=col;


				GeoDataMultiGeometry* pMulti = dynamic_cast<GeoDataMultiGeometry*>(placemark->geometry());
				if(pMulti!=NULL)
				{
					QVector<GeoDataGeometry*>::ConstIterator it = pMulti->constBegin();
					QVector<GeoDataGeometry*>::ConstIterator end = pMulti->constEnd();
					for ( ; it != end; ++it ) 
					{
						if ( (*it)->nodeType() == GeoDataTypes::GeoDataPointType ) {
							
							GeoDataCoordinates coord = GeoDataCoordinates( *static_cast<GeoDataPoint *>(*it) );
							break;
						}
					}
					GeoDataLineString gdLineString;
					//GeoDataMultiGeometry* geometry = dynamic_cast<GeoDataMultiGeometry*>( placemark->geometry() );
					for ( int i = 0; pMulti && i < pMulti->size(); ++i ) 
					{
						GeoDataLinearRing* poly = dynamic_cast<GeoDataLinearRing*>( pMulti->child( i ) );
						GeoDataLineString* line = dynamic_cast<GeoDataLineString*>( pMulti->child( i ) );
						GeoDataPolygon* polygon = dynamic_cast<GeoDataPolygon*>( pMulti->child( i ) );
						GeoDataTrack* track = dynamic_cast<GeoDataTrack*>( pMulti->child( i ) );

						gdLineString.clear();
						
						if ( poly ) 
						{
							for ( int j = 0; j < poly->size(); ++j ) {
								gdLineString << poly->at( j );
							}
						
							d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
						}
						else if ( polygon ) 
						{
							GeoDataLinearRing gdlr=polygon->outerBoundary();
							//move to viewbox  center
							GeoDataLatLonAltBox box = gdlr.latLonAltBox();
							GeoDataCoordinates cor = box.center();
							GeoDataCoordinates via;
							if(!m_bMOG)
							{
								for(int i=0;i<gdlr.size();i++)
								{
									via = gdlr.at(i);
									vblon = cor.longitude()-via.longitude();//GeoDataCoordinates::Degree);
									vblat = cor.latitude()-via.latitude();//GeoDataCoordinates::Degree);
									
									vblon = center.longitude()-vblon;
									vblat = center.latitude()-vblat;

									via.set(vblon,vblat,0);//,GeoDataCoordinates::Degree);
									gdLineString << via;
								}
							}
							else gdLineString=polygon->outerBoundary();
							/*
							for ( int j = 0; j <gdlr.size(); ++j ) {
								gdLineString << gdlr.at( j );
							}*/

					/*		qreal deltaX = (box.east()-box.west())/4;
							qreal deltaY = (box.north()-box.south())/4;

							GeoDataLatLonAltBox boxPoly = gdLineString.latLonAltBox();
							if( (boxPoly.east()-boxPoly.west())>deltaX || (boxPoly.north()-boxPoly.south())>deltaY )
							{

							}*/

						
							QString mog = ad.section('-',1,1);
							d->m_treeViewModel->AddColorPolygone(mog,groupID,linecol,fillcol,lon,lat,2,gdLineString);

						}
						else if ( track ) 
						{
							line=(GeoDataLineString*)track->lineString();
							if(line)
							{
								for ( int j = 0; j <line->size(); ++j )
									gdLineString << line->at( j );
							
								//d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,gdLineString);
								slotAddPoly(groupID,MST_ADD_POLY,linecol,gdLineString,objectID);
							}
						}
						else if(line)
						{
							QString mog = ad.section('-',1,1);
							//d->m_treeViewModel->AddColorPolygone(QString(),groupID,linecol,fillcol,lon,lat,2,*line);
							//slotAddPoly(groupID,MST_ADD_POLY,linecol,*line,objectID);
							
							//move to viewbox  center
							GeoDataLatLonAltBox box = line->latLonAltBox();
							GeoDataCoordinates cor = box.center();
							GeoDataCoordinates via;
							if(!m_bMOG)
							{
								for(int i=0;i<line->size();i++)
								{
									via = line->at(i);
									vblon = cor.longitude()-via.longitude();//GeoDataCoordinates::Degree);
									vblat = cor.latitude()-via.latitude();//GeoDataCoordinates::Degree);
									
									vblon = center.longitude()-vblon;
									vblat = center.latitude()-vblat;

									via.set(vblon,vblat,0);//,GeoDataCoordinates::Degree);
									gdLineString << via;
								}
							}
							else 
							{
								for(int i=0;i<line->size();i++)
								{
									via = line->at(i);
									gdLineString << via;
								}
							}

							d->m_treeViewModel->AddNamedPolyLine(groupID,mog,lon,lat,linecol,gdLineString);

						}//line
					}//for pMulti
					
					//to center map at this point
					if(!m_bMOG)
						placemark->setCoordinate(center);
				}//if pMulti
			}//MultiGeometry

			m_tsGroupID=0;
		}//else //the taktical object
	}

	return m_tsLayerID;
}

void TreeViewWidget::resetEventFilter(bool set)
{
	//d->m_mvLayers.insert(objectID,pLayer);
	// delete all custom layers

	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
        if ( pLayer != NULL && set) 
		{
			pLayer->setInputEnabled(false);
			d->m_marbleWidget->removeEventFilter(pLayer);
			d->m_marbleWidget->installEventFilter(pLayer);
		}
		else if ( pLayer != NULL && !set)
		{
			pLayer->setInputEnabled(true);
		}
	}

	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iterMap = d->m_mvMapLayers.constBegin();
    for ( ; iterMap != d->m_mvMapLayers.constEnd(); ++iterMap ) 
	{
		pMapLayer = iterMap.value();
        if ( pMapLayer != NULL && set) 
		{
			pMapLayer->setInputEnabled(false);
			d->m_marbleWidget->removeEventFilter(pMapLayer);
			d->m_marbleWidget->installEventFilter(pMapLayer);
		}
		else if ( pMapLayer != NULL && !set)
		{
			pMapLayer->setInputEnabled(true);
		}
	}

	MapViewShadingLayer* pShadingLayer=NULL;
    QMap<qreal,MapViewShadingLayer*>::const_iterator iterShading = d->m_mvShadingLayers.constBegin();
    for ( ; iterShading != d->m_mvShadingLayers.constEnd(); ++iterShading ) 
	{
		pShadingLayer = iterShading.value();
        if ( pShadingLayer != NULL && set) 
		{
			pShadingLayer->setInputEnabled(false);
			d->m_marbleWidget->removeEventFilter(pShadingLayer);
			d->m_marbleWidget->installEventFilter(pShadingLayer);
		}
		else if ( pShadingLayer != NULL && !set)
		{
			pShadingLayer->setInputEnabled(true);
		}
	}

	MapViewTSLayer* pTSLayer=NULL;
    QMap<qreal,MapViewTSLayer*>::const_iterator iterTS = d->m_mvTsLayers.constBegin();
    for ( ; iterTS != d->m_mvTsLayers.constEnd(); ++iterTS ) 
	{
		pTSLayer = iterTS.value();
        if ( pTSLayer != NULL && set) 
		{
			pTSLayer->setInputEnabled(false);
			d->m_marbleWidget->removeEventFilter(pTSLayer);
			d->m_marbleWidget->installEventFilter(pTSLayer);
		}
		else if ( pTSLayer != NULL && !set)
		{
			pTSLayer->setInputEnabled(true);
		}
	}

	if(set)
	{
		d->m_marbleWidget->removeEventFilter((QObject*)d->m_marbleWidget->routingLayer());
		d->m_marbleWidget->installEventFilter((QObject*)d->m_marbleWidget->routingLayer());
	}
	
}

void TreeViewWidget::exportToLocationMap(QString& filename)
{
	//<?xml version="1.0" encoding="UTF-8"?>

	QString qsMap="map%1";
	QString qsName;
	int iC=1;

	QFile fp1(filename);
	//fp1.setFileName(qsXml);
	if (!fp1.open(QIODevice::WriteOnly))  
		return;

	QXmlStreamWriter xmlWriter(&fp1);  
	xmlWriter.setAutoFormatting(true);  

	xmlWriter.writeStartDocument();					//xml  
//	xmlWriter.writeStartElement("?xml");  
//	xmlWriter.writeAttribute("version", "v1.0");  
//	xmlWriter.writeAttribute("encoding", "UTF-8");    

/*	if(!m_qsGeoset.isEmpty())
	{
		qsMap=qsMap.arg(iC);
		xmlWriter.writeStartElement(qsMap);
		xmlWriter.writeStartElement("name");  
		xmlWriter.writeCharacters(m_qsGeoset);  
		xmlWriter.writeEndElement();   //name
		xmlWriter.writeEndElement();   //max[n]
	}*/

	xmlWriter.writeStartElement("LocationMap"); 

	// iterate all map layers
	MapViewMapLayer* pMapLayer=NULL;
    QMap<qreal,MapViewMapLayer*>::const_iterator iter = d->m_mvMapLayers.constBegin();
    for ( ; iter != d->m_mvMapLayers.constEnd(); ++iter ) 
	{
		pMapLayer = iter.value();
        if ( pMapLayer != NULL ) 
		{
			qint64 qID = (qint64)iter.key();
			TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(qID);
			
			if(pNode!=NULL)// && pNode->m_bMap)//get all geoset layers
			{
				qsMap="map%1";
				qsMap=qsMap.arg(iC);
				
				if(!pNode->m_qsDesc.isEmpty())
					qsName = pNode->m_qsDesc;
				else qsName = pNode->m_qsName;
				if(pNode->m_bGeoSet)
					qsName=m_qsGeoset;
				xmlWriter.writeStartElement(qsMap);
				xmlWriter.writeStartElement("name");  
				xmlWriter.writeCharacters(qsName);  
				xmlWriter.writeEndElement();   //name
				xmlWriter.writeEndElement();   //max[n]
				iC++;
			}
		}
	}

	xmlWriter.writeEndElement();//LocationMap
	xmlWriter.writeEndElement();//xml

	fp1.close();
}

void TreeViewWidget::exportToKml(long id,QString& filename)
{
	m_selectedID = (qint64)id;
	exportLayer(filename);
}

void TreeViewWidget::exportLayer(QString &filename)
{
	TreeNode* pNode=NULL;
	TreeNode* pNodeObject=NULL;

	if(m_selectedID==0)
	{
		QModelIndex idx = d->m_ui.treeTreeView->currentIndex(); 
		if(!idx.isValid())
			return;
		pNodeObject = d->m_treeViewModel->getNodeFromIndex(idx);
	}
	else
	{
		pNode = d->m_treeViewModel->getNodeFromIndexId(m_selectedID);
		if(pNode==NULL)return;
		if(pNode->m_bMap)return;
	}
		
	if(pNode!=NULL && pNode->ItemType == Overlay)
	{
		GeoDataDocument* document = new GeoDataDocument;
		QString qsOverlay = pNode->m_qsName;
		document->setName(qsOverlay);
//		GeoDataDocument container;

		MapViewLayer* pLayer=NULL;
		MapViewTSLayer* pTsLayer=NULL;
		MapViewShadingLayer* pShadingLayer=NULL;
		TreeNode* pLayerNode=NULL;

		int iLCount = pNode->children.count();
		qint64 id=0;

		for(int iL = 0;iL < iLCount; iL++)
		{
			pLayerNode = pNode->children.at(iL);
			if(pLayerNode!=NULL&&!pLayerNode->m_bTactical)
			{
				id = (qint64)pLayerNode->m_lId;
				if(d->m_mvLayers.contains(id) )
				{
					pLayer = d->m_mvLayers.value(id);
					if(pLayer!=NULL)
					{
						GeoDataFolder* result = new GeoDataFolder;
						if ( result ) 
						{
							result->setName(pLayerNode->m_qsName);
							pLayer->exportOverlay2kml(document,result);
							document->append( result );
						}
					}
				}
				else if(d->m_mvShadingLayers.contains(id) )
				{
					pShadingLayer = d->m_mvShadingLayers.value(id);
					if(pShadingLayer!=NULL)
					{
						GeoDataFolder* result = new GeoDataFolder;
						if ( result ) 
						{
							document->setFileName(filename);
							result->setName(pLayerNode->m_qsName);
							pShadingLayer->exportOverlay2kml(document,result);
							document->append( result );
						}
					}
				}

			}
			else if(pLayerNode!=NULL&&pLayerNode->m_bTactical)
			{
				id = (qint64)pLayerNode->m_lId;
				if(d->m_mvTsLayers.contains(id) )
				{
					pTsLayer = d->m_mvTsLayers.value(id);
					if(pTsLayer!=NULL)
					{
						GeoDataFolder* result = new GeoDataFolder;
						if ( result ) 
						{
							result->setName(pLayerNode->m_qsName);
							pTsLayer->exportOverlay2kml(document,result);
							document->append(result );
						}
					}
				}

			}
		}

//		container.append( new GeoDataDocument( *document ) );
		GeoWriter writer;
		writer.setDocumentType( "http://earth.google.com/kml/2.2" );

	   // QString const filename = "test.kml";
		QFile file( filename );
		if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
			qCritical() << "Cannot write to " << file.fileName();
		}

		if ( !writer.write( &file, document ) ) {
			qCritical() << "Can not write to " << file.fileName();
		}

		//writer.writeElement(
		file.close();

		delete document;

	}
	else if(pNode!=NULL && pNode->ItemType == Layer)
	{
		GeoDataDocument* document = new GeoDataDocument;
		QString qsOverlay = pNode->parent->m_qsName;
		document->setName(qsOverlay);

		if(!pNode->m_bTactical && d->m_mvLayers.contains(m_selectedID) )
		{
			MapViewLayer* pLayer = d->m_mvLayers.value(m_selectedID);
			if(pLayer==NULL)
				return;

			GeoDataFolder* result = new GeoDataFolder;
			if ( result ) 
			{
				result->setName(pNode->m_qsName);
				pLayer->exportOverlay2kml(document,result);
				document->append( result );
			}
			//pLayer->export2kml(filename);
			GeoWriter writer;
			writer.setDocumentType( "http://earth.google.com/kml/2.2" );

		   // QString const filename = "test.kml";
			QFile file( filename );
			if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
				qCritical() << "Cannot write to " << file.fileName();
			}

			if ( !writer.write( &file, document ) ) {
				qCritical() << "Can not write to " << file.fileName();
			}

			//writer.writeElement(
			file.close();

			delete document;
		}
		if(!pNode->m_bTactical && d->m_mvShadingLayers.contains(m_selectedID) )
		{
			MapViewShadingLayer* pShadingLayer = d->m_mvShadingLayers.value(m_selectedID);
			if(pShadingLayer==NULL)
				return;

			GeoDataFolder* result = new GeoDataFolder;
			if ( result ) 
			{
				result->setName(pNode->m_qsName);
				pShadingLayer->exportOverlay2kml(document,result);
				document->append( result );
			}
			//pLayer->export2kml(filename);
			GeoWriter writer;
			writer.setDocumentType( "http://earth.google.com/kml/2.2" );

		   // QString const filename = "test.kml";
			QFile file( filename );
			if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
				qCritical() << "Cannot write to " << file.fileName();
			}

			if ( !writer.write( &file, document ) ) {
				qCritical() << "Can not write to " << file.fileName();
			}

			//writer.writeElement(
			file.close();

			delete document;
		}
		else if(pNode->m_bTactical && d->m_mvTsLayers.contains(m_selectedID) )
		{
			MapViewTSLayer* pLayer = d->m_mvTsLayers.value(m_selectedID);
			if(pLayer==NULL)
				return;

			pLayer->export2kml(filename);
		}
	}
	else if(pNodeObject!=NULL)
	{
		TreeNode* pLayerNode=NULL;
		if(pNodeObject->ItemType == Group)
			pLayerNode = pNodeObject->parent;
		else pLayerNode = pNodeObject->parent->parent;
		if(pLayerNode==NULL)return;

		GeoDataDocument* document = new GeoDataDocument;
		QString qsOverlay = pLayerNode->parent->m_qsName;
		document->setName(qsOverlay);

		if(!pLayerNode->m_bTactical && d->m_mvLayers.contains((qint64)(pLayerNode->m_lId)) )
		{
			MapViewLayer* pLayer = d->m_mvLayers.value((qint64)(pLayerNode->m_lId));
			if(pLayer==NULL)
				return;

			GeoDataFolder* result = new GeoDataFolder;
			if ( result ) 
			{
				result->setName(pLayerNode->m_qsName);
				if(pNodeObject->ItemType == Group)
					pLayer->exportGroup2kml(document,result,pNodeObject->m_lId,0);
				else
					pLayer->exportGroup2kml(document,result,pNodeObject->parent->m_lId,pNodeObject->m_lId);
				document->append( result );
			}
			//pLayer->export2kml(filename);
			GeoWriter writer;
			writer.setDocumentType( "http://earth.google.com/kml/2.2" );

		   // QString const filename = "test.kml";
			QFile file( filename );
			if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
				qCritical() << "Cannot write to " << file.fileName();
			}

			if ( !writer.write( &file, document ) ) {
				qCritical() << "Can not write to " << file.fileName();
			}

			//writer.writeElement(
			file.close();

			delete document;
		}

	}

}

void TreeViewWidget::enablePanMapToObject(bool enab)
{
	m_bFocusObject=enab;
	if(m_bFocusObject && m_focusID>0)
	{
		if(d->m_treeViewModel!=NULL)
		{
			QModelIndex idx = d->m_treeViewModel->getIndexFromIndexId(m_focusID);
			if(idx.isValid())
				zoomSelectedItem (idx);
		}
		
		if(m_pFocusTimer!=NULL && !m_pFocusTimer->isActive())
			m_pFocusTimer->start();
	}
	else
	{
		if(m_pFocusTimer!=NULL)
			m_pFocusTimer->stop();
	}
}

bool TreeViewWidget::calcPlotCoords(qint64 index,const GeoDataCoordinates& src,const GeoDataCoordinates& dest)
{
		
	if(index>0)
	{
		Model::TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(index);
		if(pNode!=NULL && pNode->parent!=NULL)
		{
			GeoDataLineString gdl;
			gdl << pNode->m_poly;
			if(gdl.size()<2)return false;

			pNode->m_poly.clear();
			
			//move the polygone
			GeoDataCoordinates pos;
			qreal mLon = src.longitude();
			qreal mLat = src.latitude();
			
			//pos.set(mLon,mLat,0);//mouse pos
			qreal dLon = mLon-dest.longitude();
			qreal dLat = mLat-dest.latitude();
			for(int i=0;i<gdl.size();i++)
			{
				pos.set(gdl.at(i).longitude()-dLon,gdl.at(i).latitude()-dLat,0);
				pNode->m_poly.append(pos);
			}

			pNode->m_position.set(dest.longitude(),dest.latitude(),0);
		}//pNode

	}//index
		
	return true;
}

/********* 3D view specials **********************************************/
void TreeViewWidget::slot3Denable(bool enable)
{
	m_b3D=enable;
	sendObjectsTo3D(m_b3D);
}

void TreeViewWidget::sendObjectsTo3D(bool send)
{
	if(!send)return;

	qint64 qID=0;
	qint64 layerID=0;
	int iGC = 0;
	int iCC = 0;

	double lon=0;
	double lat=0;

	QColor col;
	long fillcol=0;
	long linecol=0;

	MapViewLayer* pLayer=NULL;
    QMap<qreal,MapViewLayer*>::const_iterator iter = d->m_mvLayers.constBegin();
    for ( ; iter != d->m_mvLayers.constEnd(); ++iter ) 
	{
		pLayer = iter.value();
		layerID = (qint64)iter.key();
        if ( pLayer != NULL && layerID) 
		{
			Model::TreeNode* pNode = d->m_treeViewModel->getNodeFromIndexId(layerID);
			if(pNode!=NULL)
			{
				iGC = pNode->children.count();

				for(int ig=0;ig<iGC;ig++)
				{
					Model::TreeNode* pGroup = pNode->children.at(ig);
					if(pGroup!=NULL)
					{
						iCC = pGroup->children.count();
						for(int ic=0;ic<iCC;ic++)
						{
							Model::TreeNode* pChild = pGroup->children.at(ic);
							if(pChild!=NULL)
							{
								lon = pChild->m_position.longitude(GeoDataCoordinates::Degree);
								lat = pChild->m_position.latitude(GeoDataCoordinates::Degree);

								if(pChild->m_lFillCol>=0)
									fillcol = pChild->m_lFillCol;
								else 
								{
									col = pLayer->getFillColor();
									fillcol = col.blue();
									fillcol <<=8;
									fillcol += col.green();
									fillcol <<=8;
									fillcol += col.red();
								}
								if(pChild->m_lLineCol>=0)
									linecol = pChild->m_lLineCol;
								else 
								{
									col = pLayer->getLineColor();
									linecol = col.blue();
									linecol <<=8;
									linecol += col.green();
									linecol <<=8;
									linecol += col.red();
								}

								if(pChild->ItemType==Icon)
								{
									emit signal3dAddImageObject(pChild->m_lId,pChild->m_qsImage,lon,lat);
								}//Icon
								if(pChild->ItemType==Plot)
								{		
									emit signal3dAddPlot(pChild->m_lId,QStringList(),lon,lat,pChild->m_dRadius,pChild->m_poly);
						
								}//Plot
								if(pChild->ItemType==PlotEx)
								{		
									emit signal3dAddPlotEx(pChild->m_lId,QStringList(),lon,lat,pChild->m_poly);
						
								}//PlotEx
								if(pChild->ItemType==Elipse)
								{		
									//type=0 - Rect, type=1 - Polygon, type=2 - Ellipse, type=3 - Polyline
									emit signal3dAddGeoObject(pChild->m_lId,2,lon,lat,fillcol,linecol,pChild->m_poly);
					
						
								}//Elipse
								if(pChild->ItemType==Rect)
								{		
									emit signal3dAddGeoObject(pChild->m_lId,0,lon,lat,fillcol,linecol,pChild->m_poly);
						
								}//Rect
								if(pChild->ItemType==Poly)
								{		
									emit signal3dAddGeoObject(pChild->m_lId,3,lon,lat,fillcol,linecol,pChild->m_poly);
						
								}//Poly
								if(pChild->ItemType==Polygone)
								{		
									emit signal3dAddGeoObject(pChild->m_lId,1,lon,lat,fillcol,linecol,pChild->m_poly);
						
								}//Polygone
								if(pChild->ItemType==Azimuth)
								{
									emit signal3dAddAzimuth(pChild->m_lId,lon,lat,0,pChild->m_poly);
								}//Azimuth

								if(pChild->ItemType==Desc)
								{
									emit signal3dAddText(pChild->m_lId,pChild->m_qsDesc,lon,lat,0,pChild->m_iXoff,pChild->m_iYoff);
								}//Desc
								if(pChild->ItemType==Text)
								{
									emit signal3dAddText(pChild->m_lId,pChild->m_qsDesc,lon,lat,0,pChild->m_iXoff,pChild->m_iYoff);
								}//Text
							}//pChild
						}//iCC
					}//pGroup
				}//iGC

			}//pNode

		}//pLayer
	}//iter
}

								
} // namespace Marble

#include "moc_TreeViewWidget.cpp"
